@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/calma/CalmaWrite.c,v 1.4 2001/07/23 17:49:49 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
