@@ -23,7 +23,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFhier.c,v 1.3 2001/03/29 19:50:04 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
