@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdtech.c,v 1.5 2001/07/23 17:49:50 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -304,7 +304,7 @@ cifReadStyleInit()
     cifCurReadStyle->crs_scaleFactor = 0;
     HashInit(&(cifCurReadStyle->cifCalmaToCif), 64,
       sizeof (CalmaLayerType) / sizeof (unsigned));
-    for (i=0; i<MAXCIFRLAYERS; i+=1)
+    for (i = 0; i < MAXCIFRLAYERS; i++)
     {
 	cifCurReadStyle->crs_labelLayer[i] = TT_SPACE;
 	cifCurReadStyle->crs_layers[i] = NULL;
@@ -546,7 +546,7 @@ CIFReadTechLine(sectionName, argc, argv)
     }
 
     /* Process mapping between CIF layers and calma layers/types */
-    if (strcmp(argv[0], "calma") == 0)
+    if ((strcmp(argv[0], "calma") == 0) || (strncmp(argv[0], "gds", 3) == 0))
     {
 	int cifnum;
 
@@ -744,8 +744,6 @@ CIFReadTechFinal()
 
     if (cifCurReadStyle == NULL) return;
 
-    CIFTechInputScale(1, 1);
-
     /* Make sure the current style has a valid scalefactor. */
 
     if (cifCurReadStyle->crs_scaleFactor <= 0)
@@ -755,10 +753,12 @@ CIFReadTechFinal()
 	 cifCurReadStyle->crs_scaleFactor = 1;
     }
 
+    CIFTechInputScale(1, 1);
+
     /* debug info --- Tim, 1/3/02 */
-    /* TxPrintf("Input style %s: scaleFactor=%d, multiplier=%d\n",
+    TxPrintf("Input style %s: scaleFactor=%d, multiplier=%d\n",
 		cifCurReadStyle->crs_name, cifCurReadStyle->crs_scaleFactor,
-		cifCurReadStyle->crs_multiplier); */
+		cifCurReadStyle->crs_multiplier);
 }
 
 /*
@@ -789,7 +789,7 @@ CIFReadLoadStyle(stylename)
     cifCurReadStyle->crs_name = stylename;
 
     invcifr = TechSectionGetMask("cifinput", NULL);
-    TechLoad(TechDefault, invcifr);
+    TechLoad(NULL, invcifr);
 
     CIFReadTechFinal();
     CIFTechInputScale(DBLambda[0], DBLambda[1]);
