@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdutils.c,v 1.6 2001/06/15 17:53:20 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -31,24 +31,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFrdutils.c,v 1.6 200
 #include "database/database.h"
 #include "cif/CIFint.h"
 #include "cif/CIFread.h"
+#include "cif/cif.h"
 #include "textio/textio.h"
 #include "signals/signals.h"
 #include "undo/undo.h"
 #include "utils/malloc.h"
 
-#define	CIF_ZERO	0
-#define	CIF_LEFT	1
-#define	CIF_UP		2
-#define	CIF_RIGHT	3
-#define	CIF_DOWN	4
-#define	CIF_DIAG	5
-
-/* Specific diagonal directions */
-#define CIF_DIAG_UL     5
-#define CIF_DIAG_UR     6
-#define CIF_DIAG_DL     7
-#define CIF_DIAG_DR     8
-
 /* The following variables are used to provide one character of
  * lookahead.  cifParseLaAvail is TRUE if cifParseLaChar contains
  * a valid character, FALSE otherwise.  The PEEK and TAKE macros
@@ -65,7 +53,10 @@ int cifParseLaChar = EOF;
  */
 
 FILE *cifInputFile;
+FILE *cifErrorFile;
 int cifLineNumber;		/* Number of current line. */
+int cifTotalWarnings;		/* Number of warnings detected */
+int cifTotalErrors;		/* Number of errors detected */
 
 /* The variables used below hold general information about what
  * we're currently working on.
@@ -73,6 +64,26 @@ int cifLineNumber;		/* Number of current line. */
 
 int cifReadScale1;			/* Scale factor:  multiply by Scale1 */
 int cifReadScale2;			/* then divide by Scale2. */
+int CIFRescaleLimit = CIFMAXRESCALE;	/* Don't increase cifReadScale1 by more
+					 * than this amount;  internal units
+					 * finer than this will be rounded.
+					 */
+int CIFRescaleAllow = TRUE;		/* Don't subdivide the magic internal
+					 * grid if this is FALSE.
+					 */
+char *CIFErrorFilename;			/* Name of file for error redirection */
+
+#ifdef NONMANHATTAN
+int  CifPolygonCount;			/* Count of generated subcells
+					 * containing polygons.  This number
+					 * is used to create a unique cell name.
+					 */
+bool CIFSubcellPolygons = FALSE;	/* If TRUE, each non-Manhattan polygon
+					 * will be put in a separate subcell
+					 * to avoid too much tile splitting
+					 */
+#endif
+
 Plane *cifReadPlane;			/* Plane into which to paint material
 					 * NULL means no layer command has
 					 * been seen for the current cell.
@@ -104,8 +115,17 @@ CIFReadError(format, arg1, arg2, arg3)
     char *format;		/* Text format string. */
     char *arg1, *arg2, *arg3;	/* Additional arguments, if needed. */
 {
-    TxError("Error at line %d of CIF file: ", cifLineNumber);
-    TxError(format, arg1, arg2, arg3);
+    cifTotalErrors++;
+    if (CIFWarningLevel == CIF_WARN_NONE) return;
+    if ((cifTotalErrors < 100) || (CIFWarningLevel != CIF_WARN_LIMIT))
+    {
+	TxError("Error at line %d of CIF file: ", cifLineNumber);
+	TxError(format, arg1, arg2, arg3);
+    }
+    else if ((cifTotalErrors == 100) && (CIFWarningLevel == CIF_WARN_LIMIT))
+    {
+	TxError("Error limit set:  Remaining errors will not be reported.\n");
+    }
 }
 
 
@@ -115,8 +135,17 @@ CIFReadWarning(format, arg1, arg2, arg3)
     char *format;		/* Text format string. */
     char *arg1, *arg2, *arg3;	/* Additional arguments, if needed. */
 {
-    TxError("Warning at line %d of CIF file: ", cifLineNumber);
-    TxError(format, arg1, arg2, arg3);
+    cifTotalWarnings++;
+    if (CIFWarningLevel == CIF_WARN_NONE) return;
+    if ((cifTotalWarnings < 100) || (CIFWarningLevel != CIF_WARN_LIMIT))
+    {
+	TxError("Warning at line %d of CIF file: ", cifLineNumber);
+	TxError(format, arg1, arg2, arg3);
+    }
+    else if ((cifTotalWarnings == 100) && (CIFWarningLevel == CIF_WARN_LIMIT))
+    {
+	TxError("Warning limit set:  Remaining warnings will not be reported.\n");
+    }
 }
 
 /*
@@ -155,6 +184,12 @@ CIFScaleCoord(cifCoord, snap_type)
 {
     int result, scale, remain, denom;
 
+    /* If internal grid subdivision is disallowed, always round to the	*/
+    /* nearest grid unit.						*/
+
+    if (!CIFRescaleAllow)
+	snap_type = COORD_ANY;
+
     scale = cifCurReadStyle->crs_scaleFactor;
 
     /* Check for non-integer result and warn of fractional-lambda violation */
@@ -512,12 +547,24 @@ CIFParsePoint(pointp, iscale)
     if (pointp->p_x % cifReadScale2 != 0)
     {
 	rescale = cifReadScale2 / FindGCF(cifReadScale2, abs(pointp->p_x));
-	cifCurReadStyle->crs_scaleFactor *= rescale;
-	cifCurReadStyle->crs_multiplier *= rescale;
-	cifReadScale1 *= rescale;
-	CIFReadWarning("CIF units rescaled by factor of %d\n", rescale);
-	CIFScalePlanes(rescale, 1);
-	pointp->p_x *= rescale;
+	if ((cifReadScale1 * rescale) > CIFRescaleLimit)
+	{
+	    CIFReadWarning("CIF units at maximum scale; value is rounded\n");
+	    /* prepare for nearest-integer rounding */
+	    if (pointp->p_x < 0)
+		pointp->p_x -= ((cifReadScale2 - 1) >> 1);
+	    else
+		pointp->p_x += (cifReadScale2  >> 1);
+	}
+	else
+	{
+	    cifCurReadStyle->crs_scaleFactor *= rescale;
+	    cifCurReadStyle->crs_multiplier *= rescale;
+	    cifReadScale1 *= rescale;
+	    CIFReadWarning("CIF units rescaled by factor of %d\n", rescale);
+	    CIFScalePlanes(rescale, 1);
+	    pointp->p_x *= rescale;
+	}
     }
     pointp->p_x /= cifReadScale2;
     if (!CIFParseSInteger(&pointp->p_y))
@@ -526,13 +573,25 @@ CIFParsePoint(pointp, iscale)
     if (pointp->p_y % cifReadScale2 != 0)
     {
 	rescale = cifReadScale2 / FindGCF(cifReadScale2, abs(pointp->p_y));
-	cifCurReadStyle->crs_scaleFactor *= rescale;
-	cifCurReadStyle->crs_multiplier *= rescale;
-	cifReadScale1 *= rescale;
-	CIFReadWarning("CIF units rescaled by factor of %d\n", rescale);
-	CIFScalePlanes(rescale, 1);
-	pointp->p_x *= rescale;
-	pointp->p_y *= rescale;
+	if ((cifReadScale1 * rescale) > CIFRescaleLimit)
+	{
+	    CIFReadWarning("CIF units at maximum scale; value is rounded\n");
+	    /* prepare for nearest-integer rounding */
+	    if (pointp->p_y < 0)
+		pointp->p_y -= ((cifReadScale2 - 1) >> 1);
+	    else
+		pointp->p_y += (cifReadScale2  >> 1);
+	}
+	else
+	{
+	    cifCurReadStyle->crs_scaleFactor *= rescale;
+	    cifCurReadStyle->crs_multiplier *= rescale;
+	    cifReadScale1 *= rescale;
+	    CIFReadWarning("CIF units rescaled by factor of %d\n", rescale);
+	    CIFScalePlanes(rescale, 1);
+	    pointp->p_x *= rescale;
+	    pointp->p_y *= rescale;
+	}
     }
     pointp->p_y /= cifReadScale2;
     return TRUE;
@@ -771,6 +830,7 @@ path_intersect(pathHead, start, respt)
     bool does_cross = FALSE, diagonal = FALSE;
     int tdist, newdist;
 
+    tdist = INFINITY;
     for (path = pathHead; path->cifp_next; path = path->cifp_next)
     {
 	/* don't compare with self */
@@ -943,9 +1003,11 @@ is_clockwise(pathHead)
  */
 
 void
-CIFMakeManhattanPath(pathHead, plane)
+CIFMakeManhattanPath(pathHead, plane, resultTbl, ui)
     CIFPath *pathHead;
     Plane *plane;
+    PaintResultType *resultTbl;
+    PaintUndoInfo *ui;
 {
     register CIFPath *new, *new2, *next, *path;
     int xinit, xdiff, xincr, xlast, x;
@@ -987,12 +1049,12 @@ CIFMakeManhattanPath(pathHead, plane)
 	    last = next;
 	}
 	edir = CIFEdgeDirection(first, last);
-	if (edir == CIF_DIAG_DR || edir == CIF_DIAG_UL)
+	if (edir == CIF_DIAG_DL || edir == CIF_DIAG_UR)
 	{
 	    new->cifp_x = first->cifp_x; 
 	    new->cifp_y = last->cifp_y;
 	}
-	else  /* edir == CIF_DIAG_UR || edir == CIF_DIAG_DL */
+	else  /* edir == CIF_DIAG_DR || edir == CIF_DIAG_UL */
 	{
 	    new->cifp_x = last->cifp_x;
 	    new->cifp_y = first->cifp_y;
@@ -1028,8 +1090,8 @@ CIFMakeManhattanPath(pathHead, plane)
 
 	/* Break out the diagonal tile from the polygon and paint it. */
 
-        type = (edir == CIF_DIAG_UL || edir == CIF_DIAG_DL) ? 0 : TT_SIDE;
-        type |= (edir == CIF_DIAG_UL || edir == CIF_DIAG_DR) ? 0 : TT_DIRECTION;
+        type = (edir == CIF_DIAG_UR || edir == CIF_DIAG_UL) ? 0 : TT_SIDE;
+        type |= (edir == CIF_DIAG_UR || edir == CIF_DIAG_DL) ? 0 : TT_DIRECTION;
 	type |= TT_DIAGONAL;
 
 	tt.r_ll = path->cifp_point;
@@ -1043,8 +1105,7 @@ CIFMakeManhattanPath(pathHead, plane)
 */
 
         if (plane)
-            DBNMPaintPlane(plane, type, &tr, CIFPaintTable,
-		(PaintUndoInfo *)NULL);
+            DBNMPaintPlane(plane, type, &tr, resultTbl, ui);
     }
 }
 
@@ -1200,7 +1261,7 @@ CIFMakeManhattanPath(pathHead)
     }
 }
 
-#endif NONMANHATTAN
+#endif /* NONMANHATTAN */
 
 
 /*
@@ -1228,26 +1289,27 @@ int
 CIFEdgeDirection(first, last)
     CIFPath *first, *last;		/* Edge to be categorized. */
 {
+
     if (first->cifp_x < last->cifp_x)
     {
 	if (first->cifp_y < last->cifp_y)
-	    return CIF_DIAG_UL;
-	if (first->cifp_y > last->cifp_y)
 	    return CIF_DIAG_UR;
-	return CIF_UP;
+	if (first->cifp_y > last->cifp_y)
+	    return CIF_DIAG_DR;
+	return CIF_RIGHT;
     }
     if (first->cifp_x > last->cifp_x)
     {
 	if (first->cifp_y < last->cifp_y)
-	    return CIF_DIAG_DL;
+	    return CIF_DIAG_UL;
 	if (first->cifp_y > last->cifp_y)
-	    return CIF_DIAG_DR;
-	return CIF_DOWN;
+	    return CIF_DIAG_DL;
+	return CIF_LEFT;
     }
     if (first->cifp_y < last->cifp_y)
-	return CIF_RIGHT;
+	return CIF_UP;
     if (first->cifp_y > last->cifp_y)
-	return CIF_LEFT;
+	return CIF_DOWN;
     return CIF_ZERO;
 }
 
@@ -1651,6 +1713,12 @@ FILE *file;			/* File from which to read CIF. */
     TxPrintf("Warning: CIF reading is not undoable!  I hope that's OK.\n");
     UndoDisable();
 
+    cifTotalWarnings = 0;
+    cifTotalErrors = 0;
+#ifdef NONMANHATTAN
+    CifPolygonCount = 0;
+#endif
+
     cifInputFile = file;
     cifReadScale1 = 1;
     cifReadScale2 = 1;
