@@ -23,10 +23,10 @@
 
 #ifndef	_DATABASE
 int err1 = Need_to_include_database/database_header;
-#endif	_DATABASE
+#endif	/* _DATABASE */
 #ifndef	_CIFINT
 int err1 = Need_to_include_cifint_header;
-#endif	_CIFINT
+#endif	/* _CIFINT */
 
 /* The structures below are built up by CIFreadtech.c to describe
  * various styles for reading CIF.
@@ -120,6 +120,19 @@ typedef struct cifrstyle
  * or polygon boundaries.  These are just linked lists of points.
  */
 
+#define CIF_ZERO        0
+#define CIF_LEFT        1
+#define CIF_UP          2
+#define CIF_RIGHT       3
+#define CIF_DOWN        4
+#define CIF_DIAG        5
+
+/* Specific diagonal directions */
+#define CIF_DIAG_UL     5
+#define CIF_DIAG_UR     6
+#define CIF_DIAG_DL     7
+#define CIF_DIAG_DR     8
+
 typedef struct cifpath
 {
     Point cifp_point;		/* A point in the path. */
