@@ -19,13 +19,14 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/cif/CIFtech.c,v 1.4 2001/07/23 17:49:50 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
 #ifdef SYSV
 #include <string.h>
 #endif
+#include <stdlib.h>	/* for atof() */
 
 #include "misc/magic.h"
 #include "utils/geometry.h"
@@ -536,6 +537,11 @@ wrongNumArgs:
 	newLayer->cl_flags = 0;
 	newLayer->cl_calmanum = newLayer->cl_calmatype = -1;
 	newLayer->min_width = 0; /* for growSlivers */
+#ifdef THREE_D
+	newLayer->cl_height = 0.0;
+	newLayer->cl_thick = 0.0;
+	newLayer->cl_renderStyle = 0;
+#endif
 	if (strcmp(argv[0], "templayer") == 0)
 	    newLayer->cl_flags |= CIF_TEMP;
 	cifCurLayer = newLayer;
@@ -580,7 +586,7 @@ wrongNumArgs:
 	return TRUE;
     }
 
-    if (strcmp(argv[0], "calma") == 0)
+    if ((strcmp(argv[0], "calma") == 0) || (strncmp(argv[0], "gds", 3) == 0))
     {
 	if (cifCurLayer == NULL)
 	{
@@ -609,11 +615,64 @@ wrongNumArgs:
 	CIFCurStyle->cs_growSlivers = TRUE;
 	return TRUE;
     }
-    /* miscellaneous cif/calma-writing boolean options */
-    if(strcmp(argv[0], "options") == 0) {
+
+    if (strcmp(argv[0], "render") == 0) /* used by specialopen wind3d client */
+    {
+#ifdef THREE_D
+	float height, thick;
+	int i, style, lcnt;
+	CIFLayer *layer;
+
+	if (argc != 5) goto wrongNumArgs;
+
+	cifCurLayer = NULL;	/* This is not in a layer definition */
+
+	style = DBWTechParseStyle(argv[2]);
+	if (style < 0)
+	{
+	    TechError("Error:  Bad render style for CIF layer.\n");
+	    goto errorReturn;
+	}
+
+	if (!StrIsNumeric(argv[3]) || !StrIsNumeric(argv[4]))
+	{
+	    TechError("Syntax: render <layer> <style> <height> <thick>\n");
+	    goto errorReturn;
+	}
+	height = (float)atof(argv[3]);
+	thick = (float)atof(argv[4]);
+
+	lcnt = 0;
+	for (i = 0; i < CIFCurStyle->cs_nLayers; i++)
+	{
+	    layer = CIFCurStyle->cs_layers[i];
+	    if (!strcmp(argv[1], layer->cl_name))
+	    {
+		layer->cl_height = height;
+		layer->cl_thick = thick;
+		layer->cl_renderStyle = style;
+		lcnt++;
+	    }
+	}
+	if (lcnt == 0)
+	{
+	    TechError("Unknown layer name.\n");
+	    goto errorReturn;
+	}
+#endif
+        return TRUE;
+    }
+
+    /* 
+     * miscellaneous cif/calma-writing boolean options
+     */
+
+    if (strcmp(argv[0], "options") == 0)
+    {
 	int i;
 	if (argc < 2) goto wrongNumArgs;
-	for(i = 1; i < argc; i++) {
+	for (i = 1; i < argc; i++)
+	{
 	    if(strcmp(argv[i], "calma-permissive-labels") == 0)
 		CIFCurStyle->cs_flags |= CWF_PERMISSIVE_LABELS;
 	}
@@ -1210,7 +1269,8 @@ CIFTechFinal()
 	    TxPrintf("Layer %s must be processed hierarchically.\n",
 			style->cs_layers[i]->cl_name);
     }
-    */
+
+     */
 }
 
 /*
@@ -1241,7 +1301,7 @@ CIFLoadStyle(stylename)
     CIFCurStyle->cs_name = stylename;
 
     invcif = TechSectionGetMask("cifoutput", NULL);
-    TechLoad(TechDefault, invcif);
+    TechLoad(NULL, invcif);
 
     CIFTechFinal();
     CIFTechOutputScale(DBLambda[0], DBLambda[1]);
