@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdCD.c,v 1.5 2001/01/12 22:12:16 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #ifdef SYSV
@@ -47,6 +47,11 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdCD.c,v 1.5 200
 #include "select/select.h"
 #include "utils/malloc.h"
 
+#ifdef NONMANHATTAN
+#include "cif/CIFint.h"
+#include "cif/CIFread.h"
+#endif
+
 /* The following structure is used by CmdCorner to keep track of
  * areas to be filled.
  */
@@ -67,9 +72,10 @@ int cmdDumpFunc();
  *
  * CmdCalma --
  *
- * Implement the "calma" command.
+ * Implement the "gds" or "calma" command.
  *
  * Usage:
+ *	gds option args
  *	calma option args
  *
  * Results:
@@ -90,19 +96,30 @@ int cmdDumpFunc();
 #define	CALMA_NOFLATTEN	4
 #define	CALMA_NOLABELS	5
 #define	CALMA_NOLOWER	6
-#define	CALMA_READ	7
-#define	CALMA_WRITE	8
+#define CALMA_READ	7
+#define CALMA_RESCALE	8
+#define CALMA_WARNING	9
+#define CALMA_WRITE	10
+#ifdef NONMANHATTAN
+#define CALMA_POLYS	11
+#endif
+
+#define CALMA_WARN_HELP CIF_WARN_END	/* undefined by CIF module */
 
 	/* ARGSUSED */
 CmdCalma(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    int option;
-    char **msg, *namep;
+    int option, ext;
+    char **msg, *namep, *dotptr;
     CellDef *rootDef;
     FILE *f;
 
+    static char *gdsExts[] = {".gds", ".gds2", ".strm", "", NULL};
+    static char *cmdCalmaYesNo[] = { "no", "yes", 0 };
+    static char *cmdCalmaWarnOptions[] = { "default", "none", "align",
+		"limit", "redirect", "help", 0 };
     static char *cmdCalmaOption[] =
     {	
 	"help		print this help information",
@@ -112,10 +129,16 @@ CmdCalma(w, cmd)
 	"noflatten		output arrays intact; don't flatten",
 	"nolabels		labels won't be output with GDS-II",
 	"nolower	convert all labels to upper case",
-	"read file		read Calma GDS-II format from \"file\"\n\
-			into edit cell",
-	"write file		output Calma GDS-II format to \"file\"\n\
-			for the window's root cell",
+	"read file		read Calma GDS-II format from \"file\"\n"
+	"		into edit cell",
+	"rescale [yes|no]	allow or disallow internal grid subdivision",
+	"warning [option]	set warning information level",
+	"write file		output Calma GDS-II format to \"file\"\n"
+	"		for the window's root cell",
+#ifdef NONMANHATTAN
+	"polygon subcells [yes|no]\n"
+	"		put non-Manhattan polygons into subcells",
+#endif
 	NULL
     };
 
@@ -126,7 +149,7 @@ CmdCalma(w, cmd)
 	option = Lookup(cmd->tx_argv[1], cmdCalmaOption);
 	if (option < 0)
 	{
-	    TxError("\"%s\" isn't a valid calma option.\n", cmd->tx_argv[1]);
+	    TxError("\"%s\" isn't a valid gds option.\n", cmd->tx_argv[1]);
 	    option = CALMA_HELP;
 	    cmd->tx_argc = 2;
 	}
@@ -137,7 +160,7 @@ CmdCalma(w, cmd)
     switch (option)
     {
 	case CALMA_READ: case CALMA_WRITE:
-	    windCheckOnlyWindow(&w);
+	    windCheckOnlyWindow(&w, DBWclientID);
 	    if (w == (MagWindow *) NULL)
 	    {
 		TxError("Point to a window first\n");
@@ -158,7 +181,7 @@ CmdCalma(w, cmd)
     switch (option)
     {
 	case CALMA_HELP:
-	    TxPrintf("Calma commands have the form \":calma option\",");
+	    TxPrintf("GDS commands have the form \":gds option\",");
 	    TxPrintf(" where option is one of:\n");
 	    for (msg = &(cmdCalmaOption[0]); *msg != NULL; msg++)
 	    {
@@ -168,52 +191,128 @@ CmdCalma(w, cmd)
 	    TxPrintf("If no option is given, a CALMA GDS-II stream file is\n");
 	    TxPrintf("    produced for the root cell.\n");
 	    TxPrintf("The current CIF output style (\"cif ostyle\") is used\n");
-	    TxPrintf("    to select the mask layers output by :calma write.\n");
+	    TxPrintf("    to select the mask layers output by :gds write.\n");
 	    TxPrintf("The current CIF input style (\"cif istyle\") is used\n");
-	    TxPrintf("    to select the mask layers read by :calma read.\n");
+	    TxPrintf("    to select the mask layers read by :gds read.\n");
 	    return;
 
 	case CALMA_LABELS:
 	    if (cmd->tx_argc != 2)
 	    {
 		wrongNumArgs:
-		TxError("Wrong number of arguments in \"calma\" command.");
-		TxError("  Try \":calma help\" for help.\n");
+		TxError("Wrong number of arguments in \"gds\" command.");
+		TxError("  Try \":gds help\" for help.\n");
 		return;
 	    }
 	    CalmaDoLabels = TRUE;
-	    TxPrintf("Labels will be output in .strm files\n");
+	    TxPrintf("Labels will be output in .gds files\n");
 	    return;
 
 	case CALMA_NOLABELS:
 	    if (cmd->tx_argc != 2) goto wrongNumArgs;
 	    CalmaDoLabels = FALSE;
-	    TxPrintf("Labels won't be output in .strm files\n");
+	    TxPrintf("Labels won't be output in .gds files\n");
 	    return;
 
 	case CALMA_FLATTEN:
 	    if (cmd->tx_argc != 2) goto wrongNumArgs;
 	    CalmaFlattenArrays = TRUE;
 	    TxPrintf("Each element of an array will be output individually\n");
-	    TxPrintf("in .strm files\n");
+	    TxPrintf("in .gds files\n");
 	    return;
 
 	case CALMA_NOFLATTEN:
 	    if (cmd->tx_argc != 2) goto wrongNumArgs;
 	    CalmaFlattenArrays = FALSE;
-	    TxPrintf("Arrays will be output intact in .strm files\n");
+	    TxPrintf("Arrays will be output intact in .gds files\n");
 	    return;
 
 	case CALMA_LOWER:
-	    TxPrintf("Labels in .strm file will be output as upper and lower case\n");
+	    TxPrintf("Labels in .gds file will be output as upper and lower case\n");
 	    CalmaDoLower = TRUE;
 	    return;
 
 	case CALMA_NOLOWER:
-	    TxPrintf("Labels in .strm file will be converted to upper case\n");
+	    TxPrintf("Labels in .gds file will be converted to upper case\n");
 	    CalmaDoLower = FALSE;
 	    return;
 
+#ifdef NONMANHATTAN
+	case CALMA_POLYS:
+	    if (cmd->tx_argc == 3)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CalmaSubcellPolygons));
+#else
+		if (CalmaSubcellPolygons)
+		    TxPrintf("Non-manhattan polygons placed in subcells.\n");
+		else
+		    TxPrintf("Non-manhattan polygons read as-is.\n");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 4)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[3], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CalmaSubcellPolygons = option;
+	    return;
+#endif  /* NONMANHATTAN */
+
+	case CALMA_RESCALE:
+	    if (cmd->tx_argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CIFRescaleAllow));
+#else
+		TxPrintf("Internal grid rescaling %sallowed\n",
+			(CIFRescaleAllow) ?  "" : "dis");
+#endif
+		return;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto wrongNumArgs;
+
+	    option = Lookup(cmd->tx_argv[2], cmdCalmaYesNo);
+	    if (option < 0)
+		goto wrongNumArgs;
+	    CIFRescaleAllow = option;
+	    if (!CIFRescaleAllow)
+		CIFWarningLevel = CIF_WARN_LIMIT;
+	    return;
+
+	case CALMA_WARNING:
+	    if (cmd->tx_argc == 2)
+	    {
+		TxPrintf("Warning display options: %s\n",
+			cmdCalmaWarnOptions[CIFWarningLevel]);
+	    }
+	    else
+	    {
+		int suboption = Lookup(cmd->tx_argv[2], cmdCalmaWarnOptions);
+		if (suboption < 0)
+		{
+		    TxError("\"%s\" isn't a valid gds warning option.\n",
+				cmd->tx_argv[2]);
+		    suboption = CALMA_WARN_HELP;
+		}
+		if (suboption == CALMA_WARN_HELP)
+		    TxError("Valid options are: default, align, limit, "
+			"redirect, and none.\n");
+		else
+		    CIFWarningLevel = suboption;
+
+		if (suboption == CIF_WARN_REDIRECT)
+		{
+		    if (cmd->tx_argc == 4)
+			StrDup(&CIFErrorFilename, cmd->tx_argv[3]);
+		    else
+			StrDup(&CIFErrorFilename, NULL);
+		}
+	    }
+	    return;
 	case CALMA_WRITE:
 	    if (cmd->tx_argc != 3) goto wrongNumArgs;
 	    namep = cmd->tx_argv[2];
@@ -221,20 +320,21 @@ CmdCalma(w, cmd)
 
 	case CALMA_READ:
 	    if (cmd->tx_argc != 3) goto wrongNumArgs;
-	    f = PaOpen(cmd->tx_argv[2], "r", ".strm", Path,
-		    (char *) NULL, (char **) NULL);
+
+	    /* Check for various common file extensions, including	*/
+	    /* no extension (as-is), ".gds", ".gds2", and ".strm".	*/
+
+	    for (ext = 0; gdsExts[ext] != NULL; ext++)
+		if ((f = PaOpen(cmd->tx_argv[2], "r", gdsExts[ext], Path,
+		    	(char *) NULL, (char **) NULL)) != (FILE *)NULL)
+		    break;
+
 	    if (f == (FILE *) NULL)
 	    {
-		/* Most vendors use something other than .strm for GDS */
-		/* file extensions.  See if filename is complete.      */
-
-		f = PaOpen(cmd->tx_argv[2], "r", NULL, Path,
-			(char *) NULL, (char **) NULL);
-		if (f == (FILE *) NULL) {
-		    TxError("Cannot open %s.strm or %s to read Calma GDS-II.\n",
-			    cmd->tx_argv[2], cmd->tx_argv[2]);
-		    return;
-		}
+	        TxError("Cannot open %s.gds, %s.strm or %s to read "
+			"GDS-II stream input.\n",
+			cmd->tx_argv[2], cmd->tx_argv[2], cmd->tx_argv[2]);
+	        return;
 	    }
 	    CalmaReadFile(f);
 	    (void) fclose(f);
@@ -247,10 +347,15 @@ CmdCalma(w, cmd)
      */
 
 outputCalma:
-    f = PaOpen(namep, "w", ".strm", ".", (char *) NULL, (char **) NULL);
+    dotptr = strrchr(namep, '.');
+
+    f = PaOpen(namep, "w", (dotptr == NULL) ? ".gds" : "", ".",
+		(char *) NULL, (char **) NULL);
+
     if (f == (FILE *) NULL)
     {
-	TxError("Cannot open %s.strm to write Calma stream output\n", namep);
+	TxError("Cannot open %s%s to write GDS-II stream output\n", namep,
+		(dotptr == NULL) ? ".gds" : "");
 	return;
     }
 
@@ -272,7 +377,10 @@ outputCalma:
  * parent(s), or name of the selected or indicated cell or instance.
  *
  * Usage:
- *	cellname [list] children|parents|exists|instances|celldef|self|childinst [name]
+ *	cellname [list] children|parents|exists|instances|
+ *			celldef|self|childinst|delete [name]
+ * or
+ *	cellname rename [name] [newname]
  * or
  *	cellname [list] topcells|allcells|window
  *
@@ -311,10 +419,14 @@ CmdCellname(w, cmd)
 	"allcells	list all cells",
 	"topcells	list top-level cells",
 	"window		list top-level cell of a layout window",
+	"delete		delete the named cell definition",
+	"rename		rename the indicated cell",
+	NULL
     };
     typedef enum { IDX_CHILDREN, IDX_PARENTS, IDX_EXISTS, IDX_SELF,
 		   IDX_INSTANCE, IDX_CHILDINST, IDX_CELLDEF, IDX_ALLCELLS,
-		   IDX_TOPCELLS, IDX_IN_WINDOW } optionType;
+		   IDX_TOPCELLS, IDX_IN_WINDOW, IDX_DELETE,
+		   IDX_RENAME } optionType;
 
     if (strstr(cmd->tx_argv[0], "in"))
 	func = DBUsePrint;
@@ -328,12 +440,13 @@ CmdCellname(w, cmd)
 	    locargc--;
 	}
     }
-    if (locargc > 3 || locargc < 2) goto badusage;
+    if (locargc > 4 || locargc < 2) goto badusage;
 
     option = Lookup(cmd->tx_argv[1 + ((dolist) ? 1 : 0)], cmdCellOption);
     if (option < 0) goto badusage;
 
-    if (locargc == 3) {
+    if ((locargc > 3) && (option != IDX_RENAME)) goto badusage;
+    if (locargc >= 3) {
 	switch (option) {
 	    case IDX_ALLCELLS:
 	    case IDX_TOPCELLS:
@@ -380,13 +493,22 @@ CmdCellname(w, cmd)
 	case IDX_PARENTS:
 	    (*func)(cellname, PARENTS, dolist);
 	    break;
+	case IDX_DELETE:
+	    /* Unload the cell definition and free memory */
+	    DBCellDelete(cellname);
+	    break;
+	case IDX_RENAME:
+	    /* Rename the cell and mark as modified.  Do not write to disk. */
+	    DBCellRename(cellname, cmd->tx_argv[3 + ((dolist) ? 1 : 0)]);
+	    break;
     }
     return;
 
 badusage:
     TxError("Usage: %s [list] children|parents|self|exists|"
-		"instances|celldef [name]\n", cmd->tx_argv[0]);
+		"instances|celldef|delete [name]\n", cmd->tx_argv[0]);
     TxError("or:    %s [list] allcells|topcells|window\n", cmd->tx_argv[0]);
+    TxError("or:    %s rename name newname\n", cmd->tx_argv[0]);
     return;
 }
 
@@ -410,28 +532,39 @@ badusage:
  *
  * ----------------------------------------------------------------------------
  */
+
 #ifdef CIF_MODULE
-#define ARRAY		0
-#define HIER		1
-#define	AREALABELS	2
-#define HELP		3
-#define IDCELL		4
-#define ISTYLE		5
-#define CIF_LAMBDA	6
-#define PREFIX		7
-#define OSTYLE		8
-#define READ		9
-#define SEE	       10
-#define STATS	       11
-#define CIF_WRITE      12
-#define CIF_WRITE_FLAT 13
+
+#define ARRAY		 0
+#define HIER		 1
+#define	AREALABELS	 2
+#define COVERAGE	 3
+#define HELP		 4
+#define IDCELL		 5
+#define ISTYLE		 6
+#define CIF_LAMBDA	 7
+#define CIF_LIMIT	 8
+#define OSTYLE		 9
+#define PREFIX		10
+#define READ		11
+#define RESCALE		12
+#define SEE		13
+#define STATS		14
+#define WARNING		15
+#define CIF_WRITE	16
+#define CIF_WRITE_FLAT	17
+#ifdef NONMANHATTAN
+#define POLYGONS	18
+#endif
+
+#define CIF_WARN_HELP  CIF_WARN_END	/* undefined by CIF module */
+
 	/* ARGSUSED */
+
 CmdCif(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    extern bool CIFDoAreaLabels, CIFDoCellIdLabels;
-    extern char *CIFPathPrefix;
     int option, yesno;
     char **msg, *namep;
     CellDef *rootDef;
@@ -441,10 +574,13 @@ CmdCif(w, cmd)
     bool flatCif = FALSE;
     bool dolist = FALSE;
     bool doforall = FALSE;
+    int limit;
+    float curscale;
     int argc = cmd->tx_argc;
     char **argv = cmd->tx_argv;
-    float curscale;
 
+    static char *cmdCifWarnOptions[] = { "default", "none", "align",
+		"limit", "redirect", "help", 0 };
     static char *cmdCifYesNo[] = { "no", "yes", 0 };
     static char *cmdCifInOut[] = { "input", "output", 0 };
     static char *cmdCifOption[] =
@@ -452,18 +588,26 @@ CmdCif(w, cmd)
 	"*array layer		display CIF layer under box (array only)",
 	"*hier layer		display CIF layer under box (hier only)",
 	"arealabels yes|no	enable/disable us of area label extension",
+	"coverage layer		print area coverage of indicated layer",
 	"help		print this help information",
 	"idcell yes|no	enable/disable use of cell ID extension",
 	"istyle [style]	change style for reading CIF to style",
 	"lambda in|out	show microns per lambda for the current style",
-	"prefix [path]	prepend path to cell names in CIF output",
+	"limit [value]	limit internal grid subdivision by scalefactor value",
 	"ostyle [style]	change style for writing CIF to style",
+	"prefix [path]	prepend path to cell names in CIF output",
 	"read file		read CIF from \"file\" into edit cell",
+	"rescale [yes|no]	allow/disallow rescaling of internal grid",
 	"see layer		display CIF layer under box",
 	"statistics		print out statistics for CIF generator",
+	"warning [option]	set warning display options",
 	"write file		output CIF for the window's root cell to \"file\"",
 	"flat file		output flattened CIF for "
 				"the window's root cell to \"file\"",
+#ifdef NONMANHATTAN
+	"polygon subcells [yes|no]\n"
+	"			put non-Manhattan polygons in subcells",
+#endif
 	NULL
     };
 
@@ -494,10 +638,14 @@ CmdCif(w, cmd)
 
     switch (option)
     {
-	case HELP: case ISTYLE: case OSTYLE: case PREFIX: case AREALABELS:
+	case HELP: case ISTYLE: case OSTYLE: case PREFIX:
+	case AREALABELS: case WARNING:
+#ifdef NONMANHATTAN
+	case POLYGONS:
+#endif
 	   break;
 	default:
-	    windCheckOnlyWindow(&w);
+	    windCheckOnlyWindow(&w, DBWclientID);
 	    if (w == (MagWindow *) NULL)
 	    {
 		TxError("Point to a window first\n");
@@ -546,6 +694,23 @@ CmdCif(w, cmd)
 	    CIFSeeHierLayer(rootDef, &box, argv[2], FALSE, TRUE);
 	    return;
 
+	case COVERAGE:
+	    if ((argc == 4) && !strcmp(argv[3], "box"))
+	    {
+		if (!ToolGetBox(&rootDef, &box))
+		{
+		    TxError("Box requested but no cursor box exists\n");
+		    return;
+		}
+		CIFCoverageLayer(rootDef, &box, argv[2]);
+	    }
+	    else if (argc == 3)
+		CIFCoverageLayer(rootDef, &rootDef->cd_bbox, argv[2]);
+	    else
+		goto wrongNumArgs;
+
+	    return;
+
 	case CIF_LAMBDA:
 	    if (argc != 3) goto wrongNumArgs;
 	    yesno = Lookup(argv[2], cmdCifInOut); /* "input" or "output" */
@@ -563,6 +728,34 @@ CmdCif(w, cmd)
 #endif
 	    return;
 
+	case CIF_LIMIT:
+	    if (argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(CIFRescaleLimit));
+#else
+		TxPrintf("Rescale limit is %d\n", CIFRescaleLimit);
+#endif
+		return;
+	    }
+	    else if (argc != 3)
+		goto wrongNumArgs;
+	    else if (!StrIsInt(argv[2]))
+		goto wrongNumArgs;
+
+	    CIFRescaleLimit = atoi(argv[2]);
+	    if (CIFRescaleLimit > CIFMAXRESCALE)
+	    {
+		CIFRescaleLimit = CIFMAXRESCALE;
+		TxError("Warning:  rescale limit set at maximum value of %d\n",
+			CIFMAXRESCALE);
+	    }
+	    else if (CIFRescaleLimit < 1)
+	    {
+		CIFRescaleLimit = 1;
+		TxError("Warning:  rescale limit set at minimum value of 1\n");
+	    }
+
 	case AREALABELS:
 	    if (argc > 3) goto wrongNumArgs;
 	    if (argc == 3)
@@ -613,6 +806,30 @@ CmdCif(w, cmd)
 	    else goto wrongNumArgs;
 	    return;
 
+#ifdef NONMANHATTAN
+	case POLYGONS:
+	    if (argc == 3)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CIFSubcellPolygons));
+#else
+		if (CIFSubcellPolygons)
+		    TxPrintf("Non-manhattan polygons placed in subcells.\n");
+		else
+		    TxPrintf("Non-manhattan polygons read as-is.\n");
+#endif
+		return;
+	    }
+	    else if (argc != 4)
+		goto wrongNumArgs;
+
+	    yesno = Lookup(argv[3], cmdCifYesNo);
+	    if (yesno < 0)
+		goto wrongNumArgs;
+	    CIFSubcellPolygons = yesno;
+	    return;
+#endif  /* NONMANHATTAN */
+
 	case PREFIX:
 	    if (argc == 2)
 		StrDup (&CIFPathPrefix, NULL);
@@ -642,6 +859,28 @@ CmdCif(w, cmd)
 	    (void) fclose(f);
 	    return;
 	
+	case RESCALE:
+	    if (argc == 2)
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetObjResult(magicinterp, Tcl_NewBooleanObj(CIFRescaleAllow));
+#else
+		TxPrintf("Internal grid rescaling %sallowed\n",
+			(CIFRescaleAllow) ?  "" : "dis");
+#endif
+		return;
+	    }
+	    else if (argc != 3)
+		goto wrongNumArgs;
+
+	    yesno = Lookup(argv[2], cmdCifYesNo);
+	    if (yesno < 0)
+		goto wrongNumArgs;
+	    CIFRescaleAllow = yesno;
+	    if (!CIFRescaleAllow)
+		CIFWarningLevel = CIF_WARN_LIMIT;
+	    return;
+	
 	case SEE:
 	    if (argc != 3) goto wrongNumArgs;
 	    if (!ToolGetBox(&rootDef, &box))
@@ -657,6 +896,35 @@ CmdCif(w, cmd)
 	    CIFPrintStats();
 	    return;
 
+	case WARNING:
+	    if (argc == 2)
+	    {
+		TxPrintf("Warning display options: %s\n",
+			cmdCifWarnOptions[CIFWarningLevel]);
+	    }
+	    else
+	    {
+		int suboption = Lookup(argv[2], cmdCifWarnOptions);
+		if (suboption < 0)
+		{
+		    TxError("\"%s\" isn't a valid cif warning option.\n", argv[2]);
+		    suboption = CIF_WARN_HELP;
+		}
+		if (suboption == CIF_WARN_HELP)
+		    TxError("Valid options are: default, align, limit, "
+			"redirect, and none\n");
+		else
+		    CIFWarningLevel = suboption;
+		if (suboption == CIF_WARN_REDIRECT)
+		{
+		    if (argc == 4)
+			StrDup(&CIFErrorFilename, argv[3]);
+		    else
+			StrDup(&CIFErrorFilename, NULL);
+		}
+	    }
+	    return;
+
 	case CIF_WRITE:
 	    if (argc != 3) goto wrongNumArgs;
 	    namep = argv[2];
@@ -850,7 +1118,7 @@ CmdCopy(w, cmd)
 	    goto copyToPoint;
 	}
 
-	indx = GeoNameToPos(cmd->tx_argv[1], TRUE, TRUE);
+	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
 	if (indx < 0)
 	    return;
 	if (cmd->tx_argc == 3)
@@ -877,6 +1145,26 @@ CmdCopy(w, cmd)
 		xdelta = -amount;
 		ydelta = 0;
 		break;
+	    case GEO_NORTHWEST:
+		xdelta = -amount;
+		ydelta = amount;
+		break;
+	    case GEO_NORTHEAST:
+		xdelta = amount;
+		ydelta = amount;
+		break;
+	    case GEO_SOUTHWEST:
+		xdelta = -amount;
+		ydelta = -amount;
+		break;
+	    case GEO_SOUTHEAST:
+		xdelta = amount;
+		ydelta = -amount;
+		break;
+	    case GEO_CENTER:
+		xdelta = 0;
+		ydelta = 0;
+		break;
 	    default:
 		ASSERT(FALSE, "Bad direction in CmdCopy");
 		return;
@@ -960,6 +1248,22 @@ int cmdCornerDir2;			/* Second direction each wire must
 Rect cmdCornerRootBox;			/* Root coords of box. */
 struct cmdCornerArea *cmdCornerList;	/* List of areas to fill. */
 
+#ifdef NONMANHATTAN
+typedef struct cifpathlist
+{
+    TileType pathtype;
+    CIFPath *pathhead;
+    struct cifpathlist *cpl_next;
+} CIFPathList;
+
+typedef struct nmcornerpath
+{
+    bool hasErr;
+    CIFPathList *pathlist;
+} NMCornerPath;
+
+#endif
+
 	/*ARGSUSED*/
 Void
 CmdCorner(w, cmd)
@@ -971,14 +1275,26 @@ CmdCorner(w, cmd)
     SearchContext scx;
     extern int cmdCornerFunc();
     bool hasErr = FALSE;
+    int locargc = cmd->tx_argc;
 
+#ifdef NONMANHATTAN
+    extern int cmdBevelFunc();
+    bool dobevel = FALSE;
+    NMCornerPath cmdPathList;
+
+    if (cmd->tx_argc < 3 || cmd->tx_argc > 5)
+    {
+	TxError("Usage: %s direction1 direction2 [layers]\n",
+			cmd->tx_argv[0]);
+#else
     if (cmd->tx_argc < 3 || cmd->tx_argc > 4)
     {
 	TxError("Usage: %s direction1 direction2 [layers]\n", cmd->tx_argv[0]);
+#endif
 	return;
     }
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ( w == (MagWindow *) NULL )
     {
 	TxError("Point to a window\n");
@@ -1012,9 +1328,22 @@ CmdCorner(w, cmd)
 	}
     }
 
+#ifdef NONMANHATTAN
+    /* Check for "bevel" keyword */
+
+    if (locargc > 3)
+    {
+	if (!strncmp(cmd->tx_argv[locargc - 1], "bevel", 5))
+	{
+	    locargc--;
+	    dobevel = TRUE;
+	}
+    }
+#endif  /* NONMANHATTAN */
+
     /* Figure out which layers to fill. */
 
-    if (cmd->tx_argc < 4)
+    if (locargc < 4)
 	maskBits = DBAllButSpaceAndDRCBits;
     else
     {
@@ -1050,15 +1379,59 @@ CmdCorner(w, cmd)
     }
     scx.scx_use = (CellUse *) w->w_surfaceID;
     scx.scx_trans = GeoIdentityTransform;
+
+#ifdef NONMANHATTAN
+    if (dobevel)
+    {
+	cmdPathList.hasErr = FALSE;
+	cmdPathList.pathlist = NULL;
+
+	(void) DBTreeSrTiles(&scx, &maskBits,
+		((DBWclientRec *) w->w_clientData)->dbw_bitmask,
+		cmdBevelFunc, (ClientData) &cmdPathList);
+
+	if (cmdPathList.hasErr)
+	    TxError("There's not enough room in the box for all the wires.\n");
+
+	while (cmdPathList.pathlist != NULL)
+	{
+	    /* It is a bit of a hack to use CIFPolyToRects here, but it	*/
+	    /* beats rewriting the entire path parsing routine.		*/
+
+	    LinkedRect *rectp;
+	    PaintUndoInfo ui;
+	    int pNum =  DBPlane(cmdPathList.pathlist->pathtype);
+	    PaintResultType  *resultTbl =
+			DBStdPaintTbl(cmdPathList.pathlist->pathtype, pNum);
+	    Plane *plane = EditRootDef->cd_planes[pNum];
+	
+	    ui.pu_def = EditRootDef;
+	    ui.pu_pNum = pNum;
+
+	    rectp = CIFPolyToRects(cmdPathList.pathlist->pathhead, plane,
+			resultTbl, &ui);
+	    for (; rectp != NULL; rectp = rectp->r_next)
+	    {
+		DBPaintPlane(plane, &rectp->r_r, resultTbl, &ui);
+		freeMagic((char *)rectp);
+	    }
+	    CIFFreePath(cmdPathList.pathlist->pathhead);
+	    freeMagic((char *)cmdPathList.pathlist);
+	    cmdPathList.pathlist = cmdPathList.pathlist->cpl_next;
+	}
+    }
+    else
+    {
+#endif
+
     cmdCornerList = (struct cmdCornerArea *) NULL;
 
     (void) DBTreeSrTiles(&scx, &maskBits,
 	    ((DBWclientRec *) w->w_clientData)->dbw_bitmask,
 	    cmdCornerFunc, (ClientData) &hasErr);
+
     if (hasErr)
-    {
 	TxError("There's not enough room in the box for all the wires.\n");
-    }
 
     /* Now that we've got all the material, scan over the list
      * painting the material and freeing up the entries on the list.
@@ -1071,6 +1444,10 @@ CmdCorner(w, cmd)
 	cmdCornerList = cmdCornerList->cca_next;
     }
 
+#ifdef NONMANHATTAN
+    }
+#endif
+
     SelectClear();
     DBAdjustLabels(EditCellUse->cu_def, &editBox);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editBox);
@@ -1224,6 +1601,539 @@ cmdCornerFunc(tile, cxp)
     }
     return 0;
 }
+
+#ifdef NONMANHATTAN
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * Convenience function for cmdBevelFunc.  Allocates a CIFPath record.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+AddNewPoint(CIFPath **pathhead)
+{
+    CIFPath *newpoint;
+
+    newpoint = (CIFPath *)mallocMagic(sizeof(CIFPath));
+    newpoint->cifp_next = *pathhead;
+    *pathhead = newpoint;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * cmdBevelFunc --
+ *
+ * 	Search procedure called by DBTreeSrTiles from CmdCorner.  Called once
+ *	for each tile that crosses the appropriate boundary of the box.
+ *	Makes a beveled 90 degree turn to extend a wire out of an
+ *	adjacent side.
+ *
+ * Results:
+ *	Returns 0 to keep the search alive.
+ *
+ * Side effects:
+ *	Adds paint tiles to the display list.  If there are tiles found
+ *	that can't be cornered correctly, the clientData value is set
+ *	to TRUE.
+ *
+ * ----------------------------------------------------------------------------
+ */
+int
+cmdBevelFunc(tile, cxp)
+    Tile *tile;			/* Tile to fill with. */
+    TreeContext *cxp;		/* Describes state of search. */
+{
+    Rect r1, r2, r3;
+    CIFPathList *cpl;
+    CIFPath *pptr, *pathhead = NULL;
+    NMCornerPath *pathrecord = (NMCornerPath *) cxp->tc_filter->tf_arg;
+    int wirewidth, boxwidth, boxheight;
+    TileType pathtype;
+
+    /* Get the tile dimensions in root coordinates.  Clip to the box.
+     */
+    TiToRect(tile, &r1);
+    GeoTransRect(&cxp->tc_scx->scx_trans, &r1, &r2);
+    GeoClip(&r2, &cmdCornerRootBox);
+
+    /* Find the Manhattan rectangles like cmdCornerFunc.  Shrink
+     * these to a minimum square or rectangle.  Finally, pick the
+     * path points off of the two boxes to generate the nonManhattan
+     * path.
+     */
+
+    r3 = r2;
+    switch (cmdCornerDir1)
+    {
+	case GEO_NORTH:
+
+	    pathtype = TiGetTopType(tile);
+	    if (pathtype == TT_SPACE) return 0;
+	    wirewidth = r2.r_xtop - r2.r_xbot;
+	    if (cmdCornerDir2 == GEO_EAST)
+	    {
+		r3.r_ytop = cmdCornerRootBox.r_ytop
+			- (r2.r_xbot - cmdCornerRootBox.r_xbot);
+		r3.r_xtop = r3.r_xbot = cmdCornerRootBox.r_xtop;
+		boxwidth = r3.r_xtop - r2.r_xbot;
+		boxheight = r3.r_ytop - r3.r_ybot;
+
+		if (boxwidth > boxheight)
+		    r3.r_xbot = r2.r_xbot + boxheight;
+		r3.r_xbot -= wirewidth;
+	    }
+	    else
+	    {
+		r3.r_ytop = cmdCornerRootBox.r_ytop
+			- (cmdCornerRootBox.r_xtop - r2.r_xtop);
+		r3.r_xbot = r3.r_xtop = cmdCornerRootBox.r_xbot;
+		boxwidth = r2.r_xtop - r3.r_xbot;
+		boxheight = r3.r_ytop - r3.r_ybot;
+
+		if (boxwidth > boxheight)
+		    r3.r_xtop = r2.r_xtop - boxheight;
+		r3.r_xtop += wirewidth;
+	    }
+	    r3.r_ybot = r3.r_ytop - wirewidth;
+	    if (r3.r_ybot < cmdCornerRootBox.r_ybot)
+		 pathrecord->hasErr = TRUE;
+
+	    if (boxheight > boxwidth)
+		r2.r_ytop = r3.r_ytop - boxwidth;
+	    else
+		r2.r_ytop = r2.r_ybot;
+	    r2.r_ytop += wirewidth;
+
+	    /* Create nonManhattan path */
+	    if (cmdCornerDir2 == GEO_EAST)
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ybot;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xtop;
+		    pathhead->cifp_y = r2.r_ytop - wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xbot + wirewidth;
+		    pathhead->cifp_y = r3.r_ybot;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ytop;
+	    }
+	    else
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ybot;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xbot;
+		    pathhead->cifp_y = r2.r_ytop - wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xtop - wirewidth;
+	    	    pathhead->cifp_y = r3.r_ybot;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ytop;
+	    }
+	    break;
+
+	case GEO_SOUTH:
+	    pathtype = TiGetBottomType(tile);
+	    if (pathtype == TT_SPACE) return 0;
+	    wirewidth = r2.r_xtop - r2.r_xbot;
+	    if (cmdCornerDir2 == GEO_EAST)
+	    {
+		r3.r_ybot = cmdCornerRootBox.r_ybot
+			+ (r2.r_xbot - cmdCornerRootBox.r_xbot);
+		r3.r_xtop = r3.r_xbot = cmdCornerRootBox.r_xtop;
+		boxwidth = r3.r_xtop - r2.r_xbot;
+		boxheight = r3.r_ytop - r3.r_ybot;
+
+		if (boxwidth > boxheight)
+		    r3.r_xbot = r2.r_xbot + boxheight;
+		r3.r_xbot -= wirewidth;
+	    }
+	    else
+	    {
+		r3.r_ybot = cmdCornerRootBox.r_ybot
+			+ (cmdCornerRootBox.r_xtop - r2.r_xtop);
+		r3.r_xbot = r3.r_xtop = cmdCornerRootBox.r_xbot;
+		boxwidth = r2.r_xtop - r3.r_xbot;
+		boxheight = r3.r_ytop - r3.r_ybot;
+
+		if (boxwidth > boxheight)
+		    r3.r_xtop = r2.r_xtop - boxheight;
+		r3.r_xtop += wirewidth;
+	    }
+	    r3.r_ytop = r3.r_ybot + wirewidth;
+	    if (r3.r_ytop > cmdCornerRootBox.r_ytop)
+		pathrecord->hasErr = TRUE;
+
+	    if (boxheight > boxwidth)
+		r2.r_ybot = r3.r_ybot + boxwidth;
+	    else
+		r2.r_ybot = r2.r_ytop;
+	    r2.r_ybot -= wirewidth;
+
+	    /* Create nonManhattan path */
+	    if (cmdCornerDir2 == GEO_EAST)
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ytop;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xtop;
+		    pathhead->cifp_y = r2.r_ybot + wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xbot + wirewidth;
+		    pathhead->cifp_y = r3.r_ytop;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ybot;
+	    }
+	    else
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ytop;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xbot;
+		    pathhead->cifp_y = r2.r_ybot + wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xtop - wirewidth;
+		    pathhead->cifp_y = r3.r_ytop;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ybot;
+	    }
+	    break;
+
+	case GEO_EAST:
+	    pathtype = TiGetRightType(tile);
+	    if (pathtype == TT_SPACE) return 0;
+	    wirewidth = r2.r_ytop - r2.r_ybot;
+	    if (cmdCornerDir2 == GEO_NORTH)
+	    {
+		r3.r_xtop = cmdCornerRootBox.r_xtop
+			- (r2.r_ybot - cmdCornerRootBox.r_ybot);
+		r3.r_ytop = r3.r_ybot = cmdCornerRootBox.r_ytop;
+		boxwidth = r3.r_xtop = r3.r_xbot;
+		boxheight = r3.r_xtop = r2.r_ybot;
+
+		if (boxheight > boxwidth)
+		    r3.r_ybot = r2.r_ybot + boxwidth;
+		r3.r_ybot -= wirewidth;
+	    }
+	    else
+	    {
+		r3.r_xtop = cmdCornerRootBox.r_xtop
+			- (cmdCornerRootBox.r_ytop - r2.r_ytop);
+		r3.r_ybot = r3.r_ytop = cmdCornerRootBox.r_ybot;
+		boxwidth = r3.r_xtop = r3.r_xbot;
+		boxheight = r2.r_xtop = r3.r_ybot;
+
+		if (boxheight > boxwidth)
+		    r3.r_ytop = r2.r_ytop - boxwidth;
+		r3.r_ytop += wirewidth;
+	    }
+	    r3.r_xbot = r3.r_xtop - wirewidth;
+	    if (r3.r_xbot < cmdCornerRootBox.r_xbot)
+		pathrecord->hasErr = TRUE;
+
+	    if (boxwidth > boxheight)
+		r2.r_xtop = r3.r_xtop - boxheight;
+	    else
+		r2.r_xtop = r2.r_xbot;
+	    r2.r_xtop += wirewidth;
+
+	    /* Create nonManhattan path */
+	    if (cmdCornerDir2 == GEO_NORTH)
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ytop;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xbot;
+		    pathhead->cifp_y = r3.r_ybot + wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xtop - wirewidth;
+		    pathhead->cifp_y = r2.r_ytop;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ybot;
+	    }
+	    else
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ybot;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xbot;
+		    pathhead->cifp_y = r3.r_ytop - wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xtop - wirewidth;
+		    pathhead->cifp_y = r2.r_ybot;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ytop;
+	    }
+	    break;
+
+	case GEO_WEST:
+	    pathtype = TiGetLeftType(tile);
+	    if (pathtype == TT_SPACE) return 0;
+	    wirewidth = r2.r_ytop - r2.r_ybot;
+	    if (cmdCornerDir2 == GEO_NORTH)
+	    {
+		r3.r_xbot = cmdCornerRootBox.r_xbot
+			+ (r2.r_ybot - cmdCornerRootBox.r_ybot);
+		r3.r_ytop = r3.r_ybot = cmdCornerRootBox.r_ytop;
+		boxwidth = r3.r_xtop = r3.r_xbot;
+		boxheight = r3.r_xtop = r2.r_ybot;
+
+		if (boxheight > boxwidth)
+		    r3.r_ybot = r2.r_ybot + boxwidth;
+		r3.r_ybot -= wirewidth;
+	    }
+	    else
+	    {
+		r3.r_xbot = cmdCornerRootBox.r_xbot
+			+ (cmdCornerRootBox.r_ytop - r2.r_ytop);
+		r3.r_ybot = r3.r_ytop = cmdCornerRootBox.r_ybot;
+		boxwidth = r3.r_xtop = r3.r_xbot;
+		boxheight = r2.r_xtop = r3.r_ybot;
+
+		if (boxheight > boxwidth)
+		    r3.r_ytop = r2.r_ytop - boxwidth;
+		r3.r_ytop += wirewidth;
+	    }
+	    r3.r_xtop = r2.r_xbot + wirewidth;
+	    if (r3.r_xtop > cmdCornerRootBox.r_xtop)
+		pathrecord->hasErr = TRUE;
+
+	    if (boxwidth > boxheight)
+		r2.r_xbot = r3.r_xbot + boxheight;
+	    else
+		r2.r_xbot = r2.r_xtop;
+	    r2.r_xbot -= wirewidth;
+
+	    /* Create nonManhattan path */
+	    if (cmdCornerDir2 == GEO_NORTH)
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ytop;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xtop;
+		    pathhead->cifp_y = r3.r_ybot + wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xbot + wirewidth;
+		    pathhead->cifp_y = r2.r_ytop;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ybot;
+	    }
+	    else
+	    {
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xbot;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ytop;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r2.r_xtop;
+		pathhead->cifp_y = r2.r_ybot;
+		if (boxheight > boxwidth)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r3.r_xtop;
+		    pathhead->cifp_y = r3.r_ytop - wirewidth;
+		}
+		else if (boxwidth > boxheight)
+		{
+		    AddNewPoint(&pathhead);
+		    pathhead->cifp_x = r2.r_xbot + wirewidth;
+		    pathhead->cifp_y = r2.r_ybot;
+		}
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xtop;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ybot;
+		AddNewPoint(&pathhead);
+		pathhead->cifp_x = r3.r_xbot;
+		pathhead->cifp_y = r3.r_ytop;
+	    }
+	    break;
+    }
+
+    /* If either rectangle is NULL after clipping, free the point */
+    /* memory and return without adding the path.		  */
+
+    GeoClip(&r2, &cmdCornerRootBox);
+    GeoClip(&r3, &cmdCornerRootBox);
+    if (GEO_RECTNULL(&r2) || GEO_RECTNULL(&r3))
+    {
+	for (pptr = pathhead; pptr != NULL; pptr = pptr->cifp_next)
+	    freeMagic((char *)pptr);
+	return 0;
+    }
+
+    /* Clip the point geometry to the box, and translate to edit cell
+     * coords.
+     */
+
+    for (pptr = pathhead; pptr != NULL; pptr = pptr->cifp_next)
+    {
+	Point p = pptr->cifp_point;
+	GeoClipPoint(&p, &cmdCornerRootBox);
+	GeoTransPoint(&RootToEditTransform, &p, &pptr->cifp_point);
+    }
+
+    /* Create the path and add it to the list. */
+
+    cpl = (CIFPathList *) mallocMagic(sizeof(CIFPathList));
+    cpl->pathtype = pathtype;
+    cpl->cpl_next = pathrecord->pathlist;
+    cpl->pathhead = pathhead;
+    pathrecord->pathlist = cpl;
+
+    return 0;	/* Keep the search going */
+}
+#endif  /* NONMANHATTAN */
+
 
 /*
  * ----------------------------------------------------------------------------
