@@ -18,10 +18,13 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdE.c,v 1.4 2001/01/12 22:12:16 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 
+#ifdef MAGIC_WRAPPER
+#include "magic/tclmagic.h"
+#endif
 #include "misc/magic.h"
 #include "misc/styles.h"
 #include "utils/geometry.h"
@@ -76,10 +79,8 @@ CmdEdit(w, cmd)
     TxCommand *cmd;
 {
     Rect area, pointArea;
-#ifdef FILE_LOCKS
     CellUse *usave;
     CellDef *csave;
-#endif
     int cmdEditRedisplayFunc();		/* Forward declaration. */
     int cmdEditEnumFunc();		/* Forward declaration. */
 
@@ -88,7 +89,6 @@ CmdEdit(w, cmd)
 	TxError("Usage: edit\nMaybe you want the \"load\" command\n");
 	return;
     }
-#ifdef FILE_LOCKS
     if (EditCellUse->cu_def->cd_flags & CDNOEDIT)
     {
 	TxError("File %s is not writeable.  EDIT not set.\n",
@@ -96,7 +96,6 @@ CmdEdit(w, cmd)
 	cmdFoundNewEdit = FALSE;
 	return;
     }
-#endif
 
     /* Record the current edit cell's area for redisplay (now that it's
      * not the edit cell, it will be displayed differently).  Do this
@@ -117,13 +116,11 @@ CmdEdit(w, cmd)
     DBWUndoOldEdit(EditCellUse, EditRootDef,
 	    &EditToRootTransform, &RootToEditTransform);
     cmdFoundNewEdit = FALSE;
-#ifdef FILE_LOCKS
     usave = EditCellUse;
     csave = EditRootDef;
-#endif
     (void) SelEnumCells(FALSE, (bool *) NULL, (SearchContext *) NULL,
 	    cmdEditEnumFunc, (ClientData) &pointArea);
-#ifdef FILE_LOCKS
+
     if (EditCellUse->cu_def->cd_flags & CDNOEDIT)
     {
 	TxError("File %s is not writeable.  Edit not set.\n",
@@ -133,7 +130,6 @@ CmdEdit(w, cmd)
 	EditRootDef = csave;
 	return;
     }
-#endif
     if (!cmdFoundNewEdit)
 	TxError("You haven't selected a new cell to edit.\n");
 
@@ -262,6 +258,7 @@ cmdEditEnumFunc(selUse, use, transform, area)
 #define OPTION_TEXT	0
 #define OPTION_STYLE	1
 #define OPTION_POSITION	2
+#define OPTION_FLAGS	3
 
 Void
 CmdElement(w, cmd)
@@ -276,7 +273,6 @@ CmdElement(w, cmd)
     CellDef *def;
     CellUse *use;
     bool getopt;
-    int maxstyles = TECHBEGINSTYLES + (DBWNumStyles * 2);
 
     static char *cmdElementOption[] = {
 	"add		create a new element",
@@ -299,11 +295,12 @@ CmdElement(w, cmd)
 	"text		get or replace <string>",
 	"style		get or add <style> or remove <style>",
 	"position	get or <point> or <rect>",
+	"flags		set element flags",
 	NULL
     };
 
     option = ELEMENT_HELP;
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL) return;
 
     use = (CellUse *)w->w_surfaceID;
@@ -341,12 +338,8 @@ CmdElement(w, cmd)
 		    style = atoi(cmd->tx_argv[4]);
 		else
 		{
-		    for (style = 0; style < maxstyles; style++)
-			if (GrStyleTable[style].longname != NULL)
-			    if (!strcmp(cmd->tx_argv[4], GrStyleTable[style].longname))
-				break;
-
-		    if (style == maxstyles)
+		    style = GrGetStyleFromName(cmd->tx_argv[4]);
+		    if (style < 0)
 		    {
 			TxError("Unknown style \"%s\".", cmd->tx_argv[4]);
 			TxError("Use a number or one of the long names in the"
@@ -464,13 +457,8 @@ CmdElement(w, cmd)
 			    style = atoi(cmd->tx_argv[5]);
 			else
 			{
-		    	    for (style = 0; style < maxstyles; style++)
-				if (GrStyleTable[style].longname != NULL)
-			    	    if (!strcmp(cmd->tx_argv[5],
-						GrStyleTable[style].longname))
-					break;
-
-		    	    if (style == maxstyles)
+			    style = GrGetStyleFromName(cmd->tx_argv[5]);
+		    	    if (style < 0)
 		    	    {
 				TxError("Unknown style \"%s\".", cmd->tx_argv[5]);
 				TxError("Use a number or one of the long names "
@@ -523,6 +511,23 @@ badrect:
 			}
 		    }
 		    break;
+		case OPTION_FLAGS:
+		    if (getopt)
+		    {
+			TxError("(unimplemented function)\n");
+		    }
+		    else
+		    {
+			if (cmd->tx_argc >= 5)
+			    DBWElementParseFlags(w, cmd->tx_argv[2], cmd->tx_argv[4]);
+			else
+			{
+			    TxError("Usage: element configure <name> flags "
+					"(flag)\n");
+			    return;
+			}
+		    }
+		    break;
 	    }
 	    break;
 	case ELEMENT_NAMES:
@@ -587,7 +592,7 @@ CmdErase(w, cmd)
     TileTypeBitMask mask;
     extern int cmdEraseCellsFunc();
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL) return;
 
     if (cmd->tx_argc > 2)
@@ -719,7 +724,7 @@ CmdExpand(w, cmd)
 	return;
     }
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL)
     {
 	TxError("Point to a window first.\n");
@@ -892,7 +897,7 @@ CmdExtract(w, cmd)
 
     if ((option != EXTSTYLE) && (option != EXTHELP))
     {
-	windCheckOnlyWindow(&w);
+	windCheckOnlyWindow(&w, DBWclientID);
 	if (w == (MagWindow *) NULL)
 	{
 	    if (ToolGetBox(&selectedDef,NULL) == FALSE)
