@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdLQ.c,v 1.11 2001/07/23 20:37:00 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #ifdef SYSV
@@ -99,12 +99,13 @@ CmdLabelProc(text, pos, type)
 
     DBEraseLabelsByContent(EditCellUse->cu_def, &editBox, pos, -1, text);
 
-    pos = DBPutLabel(EditCellUse->cu_def, &editBox, pos, text, type);
+    pos = DBPutLabel(EditCellUse->cu_def, &editBox, pos, text, type, 0);
     DBAdjustLabels(EditCellUse->cu_def, &editBox);
     DBReComputeBbox(EditCellUse->cu_def);
     DBWLabelChanged(EditCellUse->cu_def, text, &editBox, pos, DBW_ALLWINDOWS);
 }
 
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -158,6 +159,7 @@ CmdLabel(w, cmd)
 
     p = cmd->tx_argv[1];
 
+
     /*
      * Find and check validity of type parameter
      */
@@ -194,10 +196,10 @@ CmdLabel(w, cmd)
  *
  * CmdLoad --
  *
- * Implement the "loadcell" command.
+ * Implement the "load" command.
  *
  * Usage:
- *	loadcell [name]
+ *	load [name]
  *
  * If name is supplied, then the window containing the point tool is
  * remapped so as to edit the cell with the given name.
@@ -221,7 +223,7 @@ CmdLoad(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL)
     {
 	TxError("Point to a window first.\n");
@@ -298,7 +300,7 @@ CmdMove(w, cmd)
 	    goto moveToPoint;
 	}
 
-	indx = GeoNameToPos(cmd->tx_argv[1], TRUE, TRUE);
+	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
 	if (indx < 0)
 	    return;
 	if (cmd->tx_argc == 3)
@@ -325,6 +327,22 @@ CmdMove(w, cmd)
 		xdelta = -amount;
 		ydelta = 0;
 		break;
+	    case GEO_NORTHEAST:
+		xdelta = amount;
+		ydelta = amount;
+		break;
+	    case GEO_NORTHWEST:
+		xdelta = -amount;
+		ydelta = amount;
+		break;
+	    case GEO_SOUTHEAST:
+		xdelta = amount;
+		ydelta = -amount;
+		break;
+	    case GEO_SOUTHWEST:
+		xdelta = -amount;
+		ydelta = -amount;
+		break;
 	    default:
 		ASSERT(FALSE, "Bad direction in CmdMove");
 		return;
@@ -333,8 +351,10 @@ CmdMove(w, cmd)
 
 	/* Move the box by the same amount as the selection, if the
 	 * box exists.
-         * if no selection exists, but the box does, then move the box
+         * If no selection exists, but the box does, then move the box
          * anyway (hace 10/8/97)
+	 * The above method is superceded by "box move <dir> <dist>"
+	 * but is retained for backward compatibility.
 	 */
 
 	if (ToolGetBox(&rootDef, &rootBox) && ((rootDef == SelectRootDef)
@@ -409,7 +429,7 @@ CmdPaint(w, cmd)
     Rect editRect;
     TileTypeBitMask mask;
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Put the cursor in a layout window\n");
@@ -473,7 +493,7 @@ CmdPaintButton(w, butPoint)
     TileTypeBitMask mask;
     DBWclientRec *crec;
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Put the cursor in a layout window\n");
@@ -656,71 +676,554 @@ usage:
     return 0;
 }
 
-#ifdef PLOT_AUTO
 /*
  * ----------------------------------------------------------------------------
- * CmdAutoPlot() --
  *
- *	Tcl auto-loading module routine.  If "plot" is configured as a
- *	Tcl module, then this routine replaces CmdPlot.  Plot routines
- *	are not loaded at runtime, but are loaded the first time the
- *	"plot" command is invoked.
+ * CmdPort --
  *
- *	The job of replacing the appropriate entries in CmdFuncs[] is
- *	the responsibility of the initialization function.  If something
- *	goes wrong with the auto-load, it MUST set an error condition in
- *	Tcl or this routine will infinitely recurse!
+ * Implement the "port" command.  Finds a label inside the cursor box.
+ * If this label is unique (only one label found), then its flags are
+ * set according to "num" and one or more "connect_directions" (see
+ * usage, below).
+ *
+ * Usage:
+ *	port [num] [connect_direction(s)]
+ * or
+ *	port class|use [value]
+ *
+ * num is the index of the port, usually beginning with 1.  This indicates
+ *	the order in which ports should be written to a subcircuit record
+ *	in extracted output.  The edit cell is searched for ports to make
+ *	sure that "num" is unique.  If not specified, "num" will use the
+ *	first available index starting at 1.
+ *
+ * connect_directions is one or more of the manhattan (n, s, e, w)
+ *	directions.  This indicates to place and route tools where
+ *	networks may connect to the subcircuit.  To extresis, the
+ *	direction indicates the direction of current flow into the
+ *	subcircuit.  If not specified, connect_directions will use
+ *	the direction of the label text as the direction of connection.
+ *
+ * "value" is a value string representing one of the valid port classes
+ *	or uses.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modified label record.
  *
  * ----------------------------------------------------------------------------
  */
 
-CmdAutoPlot(w, cmd)
+#define PORT_CLASS	0
+#define PORT_USE	1
+#define PORT_INDEX	2
+#define PORT_EQUIV	3
+#define PORT_CONNECT	4
+#define PORT_MAKE	5
+#define PORT_REMOVE	6
+#define PORT_HELP	7
+
+    /* ARGSUSED */
+
+Void
+CmdPort(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    int result;
+    char **msg;
+    int posmask, posstart;
+    int i, idx, pos, type, option;
+    unsigned short dirmask;
+    bool found;
+    Label *lab, *sl;
+    Rect r, editBox;
+    CellDef *editDef = EditCellUse->cu_def;
 
-    /* Auto-load */
-    result = Tcl_Eval(magicinterp, "load " TCL_DIR "/tclplot.so");
+    static char *cmdPortOption[] =
+    {
+	"class	[type]		get [set] port class type",
+	"use	[type]		get [set] port use type",
+	"index	[number]	get [set] port number",
+	"equivalent [number]	make port equivalent to another port",
+	"connections [dir...]	get [set] port connection directions",
+	"make [index] [dir...]	turn a label into a port",
+	"remove			turn a port back into a label",
+	"help			print this help information",
+	NULL
+    };
 
-    /* Call function which was originally intended */
+    static char *cmdPortClassTypes[] =
+    {
+	"default",
+	"input",
+	"output",
+	"tristate",
+	"bidirectional",
+	"inout",
+	"feedthrough",
+	"feedthru",
+	NULL
+    };
 
-    if (result == TCL_OK)
-	WindSendCommand((WindClient)NULL, cmd);
-}
+    static int cmdClassToBitmask[] =
+    {
+	PORT_CLASS_DEFAULT,
+	PORT_CLASS_INPUT,
+	PORT_CLASS_OUTPUT,
+	PORT_CLASS_TRISTATE,
+	PORT_CLASS_BIDIRECTIONAL,
+	PORT_CLASS_BIDIRECTIONAL,
+	PORT_CLASS_FEEDTHROUGH,
+	PORT_CLASS_FEEDTHROUGH
+    };
+
+    static char *cmdPortUseTypes[] =
+    {
+	"default",
+	"analog",
+	"signal",
+	"digital",
+	"power",
+	"ground",
+	"clock",
+	NULL
+    };
+
+    static int cmdUseToBitmask[] =
+    {
+	PORT_USE_DEFAULT,
+	PORT_USE_ANALOG,
+	PORT_USE_SIGNAL,
+	PORT_USE_SIGNAL,
+	PORT_USE_POWER,
+	PORT_USE_GROUND,
+	PORT_USE_CLOCK
+    };
+
+    if (cmd->tx_argc > 6)
+    {
+	TxError("Usage: %s [num] [connect_direction...]\n", cmd->tx_argv[0]);
+	return;
+    }
+
+    /*
+     * Check for unique label in box area
+     */
+
+    ToolGetEditBox(&editBox);
+    found = FALSE;
+    for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
+    {
+	if (GEO_OVERLAP(&editBox, &sl->lab_rect))
+	{
+	    if (found == TRUE)
+	    {
+		/* Let's do this again with the GEO_SURROUND function	*/
+		/* and see if we come up with only one label.		*/
+
+		found = FALSE;
+		for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
+		{
+		    if (GEO_SURROUND(&editBox, &sl->lab_rect))
+		    {
+			if (found == TRUE)
+			{
+			    TxError("Only one label may be present "
+					"under the cursor box.\n");
+			    return;
+			}
+			lab = sl;
+			found = TRUE;
+		    }
+		}
+		break;
+	    }
+	    lab = sl;
+	    found = TRUE;
+	}
+    }
+    if (found == FALSE)
+    {
+	TxError("The edit cell contains no labels inside the cursor box.\n");
+	return;
+    }
+
+    posstart = 1;
+    option = Lookup(cmd->tx_argv[1], cmdPortOption);
+    if (option >= 0)
+    {
+	if ((option != PORT_HELP) && (option != PORT_MAKE))
+	{
+	    /* label "lab" must already be a port */
+	    if (!(lab->lab_flags & PORT_DIR_MASK))
+	    {
+		TxError("The selected label is not a port.\n");
+		return;
+	    }
+	}
+
+	/* All commands except the combined index + direction(s) */
+	switch (option)
+	{
+	    case PORT_CLASS:
+		if (cmd->tx_argc == 2)
+		{
+		    type = lab->lab_flags & PORT_CLASS_MASK;
+		    for (idx = 0; cmdPortClassTypes[idx] != NULL; idx++)
+			if (cmdClassToBitmask[idx] == type)
+			{
+#ifdef MAGIC_WRAPPER
+			    Tcl_AppendResult(magicinterp, cmdPortClassTypes[idx],
+					NULL);
+#else
+			    TxPrintf("Class = %s\n", cmdPortClassTypes[idx]);
 #endif
+			    break;
+			}
+		}
+		else if (cmd->tx_argc == 3)
+		{
+		    type = Lookup(cmd->tx_argv[2], cmdPortClassTypes);
+		    if (type < 0)
+		    {
+		        TxError("Usage:  port class <type>, where <type> is one of:\n");
+			for (msg = &(cmdPortClassTypes[0]); *msg != NULL; msg++)
+			{
+                	    TxError("    %s\n", *msg);
+			}
+		    }
+		    else
+		    {
+		        lab->lab_flags &= (~PORT_CLASS_MASK);
+			lab->lab_flags |= (PORT_CLASS_MASK & cmdClassToBitmask[type]);
+			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+		    }
+		}
+		else
+		    goto portWrongNumArgs;
+		break;
+	    case PORT_USE:
+		if (cmd->tx_argc == 2)
+		{
+		    type = lab->lab_flags & PORT_USE_MASK;
+		    for (idx = 0; cmdPortUseTypes[idx] != NULL; idx++)
+			if (cmdUseToBitmask[idx] == type)
+			{
+#ifdef MAGIC_WRAPPER
+			    Tcl_AppendResult(magicinterp, cmdPortUseTypes[idx],
+					NULL);
+#else
+			    TxPrintf("Use = %s\n", cmdPortUseTypes[idx]);
+#endif
+			    break;
+			}
+		}
+		else if (cmd->tx_argc == 3)
+		{
+		    type = Lookup(cmd->tx_argv[2], cmdPortUseTypes);
+		    if (type < 0)
+		    {
+		        TxError("Usage:  port use <type>, where <type> is one of:\n");
+			for (msg = &(cmdPortUseTypes[0]); *msg != NULL; msg++)
+			{
+                	    TxError("    %s\n", *msg);
+			}
+		    }
+		    else
+		    {
+		        lab->lab_flags &= (~PORT_USE_MASK);
+			lab->lab_flags |= (PORT_USE_MASK & cmdUseToBitmask[type]);
+			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+		    }
+		}
+		else
+		    goto portWrongNumArgs;
+		break;
+
+	    case PORT_INDEX:
+		if (cmd->tx_argc == 2)
+		{
+		    idx = lab->lab_flags & PORT_NUM_MASK;
+#ifdef MAGIC_WRAPPER
+		    Tcl_SetObjResult(magicinterp, Tcl_NewIntObj(idx));
+#else
+		    TxPrintf("Index = %d\n", idx);
+#endif
+		}
+		else if (cmd->tx_argc == 3)
+		{
+		    if (!StrIsInt(cmd->tx_argv[2]))
+		        TxError("Usage:  port index <integer>\n");
+		    else
+		    {
+		        lab->lab_flags &= (~PORT_NUM_MASK);
+			lab->lab_flags |= atoi(cmd->tx_argv[2]);
+			editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+		    }
+		}
+		else
+		    goto portWrongNumArgs;
+		break;
+
+	    case PORT_CONNECT:
+		if (cmd->tx_argc == 2)
+		{
+		    char cdir[5];
+
+		    cdir[0] = '\0';
+		    pos = lab->lab_flags & PORT_DIR_MASK;
+		    if (pos & PORT_DIR_NORTH) strcat(cdir, "n");
+		    if (pos & PORT_DIR_EAST) strcat(cdir, "e");
+		    if (pos & PORT_DIR_SOUTH) strcat(cdir, "s");
+		    if (pos & PORT_DIR_WEST) strcat(cdir, "w");
+#ifdef MAGIC_WRAPPER
+		    Tcl_AppendResult(magicinterp, cdir, NULL);
+#else
+		    TxPrintf("Directions = %s\n", cdir);
+#endif
+		}
+		else
+		{
+		    posstart = 2;
+		    goto parsepositions;
+		}
+		break;
+
+	    case PORT_REMOVE:
+	        lab->lab_flags = 0;
+		DBWLabelChanged(editDef, lab->lab_text, &editBox,
+			lab->lab_pos, DBW_ALLWINDOWS);
+		editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+		break;
+
+	    case PORT_MAKE:
+		posstart = 2;
+		goto parseindex;
+		break;
+
+	    case PORT_HELP:
+portWrongNumArgs:
+		TxError("Usage:  port [option], where [option] is one of:\n");
+		for (msg = &(cmdPortOption[0]); *msg != NULL; msg++)
+		{
+                    TxError("    %s\n", *msg);
+		}
+		TxError("    <index> <directions>\n");
+		break;
+	}
+	return;
+    }
+    
+parseindex:
+
+    /* For this syntax, the label must not already be a port */
+    if ((lab->lab_flags & PORT_DIR_MASK))
+    {
+	TxError("The selected label is already a port.\n");
+	TxError("Do \"port help\" to get a list of options.\n");
+	return;
+    }
+
+    if ((cmd->tx_argc > posstart) && StrIsInt(cmd->tx_argv[posstart]))
+    {
+	idx = atoi(cmd->tx_argv[posstart]);
+	for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
+	{
+	    if (sl == lab) continue;	/* don't consider self */
+	    if (sl->lab_flags & PORT_DIR_MASK)
+	    {
+		if ((sl->lab_flags & PORT_NUM_MASK) == idx)
+		{
+		    TxError("Port index %d is already used by port %s.\n"
+				"Use command \"port index %d\" to force "
+				"equivalence after defining the port.\n",
+				idx, sl->lab_text, idx);
+		    return;
+		}
+	    }
+	}
+	posstart++;
+    }
+    else
+    {
+	idx = 0;
+	for (sl = editDef->cd_labels; sl != NULL; sl = sl->lab_next)
+	{
+	    if (sl == lab) continue;	/* don't consider self */
+	    if (sl->lab_flags & PORT_DIR_MASK)
+		if ((sl->lab_flags & PORT_NUM_MASK) > idx)
+		    idx = (sl->lab_flags & PORT_NUM_MASK);
+	}
+	idx++;
+    }
+
+    lab->lab_flags &= ~PORT_NUM_MASK;
+    lab->lab_flags |= idx;
+
+    /*
+     * Find positions.
+     */
+
+parsepositions:
+
+    dirmask = 0;
+    if (cmd->tx_argc == posstart)
+    {
+	/* Get position from label */
+
+	pos = lab->lab_pos;
+        switch (pos)
+	{
+	    case GEO_NORTH:
+		dirmask = PORT_DIR_NORTH;
+		break;
+	    case GEO_SOUTH:
+		dirmask = PORT_DIR_SOUTH;
+		break;
+	    case GEO_EAST:
+		dirmask = PORT_DIR_EAST;
+		break;
+	    case GEO_WEST:
+		dirmask = PORT_DIR_WEST;
+		break;
+	    case GEO_NORTHEAST:
+		dirmask = PORT_DIR_NORTH | PORT_DIR_EAST;
+		break;
+	    case GEO_NORTHWEST:
+		dirmask = PORT_DIR_NORTH | PORT_DIR_WEST;
+		break;
+	    case GEO_SOUTHEAST:
+		dirmask = PORT_DIR_SOUTH | PORT_DIR_EAST;
+		break;
+	    case GEO_SOUTHWEST:
+		dirmask = PORT_DIR_SOUTH | PORT_DIR_WEST;
+		break;
+	    case GEO_CENTER:
+		dirmask = PORT_DIR_MASK;
+		break;
+	}
+    }
+    else
+    {
+	/* Parse one or more positions */
+
+	for (i = posstart; i < cmd->tx_argc; i++)
+	{
+	    pos = GeoNameToPos(cmd->tx_argv[i], TRUE, TRUE);
+	    if (pos < 0)
+		return;
+            pos = GeoTransPos(&RootToEditTransform, pos);
+	    switch (pos)
+	    {
+		case GEO_NORTH:
+		    dirmask |= PORT_DIR_NORTH;
+		    break;
+		case GEO_SOUTH:
+		    dirmask |= PORT_DIR_SOUTH;
+		    break;
+		case GEO_EAST:
+		    dirmask |= PORT_DIR_EAST;
+		    break;
+		case GEO_WEST:
+		    dirmask |= PORT_DIR_WEST;
+		    break;
+	    }
+	}
+    }
+
+    lab->lab_flags &= ~PORT_DIR_MASK;
+    lab->lab_flags |= dirmask;
+
+    DBWLabelChanged(editDef, lab->lab_text, &editBox, lab->lab_pos, DBW_ALLWINDOWS);
+    editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+}
 
-#ifdef PLOW_AUTO
 /*
  * ----------------------------------------------------------------------------
- * CmdAutoPlow() --
  *
- *	Tcl auto-loading module routine.  If "plow" is configured as a
- *	Tcl module, then this routine replaces CmdPlow, CmdPlowTest, and
- *	CmdStraighten.  Plow routines are not loaded at runtime, but are
- *	loaded the first time one of these commands ("plow", "*plowtest",
- *	or "straighten") is invoked.
+ * CmdProperty --
  *
- *	The job of replacing the appropriate entries in CmdFuncs[] is
- *	the responsibility of the initialization function.  If something
- *	goes wrong with the auto-load, it MUST set an error condition in
- *	Tcl or this routine will infinitely recurse!
+ * Implement the "property" command.  Adds a property string to the
+ * current edit cell, using the cd_props hash table and functions
+ * defined in database/DBprop.c.
+ *
+ * Usage:
+ *	property [name] [value]
+ *
+ * "name" is a unique string tag for the property, and "value" is its
+ * string value.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modified cd_prop record in the current edit cell's def.
  *
  * ----------------------------------------------------------------------------
  */
 
-CmdAutoPlow(w, cmd)
+Void
+CmdProperty(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    int result;
+    CellDef *editDef = EditCellUse->cu_def;
+    int printPropertiesFunc();
+    char *value;
+    bool propfound;
 
-    /* Auto-load */
-    result = Tcl_Eval(magicinterp, "load " TCL_DIR "/tclplow.so");
-
-    /* Call function which was originally intended */
-
-    if (result == TCL_OK)
-	WindSendCommand((WindClient)NULL, cmd);
-}
+    if (cmd->tx_argc == 1)
+    {
+	/* print all properties and their values */
+	DBPropEnum(editDef, printPropertiesFunc);
+    }
+    else if (cmd->tx_argc == 2)
+    {
+	/* print the value of the indicated property */
+	value = DBPropGet(editDef, cmd->tx_argv[1], &propfound);
+	if (propfound)
+#ifdef MAGIC_WRAPPER
+	    Tcl_SetResult(magicinterp, value, NULL);
+#else
+	    TxPrintf("%s", value);
 #endif
+	else
+	    TxError("Property name %s is not defined\n");
+    }
+    else if (cmd->tx_argc == 3)
+    {
+	if (strlen(cmd->tx_argv[2]) == 0)
+	    DBPropPut(editDef, cmd->tx_argv[1], NULL);
+	else
+	{
+	    value = StrDup((char **)NULL, cmd->tx_argv[2]);
+	    DBPropPut(editDef, cmd->tx_argv[1], value);
+	}
+	editDef->cd_flags |= (CDMODIFIED | CDGETNEWSTAMP);
+    }    
+    else
+    {
+	TxError("Usage: property [name] [value]\n");
+	TxError("If value is more than one word, enclose in quotes\n");
+    }
+}
+
+int
+printPropertiesFunc(name, value)
+    char *name;
+    ClientData value;
+{
+#ifdef MAGIC_WRAPPER
+    char *keyvalue;
+
+    keyvalue = (char *)mallocMagic(strlen(name) + strlen((char *)value) + 2);
+    sprintf(keyvalue, "%s %s", name, (char *)value);
+    Tcl_AppendElement(magicinterp, keyvalue);
+#else
+    TxPrintf("%s = %s\n", name, value);
+#endif
+
+    return 0;	/* keep the search alive */
+}
