@@ -18,9 +18,10 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdRS.c,v 1.8 2001/03/29 19:50:05 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #ifdef SYSV
 #include <string.h>
@@ -50,8 +51,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdRS.c,v 1.8 200
 #include "sim/sim.h"
 
 extern void DisplayWindow();
-extern double atof();
-extern char *strchr();
 
 
 #if !defined(NO_SIM_MODULE) && defined(RSIM_MODULE)
@@ -85,7 +84,7 @@ CmdRsim(w, cmd)
 	TxPrintf("Simulator already running.  You cannot start another.\n");
 	return;
     }
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID)) {
 	TxError("Put the cursor in a layout window.\n");
 	return;
@@ -99,41 +98,6 @@ CmdRsim(w, cmd)
 }
 #endif
 
-#ifdef ROUTE_AUTO
-/*
- * ----------------------------------------------------------------------------
- * CmdAutoRoute() --
- *
- *	Tcl auto-loading module routine.  If "route" is configured as a
- *	Tcl module, then this routine replaces CmdRoute and the other
- *	command routines associated with the routing functions.  Router
- *	routines are not loaded at runtime, but are loaded the first time
- *	the "route" ("iroute", "garoute", etc.) command is invoked.
- *
- *	The job of replacing the appropriate entries in CmdFuncs[] is
- *	the responsibility of the initialization function.  If something
- *	goes wrong with the auto-load, it MUST set an error condition in
- *	Tcl or this routine will infinitely recurse!
- *
- * ----------------------------------------------------------------------------
- */
-
-CmdAutoRoute(w, cmd)
-    MagWindow *w;
-    TxCommand *cmd;
-{
-    int result;
-
-    /* Auto-load */
-    result = Tcl_Eval(magicinterp, "load " TCL_DIR "/tclroute.so");
-
-    /* Call function which was originally intended */
-
-    if (result == TCL_OK)
-	WindSendCommand((WindClient)NULL, cmd);
-}
-#endif
-
 
 /*
  * ----------------------------------------------------------------------------
@@ -363,7 +327,7 @@ CmdSee(w, cmd)
     TileTypeBitMask mask;
     DBWclientRec *crec;
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Point to a layout window first.\n");
@@ -641,14 +605,17 @@ CmdSelect(w, cmd)
      * second table, due to a help message for ":select" with no arguments.
      */
 
-#define AREA	0
-#define VISIBLE	1
-#define CELL	2
-#define SCLEAR	3
-#define HELP	4
-#define SAVE	5
-#define SBOX	6
-#define DEFAULT	7
+#define AREA	 0
+#define VISIBLE	 1
+#define CELL	 2
+#define SCLEAR	 3
+#define HELP	 4
+#define SAVE	 5
+#define SBOX	 6
+#define CHUNK	 7
+#define REGION   8
+#define NET	 9
+#define DEFAULT	10 
 
     static char *cmdSelectOption[] =
     {
@@ -659,23 +626,25 @@ CmdSelect(w, cmd)
 	"help",
 	"save",
 	"box",
+	"chunk",
+	"region",
+	"net",
 	NULL
     };
     static char *cmdSelectMsg[] =
     {
-	"[more | less] [layers]          [de]select paint chunk/region/net under\n\
-                                           cursor, or [de]select subcell if cursor\n\
-		                           over space",
+	"[more | less] [layers]          [de]select paint chunk/region/net under\n"
+ 	"                                cursor, or [de]select subcell if cursor\n"
+	"	                         over space",
 	"[more | less] area [layers]     [de]select all info under box in layers",
 	"[more | less] visible [layers]  [de]select all visible info under box in layers",
 	"[more | less | top] cell [name] [de]select cell under cursor, or \"name\"",
-	"get				 get current selection",
 	"clear                           clear selection",
 	"help                            print this message",
 	"save file                       save selection on disk in file.mag",
-	"[more | less] box [layers]      [de]select chunk/region/net \n\
-					 specified by lower left corner\n\
-					 of the current box",
+	"[more | less] box | chunk | region | net [layers]\n"
+	"				 [de]select chunk/region/net specified by"
+	"				 the lower left corner of the current box",
 	NULL
     };
 
@@ -701,9 +670,6 @@ CmdSelect(w, cmd)
     static int level;		/* How big a piece to select.  See definitions
 				 * below.
 				 */
-#define CHUNK 0
-#define REGION 1
-#define NET 2
     static CellUse *lastUse;	/* The last cellUse selected.  Used to step
 				 * through multiple uses underneath the cursor.
 				 */
@@ -728,7 +694,7 @@ CmdSelect(w, cmd)
 #define MARGIN 2
 
     bzero(&scx, sizeof(SearchContext));
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Put the cursor in a layout window\n");
@@ -879,7 +845,7 @@ CmdSelect(w, cmd)
 	    cmdSaveCell(SelectDef, cmd->tx_argv[2], FALSE, FALSE);
 	    return 0;
 	
-	case SBOX:
+	case SBOX: case CHUNK: case REGION: case NET:
 	    if (cmd->tx_argc > 3) goto usageError;
 	    if (cmd->tx_argc == 3)
 		layerspec = TRUE;
@@ -907,7 +873,8 @@ CmdSelect(w, cmd)
 	    layerspec = FALSE;
 Okay:
 	    if (!(more || less)) SelectClear();
-	    if (option == SBOX)
+	    if (option == SBOX || option == CHUNK || option == REGION
+			|| option == NET)
 	    {
 		CellDef *rootBoxDef;
 		int windowMask, xMask;
@@ -924,7 +891,8 @@ Okay:
 		    window = CmdGetRootPoint ((Point *) NULL, (Rect *) NULL);
 		    xMask = ((DBWclientRec *)window->w_clientData)->dbw_bitmask;
 		    if (windowMask & xMask == 0) {
-			TxError("Box present in multiple windows; use the cursor\nto select the one you want\n");
+			TxError("Box present in multiple windows; use the"
+				"cursor\nto select the one you want\n");
 			return 0;
 		    }
 		}
@@ -993,19 +961,30 @@ Okay:
 			return 0;
 		    }
 		}
-		if (samePlace && lessCycle == less)
-		{
-		    level += 1;
-		    if (level > NET) level = CHUNK;
-		}
-		else level = CHUNK;
 
-		if (level != CHUNK && !TTMaskHasType (&mask, type)) {
-		    /* User specified a new mask, and the current tile
-		     * type being expanded is not in the set of types
-		     * which the user wants us to use => reset level
-		     */
-		    level = CHUNK;
+		/* Set connectivity searching level */
+
+		if (option == CHUNK || option == REGION || option == NET)
+		{
+		    samePlace = TRUE;
+		    level = option;
+		}
+		else
+		{
+		    if (samePlace && lessCycle == less)
+		    {
+			level += 1;
+			if (level > NET) level = CHUNK;
+		    }
+		    else level = CHUNK;
+
+		    if (level != CHUNK && !TTMaskHasType (&mask, type)) {
+			/* User specified a new mask, and the current tile
+			 * type being expanded is not in the set of types
+			 * which the user wants us to use => reset level
+			 */
+			level = CHUNK;
+		    }
 		}
 
 		lessCycle = less;
@@ -1315,7 +1294,7 @@ CmdSgraph(w, cmd)
 	TxError("(Magic was not linked with stretchgraph module.)\n");
     }
 }
-#endif	LLNL
+#endif	/* LLNL */
 
 #if !defined(NO_SIM_MODULE) && defined(RSIM_MODULE)
 /*
@@ -1349,7 +1328,7 @@ CmdStartRsim(w, cmd)
 	TxPrintf("Simulator already running.  You cannont start another.\n");
 	return;
     }
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID)) {
 	TxError("Put the cursor in a layout window.\n");
 	return;
@@ -1398,7 +1377,7 @@ CmdSimCmd(w, cmd)
 	TxPrintf("usage: simcmd command [options]\n");
 	return;
     }
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID)) {
 	TxError("Put the cursor in a layout window.\n");
 	return;
@@ -1521,7 +1500,7 @@ CmdSplit(w, cmd)
     int pNum, direction;
     PaintUndoInfo ui;
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Put the cursor in a layout window\n");
@@ -1612,7 +1591,7 @@ CmdSplitErase(w, cmd)
     int pNum, direction;
     PaintUndoInfo ui;
 
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
     {
 	TxError("Put the cursor in a layout window\n");
@@ -1715,7 +1694,7 @@ CmdStretch(w, cmd)
     {
 	int indx, amount;
 
-	indx = GeoNameToPos(cmd->tx_argv[1], TRUE, TRUE);
+	indx = GeoNameToPos(cmd->tx_argv[1], FALSE, TRUE);
 	if (indx < 0)
 	    return;
 	if (cmd->tx_argc == 3)
@@ -1740,6 +1719,22 @@ CmdStretch(w, cmd)
 		xdelta = -amount;
 		ydelta = 0;
 		break;
+	    case GEO_NORTHEAST:
+		xdelta = amount;
+		ydelta = amount;
+		break;
+	    case GEO_NORTHWEST:
+		xdelta = -amount;
+		ydelta = amount;
+		break;
+	    case GEO_SOUTHEAST:
+		xdelta = amount;
+		ydelta = -amount;
+		break;
+	    case GEO_SOUTHWEST:
+		xdelta = -amount;
+		ydelta = -amount;
+		break;
 	    default:
 		ASSERT(FALSE, "Bad direction in CmdStretch");
 		return;
