@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdTZ.c,v 1.7 2001/03/29 19:50:05 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -48,7 +48,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/commands/CmdTZ.c,v 1.7 200
 
 #ifdef	LLNL
 #include "yacr.h"
-#endif	LLNL
+#endif	/* LLNL */
 
 extern void DisplayWindow();
 
@@ -90,11 +90,12 @@ CmdTech(w, cmd)
     MagWindow *w;		/* Window in which command was invoked. */
     TxCommand *cmd;		/* Info about command options. */
 {
-    int	option, action, i;
+    int	option, action, i, locargc;
     char **msg;
 #ifdef MAGIC_WRAPPER
     Tcl_Obj *lobj;
 #endif
+    bool noprompt = FALSE;
     int saveNumPlanes;
     int changePlanesFunc();	/* forward declaration */
 
@@ -103,7 +104,8 @@ CmdTech(w, cmd)
 
     static char *cmdTechOption[] =
     {	
-	"load filename		Load new technology",
+	"load filename [-noprompt][-nooverride]\n\
+				Load a new technology",
 	"help			Display techinfo command options",
 	"name			Show current technology name",
 	"filename		Show current technology filename",
@@ -190,9 +192,29 @@ CmdTech(w, cmd)
 	    break;
 
 	case TECH_LOAD:
-	    if (cmd->tx_argc != 3)
+	    locargc = cmd->tx_argc;
+	    while ((*cmd->tx_argv[locargc - 1] == '-') && (locargc > 3))
 	    {
-		TxError("Usage: tech load <filename>\n");
+		if (!strcmp(cmd->tx_argv[locargc - 1], "-nooverride"))
+		{
+		    /* Disallow the "tech load" command from overriding */
+		    /* a technology specified on the command line.	*/
+		    /* This is used when "tech load" sets up a specific	*/
+		    /* project technology from a .magic startup script.	*/
+
+		    if (TechOverridesDefault)
+			return;
+		}
+		else if (!strcmp(cmd->tx_argv[locargc - 1], "-noprompt"))
+		    noprompt = TRUE;
+		else
+		    break;
+		locargc--;
+	    }
+
+	    if (locargc != 3)
+	    {
+		TxError("Usage: tech load <filename> [-noprompt] [-nooverride]\n");
 		break;
 	    }
 
@@ -203,10 +225,12 @@ CmdTech(w, cmd)
 
 	    /* For now: warn without checking */
 
-	    action = TxDialog("Technology file (re)loading may invalidate"
-			" any existing layout.  Continue? ",
-			actionNames, 0);
-	    if (action == 0) return;
+	    if (!noprompt)
+	    {
+		action = TxDialog("Technology file (re)loading may invalidate"
+			" any existing layout.  Continue? ", actionNames, 0);
+		if (action == 0) return;
+	    }
 	
 	    /* Changing number of planes requires handling on every	*/
 	    /* celldef.  So we need to save the original number of	*/
@@ -224,7 +248,35 @@ CmdTech(w, cmd)
 #endif
 	    ExtTechInit();
 
-	    TechLoad(cmd->tx_argv[2], 0);
+	    if (!TechLoad(cmd->tx_argv[2], 0))
+	    {
+#ifdef MAGIC_WRAPPER
+		Tcl_SetResult(magicinterp, "Error in loading technology file\n", NULL);
+#else
+		TxError("Error in loading technology file\n");
+#endif
+		break;
+	    }
+
+	    /* If internal scalefactor is not the default 1:1, then we  */
+	    /* need to scale the techfile numbers accordingly.          */
+
+	    if ((DBLambda[0] != 1) || (DBLambda[1] != 1))
+	    {
+		int d = DBLambda[0];
+		int n = DBLambda[1];
+
+		CIFTechInputScale(d, n);      
+		CIFTechOutputScale(d, n);
+		DRCTechScale(d, n);
+		ExtTechScale(d, n);
+		WireTechScale(d, n);
+#ifdef ROUTE_MODULE
+		RtrTechScale(d, n);
+#endif
+		TxPrintf("Scaled tech values by %d / %d to" 
+				" match internal grid scaling\n", n, d);
+	    }
 
 	    if (saveNumPlanes != DBNumPlanes)
 	    {
@@ -384,7 +436,7 @@ CmdUnexpand(w, cmd)
 	return;
     }
     
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL)
     {
 	TxError("Point to a window first.\n");
@@ -517,10 +569,12 @@ CmdWhat(w, cmd)
     MagWindow *w;		/* Window in which command was invoked. */
     TxCommand *cmd;		/* Information about the command. */
 {
+    int i;
     bool foundAny;
     TileTypeBitMask layers;
     extern int cmdWhatPaintFunc(), cmdWhatLabelFunc(), cmdWhatCellFunc();
     extern int cmdWhatLabelPreFunc(), orderLabelFunc();
+
     if (cmd->tx_argc > 1)
     {
 	TxError("Usage: what\n");
@@ -532,10 +586,23 @@ CmdWhat(w, cmd)
     TTMaskZero(&layers);
     (void) SelEnumPaint(&DBAllButSpaceAndDRCBits, FALSE, (bool *) NULL,
 	    cmdWhatPaintFunc, (ClientData) &layers);
+
     if (!TTMaskIsZero(&layers))
     {
-	int i;
+	/* If there are any stacked types in the list, decompose them	*/
+	/* into their residues (which are the two contact types that	*/
+	/* are stacked).						*/
 
+	for (i = DBNumUserLayers; i < DBNumTypes; i++)
+	{
+	    if (TTMaskHasType(&layers, i))
+		TTMaskSetMask(&layers, DBResidueMask(i));
+	    TTMaskClearType(&layers, i);
+	}
+    }
+
+    if (!TTMaskIsZero(&layers))
+    {
 	TxPrintf("Selected mask layers:\n");
 	for (i = TT_SELECTBASE; i < DBNumUserLayers; i++)
 	{
@@ -1013,7 +1080,7 @@ CmdXload(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    windCheckOnlyWindow(&w);
+    windCheckOnlyWindow(&w, DBWclientID);
     if (w == (MagWindow *) NULL)
     {
 	TxError("Point to a window first.\n");
