@@ -24,11 +24,11 @@
 
 #ifndef	_WINDOWS
 int err0 = Need_to_include_window_header;
-#endif	_WINDOWS
+#endif	/* _WINDOWS */
 
 #ifndef	_DATABASE
 int err2 = Need_to_include_database/database_header;
-#endif	_DATABASE
+#endif	/* _DATABASE */
 
 /*
  * Name of default yank buffer
@@ -36,14 +36,6 @@ int err2 = Need_to_include_database/database_header;
 
 #define YANKBUFFERNAME	"y"
 
-/*
- * The table of commands that we will accept
- *
- */
-
-extern char *CmdLongCommands[];
-extern Void (*CmdFuncs[])();
-
 /*
  * Manipulation of user-supplied "layer" masks.
  * These may include both layers specifiable in a TileTypeMask,
