@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellbox.c,v 1.2 2001/01/12 22:12:19 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -598,6 +598,10 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
     register bool foundAny;
     int pNum;
 
+    /* Cells which declare their bounding box to be fixed	*/
+    /* must return immediately.					*/
+    if (cellDef->cd_flags & CDFIXEDBBOX) return;
+
     foundAny = FALSE;
     for (pNum = 0; pNum < DBNumPlanes; pNum++)
 	if (pNum != PL_DRC_CHECK)
