@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellname.c,v 1.4 2001/05/23 20:57:42 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #ifdef SYSV
@@ -39,6 +39,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellname.c,v 1.
 #include "utils/malloc.h"
 #include "windows/windows.h"
 #include "textio/textio.h"
+#include "dbwind/dbwind.h"
 #include "main/main.h"
 
 /*
@@ -65,6 +66,168 @@ static void dbUsePrintInfo();
  */
 extern void dbComputeBbox();
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBCellRename --
+ *
+ *	Rename a cell.  This differs from the "save" command in that the
+ *	cell is not immediately written to disk.  However, the cell is
+ *	marked as modified.
+ *
+ * Results:
+ *	Return TRUE if the cell was successfully renamed.  Return FALSE
+ *	otherwise.
+ *
+ * Side Effects:
+ *	Cellname is changed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+bool
+DBCellRename(cellname, newname)
+    char *cellname;
+    char *newname;
+{
+    HashEntry *entry;
+    CellDef *celldef;
+    bool result;
+
+    entry = HashLookOnly(&dbCellDefTable, cellname);
+    if (entry == NULL)
+    {
+	TxError("No such cell \"%s\"\n",  cellname);
+	return FALSE;
+    }
+
+    celldef = (CellDef *)HashGetValue(entry);
+    if (celldef == NULL) return FALSE;
+
+    if ((celldef->cd_flags & CDINTERNAL) == CDINTERNAL)
+    {
+	TxError("Attempt to rename internal cell \"%s\"\n", cellname);
+	return FALSE;
+    }
+
+    /* Good to go! */
+
+    UndoDisable();
+    result = DBCellRenameDef(celldef, newname);
+    DBWAreaChanged(celldef, &celldef->cd_bbox, DBW_ALLWINDOWS,
+        (TileTypeBitMask *) NULL);
+    UndoEnable();
+    return result;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBCellDelete --
+ *
+ *	Destroy a cell.
+ *
+ * Results:
+ *	Return TRUE if the cell was successfully eliminated.  Return FALSE
+ *	if the cell could not be deleted (i.e., has dependencies).
+ *
+ * Side Effects:
+ *	Frees memory.  May prompt the user to save, if the cell has been
+ *	changed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+bool
+DBCellDelete(cellname)
+    char *cellname;
+{
+    HashEntry *entry;
+    CellDef *celldef;
+    CellUse *celluse;
+    bool result;
+
+    entry = HashLookOnly(&dbCellDefTable, cellname);
+    if (entry == NULL)
+    {
+	TxError("No such cell \"%s\"\n",  cellname);
+	return FALSE;
+    }
+
+    celldef = (CellDef *)HashGetValue(entry);
+    if (celldef == NULL) return FALSE;
+
+    if ((celldef->cd_flags & CDINTERNAL) == CDINTERNAL)
+    {
+	TxError("Attempt to delete internal cell \"%s\"\n", cellname);
+	return FALSE;
+    }
+
+    /* If the cell has any instances which are not top-level	*/
+    /* uses, don't allow the deletion.				*/
+
+    for (celluse = celldef->cd_parents; celluse != (CellUse *) NULL;
+	 	celluse = celluse->cu_nextuse)
+	if (celluse->cu_parent != (CellDef *) NULL)
+	    if ((celluse->cu_parent->cd_flags & CDINTERNAL) != CDINTERNAL)
+		break;
+
+    if (celluse != (CellUse *)NULL)
+    {
+	TxError("Cell has non-top-level dependency in use \"%s\"\n",
+		celluse->cu_id);
+	return FALSE;
+    }
+
+    /* Cleared to delete. . . now prompt user if the cell has changes. */
+    /* Last chance to save! */
+
+    if (celldef->cd_flags & (CDMODIFIED|CDBOXESCHANGED|CDSTAMPSCHANGED))
+    {
+	static char *yesno[] = { "no", "yes", 0 };
+	int code;
+	char *prompt = TxPrintString("Cell %s has been modified.\n  Do you"
+		" want to delete it and lose all changes? ",
+		cellname);
+	code = TxDialog(prompt, yesno, 0);
+	if (code == 0) return FALSE;
+    }
+
+    /* If we're destroying the cell pointed to by dbUndoLastCell, we	*/
+    /* need to NULL it and flush the Undo queue.			*/
+
+    DBUndoReset(celldef);
+
+    /* If we're destroying "UNNAMED", then we want to rename it first	*/
+    /* so that WindUnload() will create a new one.			*/ 
+
+    if (!strcmp(cellname, UNNAMED))
+	DBCellRename(cellname, "__UNNAMED__");
+
+    /* For all top-level cell uses, check if any windows have this	*/
+    /* use.  If so, load the window with (UNNAMED).			*/
+
+    UndoDisable();
+    for (celluse = celldef->cd_parents; celluse != (CellUse *) NULL;
+	 	celluse = celluse->cu_nextuse)
+    {
+	if (celluse->cu_parent == (CellDef *) NULL)
+	{
+	    WindUnload(celluse);
+	    FREE(celluse->cu_id);
+	}
+	FREE((char *)celluse);
+    }
+    celldef->cd_parents = (CellUse *)NULL;
+
+    result = DBCellDeleteDef(celldef);
+
+    if (result == FALSE)
+	TxError("Error:  Deleted all cell uses, but could not delete cell.\n");
+
+    UndoEnable();
+    return result;
+}
+
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -248,7 +411,7 @@ dbCellPrintInfo(StartDef, who, dolist)
 			== CDINTERNAL))
 		    continue;
 
-		if (!strncmp(celluse->cu_id, "Topmost ", 8))
+		if (celluse->cu_id && (!strncmp(celluse->cu_id, "Topmost ", 8)))
 		{
 		    if (topdone == TRUE) continue;
 		    topdone = TRUE;
