@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBcellsubr.c,v 1.2 2001/01/12 22:12:20 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
