@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBconnect.c,v 1.7 2001/11/29 20:08:51 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -44,7 +44,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBconnect.c,v 1.7
 
 /* The following structure is used to hold several pieces
  * of information that must be passed through multiple
- * levels of search function.
+ * levels of search function (used by dbSrConnectFunc).
  */
 	
 struct conSrArg
@@ -62,28 +62,33 @@ struct conSrArg
     Rect csa_bounds;			/* Area that limits search. */
 };
 
-/* For DBTreeSrConnect, the extraction proceeds in one pass, copying
- * all connected stuff from a hierarchy into a single cell.  A list
- * is kept to record areas that still have to be searched for
- * hierarchical stuff.
+/* The following structure is used to hold several pieces
+ * of information that must be passed through multiple
+ * levels of search function (used by dbcConnectFunc).
  */
+	
+typedef struct
+{
+    Rect		area;		/* Area to process */
+    TileTypeBitMask	*connectMask;	/* Connection mask for search */
+} conSrArea;
 
 struct conSrArg2
 {
-    Rect csa2_area;			/* Area that we know is connected, but
-					 * which may be connected to other
-					 * stuff.
+    CellUse		*csa2_use;	/* Destination use */
+    TileTypeBitMask	*csa2_connect;	/* Table indicating what connects
+					 * to what.
 					 */
-    TileType csa2_type;			/* Type of material we found. */
-    struct conSrArg2 *csa2_next;	/* Pointer to next in list. */
+    SearchContext	*csa2_topscx;	/* Original top-level search context */
+    int			 csa2_xMask;	/* Cell window mask for search */
+    Rect		*csa2_bounds;	/* Area that limits the search */
+
+    conSrArea		*csa2_list;	/* List of areas to process */
+    int			csa2_top;	/* Index of next area to process */
+    int			csa2_size;	/* Max. number bins in area list */
 };
 
-int dbcReturn;				/* dbcConnectFunc always returns this
-					 * value.  It can be set to 1 by a
-					 * higher-level procedure to abort
-					 * searches after the first value is
-					 * found.
-					 */
+#define CSA2_LIST_START_SIZE 256
 
 
 /*
@@ -208,6 +213,7 @@ dbSrConnectStartFunc(tile, pTile)
     *pTile = tile;
     return 1;
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -462,9 +468,10 @@ donesides:
  */
 
 int
-dbcUnconnectFunc(tile, cdata)
-    Tile *tile;			/* Current tile. */
-    Cvals *cdata;		/* Passed data with overlap area and type mask */
+dbcUnconnectFunc(tile, connectMask)
+    Tile *tile;			    /* Current tile. */
+    TileTypeBitMask *connectMask;
+    
 {
     Rect tileArea;
 
@@ -478,16 +485,77 @@ dbcUnconnectFunc(tile, cdata)
     if (IsSplit(tile))
     {
 	loctype = (SplitSide(tile)) ? SplitLeftType(tile) : SplitRightType(tile);
-	if (TTMaskHasType(cdata->mask, loctype))
+	if (TTMaskHasType(connectMask, loctype))
 	    return 0;
     }
 #endif
 
-    TiToRect(tile, &tileArea);
-    if (GEO_OVERLAP(&tileArea, cdata->area)) return 1;
-    else return 0;
+    return 1;
 }
 
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbcConnectLabelFunc --
+ *
+ * 	This function is invoked by DBTreeSrTiles from DBTreeCopyConnect;
+ *	when a label is found which is connected to paint belonging to the
+ *	network, it adds it to the destination definition.
+ *
+ * Results:
+ *	Always 0.
+ *
+ * Side effects:
+ *	Adds a label to the destination definition "def".
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+dbcConnectLabelFunc(scx, lab, tpath, csa2)
+    SearchContext *scx;
+    Label *lab;
+    TerminalPath *tpath;
+    struct conSrArg2 *csa2;
+{
+    CellDef *def = csa2->csa2_use->cu_def;
+    Rect r;
+    int pos;
+    int dbcConnectFunc();		/* Forward declaration */
+
+    GeoTransRect(&scx->scx_trans, &lab->lab_rect, &r);
+    pos = GeoTransPos(&scx->scx_trans, lab->lab_pos);
+
+    DBEraseLabelsByContent(def, &r, pos, -1, lab->lab_text);
+    DBPutLabel(def, &r, pos, lab->lab_text, lab->lab_type, lab->lab_flags);
+
+    if (lab->lab_flags & PORT_DIR_MASK)
+    {
+	CellDef *orig_def = scx->scx_use->cu_def;
+	Label *slab;
+	int lidx = lab->lab_flags & PORT_NUM_MASK;
+
+	/* Check for equivalent ports. For any found, call	*/
+	/* DBTreeSrTiles recursively on the type and area	*/
+	/* of the label.					*/
+
+	for (slab = orig_def->cd_labels; slab != NULL; slab = slab->lab_next)
+	    if ((slab->lab_flags & PORT_DIR_MASK) && (slab != lab))
+		if ((slab->lab_flags & PORT_NUM_MASK) == lidx)
+		{
+		    SearchContext scx2 = *csa2->csa2_topscx;
+		    TileTypeBitMask mask;
+
+		    TTMaskSetOnlyType(&mask, slab->lab_type);
+		    GeoTransRect(&scx->scx_trans, &slab->lab_rect, &scx2.scx_area);
+		    DBTreeSrTiles(&scx2, &mask, csa2->csa2_xMask,
+				dbcConnectFunc, (ClientData) csa2);
+		}
+    }
+    return 0;
+}
+			
 /*
  * ----------------------------------------------------------------------------
  *
@@ -496,16 +564,19 @@ dbcUnconnectFunc(tile, cdata)
  * 	This procedure is invoked by DBTreeSrTiles from DBTreeCopyConnect,
  *	whenever a tile is found that is connected to the current area
  *	being processed.  If the tile overlaps the search area in a non-
- *	trivial way (i.e. more than a 1x1 square of overlap at a corner)
- *	then the area of the tile is added onto the list of things to check.
- *	The "non-trivial" overlap check is needed to prevent caddy-corner
- *	tiles from being considered as connected.
+ *	trivial way (i.e. more than a 1x1 square of overlap at a corner),
+ *	then its area is checked against the equivalent destination area.
+ *	If the destination area contains unconnected portions, then the
+ *	area of the tile is painted into the destination, and an area 1
+ *	unit larger than the tile is recursively checked for connecting
+ *	tiles.  The "non-trivial" overlap check is needed to prevent
+ *	catecorner tiles from being considered as connected.
  *
  * Results:
  *	Always returns 0 to keep the search from aborting.
  *
  * Side effects:
- *	Adds a new record to the current check list.
+ *	Adds paint to the destination definition.
  *
  * ----------------------------------------------------------------------------
  */
@@ -514,19 +585,23 @@ int
 dbcConnectFunc(tile, cx)
     Tile *tile;			/* Tile found. */
     TreeContext *cx;		/* Describes context of search.  The client
-				 * data is a pointer to the list head of
-				 * the conSrArg2's describing the areas
-				 * left to check.
+				 * data is a pointer to a conSrArg2 record
+				 * containing various required information.
 				 */
 {
-    struct conSrArg2 **pHead = (struct conSrArg2 **) cx->tc_filter->tf_arg;
-    struct conSrArg2 *newCsa2;
-    Rect tileArea;
-    register Rect *srArea;
-    register SearchContext *scx = cx->tc_scx;
+    struct conSrArg2 *csa2;
+    Rect tileArea, newarea;
+    TileTypeBitMask *connectMask, notConnectMask;
+    Rect *srArea;
+    SearchContext *scx = cx->tc_scx;
+    SearchContext scx2;
+    TileType newtype, loctype;
+    int pNum;
+    CellDef *def;
 
     TiToRect(tile, &tileArea);
     srArea = &scx->scx_area;
+
     if (((tileArea.r_xbot >= srArea->r_xtop-1) ||
 	(tileArea.r_xtop <= srArea->r_xbot+1)) &&
 	((tileArea.r_ybot >= srArea->r_ytop-1) ||
@@ -540,9 +615,15 @@ dbcConnectFunc(tile, cx)
 	if (((srArea->r_xtop-1) != srArea->r_xbot)
 	    && ((srArea->r_ytop-1) != srArea->r_ybot)) return 0;
     }
-    MALLOC(struct conSrArg2 *, newCsa2, sizeof (struct conSrArg2));
-    GeoTransRect(&scx->scx_trans, &tileArea, &newCsa2->csa2_area);
-    newCsa2->csa2_type = TiGetTypeExact(tile);
+    GeoTransRect(&scx->scx_trans, &tileArea, &newarea);
+
+    /* Clip the current area down to something that overlaps the
+     * area of interest.
+     */
+	
+    csa2 = (struct conSrArg2 *)cx->tc_filter->tf_arg;
+    GeoClip(&newarea, csa2->csa2_bounds);
+    if (GEO_RECTNULL(&newarea)) return 0;
 
 #ifdef NONMANHATTAN
     /* Resolve geometric transformations on diagonally-split tiles */
@@ -550,7 +631,6 @@ dbcConnectFunc(tile, cx)
     if (IsSplit(tile))
     {
 	int o1, o2, o3, dir, side;
-	TileType loctype = newCsa2->csa2_type;
 	
 	o1 = ((scx->scx_trans.t_e > 0) || (scx->scx_trans.t_d > 0)) ? 1 : 0;
 	o2 = ((scx->scx_trans.t_a > 0) || (scx->scx_trans.t_b > 0)) ? 1 : 0;
@@ -559,20 +639,150 @@ dbcConnectFunc(tile, cx)
 	side = SplitSide(tile) ^ o2 ^ (SplitDirection(tile) | o3);
 
 	if (side == SplitSide(tile))
-	    newCsa2->csa2_type = TT_DIAGONAL | SplitLeftType(tile) |
+	    newtype = TT_DIAGONAL | SplitLeftType(tile) |
 			(SplitRightType(tile) << 14);
 	else
-	    newCsa2->csa2_type = TT_DIAGONAL | SplitRightType(tile) |
+	    newtype = TT_DIAGONAL | SplitRightType(tile) |
 			(SplitLeftType(tile) << 14);
 
-	if (side) newCsa2->csa2_type |= TT_SIDE;
-	if (dir) newCsa2->csa2_type |= TT_DIRECTION;
+	if (dir) newtype |= TT_DIRECTION;
+	if (side)
+	{
+	    newtype |= TT_SIDE;
+	    loctype = (newtype & TT_RIGHTMASK) >> 14;
+	}
+	else
+	    loctype = newtype & TT_LEFTMASK;
+    }
+    else
+#endif
+    loctype = newtype = TiGetTypeExact(tile);
+
+    /* See if the destination cell contains stuff over the whole
+     * current area (on its home plane) that is connected to it.
+     * If so, then there's no need to process the current area,
+     * since any processing that is needed was already done before.
+     */
+	
+    pNum = DBPlane(loctype);
+    connectMask = &csa2->csa2_connect[loctype];
+
+    /* In the case of contact bits, the types underneath
+     * must be constituents of the contact before we punt
+     */
+
+    if (DBIsContact(loctype))
+    {
+	TileType ctype;
+
+	TTMaskSetOnlyType(&notConnectMask, loctype);
+
+	/* The mask of contact types must include all stacked contacts */
+	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
+	    if TTMaskHasType(DBResidueMask(ctype), loctype)
+		TTMaskSetType(&notConnectMask, ctype);
+	TTMaskCom(&notConnectMask);
+    }
+    else
+    {
+	TTMaskCom2(&notConnectMask, connectMask);
+    }
+
+    /* Only check those tiles in the destination (select)	*/
+    /* which have not already been painted.			*/
+
+    def = csa2->csa2_use->cu_def;
+    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+		&newarea, &notConnectMask, dbcUnconnectFunc,
+		(ClientData) connectMask) == 0)
+	return 0;
+
+    /* Paint this tile into the destination cell.  This
+     * marks its area has having been processed.  Then recycle
+     * the storage for the current list element.
+     */
+
+#ifdef NONMANHATTAN
+    DBNMPaintPlane(def->cd_planes[pNum], newtype,
+		&newarea, DBStdPaintTbl(loctype, pNum),
+		(PaintUndoInfo *) NULL);
+#else
+    DBPaintPlane(def->cd_planes[pNum], &newarea,
+	    DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
+#endif
+
+    /* Copy information from original context into new search context */
+    scx2 = *csa2->csa2_topscx;
+    scx2.scx_area = newarea;
+
+    /* Check the source def for any labels belonging to this	*/
+    /* tile area and plane, and add them to the destination.	*/
+
+    DBTreeSrLabels(&scx2, connectMask, csa2->csa2_xMask, NULL,
+			dbcConnectLabelFunc, (ClientData) csa2);
+
+    /* Since the whole area of this tile hasn't been recorded,
+     * we must process its area to find any other tiles that
+     * connect to it.  Add each of them to the list of things
+     * to process.  We have to expand the search area by 1 unit
+     * on all sides because DBTreeSrTiles only returns things
+     * that overlap the search area, and we want things that
+     * even just touch.
+     */
+
+#ifdef NONMANHATTAN
+    /* Only extend those sides bordering the diagonal tile */
+	  
+    if (newtype & TT_DIAGONAL)
+    {
+	if (newtype & TT_SIDE)			/* right */
+	    newarea.r_xtop += 1;
+	else					/* left */
+	    newarea.r_xbot -= 1;
+	if (((newtype & TT_SIDE) >> 1)
+		== (newtype & TT_DIRECTION))	/* top */
+	    newarea.r_ytop += 1;
+	else					/* bottom */
+	    newarea.r_ybot -= 1;
+    }
+    else
+    {
+#endif
+    newarea.r_ybot -= 1;
+    newarea.r_ytop += 1;
+    newarea.r_xbot -= 1;
+    newarea.r_xtop += 1;
+#ifdef NONMANHATTAN
     }
 #endif
-	
-    newCsa2->csa2_next = *pHead;
-    *pHead = newCsa2;
-    return dbcReturn;
+
+    /* Register the area and connection mask as needing to be processed */
+
+    if (++csa2->csa2_top == csa2->csa2_size)
+    {
+	/* Reached list size limit---need to enlarge the list	   */
+	/* Double the size of the list every time we hit the limit */
+
+	conSrArea *newlist;
+	int i, lastsize = csa2->csa2_size;
+
+	csa2->csa2_size *= 2;
+
+	newlist = (conSrArea *)mallocMagic(csa2->csa2_size * sizeof(conSrArea));
+	for (i = 0; i < lastsize; i++)
+	{
+	    newlist[i].area = csa2->csa2_list[i].area;
+	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
+	    
+	}
+	freeMagic((char *)csa2->csa2_list);
+	csa2->csa2_list = newlist;
+    }
+
+    csa2->csa2_list[csa2->csa2_top].area = newarea;
+    csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
+
+    return 0;
 }
 
 
@@ -631,133 +841,37 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
 					 * type mask in area of rootUse.
 					 */
 {
-    SearchContext scx2;
-    TileTypeBitMask notConnectMask, *connectMask;
-    TileType loctype;
-    struct conSrArg2 *list, *current;
-    CellDef *def = destUse->cu_def;
-    Cvals cdata;
-    int pNum;
+    struct conSrArg2 csa2;
+    TileTypeBitMask *newmask;
 
-    /* Get one initial tile and put it our list. */
+    csa2.csa2_use = destUse;
+    csa2.csa2_xMask = xMask;
+    csa2.csa2_bounds = area;
+    csa2.csa2_connect = connect;
+    csa2.csa2_topscx = scx;
 
-    list = NULL;
-    dbcReturn = 1;
-    (void) DBTreeSrTiles(scx, mask, xMask, dbcConnectFunc, (ClientData) &list);
-    dbcReturn = 0;
+    /* Instead of using a linked list, we keep down the number of	*/
+    /* malloc calls by maintaining a small list and expanding it only	*/
+    /* when necessary.							*/
 
-    /* Enter the main processing loop, pulling areas from the list
-     * and processing them one at a time until the list is empty.
-     */
-    
-    scx2 = *scx;
-    while (list != NULL)
+    csa2.csa2_size = CSA2_LIST_START_SIZE;
+    csa2.csa2_list = (conSrArea *)mallocMagic(CSA2_LIST_START_SIZE
+			* sizeof(conSrArea));
+    csa2.csa2_top = -1;
+
+    DBTreeSrTiles(scx, mask, xMask, dbcConnectFunc, (ClientData) &csa2);
+    while (csa2.csa2_top >= 0)
     {
-	current = list;
-	list = current->csa2_next;
-
-	/* Clip the current area down to something that overlaps the
-	 * area of interest.
-	 */
-	
-	GeoClip(&current->csa2_area, area);
-	if (GEO_RECTNULL(&current->csa2_area)) goto endOfLoop;
-
-	/* See if the destination cell contains stuff over the whole
-	 * current area (on its home plane) that is connected to it.
-	 * If so, then there's no need to process the current area,
-	 * since any processing that is needed was already done before.
-	 */
-	
-#ifdef NONMANHATTAN
-	if (current->csa2_type & TT_DIAGONAL)
-	{
-	    loctype = (current->csa2_type & TT_SIDE) ?
-			((current->csa2_type & TT_RIGHTMASK) >> 14) :
-			current->csa2_type & TT_LEFTMASK;
-	}
-	else
-#endif
-	loctype = current->csa2_type;
-
-	connectMask = &connect[loctype];
-	/* in the case of contact bits, the types underneath
-	   must be constituents of the contact before we  punt */
-	if (TTMaskHasType(&DBContactBits,loctype))
-	{
-	     TTMaskSetOnlyType(&notConnectMask,loctype);
-	     TTMaskCom(&notConnectMask);
-	}
-	else
-	{
-	     TTMaskCom2(&notConnectMask, connectMask);
-	}
-	pNum = DBPlane(loctype);
-	cdata.area = &(current->csa2_area);
-	cdata.mask = connectMask;
-	if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-	    &current->csa2_area, &notConnectMask,
-	    dbcUnconnectFunc, (ClientData) &cdata) != 0)
-	{
-	    /* Since the whole area of this tile hasn't been recorded,
-	     * we must process its area to find any other tiles that
-	     * connect to it.  Add each of them to the list of things
-	     * to process.  We have to expand the search area by 1 unit
-	     * on all sides because DBTreeSrTiles only returns things
-	     * that overlap the search area, and we want things that
-	     * even just touch.
-	     */
-
-	    scx2.scx_area = current->csa2_area;
-
-#ifdef NONMANHATTAN
-	    /* Only extend those sides bordering the diagonal tile */
-	  
-	    if (current->csa2_type & TT_DIAGONAL)
-	    {
-		if (current->csa2_type & TT_SIDE)		/* right */
-		    scx2.scx_area.r_xtop += 1;
-		else						/* left */
-		    scx2.scx_area.r_xbot -= 1;
-		if (((current->csa2_type & TT_SIDE) >> 1)
-			== (current->csa2_type & TT_DIRECTION)) /* top */
-		    scx2.scx_area.r_ytop += 1;
-		else						/* bottom */
-		    scx2.scx_area.r_ybot -= 1;
-	    }
-	    else
-	    {
-#endif
-	    scx2.scx_area.r_ybot -= 1;
-	    scx2.scx_area.r_ytop += 1;
-	    scx2.scx_area.r_xbot -= 1;
-	    scx2.scx_area.r_xtop += 1;
-#ifdef NONMANHATTAN
-	    }
-#endif
-
-	    (void) DBTreeSrTiles(&scx2, connectMask,
-		xMask, dbcConnectFunc, (ClientData) &list);
-	}
-
-	/* Lastly, paint this tile into the destination cell.  This
-	 * marks its area has having been processed.  Then recycle
-	 * the storage for the current list element.
-	 */
-#ifdef NONMANHATTAN
-	DBNMPaintPlane(def->cd_planes[pNum], current->csa2_type,
-		&current->csa2_area, DBStdPaintTbl(loctype, pNum),
-		(PaintUndoInfo *) NULL);
-#else
-	DBPaintPlane(def->cd_planes[pNum], &current->csa2_area,
-	    DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
-#endif
-	endOfLoop: FREE((char *) current);
+	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
+	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
+	csa2.csa2_top--;
+	DBTreeSrTiles(scx, newmask, xMask, dbcConnectFunc, (ClientData) &csa2);
     }
+    freeMagic((char *)csa2.csa2_list);
 
-    /* Finally, when all done, recompute the bounding box of the
-     * destination and record its area for redisplay.
+    /* Recompute the bounding box of the destination and record its area
+     * for redisplay.
      */
     
-    DBReComputeBbox(def);
+    DBReComputeBbox(destUse->cu_def);
 }
