@@ -18,10 +18,11 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001/06/07 21:20:19 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #ifdef __STDC__
 #include <unistd.h>
 #endif
@@ -39,6 +40,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBio.c,v 1.5 2001
 #include "utils/hash.h"
 #include "database/database.h"
 #include "database/databaseInt.h"
+#include "windows/windows.h"
+#include "dbwind/dbwind.h"
 #include "tech/tech.h"
 #include "textio/textio.h"
 #include "drc/drc.h"
@@ -178,6 +181,11 @@ file_is_not_writeable(name)
  *	form "<< layer >>".	The layer name is matched against the
  *	current technology.  Each line after the header has the
  *	format "rect <xbot> <ybot> <xtop> <ytop>".
+ *	Nonmanhattan geometry is covered by the entry
+ *	"try <xbot> <ybot> <xtop> <ytop> <dir>" with <dir> indicating
+ *	the direction of the corner made by the right triangle.
+ *	If the split tile contains more than one type, separate entries
+ *	are output for each.
  *	
  *	6. Zero or more groups of lines describing cell uses.  Each group
  *	is of the form
@@ -201,11 +209,20 @@ file_is_not_writeable(name)
  *		label <layer> <x> <y> <position> <text>
  *	(the second form is obsolete and is for point labels only).
  *
- *	8. The file is terminated by the line "<< end >>".
+ *	Ports are declared in the label line after the label they refer
+ *	to.  The syntax is
+ *		port <index> <dirs>
+ *	where <dir> is a string with one to four characters "nsew",
+ *	denoting the position of allowed connections to the port.  The
+ *	index must be unique to each port in a cell.
+ *
+ *	9. Elements are saved in a section "<< elements >>".
+ *
+ *     10. The file is terminated by the line "<< end >>".
  *
  *	Note: be careful about any changes to this format:  there are
  *	several previous file formats still in use in old files, and
- *	the current one is upward compatible with all of them.
+ *	the current one is backwardly compatible with all of them.
  *
  *
  * ----------------------------------------------------------------------------
@@ -372,15 +389,27 @@ DBCellRead(cellDef, name, setFileName)
 	{
 	    /*
 	     * Look for special layer names:
-	     *		labels	-- begins a list of labels
-	     *		end	-- marks the end of this file
+	     *		labels	   -- begins a list of labels and ports
+	     *		elements   -- begins a list of elements
+	     *		properties -- begins a list of properties
+	     *		end	   -- marks the end of this file
 	     */
-	    if (strcmp(layername, "labels") == 0)
+	    if (!strcmp(layername, "labels"))
 	    {
 		if (!dbReadLabels(cellDef, line, sizeof line, f, n)) goto badfile;
 		continue;
 	    }
-	    if (strcmp(layername, "end") == 0) goto done;
+	    else if (!strcmp(layername, "elements"))
+	    {
+		if (!dbReadElements(cellDef, line, sizeof line, f, n)) goto badfile;
+		continue;
+	    }
+	    else if (!strcmp(layername, "properties"))
+	    {
+		if (!dbReadProperties(cellDef, line, sizeof line, f)) goto badfile;
+		continue;
+	    }
+	    else if (!strcmp(layername, "end")) goto done;
 	    TxError("Unknown layer %s ignored in %s\n", layername,
 			cellDef->cd_name);
 	}
@@ -396,11 +425,6 @@ DBCellRead(cellDef, name, setFileName)
 	     */
 	    
 	    TTMaskSetType(&cellDef->cd_types, type);
-
-	    /*
-	     * Check for "subcircuit" boundary
-	     */ 
-	    if (type == TT_SUBCIRCUIT) cellDef->cd_flags |= CDSUBCIRCUIT;
 	}
 	else plane = NULL;
 
@@ -569,6 +593,9 @@ done:
 badfile:
     TxError("File %s contained format error\n", cellDef->cd_name);
     DRCCheckThis(cellDef, TT_CHECKPAINT, (Rect *) NULL);
+#ifdef FILE_LOCKS
+    flock_flag = 1;
+#endif
     result = FALSE;
     goto done;
 }
@@ -610,6 +637,9 @@ dbReadOpen(cellDef, name, setFileName)
 {
     FILE *f;
     char *filename;
+#ifdef FILE_LOCKS
+    char *realname;
+#endif
 
     if (name != (char *) NULL)
     {
@@ -638,13 +668,13 @@ dbReadOpen(cellDef, name, setFileName)
 	else {
 	    TxError("Cell %s couldn't be found\n", cellDef->cd_name);
 #ifdef FILE_LOCKS
-	    f = PaOpen(cellDef->cd_name, "r", DBSuffix, ".", (char *) NULL, &filename);
+	    MALLOC(char *, realname, strlen(cellDef->cd_name) + strlen(DBSuffix) + 1);
+	    (void) sprintf(realname, "%s%s", cellDef->cd_name, DBSuffix);
+	    cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
 #endif
 	}
 	cellDef->cd_flags |= CDNOTFOUND;
-#ifndef FILE_LOCKS
 	return ((FILE *) NULL);
-#endif
     }
 #ifdef FILE_LOCKS
     else
@@ -652,10 +682,11 @@ dbReadOpen(cellDef, name, setFileName)
 	if (file_is_not_writeable (filename) || (flock_flag != 0))
 	{
 	    cellDef->cd_flags |= CDNOEDIT;
-	    if ((flock_flag == 1) && DBVerbose)
+	    if ((flock_flag == 0) && DBVerbose)
 		TxPrintf("Warning: cell <%s> from file %s is not writeable\n",
 			cellDef->cd_name, filename);
 	}
+	flock_flag = 0;
 	cellDef->cd_flags &= ~CDNOTFOUND;
     }
 #else
@@ -891,7 +922,231 @@ badTransform:
 nextLine:
     return (dbFgets(line, len, f) != NULL);
 }
+
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbReadProperties --
+ *
+ * Starting with the line << properties >>, read properties for 'cellDef'
+ * up to the end of the properties section.  On exit, 'line' contains
+ * the line that terminated the properties section, which will be either
+ * a line of the form "<< something >>" or one beginning with a
+ * character other than 'r', 'l', or 't'.
+ *
+ * Results:
+ *	Returns TRUE normally, or FALSE on error or EOF.
+ *
+ * Side effects:
+ *	See above.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+dbReadProperties(cellDef, line, len, f)
+    CellDef *cellDef;	/* Cell whose elements are being read */
+    char *line;		/* Line containing << elements >> */
+    int len;		/* Size of buffer pointed to by line */
+    FILE *f;		/* Input file */
+{
+    char propertyname[128], propertyvalue[128], *storedvalue;
+    int ntok;
+
+    /* Get first element line */
+    if (dbFgets(line, len, f) == NULL) return (FALSE);
+
+    while (TRUE)
+    {
+	/* Skip blank lines */
+	while (line[0] == '\0')
+	    if (dbFgets(line, len, f) == NULL)
+		return (TRUE);
+
+	/* Stop when at end of properties section (currently, only "string"
+	 * is defined)
+	 */
+	if (line[0] != 's') break;
+
+	/*
+	 * Properties may only be "string", for now.  This may be the only
+	 * property type ever needed.
+	 */
+	if (line[0] == 's')
+	{
+	    if ((ntok = sscanf(line, "string %127s %127[^\n]",
+		    propertyname, propertyvalue)) != 2)
+	    {
+		TxError("Skipping bad property line: %s", line);
+		goto nextproperty;
+	    }
+	    storedvalue = StrDup((char **)NULL, propertyvalue);
+	    (void) DBPropPut(cellDef, propertyname, storedvalue);
+	}
+
+nextproperty:
+	if (dbFgets(line, len, f) == NULL)
+	    break;
+    }
+
+    return (TRUE);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbReadElements --
+ *
+ * Starting with the line << elements >>, read elements for 'cellDef'
+ * up to the end of the elements section.  On exit, 'line' contains
+ * the line that terminated the elements section, which will be either
+ * a line of the form "<< something >>" or one beginning with a
+ * character other than 'r', 'l', or 't'.
+ *
+ * Results:
+ *	Returns TRUE normally, or FALSE on error or EOF.
+ *
+ * Side effects:
+ *	See above.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+dbReadElements(cellDef, line, len, f, scale)
+    CellDef *cellDef;	/* Cell whose elements are being read */
+    char *line;		/* Line containing << elements >> */
+    int len;		/* Size of buffer pointed to by line */
+    FILE *f;		/* Input file */
+    int scale;		/* Scale factor */
+{
+    char elementname[128], styles[1024], *text, flags[100];
+    int istyle, ntok;
+    Rect r;
+    char *tstr, *nstr;
+
+    /* Get first element line */
+    if (dbFgets(line, len, f) == NULL) return (FALSE);
+
+    while (TRUE)
+    {
+	/* Skip blank lines */
+	while (line[0] == '\0')
+	    if (dbFgets(line, len, f) == NULL)
+		return (TRUE);
+
+	/* Stop when at end of elements section (either paint or cell use) */
+	if (line[0] != 'r' && line[0] != 'l' && line[0] != 't') break;
+
+	/*
+	 * Elements may be either rectangles, lines, or text.
+	 */
+	if (line[0] == 'r')
+	{
+	    if ((ntok = sscanf(line, "rectangle %127s %1023s %d %d %d %d %99[^\n]",
+		    elementname, styles, &r.r_xbot, &r.r_ybot,
+				&r.r_xtop, &r.r_ytop, flags)) < 6)
+	    {
+		TxError("Skipping bad \"rectangle\" element line: %s", line);
+		goto nextelement;
+	    }
+	    if (scale > 1)
+	    {
+		r.r_xbot *= scale;
+		r.r_ybot *= scale;
+		r.r_xtop *= scale;
+		r.r_ytop *= scale;
+	    }
+	    (void) DBWElementAddRect(NULL, elementname, &r, cellDef, 0);
+	    ntok -= 6;
+	}
+	else if (line[0] == 'l')
+	{
+	    if ((ntok = sscanf(line, "line %127s %1023s %d %d %d %d %99[^\n]",
+		    elementname, styles, &r.r_xbot, &r.r_ybot,
+				&r.r_xtop, &r.r_ytop, flags)) < 6)
+	    {
+		TxError("Skipping bad \"line\" element line: %s", line);
+		goto nextelement;
+	    }
+	    if (scale > 1)
+	    {
+		r.r_xbot *= scale;
+		r.r_ybot *= scale;
+		r.r_xtop *= scale;
+		r.r_ytop *= scale;
+	    }
+	    (void) DBWElementAddLine(NULL, elementname, &r, cellDef, 0);
+	    ntok -= 6;
+	}
+	else
+	{
+	    char *textend;
+
+	    if (((ntok = sscanf(line, "text %127s %1023s %d %d",
+			elementname, styles, &r.r_xbot, &r.r_ybot)) < 4)
+			|| ((text = strchr(line, '"')) == NULL)
+			|| ((textend = strrchr(line, '"')) == text))
+	    {
+		TxError("Skipping bad \"text\" element line: %s", line);
+		goto nextelement;
+	    }
+	    text++;
+	    *textend = '\0';
+
+	    if (scale > 1)
+	    {
+		r.r_xbot *= scale;
+		r.r_ybot *= scale;
+	    }
+	    (void) DBWElementAddText(NULL, elementname, r.r_xbot, r.r_ybot, text,
+				cellDef, 0);
+	    *textend = '"';
+	    ntok += sscanf(textend + 1, "%99[^\n]", flags);
+	    ntok -= 4;
+	    DBWElementParseFlags(NULL, elementname, "persistent");
+	}
+
+	/* Set the style(s) for this element */
+	tstr = styles;
+	while ((nstr = strchr(tstr, ',')) != NULL)
+	{
+	    *nstr = '\0';
+	    istyle = GrGetStyleFromName(tstr);
+	    DBWElementStyle(NULL, elementname, istyle, TRUE);
+	    *nstr = ',';
+	    tstr = nstr + 1;
+	}
+	istyle = GrGetStyleFromName(tstr);
+	DBWElementStyle(NULL, elementname, istyle, TRUE);
+
+        /* Remove initial style 0, which was temporary */
+	DBWElementStyle(NULL, elementname, 0, FALSE);
+
+	/* Any remaining text should be a comma-separated list of flags */
+	if (ntok > 0)
+	{
+	    tstr = flags;
+	    while (isspace(*tstr)) tstr++;
+	    while ((nstr = strchr(tstr, ',')) != NULL)
+	    {
+		*nstr = '\0';
+		DBWElementParseFlags(NULL, elementname, tstr);
+		*nstr = ',';
+		tstr = nstr + 1;
+	    }
+	    DBWElementParseFlags(NULL, elementname, tstr);
+	}
+
+nextelement:
+	if (dbFgets(line, len, f) == NULL)
+	    break;
+    }
+
+    return (TRUE);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -901,7 +1156,7 @@ nextLine:
  * up until the end of a label section.  On exit, 'line' contains
  * the line that terminated the label section, which will be either
  * a line of the form "<< something >>" or one beginning with a
- * character other than 'r' or 'l'.
+ * character other than 'r', 'l', or 'p'.
  *
  * Results:
  *	Returns TRUE normally, or FALSE on error or EOF.
@@ -920,9 +1175,9 @@ dbReadLabels(cellDef, line, len, f, scale)
     FILE *f;		/* Input file */
     int scale;		/* Scale factor */
 {
-    char layername[50], text[1024];
+    char layername[50], text[1024], port_use[50], port_class[50];
     TileType type;
-    int orient;
+    int ntok, orient;
     Rect r;
 
     /* Get first label line */
@@ -936,7 +1191,7 @@ dbReadLabels(cellDef, line, len, f, scale)
 		return (TRUE);
 
 	/* Stop when at end of labels section (either paint or cell use) */
-	if (line[0] != 'r' && line[0] != 'l') break;
+	if (line[0] != 'r' && line[0] != 'l' && line[0] != 'p') break;
 
 	/*
 	 * Labels may be either point labels or rectangular ones.
@@ -953,7 +1208,96 @@ dbReadLabels(cellDef, line, len, f, scale)
 		goto nextlabel;
 	    }
 	}
-	else
+	else if (line[0] == 'p')
+	{
+	    char ppos[5], *pptr;
+	    int idx;
+	    Label *lab;
+
+	    if (((lab = cellDef->cd_lastLabel) == NULL) ||
+			(lab->lab_flags & PORT_DIR_MASK) ||
+			(((ntok = sscanf(line, "port %d %4s %49s %49s",
+				&idx, &ppos, &port_use, &port_class)) != 2) &&
+			(ntok != 4)))
+	    {
+		TxError("Skipping bad \"port\" line: %s", line);
+		goto nextlabel;
+	    }
+	    lab->lab_flags = idx;
+	    for (pptr = &ppos[0]; *pptr != '\0'; pptr++)
+	    {
+		switch(*pptr)
+		{
+		    case 'n':
+			lab->lab_flags |= PORT_DIR_NORTH;
+			break;
+		    case 's':
+			lab->lab_flags |= PORT_DIR_SOUTH;
+			break;
+		    case 'e':
+			lab->lab_flags |= PORT_DIR_EAST;
+			break;
+		    case 'w':
+			lab->lab_flags |= PORT_DIR_WEST;
+			break;
+		}
+	    }
+	    if (ntok == 4)
+	    {
+		switch(port_use[0])
+		{
+		    case 's':
+			lab->lab_flags |= PORT_USE_SIGNAL;
+			break;
+		    case 'a':
+			lab->lab_flags |= PORT_USE_ANALOG;
+			break;
+		    case 'p':
+			lab->lab_flags |= PORT_USE_POWER;
+			break;
+		    case 'g':
+			lab->lab_flags |= PORT_USE_GROUND;
+			break;
+		    case 'c':
+			lab->lab_flags |= PORT_USE_CLOCK;
+			break;
+		    case 'd':
+			lab->lab_flags |= PORT_USE_DEFAULT;
+			break;
+		    default:
+			TxError("Ignoring unknown \"port\" use: %s", port_use);
+			break;
+		}
+
+		switch(port_class[0])
+		{
+		    case 'i':
+			lab->lab_flags |= PORT_CLASS_INPUT;
+			break;
+		    case 'o':
+			lab->lab_flags |= PORT_CLASS_OUTPUT;
+			break;
+		    case 't':
+			lab->lab_flags |= PORT_CLASS_TRISTATE;
+			break;
+		    case 'b':
+			lab->lab_flags |= PORT_CLASS_BIDIRECTIONAL;
+			break;
+		    case 'f':
+			lab->lab_flags |= PORT_CLASS_FEEDTHROUGH;
+			break;
+		    case 'd':
+			lab->lab_flags |= PORT_CLASS_DEFAULT;
+			break;
+		    default:
+			TxError("Ignoring unknown \"port\" use: %s", port_use);
+			break;
+		}
+
+	    }
+	    goto nextlabel;
+	}
+	else	/* deprecated, retained for backward compatibility */
 	{
 	    if (sscanf(line, "label %1023s %d %d %d %99[^\n]",
 		    layername, &r.r_xbot, &r.r_ybot, &orient, text) != 5)
@@ -964,6 +1308,7 @@ dbReadLabels(cellDef, line, len, f, scale)
 	    r.r_xtop = r.r_xbot;
 	    r.r_ytop = r.r_ybot;
 	}
+
 	if (scale > 1)
 	{
 	    r.r_xbot *= scale;
@@ -979,7 +1324,7 @@ Warning: label \"%s\" attached to unknown type \"%s\"\n", text, layername);
 	    type = TT_SPACE;
 	}
 
-	(void) DBPutLabel(cellDef, &r, orient, text, type);
+	(void) DBPutLabel(cellDef, &r, orient, text, type, 0);
 
 nextlabel:
 	if (dbFgets(line, len, f) == NULL)
@@ -988,6 +1333,7 @@ nextlabel:
 
     return (TRUE);
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1205,13 +1551,14 @@ DBCellWriteFile(cellDef, f)
     CellDef *cellDef;	/* Pointer to definition of cell to be written out */
     FILE *f;		/* The FILE to write to */
 {
-    int dbWritePaintFunc(), dbWriteCellFunc();
+    int dbWritePaintFunc(), dbWriteCellFunc(), dbWritePropFunc();
     register Label *lab;
     struct writeArg arg;
     int pNum;
-    TileType type;
+    TileType type, stype;
     TileTypeBitMask typeMask;
     int reducer;
+    char *estring;
 
 #define FPRINTF(f,s)\
 {\
@@ -1248,16 +1595,18 @@ DBCellWriteFile(cellDef, f)
 	     sprintf(headerstring,"magic\ntech %s\nmagscale %d %d\ntimestamp %d\n",
 	 		DBTechName, DBLambda[0], DBLambda[1] / reducer,
 			cellDef->cd_timestamp);
-	 FPRINTF(f,headerstring);
+	 FPRINTF(f, headerstring);
     }
 
     /*
      * Output the paint of the cell.
      * Note that we only output up to the last layer appearing
-     * in the technology file (DBNumUserLayers-1), not any of the
-     * automatically generated contact layers (DBNumUserLayers
-     * through DBNumTypes-1).
+     * in the technology file (DBNumUserLayers-1).  Automatically
+     * generated stacked contact types are added to typeMask and
+     * will be decomposed into the residue appropriate for the
+     * plane being searched.
      */
+
     arg.wa_file = f;
     arg.wa_reducer = reducer;
     for (type = TT_PAINTBASE; type < DBNumUserLayers; type++)
@@ -1266,7 +1615,16 @@ DBCellWriteFile(cellDef, f)
 	    continue;
 	arg.wa_found = FALSE;
 	arg.wa_type = type;
+	arg.wa_plane = pNum;
 	TTMaskSetOnlyType(&typeMask, type);
+
+	/* Add to the mask all generated (stacking) types which	*/
+	/* have this type as a residue.				*/
+
+	for (stype = DBNumUserLayers; stype < DBNumTypes; stype++)
+	    if (TTMaskHasType(DBResidueMask(stype), type))
+		TTMaskSetType(&typeMask, stype);
+
 	if (DBSrPaintArea((Tile *) NULL, cellDef->cd_planes[pNum],
 		&TiPlaneRect, &typeMask, dbWritePaintFunc, (ClientData) &arg))
 	    goto ioerror;
@@ -1281,19 +1639,97 @@ DBCellWriteFile(cellDef, f)
     {
 	char lstring[256];
 
-	FPRINTF(f,"<< labels >>\n");
+	FPRINTF(f, "<< labels >>\n");
 	for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
 	{
-	       sprintf(lstring, "rlabel %s %d %d %d %d %d %s\n",
+	    sprintf(lstring, "rlabel %s %d %d %d %d %d %s\n",
 		    DBTypeLongName(lab->lab_type),
 		    lab->lab_rect.r_xbot / reducer,
 		    lab->lab_rect.r_ybot / reducer,
 		    lab->lab_rect.r_xtop / reducer,
 		    lab->lab_rect.r_ytop / reducer,
 		    lab->lab_pos, lab->lab_text);
-	        FPRINTF(f,lstring);
+	    FPRINTF(f, lstring);
+	    if (lab->lab_flags & PORT_DIR_MASK)
+	    {
+		char ppos[5];
+
+		ppos[0] = '\0';
+		if (lab->lab_flags & PORT_DIR_NORTH) strcat(ppos, "n");
+		if (lab->lab_flags & PORT_DIR_SOUTH) strcat(ppos, "s");
+		if (lab->lab_flags & PORT_DIR_EAST) strcat(ppos, "e");
+		if (lab->lab_flags & PORT_DIR_WEST) strcat(ppos, "w");
+		sprintf(lstring, "port %d %s", lab->lab_flags & PORT_NUM_MASK,
+			ppos);
+
+		if (lab->lab_flags & (PORT_USE_MASK | PORT_CLASS_MASK))
+		{
+		    switch (lab->lab_flags & PORT_USE_MASK)
+		    {
+			case PORT_USE_SIGNAL:
+			    strcat(lstring, " signal");
+			    break;
+			case PORT_USE_ANALOG:
+			    strcat(lstring, " analog");
+			    break;
+			case PORT_USE_POWER:
+			    strcat(lstring, " power");
+			    break;
+			case PORT_USE_GROUND:
+			    strcat(lstring, " ground");
+			    break;
+			case PORT_USE_CLOCK:
+			    strcat(lstring, " clock");
+			    break;
+			case PORT_USE_DEFAULT:
+			    strcat(lstring, " default");
+			    break;
+		    }
+
+		    switch (lab->lab_flags & PORT_CLASS_MASK)
+		    {
+			case PORT_CLASS_INPUT:
+			    strcat(lstring, " input");
+			    break;
+			case PORT_CLASS_OUTPUT:
+			    strcat(lstring, " output");
+			    break;
+			case PORT_CLASS_TRISTATE:
+			    strcat(lstring, " tristate");
+			    break;
+			case PORT_CLASS_BIDIRECTIONAL:
+			    strcat(lstring, " bidirectional");
+			    break;
+			case PORT_CLASS_FEEDTHROUGH:
+			    strcat(lstring, " feedthrough");
+			    break;
+			case PORT_CLASS_DEFAULT:
+			    strcat(lstring, " default");
+			    break;
+		    }
+		}
+		strcat(lstring, "\n");
+		FPRINTF(f, lstring); 
+	    }
 	}
     }
+
+    /* Now any persistent elements */
+    estring = DBWPrintElements(cellDef, DBW_ELEMENT_PERSISTENT);
+    if (estring != NULL)
+    {
+	FPRINTF(f, "<< elements >>\n");
+	FPRINTF(f, estring);
+	FREE(estring);
+    }
+
+    /* And any properties */
+    if (cellDef->cd_props != NULL)
+    {
+	FPRINTF(f, "<< properties >>\n");
+	DBPropEnum(cellDef, dbWritePropFunc, (ClientData)f);
+    }
+
     FPRINTF(f, "<< end >>\n");
 
     if (fflush(f) == EOF || ferror(f))
@@ -1307,6 +1743,45 @@ ioerror:
     SigEnableInterrupts();
     return (TRUE);
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbWritePropFunc --
+ *
+ * Filter function used to write out a single cell property.
+ *
+ * Results:
+ *	Normally returns 0; returns 1 on I/O error.
+ *
+ * Side effects:
+ *	Writes to the disk file.
+ *
+ * Warnings:
+ *	This function assumes that all property values are strings!
+ *	This is currently true;  if it changes in the future, this
+ *	function will have to check each property against a list of
+ *	expected property strings and output the value based on the
+ *	known format of what it points to.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+dbWritePropFunc(key, value, cdata)
+    char *key;
+    ClientData value;
+    ClientData cdata;
+{
+    FILE *f = (FILE *)cdata;
+    char lstring[256];
+
+    sprintf(lstring, "string %s %s\n", key, (char *)value);
+    FPRINTR(f, lstring);
+
+    return 0;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1361,6 +1836,7 @@ DBCellWrite(cellDef, fileName)
     char *cp1, *cp2;
     char expandbuf[NAME_SIZE];
     FILE *realf, *tmpf;
+    int tmpres;
     struct stat statb;
     bool result;
 
@@ -1398,33 +1874,40 @@ DBCellWrite(cellDef, fileName)
     cp2 = expandname;
     if (PaExpand(&cp1, &cp2, NAME_SIZE) == -1)
 	expandname = realname;
-#ifdef FILE_LOCKS
+
     /*
-     * If the locking logic works, this should not happen.  But it is
+     * If the locking logic works, this should not happen.  Files which
+     * are not editable should never be set to MODIFIED.  But it is
      * better to be extra safe.  If it does happen, it would be good to
-     * figure out why. */
+     * figure out why.
+     */
+
     if(cellDef->cd_flags & CDNOEDIT)
     {
-	TxPrintf(
-	 "File %s is locked by another user or is read_only and cannot be written\n",
-		realname);
+#ifdef FILE_LOCKS
+	TxPrintf("File %s is locked by another user or "
+		"is read_only and cannot be written\n", realname);
+#else
+	TxPrintf("File %s is read_only and cannot be written\n", realname);
+#endif
 	return(FALSE);
     }
-#endif
 
     /*
      * Figure name of the temp file to write.
      */
     MALLOC(char *, tmpname, strlen(expandname) + strlen(template) + 1);
     (void) sprintf(tmpname, "%s%s", expandname, template);
-    mktemp(tmpname);
+    tmpres = mkstemp(tmpname);
 
     /* Critical: disable interrupts while we do our work */
     SigDisableInterrupts();
 
-    /* Don't allow a write if the file isn't writeable! */
+    /* Don't allow a write if the file isn't writeable, or if mkstemp returned	*/
+    /* an error condition.							*/
 
-    if (access(expandname, F_OK) == 0 && file_is_not_writeable (expandname))
+    if ((tmpres == -1) || (access(expandname, F_OK) == 0 &&
+		file_is_not_writeable (expandname)))
     {
 	perror(expandname);
 	goto cleanup;
@@ -1586,19 +2069,37 @@ dbWritePaintFunc(tile, cdarg)
     TileType type = TiGetType(tile);
 
 #ifdef NONMANHATTAN
+    int dir;
+
     if (IsSplit(tile))      
     { 
-	if (SplitLeftType(tile) == arg->wa_type)
-	    type = SplitLeftType(tile);
-	else if (SplitRightType(tile) == arg->wa_type)
-	    type = SplitRightType(tile);
-	else 
-    return 0;
+	if ((SplitLeftType(tile) == arg->wa_type) ||
+		((SplitLeftType(tile) >= DBNumUserLayers) &&
+		TTMaskHasType(DBResidueMask(SplitLeftType(tile)), arg->wa_type)))
+	{
+	    type = arg->wa_type;
+	    dir = 0x0;
+	}
+	else if ((SplitRightType(tile) == arg->wa_type) ||
+		((SplitRightType(tile) >= DBNumUserLayers) &&
+		TTMaskHasType(DBResidueMask(SplitRightType(tile)), arg->wa_type)))
+	{
+	    type = arg->wa_type;
+	    dir = 0x2;
+	}
+	else
+	    return 0;
     }
     else  
 #endif   
     if (type != arg->wa_type)
-	return 0;
+    {
+	if ((type < DBNumUserLayers) ||
+	    	(!TTMaskHasType(DBResidueMask(type), arg->wa_type)))
+	    return 0;
+
+	type = arg->wa_type;
+    }
 
     if (!arg->wa_found)
     {
@@ -1611,8 +2112,7 @@ dbWritePaintFunc(tile, cdarg)
     if (IsSplit(tile))
     {
 	static char *pos_diag[] = {"nw", "sw", "se", "ne"};
-	int dir = SplitDirection(tile) |
-		((SplitLeftType(tile) == arg->wa_type) ? 0x0 : 0x2);
+	dir |= SplitDirection(tile);
 	sprintf(pstring, "tri %d %d %d %d %s\n",
 	    	LEFT(tile) / arg->wa_reducer, BOTTOM(tile) / arg->wa_reducer,
 		RIGHT(tile) / arg->wa_reducer, TOP(tile) / arg->wa_reducer,
@@ -1713,7 +2213,10 @@ DBGetTech(cellName)
     static char line[512];
     char *p;
 
-    f = PaOpen(cellName, "r", DBSuffix, Path, CellLibPath, (char **) NULL);
+    /* The use of "rb" below is a kludge to prevent this call from	*/
+    /* checking if the cell is locked. mdg 09/14/03			*/
+
+    f = PaOpen(cellName, "rb", DBSuffix, Path, CellLibPath, (char **) NULL);
     if (f == NULL) return NULL;
 
     p = (char *) NULL;
