@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBlabel.c,v 1.4 2001/07/23 17:49:51 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -36,6 +36,36 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBlabel.c,v 1.4 2
 
 static TileType DBPickLabelLayer(/* CellDef *def, Label *lab, int noreconnect */);
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBIsSubcircuit --
+ *
+ * Check if any labels in a CellDef declare port attributes, indicating
+ * that the CellDef should be treated as a subcircuit.
+ *
+ * Results:
+ *	True if CellDef contains ports, False otherwise.
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBIsSubcircuit(cellDef)
+    CellDef *cellDef;
+{
+    Label *lab;
+
+    for (lab = cellDef->cd_labels; lab != NULL; lab = lab->lab_next)
+	if (lab->lab_flags & PORT_DIR_MASK)
+	    return TRUE;
+ 
+    return FALSE;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -62,7 +92,7 @@ static TileType DBPickLabelLayer(/* CellDef *def, Label *lab, int noreconnect */
  */
 
 int
-DBPutLabel(cellDef, rect, align, text, type)
+DBPutLabel(cellDef, rect, align, text, type, flags)
     CellDef *cellDef;	/* Cell in which label is placed */
     Rect *rect;		/* Location of label; see above for description */
     int align;		/* Orientation/alignment of text.  If this is < 0,
@@ -71,6 +101,7 @@ DBPutLabel(cellDef, rect, align, text, type)
 			 */
     char *text;		/* Pointer to actual text of label */
     TileType type;	/* Type of tile to be labelled */
+    int flags;		/* Label flags */
 {
     register Label *lab;
     int len, x1, x2, y1, y2, tmp, labx, laby;
@@ -122,6 +153,7 @@ DBPutLabel(cellDef, rect, align, text, type)
 
     lab->lab_pos = align;
     lab->lab_type = type;
+    lab->lab_flags = flags;
     lab->lab_rect = *rect;
     lab->lab_next = NULL;
     if (cellDef->cd_labels == NULL)
@@ -133,7 +165,7 @@ DBPutLabel(cellDef, rect, align, text, type)
     }
     cellDef->cd_lastLabel = lab;
 
-    DBUndoPutLabel(cellDef, rect, align, text, type);
+    DBUndoPutLabel(cellDef, rect, align, text, type, flags);
     cellDef->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
     return align;
 }
@@ -205,7 +237,7 @@ DBEraseLabel(cellDef, area, mask)
 	if (cellDef->cd_lastLabel == lab)
 	    cellDef->cd_lastLabel = labPrev;
 	DBUndoEraseLabel(cellDef, &lab->lab_rect, lab->lab_pos,
-	    lab->lab_text, lab->lab_type);
+	    lab->lab_text, lab->lab_type, lab->lab_flags);
 	DBWLabelChanged(cellDef, lab->lab_text, &lab->lab_rect,
 	    lab->lab_pos, DBW_ALLWINDOWS);
 	FREE((char *) lab);
@@ -274,7 +306,7 @@ DBEraseLabelsByContent(def, rect, pos, type, text)
 	if ((pos >= 0) && (pos != lab->lab_pos)) continue;
 	if ((text != NULL) && (strcmp(text, lab->lab_text) != 0)) continue;
 	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
-	    lab->lab_text, lab->lab_type);
+	    lab->lab_text, lab->lab_type, lab->lab_flags);
 	DBWLabelChanged(def, lab->lab_text, &lab->lab_rect,
 	    lab->lab_pos, DBW_ALLWINDOWS);
 	if (labPrev == NULL)
@@ -337,7 +369,7 @@ DBEraseLabelsByFunction(def, func)
 	nextCheck:
 	if (!(*func)(lab)) continue;
 	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
-	    lab->lab_text, lab->lab_type);
+	    lab->lab_text, lab->lab_type, lab->lab_flags);
 	DBWLabelChanged(def, lab->lab_text, &lab->lab_rect,
 	    lab->lab_pos, DBW_ALLWINDOWS);
 	if (labPrev == NULL)
@@ -391,12 +423,12 @@ DBReOrientLabel(cellDef, area, newPos)
 	if (GEO_TOUCH(area, &lab->lab_rect))
 	{
 	    DBUndoEraseLabel(cellDef, &lab->lab_rect, lab->lab_pos,
-		lab->lab_text, lab->lab_type);
+		lab->lab_text, lab->lab_type, lab->lab_flags);
 	    DBWLabelChanged(cellDef, lab->lab_text, &lab->lab_rect,
 		lab->lab_pos, DBW_ALLWINDOWS);
 	    lab->lab_pos = newPos;
 	    DBUndoPutLabel(cellDef, &lab->lab_rect, lab->lab_pos,
-		lab->lab_text, lab->lab_type);
+		lab->lab_text, lab->lab_type, lab->lab_flags);
 	    DBWLabelChanged(cellDef, lab->lab_text, &lab->lab_rect,
 		lab->lab_pos, DBW_ALLWINDOWS);
 	}
@@ -449,10 +481,10 @@ DBAdjustLabels(def, area)
 		    DBTypeLongName(newType), def->cd_name);
 	};
 	DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
-		lab->lab_text, lab->lab_type);
+		lab->lab_text, lab->lab_type, lab->lab_flags);
 	lab->lab_type = newType;
 	DBUndoPutLabel(def, &lab->lab_rect, lab->lab_pos,
-		lab->lab_text, lab->lab_type);
+		lab->lab_text, lab->lab_type, lab->lab_flags);
 	modified = TRUE;
     }
 
@@ -503,7 +535,7 @@ DBAdjustLabelsNew(def, area, noreconnect)
 		    if (def->cd_lastLabel == lab)
 			    def->cd_lastLabel = labPrev;
 		    DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
-				     lab->lab_text, lab->lab_type);
+				     lab->lab_text, lab->lab_type, lab->lab_flags);
 		    DBWLabelChanged(def, lab->lab_text, &lab->lab_rect,
 				    lab->lab_pos, DBW_ALLWINDOWS);
 		    FREE((char *) lab);
@@ -517,10 +549,10 @@ DBAdjustLabelsNew(def, area, noreconnect)
 				     DBTypeLongName(newType), def->cd_name);
 		    }
 		    DBUndoEraseLabel(def, &lab->lab_rect, lab->lab_pos,
-				     lab->lab_text, lab->lab_type);
+				     lab->lab_text, lab->lab_type, lab->lab_flags);
 		    lab->lab_type = newType;
 		    DBUndoPutLabel(def, &lab->lab_rect, lab->lab_pos,
-				   lab->lab_text, lab->lab_type);
+				   lab->lab_text, lab->lab_type, lab->lab_flags);
 		    modified = TRUE;
 	    }
     nextLab: 
