@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtechname.c,v 1.3 2001/11/29 20:08:51 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -69,6 +69,7 @@ DBTechNameType(typename)
     char *slash;
     TileType type;
     int plane, i;
+    LayerInfo *lp;
 
     slash = index(typename, '/');
     if (slash != NULL) *slash = 0;
@@ -77,20 +78,16 @@ DBTechNameType(typename)
     *slash = '/';
     if (type < 0) return type;
 
-    /* There's a plane qualification.  Locate the image. */
+    /* There's a plane qualification.  Make sure the type exists */
+    /* on the indicated plane.  If not, return an error.	 */
 
-    plane = (int) dbTechNameLookup(slash+1, &dbPlaneNameLists);
+    plane = (int) dbTechNameLookup(slash + 1, &dbPlaneNameLists);
     if (plane < 0) return -2;
-    if (DBPlane(type) == plane) return type;
 
-    for (i = DBNumUserLayers; i < DBNumTypes; i++)
-    {
-	if (TTMaskHasType(&DBLayerTypeMaskTbl[type], i)
-		&& (DBTypePlaneTbl[i] == plane))
-	{
-	    return (TileType) i;
-	}
-    }
+    lp = &dbLayerInfo[type];
+    if (PlaneMaskHasPlane(lp->l_pmask, plane))
+	return type;
+
     return -2;
 }
 
@@ -114,8 +111,9 @@ DBTechNameTypes(typename,bitmask)
     TileTypeBitMask	*bitmask;
 {
     char *slash;
-    TileType type,returntype;
+    TileType type;
     int plane, i;
+    LayerInfo *lp;
 
     TTMaskZero(bitmask);
     slash = index(typename, '/');
@@ -131,25 +129,18 @@ DBTechNameTypes(typename,bitmask)
 
     /* There's a plane qualification.  Locate the image. */
 
-    plane = (int) dbTechNameLookup(slash+1, &dbPlaneNameLists);
+    plane = (int) dbTechNameLookup(slash + 1, &dbPlaneNameLists);
     if (plane < 0) return -2;
-    if (DBPlane(type) == plane)
+
+    lp = &dbLayerInfo[type];
+    if (PlaneMaskHasPlane(lp->l_pmask, plane))
     {
-    	 TTMaskSetType(bitmask,type);
+    	 TTMaskSetType(bitmask, type);
     	 return type;
     } 
-    returntype = -2;
-    for (i = DBNumUserLayers; i < DBNumTypes; i++)
-    {
-	if (TTMaskHasType(&DBLayerTypeMaskTbl[type], i)
-		&& (DBTypePlaneTbl[i] == plane))
-	{
-	    TTMaskSetType(bitmask,i);
-	    if (returntype == -2) returntype = i;
-	}
-    }
-    return returntype;
+    return -2;
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -522,7 +513,7 @@ DBTechPrintCanonicalType(layername)
  *	all layers on plane "foo" except "x" and "y".
  *
  * Results:
- *	None.
+ *	Returns a plane mask of all the planes specified.
  *
  * Side effects:
  *	Error messages are output if layers aren't understood.
@@ -531,7 +522,7 @@ DBTechPrintCanonicalType(layername)
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 DBTechNoisyNameMask(layers, mask)
     char *layers;			/* String to be parsed. */
     TileTypeBitMask *mask;		/* Where to store the layer mask. */
@@ -543,6 +534,7 @@ DBTechNoisyNameMask(layers, mask)
                                          */
     char save;
     bool allBut;
+    int planemask;
 
     TTMaskZero(mask);
     p = layers;
@@ -640,12 +632,26 @@ DBTechNoisyNameMask(layers, mask)
     	    plane = DBTechNoisyNamePlane(p+1);
 	    *p2 = save;
 	    p = p2;
-	    if (plane > 0) TTMaskAndMask(&m2, &DBPlaneTypes[plane]);
+	    if (plane > 0)
+	    {
+		TTMaskAndMask(&m2, &DBPlaneTypes[plane]);
+		planemask = PlaneNumToMaskBit(plane);
+	    }
+	}
+	else
+	{
+	    TileType t; 
+
+	    planemask = 0;
+	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+		if (TTMaskHasType(&m2, t))
+		    planemask |= DBTypePlaneMaskTbl[t];
 	}
 
 	TTMaskSetMask(mask, &m2);
 	while (*p == ',') p++;
     }
+    return planemask;
 }
 
 /*
@@ -691,7 +697,7 @@ DBTechMinSetPlanes(typeMask, newTypes)
 	if (!TTMaskHasType(&typeMask, i) || DBConnPlanes[i] != 0)
 	    continue;
 	TTMaskSetType(newTypes, i);
-	planes |= PlaneNumToMaskBit(DBTypePlaneTbl[i]);
+	planes |= PlaneNumToMaskBit(DBPlane(i));
     }
 
     /*
@@ -718,7 +724,7 @@ DBTechMinSetPlanes(typeMask, newTypes)
 	else
 	{
 	    TTMaskSetType(newTypes, i);
-	    planes |= PlaneNumToMaskBit(DBTypePlaneTbl[i]);
+	    planes |= PlaneNumToMaskBit(DBPlane(i));
 	}
     }
 
