@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtechtype.c,v 1.5 2001/01/12 22:12:21 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -40,7 +40,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtechtype.c,v 1.
     /* Types and their names */
 int DBNumTypes;
 char *DBTypeLongNameTbl[NT];
-int DBTypePlaneTbl[NT];
+int DBTypePlaneTbl[NT];		/* Normally accessed as macro "DBPlane(x)" */
 NameList dbTypeNameLists = {NULL, NULL, NULL, (ClientData)0, FALSE};
 
     /* Planes and their names */
@@ -73,7 +73,6 @@ DefaultPlane dbTechDefaultPlanes[] =
     PL_M_HINT,		"mhint",
     PL_F_HINT,		"fhint",
     PL_R_HINT,		"rhint",
-    PL_SUBCIRCUIT,	"subcircuit",
     0,			0,	0
 };
 
@@ -89,7 +88,6 @@ DefaultType dbTechDefaultTypes[] =
     TT_MAGNET,		PL_M_HINT,	"magnet,mag",		TRUE,
     TT_FENCE,		PL_F_HINT,	"fence,f",		TRUE,
     TT_ROTATE,		PL_R_HINT,	"rotate,r",		TRUE,
-    TT_SUBCIRCUIT,	PL_SUBCIRCUIT,	"subcircuit",		TRUE,
     0,			0,		NULL,			0
 };
 
@@ -207,7 +205,7 @@ DBTechInitType()
 	    niceabort();
 	}
 	DBTypeLongNameTbl[dtp->dt_type] = cp;
-	DBTypePlaneTbl[dtp->dt_type] = dtp->dt_plane;
+	DBPlane(dtp->dt_type) = dtp->dt_plane;
     }
 
     DBNumTypes = TT_TECHDEPBASE;
@@ -306,85 +304,58 @@ DBTechAddType(sectionName, argc, argv)
 	return FALSE;
 
     DBTypeLongNameTbl[DBNumTypes] = cp;
-    DBTypePlaneTbl[DBNumTypes] = pNum;
+    DBPlane(DBNumTypes) = pNum;
     DBNumTypes++;
 
     return TRUE;
 }
+
 
+
 /*
  * ----------------------------------------------------------------------------
+ *  dbTechNewStackedType --
  *
- * dbTechNewGeneratedType --
+ *	Generate a new tiletype for a stacked layer (generally speaking,
+ *	a stacked contact).  Return the new tile type for the stacked
+ *	contact type.
  *
- * Create the name for a type generated as the image of a contact on
- * another plane.  The generated name is of the form 'type/plane',
- * where 'type' is the short name of the type 'base', and 'plane'
- * is the short name of the plane 'p'.  Install the new name and
- * return the new TileType.
- *
- * Results:
- *	Returns the new tile type, or -1 if none could be allocated.
- *
- * Side effects:
- *	See above.
+ *  Results:
+ *	The new tile type
  *
+ *  Side effects:
+ *	Updates the name tables with the new type name.  Note that other
+ *	tables, such as the home plane table, are not affected by this
+ *	routine.
  * ----------------------------------------------------------------------------
  */
 
-int
-dbTechNewGeneratedType(base,base2,p)
-    TileType base,base2;/* Base type of the contact being named */
-    int p;		/* Plane on which the new image will reside */
+TileType
+dbTechNewStackedType(type1, type2)
+    TileType type1, type2;
 {
     char buf[1024], *cp;
 
-    if (DBNumTypes >= TT_MAXTYPES-TT_RESERVEDTYPES)
+    if (DBNumTypes >= TT_MAXTYPES - TT_RESERVEDTYPES)
     {
 	TechError("Too many types to generate a new contact.  Maximum=%d\n",
-			TT_MAXTYPES-TT_RESERVEDTYPES);
+			TT_MAXTYPES - TT_RESERVEDTYPES);
 	return (TileType) -1;
     }
 
-    if (base2)
-    {
-         char	*backslash1;
-	 char	*backslash2;
-	 if (backslash1 = index(DBTypeShortName(base),'/')) *backslash1 = 0;
-	 if (backslash2 = index(DBTypeShortName(base),'/')) *backslash2 = 0;
-	 if (strcmp(DBTypeShortName(base),DBTypeShortName(base2)) >0)
-	 {
-	      (void) sprintf(buf, "%s+%s/%s", 
-	 		DBTypeShortName(base),
-	 		DBTypeShortName(base2),DBPlaneShortName(p));
-	 }
-	 else
-	 {
-	      (void) sprintf(buf, "%s+%s/%s", 
-	 		DBTypeShortName(base2),
-	 		DBTypeShortName(base),DBPlaneShortName(p));
-	 }
-	 if (backslash1) *backslash1 = '/';
-	 if (backslash2) *backslash2 = '/';
-    }
-    else
-    {
-         (void) sprintf(buf, "%s/%s", 
-	 	DBTypeShortName(base),DBPlaneShortName(p));
-    }
+    sprintf(buf, "%s+%s", DBTypeShortName(type1), DBTypeShortName(type2));
     cp = dbTechNameAdd(buf, (ClientData) DBNumTypes, &dbTypeNameLists);
     if (cp == NULL)
     {
-	TechError("Couldn't generate new image type %s\n", buf);
+	TechError("Couldn't generate new stacking type %s\n", buf);
 	return (TileType) -1;
     }
 
     DBTypeLongNameTbl[DBNumTypes] = cp;
-    DBTypePlaneTbl[DBNumTypes] = p;
 
     return DBNumTypes++;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
