@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtiles.c,v 1.3 2001/03/29 19:50:06 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -890,136 +890,3 @@ dbCheckMaxVFunc(tile, dbc)
     return (0);
 }
 
-/*
- *-------------------------------------------------------------------------
- * Free linked segment list gathered by DBGetLayerBoundary
- *-------------------------------------------------------------------------
- */
-void
-DBFreeLayerBoundary(slist)
-    SegList *slist;
-{
-    SegList *sl;
-
-    sl = slist;
-    while (sl != NULL)
-    {
-	freeMagic((char *)sl);
-	sl = sl->next;
-    }
-}
-
-/*
- *-------------------------------------------------------------------------
- * Gets all the segments belonging to a particular layer
- *-------------------------------------------------------------------------
- */
-SegList *
-DBGetLayerBoundary(hintTile, plane, mask, area)
-    Tile  *hintTile;
-    register Plane *plane;
-    TileTypeBitMask *mask;
-    register Rect  *area;
-{
-    SegList *slist = NULL;
-
-    extern int dbFindBoundaryFunc();   /* Forward declaration */
-
-    DBSrPaintArea(hintTile, plane, area, mask, dbFindBoundaryFunc,
-		(ClientData) &slist);
-
-    return slist;	/* Caller will need to free memory */
-}
-
-/*
- *-------------------------------------------------------------------------
- * Function which adds the tile boundary segments to the linked list
- *-------------------------------------------------------------------------
- */
-
-int
-dbFindBoundaryFunc(tile, slist)
-   Tile *tile;
-   SegList **slist;
-{
-    SegList *sl;
-    register Tile *tpout;
-    int edgeTop, edgeBottom, edgeLeft, edgeRight;
-
-    /* Search all boundaries of each tile, adding a segment to the	*/
-    /* linked list for each boundary segment touching space tiles.	*/
-
-    /* The edge-traversing routine was lifted from the DRC stuff */
-
-    for (tpout = BL(tile); BOTTOM(tpout) < TOP(tile); tpout = RT(tpout))
-    {
-        if(TiGetRightType(tpout) == TiGetLeftType(tile))
-            continue;
-        edgeTop = MIN(TOP(tpout), TOP(tile));
-        edgeBottom = MAX(BOTTOM(tpout), BOTTOM(tile));
-        if (edgeTop <= edgeBottom)
-            continue;
-
-	sl = (SegList *) mallocMagic(sizeof(SegList));
-	sl->sp1.p_x = LEFT(tile);
-	sl->sp1.p_y = edgeBottom;
-	sl->sp2.p_x = LEFT(tile);
-	sl->sp2.p_y = edgeTop;
-	sl->next = (*slist);
-	(*slist) = sl;
-    }
-
-    for (tpout = TR(tile); TOP(tpout) > BOTTOM(tile); tpout = LB(tpout))
-    {
-        if(TiGetLeftType(tpout) == TiGetRightType(tile))
-            continue;
-        edgeTop = MIN(TOP(tpout), TOP(tile));
-        edgeBottom = MAX(BOTTOM(tpout), BOTTOM(tile));
-        if (edgeTop <= edgeBottom)
-            continue;
-
-	sl = (SegList *) mallocMagic(sizeof(SegList));
-	sl->sp1.p_x = RIGHT(tile);
-	sl->sp1.p_y = edgeBottom;
-	sl->sp2.p_x = RIGHT(tile);
-	sl->sp2.p_y = edgeTop;
-	sl->next = (*slist);
-	(*slist) = sl;
-    }
-
-    for (tpout = LB(tile); LEFT(tpout) < RIGHT(tile); tpout = TR(tpout))
-    {
-        if(TiGetTopType(tpout) == TiGetBottomType(tile))
-            continue;
-        edgeLeft = MAX(LEFT(tpout), LEFT(tile));
-        edgeRight = MIN(RIGHT(tpout), RIGHT(tile));
-        if (edgeLeft >= edgeRight)
-            continue;
-
-	sl = (SegList *) mallocMagic(sizeof(SegList));
-	sl->sp1.p_x = edgeLeft;
-	sl->sp1.p_y = BOTTOM(tile);
-	sl->sp2.p_x = edgeRight;
-	sl->sp2.p_y = BOTTOM(tile);
-	sl->next = (*slist);
-	(*slist) = sl;
-    }
-
-    for (tpout = RT(tile); RIGHT(tpout) > LEFT(tile); tpout = BL(tpout))
-    {
-        if(TiGetBottomType(tpout) == TiGetTopType(tile))
-            continue;
-        edgeLeft = MAX(LEFT(tpout), LEFT(tile));
-        edgeRight = MIN(RIGHT(tpout), RIGHT(tile));
-        if (edgeLeft >= edgeRight)
-            continue;
-
-	sl = (SegList *) mallocMagic(sizeof(SegList));
-	sl->sp1.p_x = edgeLeft;
-	sl->sp1.p_y = TOP(tile);
-	sl->sp2.p_x = edgeRight;
-	sl->sp2.p_y = TOP(tile);
-	sl->next = (*slist);
-	(*slist) = sl;
-    }
-}
