@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/database/DBtpaint.c,v 1.5 2001/01/12 22:12:22 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -194,7 +194,7 @@ DBTechInitCompose()
     {
 	if ((ps = DBPlane(s)) > 0)
 	{
-	    for (t = 0; t < DBNumUserLayers; t++)
+	    for (t = 0; t < DBNumTypes; t++)
 	    {
 		if (DBPlane(t) > 0)
 		{
@@ -534,8 +534,7 @@ dbTechCheckImages(t, r, s)
     if (pMask = (PlaneMask(r) & ~PlaneMask(s)))
     {
 	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
-	    if (PlaneMaskHasPlane(pMask, pNum)
-		    && dbImageOnPlane(t, pNum) != dbImageOnPlane(r, pNum))
+	    if (PlaneMaskHasPlane(pMask, pNum) && (t != r))
 	    {
 		TechError("\
 Result image on plane %s must be the same as image of %s on plane %s\n",
@@ -758,9 +757,11 @@ dbTechPrintPaint(where, doPaint, contactsOnly)
     bool contactsOnly;
 {
     TileType have, paint, erase, result;
+    int plane;
+    LayerInfo *lp;
 
     if (where)
-	TxPrintf("\f\n%s:\n\n", where);
+	TxPrintf("\n%s:\n\n", where);
 
     if (doPaint)
     {
@@ -768,16 +769,27 @@ dbTechPrintPaint(where, doPaint, contactsOnly)
 	for (have = TT_TECHDEPBASE; have < DBNumTypes; have++)
 	{
 	    if (contactsOnly && !IsContact(have)) continue;
-	    for (paint = TT_TECHDEPBASE; paint < DBNumUserLayers; paint++)
+	    for (paint = TT_TECHDEPBASE; paint < DBNumTypes; paint++)
 	    {
 		if (contactsOnly && !IsContact(paint)) continue;
-		result = DBStdPaintEntry(have, paint, DBPlane(have));
-		if (result != have)
+		for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
 		{
-		    TxPrintf("%s + %s -> %s\n",
-			    DBTypeShortName(have),
-			    DBTypeShortName(paint),
-			    DBTypeShortName(result));
+		    lp = &dbLayerInfo[have];
+		    if (!PlaneMaskHasPlane(lp->l_pmask, plane))
+			continue;
+
+		    result = DBStdPaintEntry(have, paint, plane);
+		    if (result != have)
+		    {
+		        TxPrintf("%s ",
+				DBTypeShortName(have));
+			if (IsContact(have))
+		            TxPrintf("(on %s) ",
+					DBPlaneLongName(plane));
+		        TxPrintf(" + %s -> %s\n",
+				DBTypeShortName(paint),
+				DBTypeShortName(result));
+		    }
 		}
 	    }
 	}
@@ -788,16 +800,27 @@ dbTechPrintPaint(where, doPaint, contactsOnly)
 	for (have = TT_TECHDEPBASE; have < DBNumTypes; have++)
 	{
 	    if (contactsOnly && !IsContact(have)) continue;
-	    for (erase = TT_TECHDEPBASE; erase < DBNumUserLayers; erase++)
+	    for (erase = TT_TECHDEPBASE; erase < DBNumTypes; erase++)
 	    {
 		if (contactsOnly && !IsContact(erase)) continue;
-		result = DBStdEraseEntry(have, erase, DBPlane(have));
-		if (result != have)
+		for (plane = PL_TECHDEPBASE; plane < DBNumPlanes; plane++)
 		{
-		    TxPrintf("%s - %s -> %s\n",
-			    DBTypeShortName(have),
-			    DBTypeShortName(erase),
-			    DBTypeShortName(result));
+		    lp = &dbLayerInfo[have];
+		    if (!PlaneMaskHasPlane(lp->l_pmask, plane))
+			continue;
+
+		    result = DBStdEraseEntry(have, erase, plane);
+		    if (result != have)
+		    {
+		        TxPrintf("%s ",
+				DBTypeShortName(have));
+			if (IsContact(have))
+		            TxPrintf("(on %s) ",
+					DBPlaneLongName(plane));
+		        TxPrintf(" - %s -> %s\n",
+				DBTypeShortName(erase),
+				DBTypeShortName(result));
+		    }
 		}
 	    }
 	}
