@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/dbwind/DBWdisplay.c,v 1.5 2001/03/29 19:50:07 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -331,7 +331,7 @@ DBWredisplay(w, rootArea, clipArea)
 	    disStyle = i + TECHBEGINSTYLES;
 	    disWasPale = FALSE;		/* Using normal solid style */
 	    dbwNeedStyle = TRUE;	/* Haven't set style yet */
-	    (void) DBTreeSrTiles(&scontext, &layers, bitMask,
+	    (void) DBTreeSrUniqueTiles(&scontext, &layers, bitMask,
 		    dbwPaintFunc, (ClientData) NULL);
 	    if (dbwIsLocked)
 	    {
@@ -492,7 +492,7 @@ DBWredisplay(w, rootArea, clipArea)
 #ifdef	notdef
     /* finally, display the plowing debugging information */
     cmdPlowDisplayEdges(w, rootArea, clipArea);
-#endif	notdef
+#endif	/* notdef */
 
     GrUnlock(w);
 }
@@ -724,8 +724,32 @@ dbwLabelFunc(scx, label, tpath)
     GeoTransRect(&scx->scx_trans, &label->lab_rect, &tmp);
     WindSurfaceToScreen(dbwWindow, &tmp, &labRect);
     if (!GEO_TOUCH(&labRect, &dbwWindow->w_screenArea)) return 0;
+    if (label->lab_flags & PORT_DIR_MASK)
+        GrSetStuff(STYLE_PORT);
+	
     DBWDrawLabel(label->lab_text, &labRect, screenPos, -1,
 	dbwLabelSize, dbwExpandAmounts);
+
+    if (label->lab_flags & PORT_DIR_MASK)
+    {
+	/* Temporarily set the style for port connection lines */
+        GrSetStuff(STYLE_PORT_CONNECT);
+	if (label->lab_flags & PORT_DIR_NORTH)
+	    GrClipLine(labRect.r_xbot, labRect.r_ytop,
+			labRect.r_xtop, labRect.r_ytop);
+	if (label->lab_flags & PORT_DIR_SOUTH)
+	    GrClipLine(labRect.r_xbot, labRect.r_ybot,
+			labRect.r_xtop, labRect.r_ybot);
+	if (label->lab_flags & PORT_DIR_EAST)
+	    GrClipLine(labRect.r_xtop, labRect.r_ybot,
+			labRect.r_xtop, labRect.r_ytop);
+	if (label->lab_flags & PORT_DIR_WEST)
+	    GrClipLine(labRect.r_xbot, labRect.r_ybot,
+			labRect.r_xbot, labRect.r_ytop);
+
+	/* Revert back to label style */
+        GrSetStuff(STYLE_LABEL);
+    }
     return 0;
 }
 
@@ -1373,6 +1397,47 @@ DBWTechInitStyles()
 	TTMaskZero(DBWStyleToTypesTbl + i);
 }
 
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBWTechParseStyle --
+ *
+ *	Turn a style entry (either a style number or name) into an index
+ *	into the styles array.
+ *
+ * Results:  Index into the DBW styles table, or -1 if no style found.
+ *
+ * Side Effects:  None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+DBWTechParseStyle(stylestr)
+    char *stylestr;
+{
+    int sidx, style;
+
+    if (StrIsInt(stylestr))
+    {
+	style = atoi(stylestr);
+	for (sidx = 0; sidx < DBWNumStyles; sidx++)
+	    if (GrStyleTable[sidx + TECHBEGINSTYLES].idx == style)
+		break;
+    }
+    else {
+	for (sidx = 0; sidx < DBWNumStyles; sidx++)
+	    if (!strcmp(GrStyleTable[sidx + TECHBEGINSTYLES].longname,
+			stylestr))
+		break;
+    }
+    if (sidx >= DBWNumStyles)
+	return -1;
+    else
+	return sidx;
+}
+
 /*
  * ----------------------------------------------------------------------------
  * DBWTechAddStyle --
@@ -1394,8 +1459,8 @@ DBWTechAddStyle(sectionName, argc, argv)
     int argc;
     char *argv[];
 {
-    TileType t;
-    int style, sidx;
+    TileType t, r;
+    int sidx;
     static char styleType[50];
 
     if (argc != 2)
@@ -1431,26 +1496,26 @@ DBWTechAddStyle(sectionName, argc, argv)
     }
     if ((t = DBTechNoisyNameType(argv[0])) < 0) return (FALSE);
 
-    if (StrIsInt(argv[1]))
-    {
-	style = atoi(argv[1]);
-	for (sidx = 0; sidx < DBWNumStyles; sidx++)
-	    if (GrStyleTable[sidx + TECHBEGINSTYLES].idx == style)
-		break;
-    }
-    else {
-	for (sidx = 0; sidx < DBWNumStyles; sidx++)
-	    if (!strcmp(GrStyleTable[sidx + TECHBEGINSTYLES].longname,
-			argv[1]))
-		break;
-    }
+    sidx = DBWTechParseStyle(argv[1]);
 
-    if (sidx >= DBWNumStyles)
+    if (sidx < 0)
     {
-	TechError("Invalid style %d for tile type %s\n", argv[1], argv[0]);
+	TechError("Invalid style \"%s\" for tile type %s\n", argv[1], argv[0]);
 	return TRUE;
     }
 
     TTMaskSetType(&DBWStyleToTypesTbl[sidx], t);
+
+    /* If type t is a contact, then any stacked contact type which has	*/
+    /* t as a residue, and for which the home plane of the stacked	*/
+    /* contact is also the home plane of t, should be added to the list	*/
+    /* of types to be painted with this style.				*/
+
+    if (DBIsContact(t) && (t < DBNumUserLayers))
+	for (r = DBNumUserLayers; r < DBNumTypes; r++)
+	    if (TTMaskHasType(DBResidueMask(r), t) && (DBPlane(r) ==
+			DBPlane(t)))
+		TTMaskSetType(&DBWStyleToTypesTbl[sidx], r);
+
     return TRUE;
 }
