@@ -19,9 +19,10 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/dbwind/DBWprocs.c,v 1.4 2001/01/12 22:12:24 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
 
 #include "misc/magic.h"
 #include "utils/geometry.h"
@@ -246,6 +247,7 @@ dbwReloadFunc(w, name)
     return (0);
 }
 
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -310,6 +312,14 @@ DBWloadWindow(window, name)
 	 * First try to find it in main memory, then try to
 	 * read it from disk.
 	 */
+
+	/* Strip off any ".mag" extension from the name */
+	char *dotptr = strrchr(name, '.');
+
+	if (dotptr != NULL)
+	    if (!strcmp(dotptr, ".mag"))
+		*dotptr = '\0';
+
 	newEditDef = DBCellLookDef(name);
 	if (newEditDef == (CellDef *) NULL)
 	    newEditDef = DBCellNewDef(name, (char *) NULL);
@@ -319,7 +329,8 @@ DBWloadWindow(window, name)
 	    TxPrintf("Creating new cell\n");
 	    DBCellSetAvail(newEditDef);
 #ifdef FILE_LOCKS
-	    flock_test_set(newEditDef->cd_file, dummy);
+	    if (flock_flag == 0) flock_test_set(newEditDef->cd_file, dummy);
+	    flock_flag = 0;
 #endif
 	}
 	else
@@ -350,17 +361,15 @@ DBWloadWindow(window, name)
 	    DBWUndoNewEdit(newEditUse, newEditDef,
 		    &GeoIdentityTransform, &GeoIdentityTransform);
 	}
-#ifdef FILE_LOCKS
 	if (newEditUse->cu_def->cd_flags & CDNOEDIT)
 	{
 	    newEdit = FALSE;
-	} else {
-#endif
-	EditCellUse = newEditUse;
-	EditRootDef = newEditDef;
-#ifdef FILE_LOCKS
 	}
-#endif
+	else
+	{
+	    EditCellUse = newEditUse;
+	    EditRootDef = newEditDef;
+	}
 	EditToRootTransform = GeoIdentityTransform;
 	RootToEditTransform = GeoIdentityTransform;
     }
@@ -502,17 +511,16 @@ DBWxloadWindow(window, name)
 	    DBWUndoNewEdit(newEditUse, newEditDef,
 		    &GeoIdentityTransform, &GeoIdentityTransform);
 	}
-#ifdef FILE_LOCKS
+
 	if (newEditUse->cu_def->cd_flags & CDNOEDIT)
 	{
 	    newEdit = FALSE;
-	} else {
-#endif
-	EditCellUse = newEditUse;
-	EditRootDef = newEditDef;
-#ifdef FILE_LOCKS
 	}
-#endif
+	else
+	{
+	    EditCellUse = newEditUse;
+	    EditRootDef = newEditDef;
+	}
 	EditToRootTransform = GeoIdentityTransform;
 	RootToEditTransform = GeoIdentityTransform;
     }
@@ -612,12 +620,7 @@ DBWcommands(w, cmd)
 
     if (cmd->tx_button == TX_NO_BUTTON)
     {
-	if (cmd->tx_argc > 0)
-	{
-	    cmdNum = Lookup(cmd->tx_argv[0], CmdLongCommands);
-	    ASSERT(cmdNum >= 0, "DBWcommands");
-	    (void) (*CmdFuncs[cmdNum])(w, cmd);
-	}
+	WindExecute(w, DBWclientID, cmd);
     }
     else
     {
@@ -714,6 +717,7 @@ int
 DBWinit()
 {
     MagWindow *initialWindow;
+    int i;
     static char *doc =
 "You are currently using the \"box\" tool.  The button actions are:\n\
    left    - move the box so its lower-left corner is at cursor position\n\
@@ -727,8 +731,15 @@ You can move or resize the box by different corners by pressing left\n\
     /* Initialize */
     DBWclientID = WindAddClient("layout", DBWcreate, DBWdelete, 
 	    DBWredisplay, DBWcommands, DBWupdate, DBWexit,
-	    ( Void (*)() ) NULL, CmdLongCommands, (GrGlyph *) NULL);
-    
+	    ( Void (*)() ) NULL,
+	    (GrGlyph *) NULL);
+
+    /* Add all of the commands for the DBWind interface.  These	*/
+    /* are defined in DBWCommands.c.  Commands added by other	*/
+    /* modules are registered with the client by each module.	*/
+
+    DBWInitCommands();
+
     DBWHLAddClient(DBWDrawBox);
     DBWAddButtonHandler("box", DBWBoxHandler, STYLE_CURS_NORMAL, doc);
     (void) DBWChangeButtonHandler("box");
