@@ -203,6 +203,13 @@ extern char *DBWFeedbackNth();
 
 /* Exported procedures and variables relating to elements: */
 
+/* flag fields for all elements (8 bits maximum) */
+#define DBW_ELEMENT_PERSISTENT	0x01
+#define DBW_ELEMENT_TEXT_SIZE   0x0e	/* 3 bits (5 text sizes) */
+#define DBW_ELEMENT_TEXT_POS    0xf0	/* 4 bits (9 positions) */
+#define DBW_ELEMENT_LINE_HALFX	0x02	/* Add 1/2 to X position */
+#define DBW_ELEMENT_LINE_HALFY	0x04	/* Add 1/2 to Y position */
+
 extern void DBWElementAddRect();
 extern void DBWElementAddLine();
 extern void DBWElementAddText();
@@ -210,6 +217,8 @@ extern void DBWElementDelete();
 extern void DBWElementInit();
 extern void DBWElementNames();
 extern void DBWElementInbox();
+extern void DBWElementParseFlags();
+extern char *DBWPrintElements();
 
 /* Random procedures used internally to this module.  None of these
  * should ever need to be called by the outside world.
