@@ -21,7 +21,7 @@
 
 #ifndef	lint
 static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCcontin.c,v 1.2 2001/01/12 22:12:29 jsolomon Exp $";
-#endif	not lint
+#endif	/* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -369,7 +369,7 @@ DRCBreak()
     if (DRCHasWork && (DRCStatus == DRC_IN_PROGRESS))
     {
 	UndoEnable();
-
+	
 	/* fprintf(stderr, "DRC breaking. . .\n"); fflush(stderr); */
 
 	/* As a convenience for debugging DRC stuff, we pretend the DRC
