@@ -20,7 +20,7 @@
 
 #ifndef	lint
 static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCmain.c,v 1.2 2001/01/12 22:12:29 jsolomon Exp $";
-#endif	not lint
+#endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -106,7 +106,7 @@ int DRCstatArrayTiles = 0;	/* Number of tiles processed as part of
 #ifdef	DRCRULESHISTO
 int DRCstatVRulesHisto[DRC_MAXRULESHISTO];
 int DRCstatHRulesHisto[DRC_MAXRULESHISTO];
-#endif	DRCRULESHISTO
+#endif	/* DRCRULESHISTO */
 
 static int drcTotalSquares = 0;
 static int drcTotalTiles = 0;
@@ -120,7 +120,7 @@ static int drcTotalArrayTiles = 0;
 #ifdef	DRCRULESHISTO
 static int drcTotalVRulesHisto[DRC_MAXRULESHISTO];
 static int drcTotalHRulesHisto[DRC_MAXRULESHISTO];
-#endif	DRCRULESHISTO
+#endif	/* DRCRULESHISTO */
 
 
 /*
@@ -222,7 +222,7 @@ DRCPrintStats()
 {
 #ifdef	DRCRULESHISTO
     int n;
-#endif	DRCRULESHISTO
+#endif	/* DRCRULESHISTO */
 
     TxPrintf("Design-rule checker statistics (recent/total):\n");
     drcTotalSquares += DRCstatSquares;
@@ -272,7 +272,7 @@ DRCPrintStats()
 		DRCstatVRulesHisto[n], drcTotalVRulesHisto[n]);
 	DRCstatHRulesHisto[n] = DRCstatVRulesHisto[n] = 0;
     }
-#endif	DRCRULESHISTO
+#endif	/* DRCRULESHISTO */
 }
 
 /*
