@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/drc/DRCtech.c,v 1.4 2001/01/12 22:12:29 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -165,6 +165,49 @@ drcWhyDup(why)
     return new->dwl_string;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * drcFindBucket --
+ *
+ *	Find the bucket preceding the point where we with to insert a new DRC
+ *	cookie.  Don't insert a cookie in the middle of a pair of coupled
+ *	(trigger + check) rules.
+ *
+ * Results:
+ *	Returns a pointer to the location where we want to insert a rule
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+DRCCookie *
+drcFindBucket(i, j, distance)
+    int i, j, distance;
+{
+    DRCCookie *dp;
+
+    /* find bucket preceding the new one we wish to insert */
+
+    for (dp = DRCRulesTbl[i][j];
+		dp->drcc_next != (DRCCookie *) NULL;
+		dp = dp->drcc_next)
+    {
+	if (dp->drcc_next->drcc_flags & DRC_TRIGGER)
+	{
+	    if (dp->drcc_next->drcc_next->drcc_dist >= distance)
+		break;
+	    else
+		dp = dp->drcc_next;
+	}
+	else if (dp->drcc_next->drcc_dist >= distance) break;
+    }
+
+    return dp;
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -238,14 +281,14 @@ DRCTechInit()
 		if ((i == TT_ERROR_S) || (j == TT_ERROR_S))
 		    DRCPaintTable[plane][i][j] = TT_ERROR_S;
 		else if ((i == TT_SPACE) || (j == TT_SPACE)
-			|| (DBPlane(j) != plane)
-			|| !DBPaintOnPlane(i, DBPlane(j)))
+			|| !DBTypeOnPlane(j, plane)
+			|| !DBPaintOnTypePlanes(i, j))
 		    DRCPaintTable[plane][i][j] = result;
 		else if ((!TTMaskHasType(&DBLayerTypeMaskTbl[i], result)
 			&& !TTMaskHasType(&DBLayerTypeMaskTbl[j], result))
 			|| ((result != DBPaintResultTbl[plane][j][i])
-			&& (DBPlane(i) == plane)
-			&& DBPaintOnPlane(j, DBPlane(i))))
+			&& DBTypeOnPlane(i, plane)
+			|| DBPaintOnTypePlanes(j, i)))
 		{
 		    DRCPaintTable[plane][i][j] = TT_ERROR_S;
 		    /* printf("Error: %s on %s, was %s\n", DBTypeShortName(i),
@@ -335,9 +378,9 @@ DRCTechAddRule(sectionName, argc, argv)
     "step_size",
 	"width",	 4,	4,	drcWidth,
     "layers width why",
-#ifdef DRC_EXTENSIONS
 	"widespacing",	 7,	7,	drcSpacing,
     "layers1 width layers2 separation adjacency why",
+#ifdef DRC_EXTENSIONS
         "area",		 5,	5,	drcArea,
     "layers area horizon why",
         "maxwidth",	 5,	5,	drcMaxwidth,
@@ -447,16 +490,11 @@ widthOK:
 	     * to the right of any edge between a type in '~set'
 	     * and a type in 'set'.
 	     */
-	    if (SamePlane(i, j)
+	    if (DBTypesOnSamePlane(i, j)
 		    && TTMaskHasType(&setC, i) && TTMaskHasType(&set, j))
 	    {
 		/* find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl [i][j];
-			 dp->drcc_next != (DRCCookie *) NULL &&
-			 dp->drcc_next->drcc_dist < distance;
-			     dp = dp->drcc_next)
-		    ; /* null body */
-
+		dp = drcFindBucket(i, j);
 		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		ASSIGN(dpnew, distance, dp->drcc_next, set, set, why,
 			    distance, DRC_FORWARD, plane);
@@ -527,16 +565,11 @@ widthOK:
 	     * to the right of any edge between a type in '~set'
 	     * and a type in 'set'.
 	     */
-	    if (SamePlane(i, j)
+	    if (DBTypesOnSamePlane(i, j)
 		    && TTMaskHasType(&setC, i) && TTMaskHasType(&set, j))
 	    {
 		/* find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl [i][j];
-			 dp->drcc_next != (DRCCookie *) NULL &&
-			 dp->drcc_next->drcc_dist < horizon;
-			     dp = dp->drcc_next)
-		    ; /* null body */
-
+		dp = drcFindBucket(i, j, horizon);
 		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		ASSIGN(dpnew, horizon, dp->drcc_next, set, set, why,
 			    distance, DRC_AREA|DRC_FORWARD, plane);
@@ -629,16 +662,11 @@ widthOK:
 	     * to the right of any edge between a type in '~set'
 	     * and a type in 'set'.
 	     */
-	    if (SamePlane(i, j)
+	    if (DBTypesOnSamePlane(i, j)
 		    && TTMaskHasType(&setC, i) && TTMaskHasType(&set, j))
 	    {
 		/* find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl [i][j];
-			 dp->drcc_next != (DRCCookie *) NULL &&
-			 dp->drcc_next->drcc_dist < distance;
-			     dp = dp->drcc_next)
-		    ; /* null body */
-
+		dp = drcFindBucket(i, j, distance);
 		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		ASSIGN(dpnew, distance, dp->drcc_next, set, set, why,
 			    distance, DRC_MAXWIDTH|bend, plane);
@@ -714,9 +742,9 @@ drcSpacing(argc, argv)
     int needReverse = FALSE;
     TileType i, j;
 
-#ifdef DRC_EXTENSIONS
     int wwidth;
     bool widerule;
+
     if (argc == 7)
     {
 	wwidth = atoi(argv[2]);
@@ -725,22 +753,19 @@ drcSpacing(argc, argv)
 	adjacency = argv[5];
 	why = drcWhyDup(argv[6]);
 	widerule = TRUE;
-	needReverse = TRUE;  /* Requires checks in both directions! */
+	needReverse = TRUE;	/* Requires checks in both directions! */
 	/* TxPrintf("Info:  DRCtech:  widespacing rule for %s width %d:"
 		" spacing must be %d\n", layers1, wwidth, distance); */
     }
     else
     {
-#endif
-    layers2 = argv[2];
-   distance = atoi(argv[3]);
-   adjacency = argv[4];
-   why = drcWhyDup(argv[5]);
-#ifdef DRC_EXTENSIONS
+	layers2 = argv[2];
+	distance = atoi(argv[3]);
+	adjacency = argv[4];
+	why = drcWhyDup(argv[5]);
 	wwidth = distance;
 	widerule = FALSE;
     }
-#endif
 
     DBTechNoisyNameMask(layers1, &set1);
     planes1 = DBTechMinSetPlanes(set1, &set1);
@@ -791,7 +816,7 @@ drcSpacing(argc, argv)
 	setR = tmp1;
 	setRreverse = tmp1;
 
-	/* If set1 = set2, set flag to check rules in both directions */
+	/* If set1 != set2, set flag to check rules in both directions */
 	if (!TTMaskEqual(&set1, &set2))
 	    needReverse = TRUE;
     }
@@ -829,7 +854,7 @@ drcSpacing(argc, argv)
     {
 	for (j = 0; j < DBNumTypes; j++)
 	{
-	    if (i == j || !SamePlane(i, j)) continue;
+	    if ((i == j) || !DBTypesOnSamePlane(i, j)) continue;
 
 	    /* LHS is an element of set1, RHS is an element of setR */
 	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&setR, j))
@@ -842,12 +867,8 @@ drcSpacing(argc, argv)
 		 */
 
 		/* Find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl [i][j];
-			 dp->drcc_next != (DRCCookie *) NULL &&
-			 dp->drcc_next->drcc_dist < distance;
-			     dp = dp->drcc_next)
-		    ; /* null body */
-		
+		dp = drcFindBucket(i, j, distance);
+
 		/* May have to insert several buckets on different planes */
 		for (curPlane = PL_TECHDEPBASE; curPlane < DBNumPlanes; curPlane++)
 		{
@@ -855,31 +876,48 @@ drcSpacing(argc, argv)
 		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set2);
 		    TTMaskAndMask3(&tmp2, &DBPlaneTypes[curPlane], &setR);
-#ifdef DRC_EXTENSIONS
-		    ASSIGN(dpnew, distance, dp->drcc_next,
-			tmp1, tmp2, why, (widerule) ? wwidth : distance,
-			DRC_FORWARD | ((widerule) ? DRC_WIDESPACING : 0), curPlane);
-#else
-		    ASSIGN(dpnew, distance, dp->drcc_next,
-			tmp1, tmp2, why, distance, DRC_FORWARD, curPlane);
-#endif
+		    if (widerule)
+		    {
+			DRCCookie *dptrig;
+
+			/* Create two contiguous rules, one for spacing */
+			/* and one for width.  These are created in	*/
+			/* reverse order due to the stack property of	*/
+			/* the linked list.  The spacing rule does not	*/
+			/* do corner extensions; otherwise, the wide	*/
+			/* spacing rule gets triggered at endcaps of	*/
+			/* long, narrow wires.				*/
+
+		        ASSIGN(dpnew, distance, dp->drcc_next, tmp1, tmp2,
+				why, distance, DRC_FORWARD, curPlane);
+
+			MALLOC(DRCCookie *, dptrig, sizeof (DRCCookie));
+		        ASSIGN(dptrig, wwidth, dpnew, set1, set1, why,
+				wwidth, DRC_REVERSE | DRC_TRIGGER, curPlane);
+			dp->drcc_next = dptrig;
+		    }
+		    else
+		    {
+			ASSIGN(dpnew, distance, dp->drcc_next,
+				tmp1, tmp2, why, distance, DRC_FORWARD,
+				curPlane);
+			dp->drcc_next = dpnew;
+		    }
 
 		    if (i == TT_SPACE)
 		    {
-			if (curPlane != DBPlane(j))
+			if (!DBTypeOnPlane(j, curPlane))
 			    dpnew->drcc_flags |= DRC_XPLANE;
 		    }
 		    else
 		    {
-			if (curPlane != DBPlane(i))
+			if (!DBTypeOnPlane(i, curPlane))
 			    dpnew->drcc_flags |= DRC_XPLANE;
 		    }
 		    
 		    if (needReverse)
 		        dpnew->drcc_flags |= DRC_BOTHCORNERS;
 
-		    dp->drcc_next = dpnew;
-
 		    if (needReverse)
 		    {
 			/* Add check in reverse direction, 
@@ -887,28 +925,32 @@ drcSpacing(argc, argv)
 			 * rules only used with touching_ok which must be 
 			 * single plane)
 			 * so am not setting DRC_XPLANE as above.
-			 * /
+			 */
 			 
-			 /* find bucket preceding new one we wish to insert */
-			 for (dp = DRCRulesTbl [j][i];
-			          dp->drcc_next != (DRCCookie *) NULL &&
-   			          dp->drcc_next->drcc_dist < distance;
-			          dp = dp->drcc_next)
-			     ; /* null body */
-			 
-			 MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
-#ifdef DRC_EXTENSIONS
-		         ASSIGN(dpnew, distance, dp->drcc_next,
-				tmp1, tmp2, why, (widerule) ? wwidth : distance,
-				DRC_REVERSE | DRC_BOTHCORNERS | ((widerule) ?
-				DRC_WIDESPACING : 0), curPlane);
-#else
-			 ASSIGN(dpnew,distance,dp->drcc_next,
+			/* find bucket preceding new one we wish to insert */
+			dp = drcFindBucket(j, i, distance);
+			MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+			if (widerule)
+			{
+			    DRCCookie *dptrig;
+
+			    /* Assign two coupled rules (see above) */
+
+			    ASSIGN(dpnew, distance, dp->drcc_next,
+				tmp1, tmp2, why, distance,
+				DRC_REVERSE | DRC_BOTHCORNERS, curPlane);
+			    MALLOC(DRCCookie *, dptrig, sizeof (DRCCookie));
+			    ASSIGN(dptrig, wwidth, dpnew, set1, set1, why,
+				wwidth, DRC_FORWARD | DRC_TRIGGER, curPlane);
+			    dp->drcc_next = dptrig;
+			}
+			else
+			{
+			    ASSIGN(dpnew,distance,dp->drcc_next,
 				tmp1, tmp2, why, distance, 
-				DRC_REVERSE|DRC_BOTHCORNERS, curPlane);
-#endif
-			 
-			 dp->drcc_next = dpnew;
+				DRC_REVERSE | DRC_BOTHCORNERS, curPlane);
+			    dp->drcc_next = dpnew;
+			}
 		     }
 		}
 	    }
@@ -919,40 +961,33 @@ drcSpacing(argc, argv)
 	     */
 	    if (TTMaskEqual(&set1, &set2)) continue;
 
+	    if (widerule) continue;	/* Can't determine width of set1    */
+					/* when looking at the edge of set2 */
+
 	    /* LHS is an element of set2, RHS is an element of setRreverse */
 	    if (TTMaskHasType(&set2, i) && TTMaskHasType(&setRreverse, j))
 	    {
 		/* Find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl [i][j];
-			 dp->drcc_next != (DRCCookie *) NULL &&
-			 dp->drcc_next->drcc_dist < distance;
-			     dp = dp->drcc_next)
-		    ; /* null body */
+		dp = drcFindBucket(i, j, distance);
 
 		for (curPlane = PL_TECHDEPBASE; curPlane < DBNumPlanes; curPlane++)
 		{
 		    if (!PlaneMaskHasPlane(planes1, curPlane)) continue;
 
 		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
-		    TTMaskClearMask3(&tmp1,&DBPlaneTypes[curPlane],&set1);
-		    TTMaskAndMask3(&tmp2,&DBPlaneTypes[curPlane],&setRreverse);
-#ifdef DRC_EXTENSIONS
-		    ASSIGN(dpnew, distance, dp->drcc_next,
-			tmp1, tmp2, why, (widerule) ? wwidth : distance,
-			DRC_FORWARD | ((widerule) ? DRC_WIDESPACING : 0), curPlane);
-#else
+		    TTMaskClearMask3(&tmp1, &DBPlaneTypes[curPlane], &set1);
+		    TTMaskAndMask3(&tmp2, &DBPlaneTypes[curPlane], &setRreverse);
 		    ASSIGN(dpnew, distance, dp->drcc_next,
 			tmp1, tmp2, why, distance, DRC_FORWARD, curPlane);
-#endif
 
 		    if (i == TT_SPACE)
 		    {
-			if (curPlane != DBPlane(j))
+			if (!DBTypeOnPlane(j, curPlane))
 			    dpnew->drcc_flags |= DRC_XPLANE;
 		    }
 		    else
 		    {
-			if (curPlane != DBPlane(i))
+			if (!DBTypeOnPlane(i, curPlane))
 			    dpnew->drcc_flags |= DRC_XPLANE;
 		    }
 
@@ -970,27 +1005,15 @@ drcSpacing(argc, argv)
 			 * so am not setting DRC_XPLANE as above.
 			 * /
 			 
-			 /* find bucket preceding new one we wish to insert */
-			 for (dp = DRCRulesTbl [j][i];
-			          dp->drcc_next != (DRCCookie *) NULL &&
-   			          dp->drcc_next->drcc_dist < distance;
-			          dp = dp->drcc_next)
-			     ; /* null body */
-			 
-			 MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
-#ifdef DRC_EXTENSIONS
-		         ASSIGN(dpnew, distance, dp->drcc_next,
-				tmp1, tmp2, why, (widerule) ? wwidth : distance,
-				DRC_REVERSE | DRC_BOTHCORNERS | ((widerule) ?
-				DRC_WIDESPACING : 0), curPlane);
-#else
-			 ASSIGN(dpnew,distance,dp->drcc_next,
+			/* find bucket preceding new one we wish to insert */
+			dp = drcFindBucket(j, i, distance);
+			MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
+			ASSIGN(dpnew, distance, dp->drcc_next,
 				tmp1, tmp2, why, distance, 
-				DRC_REVERSE|DRC_BOTHCORNERS, curPlane);
-#endif
+				DRC_REVERSE | DRC_BOTHCORNERS, curPlane);
 			 
-			 dp->drcc_next = dpnew;
-		     }
+			dp->drcc_next = dpnew;
+		    }
 		}
 	    }
 
@@ -1034,12 +1057,12 @@ drcSpacing(argc, argv)
 		    /* filter out checks that are not cross plane */
 		    if (i == TT_SPACE) 
 		    {
-			if (curPlane == DBPlane(j))
+			if (DBTypeOnPlane(j, curPlane))
 			    continue;
 		    }
 		    else
 		    {
-			if (curPlane == DBPlane(i))
+			if (DBTypeOnPlane(i, curPlane))
 			    continue;
 		    }
 
@@ -1056,7 +1079,7 @@ drcSpacing(argc, argv)
 	}
     }
 
-    return (distance);
+    return (MAX(wwidth, distance));
 }
 
 /*
@@ -1160,12 +1183,7 @@ edgeOK:
 	    if (TTMaskHasType(&set1, i) && TTMaskHasType(&set2, j))
 	    {
 		/* Find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl [i][j];
-		    dp->drcc_next != (DRCCookie *) NULL &&
-		    dp->drcc_next->drcc_dist < distance;
-			dp = dp->drcc_next)
-		    ; /* null body */
-
+		dp = drcFindBucket(i, j, distance);
 		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		ASSIGN(dpnew, distance, dp->drcc_next,
 		    setM, setC, why, cdist, DRC_FORWARD, checkPlane);
@@ -1177,12 +1195,7 @@ edgeOK:
 		if (fourway)
 		{
 		    /* find bucket preceding new one we wish to insert */
-		    for (dp = DRCRulesTbl [j][i];
-			dp->drcc_next != (DRCCookie *) NULL &&
-			dp->drcc_next->drcc_dist < distance;
-			    dp = dp->drcc_next)
-			; /* null body */
-
+		    dp = drcFindBucket(j, i, distance);
 		    MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		    ASSIGN(dpnew,distance,dp->drcc_next,
 			    setM, setC, why, cdist, DRC_REVERSE, checkPlane);
@@ -1384,12 +1397,7 @@ drcRectangle(argc, argv)
 		int distance = 1;
 
 		/* Find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl[i][j];
-		    dp->drcc_next != (DRCCookie *) NULL &&
-		    dp->drcc_next->drcc_dist < distance;
-			dp = dp->drcc_next)
-		    ; /* null body */
-
+		dp = drcFindBucket(i, j, distance);
 		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		ASSIGN(dpnew, distance, dp->drcc_next, 
 		    nottypes, DBAllTypeBits, why, distance, 
@@ -1397,12 +1405,7 @@ drcRectangle(argc, argv)
 		dp->drcc_next = dpnew;
 
 		/* Find bucket preceding the new one we wish to insert */
-		for (dp = DRCRulesTbl[j][i];  /* note: j, i not i, j */
-		    dp->drcc_next != (DRCCookie *) NULL &&
-		    dp->drcc_next->drcc_dist < distance;
-			dp = dp->drcc_next)
-		    ; /* null body */
-
+		dp = drcFindBucket(j, i, distance); /* note: j, i not i, j */
 		MALLOC(DRCCookie *, dpnew, sizeof (DRCCookie));
 		ASSIGN(dpnew, distance, dp->drcc_next, 
 		    nottypes, DBAllTypeBits, why, distance, 
@@ -1548,7 +1551,7 @@ void
 DRCTechFinal()
 {
     TileTypeBitMask tmpMask, nextMask;
-    DRCCookie  *dummy, *dp, *next;
+    DRCCookie  *dummy, *dp, *next, *dptrig;
     DRCCookie **dpp, **dp2back;
     TileType i, j;
 
@@ -1633,16 +1636,35 @@ DRCTechFinal()
 	{
 	    for (dp = DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
 	    {
+		/* Don't optimize on trigger rules; optimize on the	*/
+		/* rule that gets triggered.				*/
+		if (dp->drcc_flags & DRC_TRIGGER)
+		{
+		    dptrig = dp;
+		    dp = dp->drcc_next;
+		}
+		else
+		    dptrig = NULL;
+		
 		/*
 		 * Check following buckets to see if any is a superset.
 		 */
 #ifdef DRC_EXTENSIONS
 		if (dp->drcc_flags & DRC_WEIRDONES) continue;
 #endif
-		
 		for (next = dp->drcc_next; next != NULL;
 			next = next->drcc_next)
 		{
+		    if (next->drcc_flags & DRC_TRIGGER)
+		    {
+			/* A triggered rule cannot be considered */
+			/* a superset of a non-triggered rule or */
+			/* a rule with a different trigger, so   */
+			/* we skip all triggered rules and their */
+			/* triggering rule.			 */
+			next = next->drcc_next;
+			continue;
+		    }
 		    tmpMask = nextMask = next->drcc_mask;
 		    TTMaskAndMask(&tmpMask, &dp->drcc_mask);
 		    if (!TTMaskEqual(&tmpMask, &nextMask)) continue;
@@ -1669,6 +1691,10 @@ DRCTechFinal()
 
 		/* "dp" is a subset of "next".  Eliminate it. */
 
+		/* For triggered rules, eliminate both the rule */
+		/* and the trigger.				*/
+		if (dptrig != NULL) dp = dptrig;
+
 		/* TxPrintf("For edge %s-%s, \"%s\" covers \"%s\"\n",
 		    DBTypeShortName(i), DBTypeShortName(j),
 		    next->drcc_why, dp->drcc_why);
@@ -1676,13 +1702,48 @@ DRCTechFinal()
 		dp2back = &(DRCRulesTbl [i][j]);
 		while (*dp2back != dp)
 		    dp2back = &(*dp2back)->drcc_next;
-		*dp2back = dp->drcc_next;
+
+		/* Trigger rules */
+		if (dptrig != NULL)
+		{
+		    FREE((char *)dp->drcc_next);
+		    *dp2back = dp->drcc_next->drcc_next;
+		}
+		else
+		    *dp2back = dp->drcc_next;
+
 		/* Don't free the shared drcc_why string here! */
 		FREE ((char *) dp);
 		drcRulesOptimized += 1;
 	    }
 	}
     }
+
+    /*
+     * Add stacked types to all the rules which affect either of the
+     * stacked contact type's residue contacts.
+     */
+
+    for (i = 0; i < DBNumUserLayers; i++)
+    {
+	int k, l;
+	TileTypeBitMask *rMask;
+
+	for (j = 0; j < DBNumUserLayers; j++)
+	    for (dp = DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
+		for (k = DBNumUserLayers; k < DBNumTypes; k++)
+		{
+		    rMask = DBResidueMask(k);
+		    for (l = 0; l < DBNumUserLayers; l++)
+			if (TTMaskHasType(rMask, l))
+			{
+			    if (TTMaskHasType(&dp->drcc_mask, l))
+				TTMaskSetType(&dp->drcc_mask, k);
+			    if (TTMaskHasType(&dp->drcc_corner, l))
+				TTMaskSetType(&dp->drcc_corner, k);
+			}
+		}
+    }
 }
 
 
@@ -1726,8 +1787,7 @@ DRCTechRuleStats()
 	    for (dp = DRCRulesTbl[i][j]; dp != NULL; dp = dp->drcc_next)
 		thisCount++;
 	    edgeRules += thisCount;
-	    if ((i != TT_SPACE) && (j != TT_SPACE) &&
-		(DBPlane(i) != DBPlane(j))) continue;
+	    if (!DBTypesOnSamePlane(i, j)) continue;
 	    if (thisCount <= MAXBIN) counts[thisCount] += 1;
 	    else overflow += 1;
 	}
