@@ -14,45 +14,53 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/ext2sim/ext2sim.c,v 1.5 2001/07/06 17:58:30 rajit Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>	/* for atof() */
 #include <ctype.h>
 #include <stdlib.h>
-#include <varargs.h>
+#include <math.h>	/* for sqrt() in bipolar L,W calculation */
 
 #ifdef MAGIC_WRAPPER
 #include "magic/tclmagic.h"
-#ifndef free_all_mem
-#define free_all_mem
-#endif
 #endif
 
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/utils.h"
-#ifdef MAGIC_WRAPPER
 #include "tiles/tile.h"
+#ifdef MAGIC_WRAPPER
 #include "database/database.h"
 #include "windows/windows.h"
 #include "textio/textio.h"
+#include "dbwind/dbwind.h"	/* for DBWclientID */
+#include "commands/commands.h"  /* for module auto-load */
+#include "textio/txcommands.h"
 #endif
 #include "extflat/extflat.h"
+#include "extract/extract.h"	/* for extDevTable */
 #include "utils/runstats.h"
 #include "utils/malloc.h"
-#include "math.h"	/* for sqrt() in bipolar L,W calculation */
 
 /* Forward declarations */
+#ifdef MAGIC_WRAPPER
+Void CmdExtToSim();
+#endif
 int simmainArgs();
-int simfetVisit(), simresistVisit(), simcapVisit(), simnodeVisit();
+int simdevVisit(), simresistVisit(), simcapVisit(), simnodeVisit();
+int simmergeVisit();
 
 /* Options specific to ext2sim */
-bool esFetNodesOnly = FALSE;
-bool esNoAlias = FALSE;
+bool esDoExtResis = FALSE;
+bool esDevNodesOnly = FALSE;
+bool esNoAlias = TRUE;
+bool esNoLabel = TRUE;
 bool esNoAttrs = FALSE;
-bool esNoLabel = FALSE;
 bool esHierAP = FALSE;
+bool esMergeDevsA = FALSE;	/* merge devices of equal length */
+bool esMergeDevsC = FALSE;	/* merge devices of equal length & width */
 char simesDefaultOut[FNSIZE];
 char *simesOutName = simesDefaultOut;
 char esDefaultAlias[FNSIZE], esDefaultLabel[FNSIZE];
@@ -69,21 +77,21 @@ FILE *esLabF = NULL;
 #define	LBL 1
 #define	SU  2
 
-static unsigned short esFormat = SU ;
+static unsigned short esFormat = MIT ;
 
 struct {
-   short resClassSD ;  /* the resistance class of the src/drn of the fet */
-   short resClassSub ; /* the resistance class of the substrate of the fet */
+   short resClassSD ;  /* the resistance class of the src/drn of the dev */
+   short resClassSub ; /* the resistance class of the substrate of the dev */
    char  *defSubs ;    /* the default substrate node */
-} fetInfo[MAXFETTYPES];
+} fetInfo[MAXDEVTYPES];
 
 typedef struct {
-	long    visitMask:MAXFETTYPES; 
+	long    visitMask:MAXDEVTYPES; 
 } nodeClient;
 
 typedef struct {
 	HierName *lastPrefix;
-	long    visitMask:MAXFETTYPES; 
+	long    visitMask:MAXDEVTYPES; 
 } nodeClientHier;
 
 #define NO_RESCLASS	-1
@@ -111,6 +119,46 @@ typedef struct {
 }
 
 
+/* device merging */
+
+#define devIsKilled(n) (esFMult[(n)] <= (float)0.0)
+#define DEV_KILLED ((float) -1.0)
+#define FMULT_SIZE (1<<10)
+static float *esFMult = NULL;
+static int    esFMIndex = 0;
+static int    esFMSize = FMULT_SIZE;
+int  esDevsMerged;
+
+/* macro to add a dev's multiplier to the table and grow it if necessary */
+#define addDevMult(f) \
+{  \
+        if ( esFMult == NULL ) { \
+          MALLOC(float *, esFMult, esFMSize*sizeof(float));  \
+        } else if ( esFMIndex >= esFMSize ) {  \
+          int i;  \
+          float *op = esFMult ;  \
+          MALLOC(float *, esFMult, (esFMSize = esFMSize*2)*sizeof(float)); \
+          for ( i = 0 ; i < esFMSize/2 ; i++ ) esFMult[i] = op[i]; \
+          if (op) FREE(op); \
+        }  \
+        esFMult[esFMIndex++] = (float)(f); \
+}
+
+#define setDevMult(i,f) { esFMult[(i)] = (float)(f); }
+
+#define getCurDevMult() ((esFMult) ? esFMult[esFMIndex-1] : (float)1.0)
+
+/* cache list used to find parallel devs */
+typedef struct _devMerge {
+        int     l, w;
+        EFNode *g, *s, *d, *b;
+        Dev * dev;
+        int       esFMIndex;
+        HierName *hierName;
+        struct _devMerge *next;
+} devMerge;
+
+devMerge *devMergeList = NULL ;
 
 /* attributes controlling the Area/Perimeter extraction of fet terminals */
 #define ATTR_FLATAP	"*[Ee][Xx][Tt]:[Aa][Pp][Ff]*"
@@ -119,6 +167,8 @@ typedef struct {
 
 #ifdef MAGIC_WRAPPER
 
+#define        atoCap(s)       ((EFCapValue)atof(s))
+
 /* Forward declaration */
 int _ext2sim_start();
 
@@ -139,36 +189,452 @@ Exttosim_Init(interp)
     if (Tcl_PkgRequire(interp, "Tclmagic", "7.2", 0) == NULL) return TCL_ERROR;
     if (Tcl_InitStubs(interp, "8.1", 0) == NULL) return TCL_ERROR;
 
-    Tcl_CreateCommand(interp, "exttosim::exttosim", _ext2sim_start,
-                (ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
+    TxPrintf("Auto-loading EXTTOSIM module\n");
+    TxFlushOut();
 
-    Tcl_Eval(interp, "namespace eval exttosim namespace export *");
+    /* Replace the auto-load function with the one defined in	*/
+    /* this package in the command functions list.		*/
+
+    if (WindReplaceCommand(DBWclientID, "exttosim", CmdExtToSim) < 0)
+	return TCL_ERROR;
 
     Tcl_PkgProvide(interp, "Exttosim", "7.2");
     return TCL_OK;
 }
 
+
 /*
  * ----------------------------------------------------------------------------
  *
- * Main Tcl callback for command "exttosim::exttosim"
+ * Main Tcl callback for command "magic::exttosim"
  *
  * ----------------------------------------------------------------------------
  */
 
-int
-_ext2sim_start(ClientData clientData,
-    Tcl_Interp *interp, int argc, char *argv[])
-{
+#define EXTTOSIM_RUN		0
+#define EXTTOSIM_ALIAS		1
+#define EXTTOSIM_LABELS		2
+#define EXTTOSIM_DEFAULT	3
+#define EXTTOSIM_FORMAT		4
+#define EXTTOSIM_RTHRESH	5
+#define EXTTOSIM_CTHRESH	6
+#define EXTTOSIM_MERGE		7
+#define EXTTOSIM_EXTRESIST	8
+#define EXTTOSIM_HELP		9
 
-#else
+Void
+CmdExtToSim(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    int i,flatFlags;
+    char *inName;
+    FILE *f;
+
+    int value;
+    int option = EXTTOSIM_RUN;
+    int argc = cmd->tx_argc;
+    char **argv = cmd->tx_argv; 
+    char **msg;
+    bool err_result;
+    char *resstr;
+
+    short sd_rclass;
+    short sub_rclass;
+    char *devname;
+    char *subname;
+    int idx;
+
+    static EFCapValue LocCapThreshold = 2;
+    static int LocResistThreshold = 10;
+
+    static char *cmdExtToSimOption[] = {
+	"[run] [options]	run exttosim on current cell\n"
+	"			use \"run -help\" to get standard options",
+	"alias on|off		enable/disable alias (.al) file",
+	"labels on|off		enable/disable labels (.nodes) file",
+	"default		reset to default values",
+	"format MIT|SU|LBL	set output format",
+	"rthresh [value]	set resistance threshold value",
+	"cthresh [value]	set capacitance threshold value",
+	"merge [option]		merge parallel transistors",
+	"extresist on|off	incorporate extresist output",
+        "help			print help information",
+	NULL
+    };
+
+    static char *sim_formats[] = {
+	"MIT",
+	"LBL",
+	"SU",
+	NULL
+    };
+
+    static char *yesno[] = {
+	"yes",
+	"true",
+	"on",
+	"no",
+	"false",
+	"off",
+	NULL
+    };
+
+    static char *cmdMergeTypes[] = {
+	"none			don't merge parallel devices",
+	"conservative		merge devices with same L, W",
+	"aggressive		merge devices with same L"
+    };
+
+    if (cmd->tx_argc > 1)
+    {
+	option = Lookup(cmd->tx_argv[1], cmdExtToSimOption);
+	if (option < 0) option = EXTTOSIM_RUN;
+	else argv++;
+    }
+
+    switch (option)
+    {
+	case EXTTOSIM_EXTRESIST:
+	    if (cmd->tx_argc == 2)
+	    {
+		Tcl_SetResult(magicinterp, (esDoExtResis) ? "on" : "off", NULL);
+		return TCL_OK;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto usage;
+	    idx = Lookup(cmd->tx_argv[2], yesno);
+	    if (idx < 0) goto usage;
+	    else if (idx < 3) esDoExtResis = TRUE;
+	    else esDoExtResis = FALSE;
+	    break;
+	case EXTTOSIM_ALIAS:
+	    if (cmd->tx_argc == 2)
+	    {
+		Tcl_SetResult(magicinterp, (esNoAlias) ? "off" : "on", NULL);
+		return TCL_OK;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto usage;
+	    idx = Lookup(cmd->tx_argv[2], yesno);
+	    if (idx < 0) goto usage;
+	    else if (idx < 3) esNoAlias = FALSE;
+	    else esNoAlias = TRUE;
+	    break;
+	case EXTTOSIM_LABELS:
+	    if (cmd->tx_argc == 2)
+	    {
+		Tcl_SetResult(magicinterp, (esNoLabel) ? "off" : "on", NULL);
+		return TCL_OK;
+	    }
+	    else if (cmd->tx_argc != 3)
+		goto usage;
+	    idx = Lookup(cmd->tx_argv[2], yesno);
+	    if (idx < 0) goto usage;
+	    else if (idx < 3) esNoLabel = FALSE;
+	    else esNoLabel = TRUE;
+	    break;
+	case EXTTOSIM_CTHRESH:
+	    if (cmd->tx_argc == 2)
+	    {
+		if (LocCapThreshold == INFINITE_THRESHOLD)
+		    Tcl_SetResult(magicinterp, "infinite", NULL);
+		else
+		    Tcl_SetObjResult(magicinterp,
+			Tcl_NewDoubleObj((double)LocCapThreshold));
+		return TCL_OK;
+	    }
+	    else if (cmd->tx_argc < 3) goto usage;
+	    if (!strncmp(cmd->tx_argv[2], "inf", 3))
+		LocCapThreshold = INFINITE_THRESHOLD;
+	    else if (StrIsNumeric(cmd->tx_argv[2]))
+		LocCapThreshold = atoCap(cmd->tx_argv[2]);
+	    else 
+		TxError("exttosim: numeric value or \"infinite\" expected.\n");
+	    break;
+	case EXTTOSIM_RTHRESH:
+	    if (cmd->tx_argc == 2)
+	    {
+		if (LocResistThreshold == INFINITE_THRESHOLD)
+		    Tcl_SetResult(magicinterp, "infinite", NULL);
+		else
+		    Tcl_SetObjResult(magicinterp,
+			Tcl_NewIntObj(LocResistThreshold));
+		return TCL_OK;
+	    }
+	    else if (cmd->tx_argc < 3) goto usage;
+	    if (StrIsInt(cmd->tx_argv[2]))
+		LocResistThreshold = atoi(cmd->tx_argv[2]);
+	    else if (!strncmp(cmd->tx_argv[2], "inf", 3))
+		LocResistThreshold = INFINITE_THRESHOLD;
+	    else 
+		TxError("exttosim: integer value or \"infinite\" expected.\n");
+	    break;
+	case EXTTOSIM_FORMAT:
+	    if (cmd->tx_argc == 2)
+	    {
+		Tcl_SetResult(magicinterp, sim_formats[esFormat], TCL_STATIC);
+		return TCL_OK;
+	    }
+	    else if (cmd->tx_argc < 3) goto usage;
+	    value = Lookup(cmd->tx_argv[2], sim_formats);
+	    if (value < 0)
+		TxError("exttosim: output formats are MIT, LBL, or SU\n");
+	    else
+		esFormat = value;
+	    break;
+	case EXTTOSIM_MERGE:
+	    if (cmd->tx_argc == 2)
+	    {
+		if (esMergeDevsA)
+		    Tcl_SetResult(magicinterp, "aggressive", NULL);
+		else if (esMergeDevsC)
+		    Tcl_SetResult(magicinterp, "conservative", NULL);
+		else
+		    Tcl_SetResult(magicinterp, "none", NULL);
+		return TCL_OK;
+	    }
+	    else if (cmd->tx_argc < 3) goto usage;
+	    value = Lookup(cmd->tx_argv[2], cmdMergeTypes);
+	    if (value < 0)
+	    {
+		TxError("Merge types are:\n");
+		for (msg = &(cmdMergeTypes[0]); *msg != NULL; msg++)
+		    TxPrintf("    %s\n", *msg);
+	    }
+	    else switch (value) {
+		case 0:
+		    esMergeDevsA = FALSE;
+		    esMergeDevsC = FALSE;
+		    break;
+		case 1:
+		    esMergeDevsA = FALSE;
+		    esMergeDevsC = TRUE;
+		    break;
+		case 2:
+		    esMergeDevsA = TRUE;
+		    esMergeDevsC = FALSE;
+		    break;
+	    }
+	    break;
+	    
+	case EXTTOSIM_DEFAULT:
+	    LocCapThreshold = 2;
+	    LocResistThreshold = 10;
+	    EFTrimFlags = 0;
+	    EFScale = 0;
+	    if (EFArgTech)
+	    {
+		FREE(EFArgTech);
+		EFArgTech = NULL;
+	    }
+	    if (EFSearchPath)
+	    {
+		FREE(EFSearchPath);
+		EFArgTech = NULL;
+	    }
+	    break;
+	case EXTTOSIM_RUN:
+	    goto runexttosim;
+	    break;
+	case EXTTOSIM_HELP:
+usage:
+	    for (msg = &(cmdExtToSimOption[0]); *msg != NULL; msg++)
+	    {
+		TxPrintf("    %s\n", *msg);
+	    }
+	    break;
+    }
+    return TCL_OK;
+
+runexttosim:
+
+    esDevsMerged = 0;
+
+    EFInit();
+
+    /* Process command line arguments */
+
+    inName = EFArgs(argc, argv, &err_result, simmainArgs, (ClientData) NULL);
+    if (err_result == TRUE)
+    {
+	EFDone();
+        return TCL_ERROR;
+    }
+
+    if (inName == NULL)
+    {
+	/* Assume that we want to do exttospice on the currently loaded cell */
+	
+	if (w == (MagWindow *) NULL)
+	    windCheckOnlyWindow(&w, DBWclientID);
+
+	if (w == (MagWindow *) NULL)
+	{
+	    TxError("Point to a window or specify a cell name.\n");
+	    EFDone();
+	    return TCL_ERROR;
+	}
+	inName = ((CellUse *) w->w_surfaceID)->cu_def->cd_name;
+    }
+
+    /*
+     * Initializations specific to this program.
+     * Make output, alias, and label names be of the form
+     * inName.suffix if they weren't explicitly specified,
+     * where suffix is .sim, .al, or .nodes.
+     */
+
+    /* Addendum:  Because extresis depends on ".sim" output, we	*/
+    /* add ".res" in the output name (the same as used for the	*/
+    /* ".res.ext" output) so that this output won't be grabbed	*/
+    /* the next time "extresist" is run.			*/
+
+    if (simesOutName == simesDefaultOut)
+	(void) sprintf(simesDefaultOut, "%s%s.sim", inName,
+		((esDoExtResis) ? ".ext" : ""));
+    if (esAliasName == esDefaultAlias)
+	(void) sprintf(esDefaultAlias, "%s%s.al", inName,
+		((esDoExtResis) ? ".ext" : ""));
+    if (esLabelName == esDefaultLabel)
+	(void) sprintf(esDefaultLabel, "%s%s.nodes", inName,
+		((esDoExtResis) ? ".ext" : ""));
+    if ((esSimF = fopen(simesOutName, "w")) == NULL)
+    {
+	char *tclres = Tcl_Alloc(128);
+	sprintf(tclres, "exttosim: Unable to open file %s for writing\n",
+		simesOutName);
+	Tcl_SetResult(magicinterp, tclres, TCL_DYNAMIC);
+	EFDone();
+	return TCL_ERROR;
+    }
+    if (!esNoAlias && (esAliasF = fopen(esAliasName, "w")) == NULL)
+    {
+	char *tclres = Tcl_Alloc(128);
+	sprintf(tclres, "exttosim: Unable to open file %s for writing\n",
+		esAliasName);
+	Tcl_SetResult(magicinterp, tclres, TCL_DYNAMIC);
+	EFDone();
+	return TCL_ERROR;
+    }
+    if (!esNoLabel && (esLabF = fopen(esLabelName, "w")) == NULL)
+    {
+	char *tclres = Tcl_Alloc(128);
+	sprintf(tclres, "exttosim: Unable to open file %s for writing\n",
+		esLabelName);
+	Tcl_SetResult(magicinterp, tclres, TCL_DYNAMIC);
+	return TCL_ERROR;
+    }
+
+    /* Set local values for capacitor and resistor thresholds */
+    EFCapThreshold = LocCapThreshold;
+    EFResistThreshold = LocResistThreshold;
+
+    /* Read the hierarchical description of the input circuit */
+    if (EFReadFile(inName, FALSE, esDoExtResis) == FALSE)
+    {
+	EFDone();
+	return TCL_ERROR;
+    }
+
+    if ((EFStyle == NULL) && (esFormat == SU))
+    {
+	TxError("Warning:  Current extraction style does not match .ext file!\n");
+	TxError("Area/Perimeter values will be zero.\n");
+    }
+
+    /* create default fetinfo entries (MOSIS) which can be overriden by
+       the command line arguments */
+
+    for ( i = 0 ; i < MAXDEVTYPES ; i++ )
+    {
+	fetInfo[i].resClassSD = NO_RESCLASS;
+	fetInfo[i].resClassSub = NO_RESCLASS;
+	fetInfo[i].defSubs = NULL;
+    }
+
+    /* Get fetInfo information from the current extraction style 	 */
+    /* (this works only for the Tcl version with the embedded exttospice */
+    /* command)								 */
+
+    idx = 0;
+    while (ExtGetDevInfo(idx++, &devname, &sd_rclass, &sub_rclass, &subname))
+    {
+	if (idx == MAXDEVTYPES)
+	{
+	    TxError("Error:  Ran out of space for device types!\n");
+	    break;
+	}
+	i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, devname);
+
+	if (EFStyle != NULL)
+	{
+	    fetInfo[i].resClassSD = sd_rclass;
+	    fetInfo[i].resClassSub = sub_rclass;
+	    fetInfo[i].defSubs = subname;
+	}
+
+	/* Tcl variable substitution for substrate node names */
+	if (subname && (subname[0] == '$'))
+	{
+	    char *resstr;
+	    resstr = Tcl_GetVar(magicinterp, &subname[1], TCL_GLOBAL_ONLY);
+	    if (resstr != NULL) fetInfo[i].defSubs = resstr;
+	}
+    }
+
+    /* Write the output file */
+
+    fprintf(esSimF, "| units: %d tech: %s format: %s\n", EFScale, EFTech,
+		   (esFormat == MIT) ? "MIT" :  
+		      ( (esFormat == LBL) ? "LBL" : "SU" ) );
+
+    /* Convert the hierarchical description to a flat one */
+    flatFlags = EF_FLATNODES;
+    if (EFCapThreshold < INFINITE_THRESHOLD) flatFlags |= EF_FLATCAPS;
+    if (EFResistThreshold < INFINITE_THRESHOLD) flatFlags |= EF_FLATRESISTS;
+    EFFlatBuild(inName, flatFlags);
+
+    if (esMergeDevsA || esMergeDevsC)
+    {
+	devMerge *p;
+
+	EFVisitDevs(simmergeVisit, (ClientData) NULL);
+	TxPrintf("Devices merged: %d\n", esDevsMerged);
+	esFMIndex = 0;
+	for (p = devMergeList; p != NULL; p = p->next)
+	    FREE(p);
+	devMergeList = NULL;
+    }
+
+    EFVisitDevs(simdevVisit, (ClientData) NULL);
+    if (flatFlags & EF_FLATCAPS) {
+    	(void) sprintf( esCapFormat, " %%.%dlf\n", esCapAccuracy);
+	EFVisitCaps(simcapVisit, (ClientData) NULL);
+    }
+    EFVisitResists(simresistVisit, (ClientData) NULL);
+    (void) sprintf( esCapFormat, " GND %%.%dlf\n", esCapAccuracy);
+    EFVisitNodes(simnodeVisit, (ClientData) NULL);
+
+    EFFlatDone();
+    EFDone();
+
+    if (esSimF) fclose(esSimF);
+    if (esLabF) fclose(esLabF);
+    if (esAliasF) fclose(esAliasF);
+
+    TxPrintf("exttosim finished.\n");
+    return TCL_OK;
+}
+
+#else	/* !MAGIC_WRAPPER */
 
 /*
  * ----------------------------------------------------------------------------
  *
  * main --
  *
- * Top level of ext2sim.
+ * Top level of ext2sim (non-Tcl version only)
  *
  * ----------------------------------------------------------------------------
  */
@@ -179,77 +645,40 @@ main(argc, argv)
     char *argv[];
 {
 
-#endif	/* MAGIC_WRAPPER */
-
     int i,flatFlags;
     char *inName;
     FILE *f;
 
-#ifdef MAGIC_WRAPPER
-    MagWindow *w;
-    int wid;
-
-    /* Did we start in the same interpreter as we initialized? */
-    if (magicinterp != interp)
-    {
-        TxError("Warning:  Switching interpreters.  Tcl-magic is not set up "
-                "to handle this.\n");
-        magicinterp = interp;
-    }
-#endif
+    esDevsMerged = 0;
 
     EFInit();
     /* create default fetinfo entries (MOSIS) which can be overriden by
        the command line arguments */
-    for ( i = 0 ; i < MAXFETTYPES ; i++ ) {
+    for ( i = 0 ; i < MAXDEVTYPES ; i++ ) {
 	fetInfo[i].resClassSD = NO_RESCLASS;
 	fetInfo[i].resClassSub = NO_RESCLASS;
 	fetInfo[i].defSubs = NULL;
     }
-    i = efBuildAddStr(EFFetTypes, &EFFetNumTypes, MAXFETTYPES, "nfet");
+    i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "nfet");
     fetInfo[i].resClassSD = 0 ;
     fetInfo[i].resClassSub = NO_RESCLASS ;
     fetInfo[i].defSubs = "Gnd!";
-    i = efBuildAddStr(EFFetTypes, &EFFetNumTypes, MAXFETTYPES, "pfet");
+    i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "pfet");
     fetInfo[i].resClassSD = 1 ;
     fetInfo[i].resClassSub = 6 ;
     fetInfo[i].defSubs = "Vdd!";
-    i = efBuildAddStr(EFFetTypes, &EFFetNumTypes, MAXFETTYPES, "nmos");
+    i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "nmos");
     fetInfo[i].resClassSD = 0 ;
     fetInfo[i].resClassSub = NO_RESCLASS ;
     fetInfo[i].defSubs = "Gnd!";
-    i = efBuildAddStr(EFFetTypes, &EFFetNumTypes, MAXFETTYPES, "pmos");
+    i = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, "pmos");
     fetInfo[i].resClassSD = 1 ;
     fetInfo[i].resClassSub = 6 ;
     fetInfo[i].defSubs = "Vdd!";
     /* Process command line arguments */
-    inName = EFArgs(argc, argv, simmainArgs, (ClientData) NULL);
+    inName = EFArgs(argc, argv, NULL, simmainArgs, (ClientData) NULL);
     if (inName == NULL)
-#ifdef MAGIC_WRAPPER
-    {
-	/* Assume that we want to do exttospice on the currently loaded cell */
-	
-	if ((wid = TxGetPoint(NULL)) >= 0)
-	{
-	    w = WindSearchWid(wid);
-	}
-	else
-	{
-	    w = NULL;
-	    windCheckOnlyWindow(&w);
-	}
-
-	if (w == (MagWindow *) NULL)
-	{
-	    TxError("Point to a window or specify a cell name.\n");
-	    return TCL_ERROR;
-	}
-	inName = ((CellUse *) w->w_surfaceID)->cu_def->cd_name;
-    }
-
-#else
 	exit (1);
-#endif
 
     /*
      * Initializations specific to this program.
@@ -266,52 +695,24 @@ main(argc, argv)
 	(void) sprintf(esDefaultLabel, "%s.nodes", inName);
     if ((esSimF = fopen(simesOutName, "w")) == NULL)
     {
-#ifdef MAGIC_WRAPPER
-	char *tclres = Tcl_Alloc(128);
-	sprintf(tclres, "exttosim: Unable to open file %s for writing\n",
-		simesOutName);
-	Tcl_SetResult(magicinterp, tclres, TCL_DYNAMIC);
-	return TCL_ERROR;
-#else
 	perror(simesOutName);
 	exit (1);
-#endif
     }
     if (!esNoAlias && (esAliasF = fopen(esAliasName, "w")) == NULL)
     {
-#ifdef MAGIC_WRAPPER
-	char *tclres = Tcl_Alloc(128);
-	sprintf(tclres, "exttosim: Unable to open file %s for writing\n",
-		esAliasName);
-	Tcl_SetResult(magicinterp, tclres, TCL_DYNAMIC);
-	return TCL_ERROR;
-#else
 	perror(esAliasName);
 	exit (1);
-#endif
     }
     if (!esNoLabel && (esLabF = fopen(esLabelName, "w")) == NULL)
     {
-#ifdef MAGIC_WRAPPER
-	char *tclres = Tcl_Alloc(128);
-	sprintf(tclres, "exttosim: Unable to open file %s for writing\n",
-		esLabelName);
-	Tcl_SetResult(magicinterp, tclres, TCL_DYNAMIC);
-	return TCL_ERROR;
-#else
 	perror(esLabelName);
 	exit (1);
-#endif
     }
 
     /* Read the hierarchical description of the input circuit */
-    if (EFReadFile(inName, FALSE) == FALSE)
+    if (EFReadFile(inName, FALSE, esDoExtResis) == FALSE)
     {
-#ifdef MAGIC_WRAPPER
-	return TCL_ERROR;
-#else
 	exit(1);
-#endif
     }
 
     fprintf(esSimF, "| units: %d tech: %s format: %s\n", EFScale, EFTech,
@@ -323,7 +724,17 @@ main(argc, argv)
     if (EFCapThreshold < INFINITE_THRESHOLD) flatFlags |= EF_FLATCAPS;
     if (EFResistThreshold < INFINITE_THRESHOLD) flatFlags |= EF_FLATRESISTS;
     EFFlatBuild(inName, flatFlags);
-    EFVisitFets(simfetVisit, (ClientData) NULL);
+
+    if (esMergeDevsA || esMergeDevsC) {
+	devMerge *p;
+
+	EFVisitDevs(simmergeVisit, (ClientData) NULL);
+	TxPrintf("Devices merged: %d\n", esDevsMerged);
+	esFMIndex = 0;
+	for (p = devMergeList; p != NULL; p = p->next) FREE(p);
+    }
+
+    EFVisitDevs(simdevVisit, (ClientData) NULL);
     if (flatFlags & EF_FLATCAPS) {
     	(void) sprintf( esCapFormat, " %%.%dlf\n", esCapAccuracy);
 	EFVisitCaps(simcapVisit, (ClientData) NULL);
@@ -332,23 +743,18 @@ main(argc, argv)
     (void) sprintf( esCapFormat, " GND %%.%dlf\n", esCapAccuracy);
     EFVisitNodes(simnodeVisit, (ClientData) NULL);
 
-#ifdef	free_all_mem
     EFFlatDone();
     EFDone();
-#endif	free_all_mem
 
     if (esSimF) fclose(esSimF);
     if (esLabF) fclose(esLabF);
     if (esAliasF) fclose(esAliasF);
 
-#ifdef MAGIC_WRAPPER
-    TxPrintf("exttosim finished.\n");
-    return TCL_OK;
-#else
     TxPrintf("Memory used: %s\n", RunStats(RS_MEM, NULL, NULL));
     exit(0);
-#endif
 }
+
+#endif		/* MAGIC_WRAPPER */
 
 /*
  * ----------------------------------------------------------------------------
@@ -374,7 +780,7 @@ main(argc, argv)
  * ----------------------------------------------------------------------------
  */
 
-Void
+int
 simmainArgs(pargc, pargv)
     int *pargc;
     char ***pargv;
@@ -391,11 +797,17 @@ simmainArgs(pargc, pargv)
 	    esNoAttrs = TRUE;
 	    break;
 	case 'F':
-	    esFetNodesOnly = TRUE;
+	    esDevNodesOnly = TRUE;
 	    break;
 	case 'L':
 	    esNoLabel = TRUE;
 	    break;
+	case 'M':
+	    esMergeDevsA = TRUE;
+	    break;
+	case 'm':
+	    esMergeDevsC = TRUE;
+	    break;
 	case 'a':
 	    if ((esAliasName = ArgStr(&argc, &argv, "filename")) == NULL)
 		goto usage;
@@ -444,6 +856,7 @@ simmainArgs(pargc, pargv)
 
 	     break;
 	     }
+#ifndef MAGIC_WRAPPER
 	case 'j':
 	    {
 	    char *rp,  subsNode[80] ;
@@ -458,7 +871,7 @@ simmainArgs(pargc, pargv)
 		rClassSub = NO_RESCLASS ;
 	    	if ( sscanf(rp, "%d/%s",  &rClass, subsNode) != 2 ) goto usage;
 	    }
-	    ndx = efBuildAddStr(EFFetTypes, &EFFetNumTypes, MAXFETTYPES, cp);
+	    ndx = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, cp);
 	    fetInfo[ndx].resClassSD = rClass;
 	    fetInfo[ndx].resClassSub = rClassSub;
 	    MALLOC(char *, fetInfo[ndx].defSubs, (strlen(subsNode)+1));
@@ -468,6 +881,7 @@ simmainArgs(pargc, pargv)
             	fetInfo[ndx].defSubs);
 	    break;
 	    }
+#endif			/* MAGIC_WRAPPER */
 	default:
 	    TxError("Unrecognized flag: %s\n", argv[0]);
 	    goto usage;
@@ -475,22 +889,22 @@ simmainArgs(pargc, pargv)
 
     *pargv = argv;
     *pargc = argc;
-    return;
+    return 0;
 
 usage:
     TxError("Usage: ext2sim [-a aliasfile] [-A] [-B] [-l labelfile] [-L]\n"
 		"[-o simfile] [-J flat|hier] [-y cap_digits]\n"
-		"[-j device:sdRclass[/subRclass]/defaultSubstrate]\n"
 		"[-f mit|lbl|su] "
 #ifdef MAGIC_WRAPPER
 		"[file]\n"
 #else
+		"[-j device:sdRclass[/subRclass]/defaultSubstrate]\n"
 		"file\n\n    or else see options to extcheck(1)\n"
 #endif
 		);
 
 #ifdef MAGIC_WRAPPER
-    return;
+    return 1;
 #else
     exit (1);
 #endif
@@ -523,10 +937,10 @@ HierName *suffix;
 /*
  * ----------------------------------------------------------------------------
  *
- * simfetVisit --
+ * simdevVisit --
  *
- * Procedure to output a single fet to the .sim file.
- * Called by EFVisitFets().
+ * Procedure to output a single dev to the .sim file.
+ * Called by EFVisitDevs().
  *
  * Results:
  *	Returns 0 always.
@@ -534,7 +948,7 @@ HierName *suffix;
  * Side effects:
  *	Writes to the file esSimF.
  *
- * Format of a .sim fet line:
+ * Format of a .sim dev line:
  *
  *	type gate source drain l w x y g= s= d=
  *
@@ -551,57 +965,87 @@ HierName *suffix;
  */
 
 int
-simfetVisit(fet, hierName, trans, l, w)
-    Fet *fet;		/* Fet being output */
-    HierName *hierName;	/* Hierarchical path down to this fet */
+simdevVisit(dev, hierName, trans)
+    Dev *dev;		/* Device being output */
+    HierName *hierName;	/* Hierarchical path down to this device */
     Transform *trans;	/* Coordinate transform for output */
-    int l, w;
 {
-    FetTerm *gate, *source, *drain;
+    DevTerm *gate, *source, *drain;
     EFNode  *subnode, *snode, *dnode;
     int scale;
+    int l, w;
     Rect r;
     char name[12];
 
     sprintf(name, "output");
 
     /* If no terminals, can't do much of anything */
-    if (fet->fet_nterm < 2)
+    if (dev->dev_nterm < 1)
 	return 0;
 
+    /* Merged devices */
+    if ((esMergeDevsA || esMergeDevsC) && devIsKilled(esFMIndex++))
+	return 0;
+
+    /* Computation of length and width has been moved from efVisitDevs */
+    EFGetLengthAndWidth(dev, &l, &w);
+
     /* If only two terminals, connect the source to the drain */
-    gate = &fet->fet_terms[0];
-    source = drain = &fet->fet_terms[1];
-    if (fet->fet_nterm >= 3)
-	drain = &fet->fet_terms[2];
-    subnode = fet->fet_subsnode;
+    gate = &dev->dev_terms[0];
+    if (dev->dev_nterm >= 2)
+	source = drain = &dev->dev_terms[1];
+    if (dev->dev_nterm >= 3)
+	drain = &dev->dev_terms[2];
+    subnode = dev->dev_subsnode;
 
     /* Kludge for .sim: transistor types can only be one character */
-    fprintf(esSimF, "%c", EFFetTypes[fet->fet_type][0]);
+    switch(dev->dev_class)
+    {
+	/* "sim" types are fixed according to the device class */
+	case DEV_BJT:
+	    fprintf(esSimF, "b");	/* sim format extension */
+	    break;
+	case DEV_RES:
+	    fprintf(esSimF, "r");
+	    break;
+	case DEV_CAP:
+	    fprintf(esSimF, "c");	/* sim format extension */
+	    break;
+	default:
+	    fprintf(esSimF, "%c", EFDevTypes[dev->dev_type][0]);
+	    break;
+    }
 
     /* Output gate, source, and drain node names */
-    simfetOutNode(hierName, gate->fterm_node->efnode_name->efnn_hier, name, esSimF);
-    simfetOutNode(hierName, source->fterm_node->efnode_name->efnn_hier, name, esSimF);
+    simdevOutNode(hierName, gate->dterm_node->efnode_name->efnn_hier, name, esSimF);
+    if (dev->dev_nterm > 1)
+	simdevOutNode(hierName, source->dterm_node->efnode_name->efnn_hier, name, esSimF);
 
     /* Hack for BiCMOS---see scmos.tech27 hack:  Tim, 7/16/96 */
+    /* Second hack, using dev_class 2/20/03 */
+    if (EFDevTypes[dev->dev_type][0] == 'b')
+	dev->dev_class = DEV_BJT;
 
-    if (EFFetTypes[fet->fet_type][0] == 'b') {
+    if (dev->dev_class == DEV_BJT && subnode)
+    {
         sprintf(name, "fet");
-        simfetOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF); 
+        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF); 
     }
-    else
-        simfetOutNode(hierName, drain->fterm_node->efnode_name->efnn_hier, name, esSimF);
+    else if (dev->dev_nterm > 2)
+        simdevOutNode(hierName, drain->dterm_node->efnode_name->efnn_hier, name, esSimF);
 
-    if ( esFormat == LBL ) { /* support gemini's substrate comparison */
+    /* Support gemini's substrate comparison */
+    if ( esFormat == LBL && subnode)
+    {
 	putc(' ', esSimF);
-	simfetSubstrate(hierName, subnode->efnode_name->efnn_hier,
-	             fet->fet_type, 0, FALSE, esSimF);
+	simdevSubstrate(hierName, subnode->efnode_name->efnn_hier,
+	             dev->dev_type, 0, FALSE, esSimF);
     }
 
-    GeoTransRect(trans, &fet->fet_rect, &r);
+    GeoTransRect(trans, &dev->dev_rect, &r);
     scale = GeoScale(trans);
 
-    if (EFFetTypes[fet->fet_type][0] == 'b') {
+    if (dev->dev_class == DEV_BJT || EFDevTypes[dev->dev_type][0] == 'b') {
 
        /* Bipolar sim format:  We don't have the length and width
 	* of the collector well, but we can get it from the area
@@ -626,6 +1070,12 @@ simfetVisit(fet, hierName, trans, l, w)
 	fprintf(esSimF, " %d %d %d %d", (int)cl, (int)cw,
 		r.r_xbot, r.r_ybot);
     }
+    else if (dev->dev_class == DEV_RES) {	/* generate a resistor */
+       fprintf(esSimF, " %f", (double)(dev->dev_values.dev_res));
+    }
+    else if (dev->dev_class == DEV_CAP) {	/* generate a capacitor */
+       fprintf(esSimF, " %f", (double)(dev->dev_values.dev_cap));
+    }
     else {
 
        /*
@@ -645,51 +1095,51 @@ simfetVisit(fet, hierName, trans, l, w)
        {
 	   bool subAP= FALSE, hierS = esHierAP, hierD = esHierAP;
 
-	   if (gate->fterm_attrs)
-	       fprintf(esSimF, " g=%s", gate->fterm_attrs);
+	   if (gate->dterm_attrs)
+	       fprintf(esSimF, " g=%s", gate->dterm_attrs);
 	   if ( esFormat == SU ) {
-	       if ( gate->fterm_attrs ) {
-	         subAP = Match(ATTR_SUBSAP, gate->fterm_attrs ) ;
+	       if ( gate->dterm_attrs ) {
+	         subAP = Match(ATTR_SUBSAP, gate->dterm_attrs ) ;
 	         fprintf(esSimF, ",");
 	       } else
 	         fprintf(esSimF, " g=");
-	       simfetSubstrate(hierName, subnode->efnode_name->efnn_hier,
-		         fet->fet_type, scale, subAP, esSimF);
+	       simdevSubstrate(hierName, subnode->efnode_name->efnn_hier,
+		         dev->dev_type, scale, subAP, esSimF);
 	   }
-	   if (source->fterm_attrs) {
-	       fprintf(esSimF, " s=%s", source->fterm_attrs);
-	       if  ( Match(ATTR_HIERAP, source->fterm_attrs ) != FALSE )
+	   if (source->dterm_attrs) {
+	       fprintf(esSimF, " s=%s", source->dterm_attrs);
+	       if  ( Match(ATTR_HIERAP, source->dterm_attrs ) != FALSE )
 		   hierS = TRUE ;
-	       else if ( Match(ATTR_FLATAP, source->fterm_attrs ) != FALSE )
+	       else if ( Match(ATTR_FLATAP, source->dterm_attrs ) != FALSE )
 		   hierS = FALSE ;
 	   }
 	   if ( esFormat == SU ) {
-	       fprintf(esSimF, "%s", (source->fterm_attrs) ? "," : " s=" );
+	       fprintf(esSimF, "%s", (source->dterm_attrs) ? "," : " s=" );
 	       if (hierS)
-	         simnAPHier(source, hierName, fetInfo[fet->fet_type].resClassSD, 
+	         simnAPHier(source, hierName, fetInfo[dev->dev_type].resClassSD, 
 		      scale, esSimF);
 	       else {
 	         snode= GetNode(hierName,
-			     source->fterm_node->efnode_name->efnn_hier);
-	         simnAP(snode, fetInfo[fet->fet_type].resClassSD, scale, esSimF);
+			     source->dterm_node->efnode_name->efnn_hier);
+	         simnAP(snode, fetInfo[dev->dev_type].resClassSD, scale, esSimF);
 	       }
 	   }
-	   if (drain->fterm_attrs) {
-	       fprintf(esSimF, " d=%s", drain->fterm_attrs);
-	       if  ( Match(ATTR_HIERAP, drain->fterm_attrs ) != FALSE )
+	   if (drain->dterm_attrs) {
+	       fprintf(esSimF, " d=%s", drain->dterm_attrs);
+	       if  ( Match(ATTR_HIERAP, drain->dterm_attrs ) != FALSE )
 		   hierD = TRUE ;
-	       else if ( Match(ATTR_FLATAP, drain->fterm_attrs ) != FALSE )
+	       else if ( Match(ATTR_FLATAP, drain->dterm_attrs ) != FALSE )
 		   hierD = FALSE ;
 	   }
 	   if ( esFormat == SU ) {
-	       fprintf(esSimF, "%s", (drain->fterm_attrs) ? "," : " d=" );
+	       fprintf(esSimF, "%s", (drain->dterm_attrs) ? "," : " d=" );
 	       if (hierD) 
-	         simnAPHier(drain, hierName, fetInfo[fet->fet_type].resClassSD, 
+	         simnAPHier(drain, hierName, fetInfo[dev->dev_type].resClassSD, 
 		      scale, esSimF);
 	       else {
 	         dnode = GetNode(hierName,
-			      drain->fterm_node->efnode_name->efnn_hier);
-	         simnAP(dnode, fetInfo[fet->fet_type].resClassSD, 
+			      drain->dterm_node->efnode_name->efnn_hier);
+	         simnAP(dnode, fetInfo[dev->dev_type].resClassSD, 
 		      scale, esSimF);
 	       }
 	   }
@@ -701,7 +1151,7 @@ simfetVisit(fet, hierName, trans, l, w)
 }
 
 Void
-simfetSubstrate( prefix, suffix, type, scale, doAP, outf)
+simdevSubstrate( prefix, suffix, type, scale, doAP, outf)
 HierName *prefix;
 HierName *suffix;
 int type, scale;
@@ -737,7 +1187,7 @@ FILE *outf;
 	if ( esFormat == SU ) {
 	  if ( doAP ) {
 	    if ( fetInfo[type].resClassSub < 0 ) {
-	     TxError("Error: subap for fettype %d required but not "
+	     TxError("Error: subap for devtype %d required but not "
 			"specified on command line\n", type);
 	     fprintf(outf,"A_0,P_0,");
 	    }
@@ -792,13 +1242,13 @@ FILE *outf;
 	return TRUE;
 }
 
-int simnAPHier(fterm, hierName, resClass, scale, outf)
-    FetTerm *fterm;
+int simnAPHier(dterm, hierName, resClass, scale, outf)
+    DevTerm *dterm;
     HierName *hierName;
     int  resClass, scale;
     FILE *outf;
 {
-	EFNode *node = fterm->fterm_node;
+	EFNode *node = dterm->dterm_node;
 	nodeClientHier   *nc ;
 	int a, p;
 
@@ -827,7 +1277,7 @@ int simnAPHier(fterm, hierName, resClass, scale, outf)
 /*
  * ----------------------------------------------------------------------------
  *
- * simfetOutNode --
+ * simdevOutNode --
  *
  * Output the name of the node whose hierarchical prefix down to this
  * point is 'prefix' and whose name from the end of suffix down to the
@@ -845,7 +1295,7 @@ int simnAPHier(fterm, hierName, resClass, scale, outf)
  */
 
 Void
-simfetOutNode(prefix, suffix, name, outf)
+simdevOutNode(prefix, suffix, name, outf)
     HierName *prefix;
     HierName *suffix;
     char *name;
@@ -982,7 +1432,7 @@ int simnodeVisit(node, res, cap)
     char *fmt;
     EFAttr *ap;
 
-    if (esFetNodesOnly && node->efnode_client == (ClientData) NULL)
+    if (esDevNodesOnly && node->efnode_client == (ClientData) NULL)
 	return 0;
 
     hierName = (HierName *) node->efnode_name->efnn_hier;
@@ -1039,3 +1489,180 @@ int simnodeVisit(node, res, cap)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * simmkDevMerge --
+ * Create a new devMerge structure.
+ *
+ * Results:
+ *    Obvious
+ *
+ * Side effects:
+ *	Allocates memory and sets the fields of the structure.
+ *
+ * ----------------------------------------------------------------------------
+ */
+devMerge *simmkDevMerge(l, w, g, s, d, b, hn, dev)
+int     l, w;
+EFNode *g, *s, *d, *b;
+HierName *hn;
+Dev    *dev;
+{
+	register    devMerge *fp;
+
+	MALLOC (devMerge *, fp, sizeof(devMerge));
+	fp->l = l; fp->w = w;
+	fp->g = g; fp->s = s;
+	fp->d = d; fp->b = b;
+	fp->dev = dev;
+	fp->esFMIndex = esFMIndex;
+	fp->hierName = hn;
+	fp->next = NULL;
+	addDevMult(1.0);
+
+	return fp;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * parallelDevs --
+ * Macro to look if two devs are in parallel
+ *
+ * Results:
+ *    NOT_PARALLEL  if not in parallel
+ *    PARALLEL      if s==s d==d and g==g and bulk=bulk
+ *    FLIP_PARALLEL if s==d d==s --->>----------------
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+#define	NOT_PARALLEL	0
+#define	PARALLEL	1
+#define	PARALLEL_R	2
+
+#define parallelDevs(f1, f2) \
+( \
+	( (f1)->g == (f2)->g && (f1)->b == (f2)->b && (f1)->l == (f2)->l && \
+	  ( esMergeDevsA || (f1)->w == (f2)->w ) )  ? \
+	   ( ((f1)->d == (f2)->d && (f1)->s == (f2)->s ) ? \
+	       PARALLEL : \
+	      (((f1)->s == (f2)->d && (f1)->d == (f2)->s ) ? PARALLEL_R : NOT_PARALLEL) )\
+	  : NOT_PARALLEL  \
+)
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * mergeAttr --
+ * Macro to merge two attribute strings
+ *
+ * Results:
+ *  The merged strings
+ *
+ * Side effects:
+ *  Might allocate and free memory.
+ *
+ * ----------------------------------------------------------------------------
+ */
+#define	mergeAttr(a1, a2) { \
+	if ( a1 == NULL ) a1 = a2 ; \
+	else { \
+		char *t; \
+		int l1 = strlen(a1); \
+		int l2 = strlen(a2); \
+		MALLOC(char *, t, (l1+l2)+1); \
+		t = (char *) strcat(a1,a2); \
+		FREE(a1); \
+		a1 = t ; \
+	} \
+}
+
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * simmergeVisit --
+ * Visits each dev throu EFVisitDevs and finds if it is in parallel with
+ * any previously visited dev.
+ *
+ * Results:
+ *  0 always to keep the caller going.
+ *
+ * Side effects:
+ *  Numerous.
+ *
+ * ----------------------------------------------------------------------------
+ */
+int
+simmergeVisit(dev, hierName, trans, l, w)
+Dev *dev;		/* Dev to examine */
+HierName *hierName;	/* Hierarchical path down to this dev */
+Transform *trans;	/* Coordinate transform not used */
+int l, w;
+{
+	DevTerm *gate, *source, *drain;
+	Dev     *cf;
+	DevTerm *cg, *cs, *cd;
+	EFNode	*subnode, *snode, *dnode, *gnode;
+	int      scale, pmode;
+    	Rect	 r;
+	float	 m;
+	register devMerge *fp, *cfp;
+
+	if (dev->dev_nterm < 2) {
+		TxError("outPremature\n");
+		return 0;
+	}
+
+	gate = &dev->dev_terms[0];
+	source = drain = &dev->dev_terms[1];
+	if (dev->dev_nterm >= 3)
+		drain = &dev->dev_terms[2];
+	subnode = dev->dev_subsnode;
+
+	gnode = GetNode (hierName, gate->dterm_node->efnode_name->efnn_hier);
+	snode = GetNode (hierName, source->dterm_node->efnode_name->efnn_hier);
+	dnode = GetNode (hierName, drain->dterm_node->efnode_name->efnn_hier);
+
+    	GeoTransRect(trans, &dev->dev_rect, &r);
+    	scale = GeoScale(trans);
+	fp = simmkDevMerge(l*scale, w*scale, gnode, snode, 
+			dnode, subnode, hierName, dev);
+
+	/*
+	 * run the list of devs. compare the current one with 
+	 * each one in the list. if they fullfill the matching requirements
+	 * merge them.
+	 */
+
+	for ( cfp = devMergeList ; cfp != NULL ; cfp = cfp->next ) {
+	  if ((pmode = parallelDevs(fp, cfp)) != NOT_PARALLEL) {
+
+		cf = cfp->dev;
+		cg = &cfp->dev->dev_terms[0];
+		cs = cd = &cfp->dev->dev_terms[1];
+		if (cfp->dev->dev_nterm >= 3) {
+			if ( pmode == PARALLEL ) 
+				cd = &cfp->dev->dev_terms[2];
+			else if ( pmode == PARALLEL_R ) 
+				cs = &cfp->dev->dev_terms[2];
+		}
+		
+		m = esFMult[cfp->esFMIndex] + ((float)fp->w/(float)cfp->w);
+		setDevMult(fp->esFMIndex, DEV_KILLED); 
+		setDevMult(cfp->esFMIndex, m);
+		esDevsMerged++;
+		FREE(fp);
+		return 0;
+	  }
+	}
+	/* No parallel devs to it yet */
+	fp->next = devMergeList;
+	devMergeList = fp;
+	return 0;
+}
+
