@@ -19,12 +19,13 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFbuild.c,v 1.2 2001/01/12 22:12:34 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #ifdef SYSV
 #include <string.h>
 #endif
+#include <stdlib.h>		/* for atof() */
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
@@ -32,30 +33,31 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFbuild.c,v 1.2 20
 #include "utils/malloc.h"
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
+#include "extract/extract.h"	/* for device class list */
 
 /*
  * To avoid allocating ridiculously large amounts of memory to hold
  * transistor types and the names of node types, we maintain the following
  * string tables.  Each string (transistor type or Magic layername) appears
- * exactly once in its respective table; each Fet structure's fet_type field
- * is an index into EFFetTypes[], and each node layer name an index into
+ * exactly once in its respective table; each Dev structure's dev_type field
+ * is an index into EFDevTypes[], and each node layer name an index into
  * EFLayerNames[].
  */
 
-    /* The following are ridiculously high */
+/* The following are ridiculously high */
 #define	MAXTYPES	100
 
-    /* Table of transistor types */
-char *EFFetTypes[MAXFETTYPES];
-int   EFFetNumTypes;
+/* Table of transistor types */
+char *EFDevTypes[MAXDEVTYPES];
+int   EFDevNumTypes;
 
-    /* Table of Magic layers */
+/* Table of Magic layers */
 char *EFLayerNames[MAXTYPES] = { "space" };
 int   EFLayerNumNames;
 
-    /* Forward declarations */
+/* Forward declarations */
 Connection *efAllocConn();
-EFNode *efBuildFetNode();
+EFNode *efBuildDevNode();
 
 /*
  * ----------------------------------------------------------------------------
@@ -301,7 +303,7 @@ efBuildDist(def, driver, receiver, min, max)
     TxError("ADD %s ", EFHNToStr(distKey.dist_1));
     TxError("%s ", EFHNToStr(distKey.dist_2));
     TxError("%d %d\n", min, max);
-#endif	notdef
+#endif	/* notdef */
 
     he = HashFind(&def->def_dists, (char *) &distKey);
     if (dist = (Distance *) HashGetValue(he))
@@ -434,10 +436,10 @@ efBuildEquiv(def, nodeName1, nodeName2)
 /*
  * ----------------------------------------------------------------------------
  *
- * efBuildFet --
+ * efBuildDevice --
  *
- * Process a "fet" line from a .ext file.
- * The number of terminals in the fet is argc/3 (which must be integral).
+ * Process a device line from a .ext file.
+ * The number of terminals in the dev is argc/3 (which must be integral).
  * Each block of 3 strings in argv describes a single terminal; see the
  * comments below for their interpretation.
  *
@@ -445,63 +447,112 @@ efBuildEquiv(def, nodeName1, nodeName2)
  *	None.
  *
  * Side effects:
- *	Prepends this fet to the list for the def 'def'.
+ *	Prepends this dev to the list for the def 'def'.
  *
  * ----------------------------------------------------------------------------
  */
 
 Void
-efBuildFet(def, type, r, area, perim, substrate, argc, argv)
+efBuildDevice(def, class, type, r, argc, argv)
     Def *def;		/* Def to which this connection is to be added */
-    char *type;		/* Type of this transistor */
-    Rect *r;		/* Coordinates of 1x1 rectangle entirely inside fet */
-    int area, perim;	/* Total area, perimeter of channel */
-    char *substrate;	/* Name of node to which substrate is connected */
+    char class;		/* Class (dev, bjt, etc.) of this device */
+    char *type;		/* Type (name) of this device */
+    Rect *r;		/* Coordinates of 1x1 rectangle entirely inside device */
     int argc;		/* Size of argv; should be a multiple of 3 */
-    char *argv[];	/* Tokens for the rest of the fet line.  These are
+    char *argv[];	/* Tokens for the rest of the dev line.  These are
 			 * taken in groups of 3, one for each terminal.
 			 * Each group of 3 consists of the node name to which
 			 * the terminal connects, the length of the terminal,
 			 * and an attribute list (or the token 0).
 			 */
 {
-    register int n, nterminals = argc / 3;
-    register FetTerm *term;
-    register Fet *newfet;
+    register int n, nterminals;
+    register DevTerm *term;
+    register Dev *newdev;
     register char **av;
+    int argstart = 1;	/* start of terminal list in argv[] */
 
-    MALLOC(Fet *, newfet, (unsigned) FetSize(nterminals));
-    newfet->fet_nterm = nterminals;
-    newfet->fet_area = area;
-    newfet->fet_perim = perim;
-    newfet->fet_rect = *r;
-    newfet->fet_type = efBuildAddStr(EFFetTypes, &EFFetNumTypes, MAXFETTYPES, type);
+    int area, perim;	/* Total area, perimeter of primary type (i.e., channel) */
+    char *substrate;	/* Name of node to which substrate is connected */
+
+    switch (class)
+    {
+	case DEV_FET:
+	case DEV_MOSFET:
+	case DEV_BJT:
+	case DEV_SUBCKT:
+	    argstart = 3;
+	    break;
+    }
+    nterminals = (argc - argstart) / 3;
+
+    MALLOC(Dev *, newdev, (unsigned) DevSize(nterminals));
+    newdev->dev_nterm = nterminals;
+    newdev->dev_rect = *r;
+    newdev->dev_type = efBuildAddStr(EFDevTypes, &EFDevNumTypes, MAXDEVTYPES, type);
+    newdev->dev_class = class;
+    newdev->dev_subsnode = NULL;
+ 
+    switch (class)
+    {
+	case DEV_FET:		/* old-style "fet" record */
+	    newdev->dev_values.dev_ap.dev_area = atoi(argv[0]);
+	    newdev->dev_values.dev_ap.dev_perim = atoi(argv[1]);
+	    newdev->dev_subsnode = efBuildDevNode(def, argv[2], TRUE);
+	    break;
+	case DEV_MOSFET:	/* new-style "device mosfet" record */
+	case DEV_BJT:
+	case DEV_SUBCKT:
+	    newdev->dev_values.dev_lw.dev_length = atoi(argv[0]);
+	    newdev->dev_values.dev_lw.dev_width = atoi(argv[1]);
+	    /* "None" in the place of the substrate name means substrate is ignored */
+	    if (strncmp(argv[2], "None", 4) != 0)
+		newdev->dev_subsnode = efBuildDevNode(def, argv[2], TRUE);
+	    break;
+	case DEV_RES:
+	    if (StrIsNumeric(argv[0]))
+		newdev->dev_values.dev_res = (float)atof(argv[0]);
+	    else
+	    {
+		efReadError("Error: expected resistance value, got %s\n", argv[0]);
+		newdev->dev_values.dev_res = 0.0;
+	    }
+	    break;
+	case DEV_CAP:
+	    if (StrIsNumeric(argv[0]))
+		newdev->dev_values.dev_cap = (float)atof(argv[0]);
+	    else
+	    {
+		efReadError("Error: expected capacitance value, got %s\n", argv[0]);
+		newdev->dev_values.dev_cap = 0.0;
+	    }
+	    break;
+    }
 
 #define	TERM_NAME	0
 #define	TERM_PERIM	1
 #define	TERM_ATTRS	2
 
-    newfet->fet_subsnode = efBuildFetNode(def, substrate, TRUE);
-    for (av = argv, n = 0; n < nterminals; n++, av += 3)
+    for (av = &argv[argstart], n = 0; n < nterminals; n++, av += 3)
     {
-	term = &newfet->fet_terms[n];
-	term->fterm_node = efBuildFetNode(def, av[TERM_NAME], FALSE);
-	term->fterm_perim = atoi(av[TERM_PERIM]);
+	term = &newdev->dev_terms[n];
+	term->dterm_node = efBuildDevNode(def, av[TERM_NAME], FALSE);
+	term->dterm_perim = atoi(av[TERM_PERIM]);
 
 	/* If the attr list is '0', this signifies no attributes */
 	if (av[TERM_ATTRS][0] == '0' && av[TERM_ATTRS][1] == '\0')
-	    term->fterm_attrs = (char *) NULL;
+	    term->dterm_attrs = (char *) NULL;
 	else
-	    term->fterm_attrs = StrDup((char **) NULL, av[TERM_ATTRS]);
+	    term->dterm_attrs = StrDup((char **) NULL, av[TERM_ATTRS]);
     }
 
 #undef	TERM_NAME
 #undef	TERM_PERIM
 #undef	TERM_ATTRS
 
-    /* Add this fet to the list for def */
-    newfet->fet_next = def->def_fets;
-    def->def_fets = newfet;
+    /* Add this dev to the list for def */
+    newdev->dev_next = def->def_devs;
+    def->def_devs = newdev;
 }
 
 /*
@@ -512,15 +563,17 @@ efBuildFet(def, type, r, area, perim, substrate, argc, argv)
  *	Look for the node named 'name' in the local table for 'def', or in
  *	the global node name table.  If it doesn't already exist, create a
  *	EFNode for it, and set capacitance and area/perimeter to zero.
- *	Set the efnode_flags value to EF_PORT.
+ *	Set the efnode_flags value to EF_PORT, with the port number encoded
+ *	in the field above the flag bits (PORT_SHIFT bits up).
  *
  * ----------------------------------------------------------------------------
  */
 
 void
-efBuildPortNode(def, name, x, y, layername)
+efBuildPortNode(def, name, idx, x, y, layername)
     Def *def;		/* Def to which this connection is to be added */
     char *name;		/* One of the names for this node */
+    int idx;		/* Port number (order) */
     int x; int y;	/* Location of a point inside this node */
     char *layername;	/* Name of tile type */
 {
@@ -537,14 +590,14 @@ efBuildPortNode(def, name, x, y, layername)
 	nn = (EFNodeName *) HashGetValue(he);
     }
     if (nn != (EFNodeName *) NULL)
-	nn->efnn_node->efnode_flags |= EF_PORT;
+	nn->efnn_node->efnode_flags |= (EF_PORT | (idx << PORT_SHIFT));
 }
 
 
 /*
  * ----------------------------------------------------------------------------
  *
- * efBuildFetNode --
+ * efBuildDevNode --
  *
  * Look for the node named 'name' in the local table for 'def', or
  * in the global node name table.  If it doesn't already exist,
@@ -562,7 +615,7 @@ efBuildPortNode(def, name, x, y, layername)
  */
 
 EFNode *
-efBuildFetNode(def, name, isSubsNode)
+efBuildDevNode(def, name, isSubsNode)
     Def *def;
     char *name;
     bool isSubsNode;
@@ -584,9 +637,9 @@ efBuildFetNode(def, name, isSubsNode)
 	if (isSubsNode)
 	{
 	    if (!EFHNIsGlob(nn->efnn_hier))
-		efReadError("Default fet substrate node %s is not a global\n",
+		efReadError("Default dev substrate node %s is not a global\n",
 		    name);
-	    nn->efnn_node->efnode_flags |= EF_FETTERM;
+	    nn->efnn_node->efnode_flags |= EF_DEVTERM;
 	}
     }
 
@@ -1168,11 +1221,11 @@ efNodeMerge(node1, node2)
     node2->efnode_next->efnhdr_prev = node2->efnode_prev;
 
     /*
-     * Only if both nodes were EF_FETTERM do we keep EF_FETTERM set
+     * Only if both nodes were EF_DEVTERM do we keep EF_DEVTERM set
      * in the resultant node.
      */
-    if ((node2->efnode_flags & EF_FETTERM) == 0)
-	node1->efnode_flags &= ~EF_FETTERM;
+    if ((node2->efnode_flags & EF_DEVTERM) == 0)
+	node1->efnode_flags &= ~EF_DEVTERM;
 
     /* Get rid of node2 */
     FREE((char *) node2);
