@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFflat.c,v 1.2 2001/01/12 22:12:35 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -66,7 +66,7 @@ char *efFlatGlobCopy();
  * EF_FLATNODES), and EF_FLATDISTS the distance table.
  *
  * Callers who want various pieces of information should call
- * the relevant EFVisit procedures (e.g., EFVisitFets(), EFVisitCaps(),
+ * the relevant EFVisit procedures (e.g., EFVisitDevs(), EFVisitCaps(),
  * EFVisitNodes(), etc).
  *
  * Results:
@@ -153,9 +153,9 @@ EFFlatDone()
 {
 #ifdef	MALLOCTRACE
     /* Hash table statistics */
-    printf("\n\nStatistics for node hash table:\n");
+    TxPrintf("\n\nStatistics for node hash table:\n");
     HashStats(&efNodeHashTable);
-#endif	MALLOCTRACE
+#endif	/* MALLOCTRACE */
 
     /* Free temporary storage */
     efFreeNodeTable(&efNodeHashTable);
@@ -209,7 +209,8 @@ efFlatNodes(hc)
     efAddNodes(hc);
 
     /* Process our own connections and adjustments */
-    (void) efAddConns(hc);
+    if (!(hc->hc_use->use_def->def_flags & DEF_SUBCIRCUIT))
+	(void) efAddConns(hc);
 
     return (0);
 }
@@ -243,6 +244,7 @@ efAddNodes(hc)
     int scale, size, asize;
     HashEntry *he;
     Transform t;
+    bool is_subcircuit = (def->def_flags & DEF_SUBCIRCUIT) ? TRUE : FALSE;
 
     scale = def->def_scale;
     t = hc->hc_trans;
@@ -254,6 +256,9 @@ efAddNodes(hc)
 	    node != &def->def_firstn;
 	    node = (EFNode *) node->efnode_next)
     {
+	/* In subcircuits, only enumerate the ports */
+	if (is_subcircuit && !(node->efnode_flags & EF_PORT)) continue;
+
 	MALLOC(EFNode *, newnode, size);
 	newnode->efnode_attrs = (EFAttr *) NULL;
 	for (ap = node->efnode_attrs; ap; ap = ap->efa_next)
@@ -288,17 +293,17 @@ efAddNodes(hc)
 	     * Construct the full hierarchical name of this node.
 	     * The path down to this point is given by hc->hc_hierName,
 	     * to which nn->efnn_hier is "appended".  Exception: nodes
-	     * marked with EF_FETTERM (fet substrate nodes used before
+	     * marked with EF_DEVTERM (fet substrate nodes used before
 	     * declared, so intended to refer to default global names)
 	     * are added as global nodes.
 	     */
-	    if (node->efnode_flags & EF_FETTERM) hierName = nn->efnn_hier;
+	    if (node->efnode_flags & EF_DEVTERM) hierName = nn->efnn_hier;
 	    else hierName = EFHNConcat(hc->hc_hierName, nn->efnn_hier);
 	    he = HashFind(&efNodeHashTable, (char *) hierName);
 
 	    /*
 	     * The name should only have been in the hash table already
-	     * if the node was marked with EF_FETTERM as described above.
+	     * if the node was marked with EF_DEVTERM as described above.
 	     */
 	    if (oldname = (EFNodeName *) HashGetValue(he))
 	    {
@@ -357,7 +362,7 @@ efAddConns(hc)
     register Connection *conn;
 
     if (efWatchNodes)
-	printf("Processing %s (%s)\n",
+	TxPrintf("Processing %s (%s)\n",
 		EFHNToStr(hc->hc_hierName),
 		hc->hc_use->use_def->def_name);
 
@@ -477,7 +482,7 @@ efFlatGlob()
      * hash table globalTable to keep track of how many times each global
      * name has been seen.  Each global name should be seen exactly once.
      * The only exceptions are fet substrate nodes (nodes marked with the
-     * flag EF_FETTERM), which automatically merge with other global nodes
+     * flag EF_DEVTERM), which automatically merge with other global nodes
      * with the same name, since they're only implicitly connected anyway.
      */
     for (nodeFlat = (EFNode *) efNodeList.efnode_next;
@@ -516,7 +521,7 @@ efFlatGlob()
 	else if (nameGlob->efnn_node != nodeFlat)
 	{
 	    /*
-	     * If either node is a fet substrate node (marked with EF_FETTERM)
+	     * If either node is a fet substrate node (marked with EF_DEVTERM)
 	     * it's OK to merge them; otherwise, it's an error, but we still
 	     * merge the nodes.  When merging, we blow away nodeGlob and
 	     * absorb it into nodeFlat for simplicity in control of the main
@@ -524,8 +529,8 @@ efFlatGlob()
 	     * for nodeGlob, we have to update its node backpointer explicitly.
 	     */
 	    nodeGlob = nameGlob->efnn_node;
-	    if ((nodeGlob->efnode_flags & EF_FETTERM) == 0
-		    && (nodeFlat->efnode_flags & EF_FETTERM) == 0)
+	    if ((nodeGlob->efnode_flags & EF_DEVTERM) == 0
+		    && (nodeFlat->efnode_flags & EF_DEVTERM) == 0)
 	    {
 		efFlatGlobError(nameGlob, nameFlat);
 	    }
@@ -578,26 +583,26 @@ efFlatGlobError(nameGlob, nameFlat)
     register EFNodeName *nn;
     int count;
 
-    printf("*** Global name %s not fully connected:\n",
+    TxPrintf("*** Global name %s not fully connected:\n",
 			nameGlob->efnn_hier->hn_name);
-    printf("One portion contains the names:\n");
+    TxPrintf("One portion contains the names:\n");
     for (count = 0, nn = nodeGlob->efnode_name;
 	    count < 10 && nn;
 	    count++, nn = nn->efnn_next)
     {
-	printf("    %s\n", EFHNToStr(nn->efnn_hier));
+	TxPrintf("    %s\n", EFHNToStr(nn->efnn_hier));
     }
-    if (nn) printf("    .... (no more names will be printed)\n");
-    printf("The other portion contains the names:\n");
+    if (nn) TxPrintf("    .... (no more names will be printed)\n");
+    TxPrintf("The other portion contains the names:\n");
     for (count = 0, nn = nodeFlat->efnode_name;
 	    count < 10 && nn;
 	    count++, nn = nn->efnn_next)
     {
-	printf("    %s\n", EFHNToStr(nn->efnn_hier));
+	TxPrintf("    %s\n", EFHNToStr(nn->efnn_hier));
     }
-    if (nn) printf("    .... (no more names will be printed)\n");
-    printf("I'm merging the two pieces into a single node, but you\n");
-    printf("should be sure eventually to connect them in the layout.\n\n");
+    if (nn) TxPrintf("    .... (no more names will be printed)\n");
+    TxPrintf("I'm merging the two pieces into a single node, but you\n");
+    TxPrintf("should be sure eventually to connect them in the layout.\n\n");
 }
 
 int
