@@ -21,7 +21,7 @@
 
 #ifndef	_MAGIC
 int err0 = Need_to_include_misc/magic_header;
-#endif	_MAGIC
+#endif	/* _MAGIC */
 
 
 /*
@@ -163,7 +163,7 @@ typedef struct def
     Connection	*def_conns;	/* Hierarchical connections/adjustments */
     Connection	*def_caps;	/* Two-terminal capacitors */
     Connection	*def_resistors;	/* Two-terminal resistors */
-    Fet		*def_fets;	/* Transistors */
+    Dev		*def_devs;	/* Devices */
     Kill	*def_kills;	/* Used to modify hierarchical structure
 				 * using information present only in the
 				 * parent, e.g, to kill an old node and
@@ -174,8 +174,7 @@ typedef struct def
 } Def;
 
 #define	DEF_AVAILABLE	0x01	/* This def has been read in */
-#define DEF_SUBCIRCUIT  0x02	/* This def is a subcircuit, so nodes	*/
-				/* should be interpreted as ports	*/
+#define	DEF_SUBCIRCUIT	0x02	/* This def defines subcircuit ports */
 
 /*
  * Every Def has a NULL-terminated list of uses that correspond
