@@ -18,10 +18,14 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFname.c,v 1.4 2001/07/06 17:58:39 rajit Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
+
+#ifdef MAGIC_WRAPPER 
+#include "magic/tclmagic.h"
+#endif
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
@@ -31,6 +35,12 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFname.c,v 1.4 200
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
 
+#ifdef MAGIC_WRAPPER 
+#define PrintErr TxError
+#else
+#define PrintErr printf
+#endif
+
 /*
  * Hash table containing all flattened node names.
  * The keys in this table are HierNames, processed by the
@@ -53,6 +63,9 @@ HashTable efHNUseHashTable;
  *
  * Determine whether a HierName is of the format of a global name,
  * i.e, it ends in a '!'.
+ * 
+ * The Tcl version of magic further refines this to include names
+ * which are defined in the global Tcl variable space.
  *
  * Results:
  *	TRUE if the name is a global.
@@ -67,6 +80,11 @@ bool
 EFHNIsGlob(hierName)
     register HierName *hierName;
 {
+#ifdef MAGIC_WRAPPER
+    char *retstr;
+    retstr = Tcl_GetVar(magicinterp, hierName->hn_name, TCL_GLOBAL_ONLY);
+    if (retstr != NULL) return TRUE;
+#endif
     return hierName->hn_name[strlen(hierName->hn_name) - 1] == '!';
 }
 
@@ -77,6 +95,9 @@ EFHNIsGlob(hierName)
  *
  * Determine whether a HierName is the same as the global signal GND.
  *
+ * The Tcl version of magic expands this to include names which are
+ * equal to the global Tcl variable $GND, if it is set.
+ *
  * Results:
  *	TRUE if the name is GND, false if not.
  *
@@ -90,8 +111,19 @@ bool
 EFHNIsGND(hierName)
     register HierName *hierName;
 {
-    return (hierName->hn_parent == (HierName *) NULL
-	    && strcmp(hierName->hn_name, "GND!") == 0);
+#ifdef MAGIC_WRAPPER
+    char *retstr;
+#endif
+
+    if (hierName->hn_parent != (HierName *) NULL) return FALSE;
+
+#ifdef MAGIC_WRAPPER
+    retstr = Tcl_GetVar(magicinterp, "GND", TCL_GLOBAL_ONLY);
+    if (retstr != NULL)
+	if (!strcmp(hierName->hn_name, retstr)) return TRUE;
+#endif
+
+    return (strcmp(hierName->hn_name, "GND!") == 0);
 }
 
 /*
@@ -319,7 +351,7 @@ EFHNLook(prefix, suffixStr, errorStr)
     if (he == NULL || HashGetValue(he) == NULL)
     {
 	if (errorStr)
-	    printf("%s: no such node %s\n", errorStr, EFHNToStr(hierName));
+	    PrintErr("%s: no such node %s\n", errorStr, EFHNToStr(hierName));
 	he = NULL;
     }
 
@@ -377,7 +409,7 @@ EFHNConcatLook(prefix, suffix, errorStr)
     he = HashLookOnly(&efNodeHashTable, (char *) suffix);
     if (he == NULL || HashGetValue(he) == NULL)
     {
-	printf("%s: no such node %s\n", errorStr, EFHNToStr(suffix));
+	PrintErr("%s: no such node %s\n", errorStr, EFHNToStr(suffix));
 	he = (HashEntry *) NULL;
     }
 
