@@ -19,30 +19,37 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFread.c,v 1.2 2001/01/12 22:12:35 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
 #include <stdlib.h>
-#include <varargs.h>
 
 #include "magic/tclmagic.h"
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/hash.h"
 #include "utils/utils.h"
+#include "tiles/tile.h"
+#include "database/database.h"
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
+#include "extract/extract.h"
 #include "graphics/graphics.h"	/* for  GrVfprintf */
 #include "misc/paths.h"
 
+#ifndef MAGIC_WRAPPER
+/* This must match the definition for extDevTable in extract/ExtBasic.c */
+char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap", "subckt", NULL};
+#endif
+
 /*
  * The following table describes the kinds of lines
  * that may be read in a .ext file.
  */
 typedef enum
 {
-    ADJUST, ATTR, CAP, DIST, EQUIV, FET, KILLNODE, MERGE, NODE, PORT,
+    ADJUST, ATTR, CAP, DEVICE, DIST, EQUIV, FET, KILLNODE, MERGE, NODE, PORT,
     RESISTOR, RESISTCLASS, RNODE, SCALE, TECH, TIMESTAMP, USE, VERSION,
     EXT_STYLE
 } Key;
@@ -58,9 +65,10 @@ keyTable[] =
     "adjust",		ADJUST,		4,
     "attr",		ATTR,		8,
     "cap",		CAP,		4,
+    "device",		DEVICE,		13,	/* effectively replaces "fet" */
     "distance",		DIST,		4,
     "equiv",		EQUIV,		3,
-    "fet",		FET,		12,
+    "fet",		FET,		12,	/* for backwards compatibility */
     "killnode",		KILLNODE,	2,
     "merge",		MERGE,		3,
     "node",		NODE,		7,
@@ -95,8 +103,9 @@ static int efReadDef();
  *
  * Main procedure to read a .ext file.  If there is no Def by the
  * name of 'name', allocates a new one.  Calls efReadDef to do the
- * work of reading the def itself.  If 'ignore' is true, then declared
- * subcircuits are ignored beyond the port mappings.
+ * work of reading the def itself.  If 'dosubckt' is true, then port
+ * mappings are kept.  If 'resist' is true, read in the .res.ext file
+ * (from extresist) if it exists, after reading the .ext file.
  *
  * Results:
  *	
@@ -111,9 +120,9 @@ static int efReadDef();
  */
 
 int
-EFReadFile(name, ignore)
+EFReadFile(name, dosubckt, resist)
 char *name; /* Name of def to be read in */
-bool ignore;
+bool dosubckt, resist;
 {
     Def *def;
     int  rc;
@@ -122,7 +131,7 @@ bool ignore;
     if (def == NULL)
 	def = efDefNew(name);
 
-    rc = efReadDef(def, ignore);
+    rc = efReadDef(def, dosubckt, resist);
     if (EFArgTech) EFTech = StrDup((char **) NULL, EFArgTech);
     if (EFScale == 0) EFScale = 1;
 
@@ -150,9 +159,9 @@ bool ignore;
  */
 
 static int
-efReadDef(def, ignore)
+efReadDef(def, dosubckt, resist)
 Def *def;
-bool ignore;
+bool dosubckt, resist;
 {
     int argc, ac, n;
     EFCapValue cap;
@@ -164,6 +173,7 @@ bool ignore;
     Use *use;
     Rect r;
     int rc = TRUE;
+    bool DoResist = resist;
 
     /* Mark def as available */
     def->def_flags |= DEF_AVAILABLE;
@@ -181,6 +191,7 @@ bool ignore;
 	return FALSE;
     }
 
+readfile:
     efReadLineNum = 0;
     while ((argc = efReadLine(line, sizeof line, inf, argv)) >= 0)
     {
@@ -196,15 +207,6 @@ bool ignore;
 	    continue;
 	}
 
-	/* This needs to be separated between SPICE and sim types.	*/
-	/* For SPICE, do the break (use external circuit definition).	*/
-	/* For sim, don't do the break (extract the subcircuit and 	*/
-	/* all of its subcells).					*/
-
-	if (ignore && (def->def_flags & DEF_SUBCIRCUIT) &&
-			(keyTable[n].k_key != PORT))
-	    break;
-
 	switch (keyTable[n].k_key)
 	{
 	    /* scale rscale cscale lscale */
@@ -255,9 +257,50 @@ bool ignore;
 		efBuildEquiv(def, argv[1], argv[2]);
 		break;
 
+	    /* replaces "fet" (below) */
+	    /* device mosfet|bjt|subckt type xlo ylo xhi yhi area perim */
+	    /*		[substrate] GATE T1 T2 ...			*/
+	    /* device res|cap type xlo ylo xhi yhi value GATE T1 T2 ... */
+	    case DEVICE:
+		r.r_xbot = atoi(argv[3]);
+		r.r_ybot = atoi(argv[4]);
+		r.r_xtop = atoi(argv[5]);
+		r.r_ytop = atoi(argv[6]);
+
+		/* Parse device class */
+		for (n = 0; extDevTable[n] != NULL; n++)
+		    if (!strcmp(argv[1], extDevTable[n]))
+			break;
+
+		if (extDevTable[n] == NULL)
+		{
+		    efReadError("Unknown device class\n");
+		    continue;
+		}
+		switch (n)
+		{
+		    case DEV_MOSFET:
+		    case DEV_BJT:
+		    case DEV_SUBCKT:
+			ac = 10;
+			break;
+		    case DEV_RES:
+		    case DEV_CAP:
+			ac = 8;
+			break;
+		}
+		if (((argc - ac) % 3) != 0)
+		{
+		    efReadError("Incomplete terminal description for device\n");
+		    continue;
+		}
+		efBuildDevice(def, (char)n, argv[2], &r, argc - 7, &argv[7]);
+		break;
+
+	    /* for backwards compatibility */
 	    /* fet type xlo ylo xhi yhi area perim substrate GATE T1 T2 ... */
 	    case FET:
-		if ( ((argc-9) % 3) != 0)
+		if (((argc - 9) % 3) != 0)
 		{
 		    efReadError("Incomplete terminal description for fet\n");
 		    continue;
@@ -266,18 +309,17 @@ bool ignore;
 		r.r_ybot = atoi(argv[3]);
 		r.r_xtop = atoi(argv[4]);
 		r.r_ytop = atoi(argv[5]),
-		efBuildFet(def, argv[1], &r,
-		    atoi(argv[6]), atoi(argv[7]), argv[8], argc-9, &argv[9]);
+		efBuildDevice(def, DEV_FET, argv[1], &r, argc - 6, &argv[6]);
 		break;
 
 	    /* merge node1 node2 C a1 p1 a2 p2 ... */
 	    case MERGE:
-		if (argc > 3 && argc - 4 < 2 * efNumResistClasses)
+		if ((argc > 4) && (argc - 4 < 2 * efNumResistClasses))
 		{
-		    efReadError(
-		    "Too few area/perim values: assuming remainder are zero\n");
+		    efReadError("Too few area/perim values: "
+				"assuming remainder are zero\n");
 		}
-		cap = argc > 3 ? atoCap(argv[3]) * cscale : 0;
+		cap = (argc > 3) ? atoCap(argv[3]) * cscale : 0;
 		efBuildConnect(def, argv[1], argv[2], (double) cap, &argv[4], argc - 4);
 		break;
 
@@ -301,9 +343,13 @@ bool ignore;
 
 	    /* port name num xl yl xh yh type */
 	    case PORT:
-		efBuildPortNode(def, argv[1], atoi(argv[3]), atoi(argv[4]),
-			argv[7]);
-		def->def_flags |= DEF_SUBCIRCUIT;
+		if (dosubckt)
+		{
+		    DoResist = FALSE;
+		    def->def_flags |= DEF_SUBCIRCUIT;
+		    efBuildPortNode(def, argv[1], atoi(argv[2]), atoi(argv[3]),
+					atoi(argv[4]), argv[7]);
+		}
 		break;
 
 	    /*
@@ -328,11 +374,10 @@ bool ignore;
 		}
 		else if (efNumResistClasses != argc-1)
 		{
-		    efReadError(
-	    "Number of resistance classes doesn't match:\n");
+		    efReadError("Number of resistance classes doesn't match:\n");
 resistChanged:
-		    efReadError(
-	    "Re-extract the entire tree with the same technology file\n");
+		    efReadError("Re-extract the entire tree with "
+				"the same technology file\n");
 		    efResistChanged = TRUE;
 		    break;
 		}
@@ -353,6 +398,16 @@ resistChanged:
 
 	    /* tech techname */
 	    case TECH:
+#ifdef MAGIC_WRAPPER
+		if (strcmp(argv[1], DBTechName))
+		{
+		    TxError("Error: .ext file has different technology %s\n",
+				argv[1]);
+		    TxError("Load this technology and repeat.\n");
+		    rc = FALSE;
+		    break;
+		}
+#else
 		if (EFTech && EFTech[0])
 		{
 		    if (strcmp(EFTech, argv[1]) != 0)
@@ -363,14 +418,28 @@ resistChanged:
 			break;
 		    }
 		}
+#endif
 		else EFTech = StrDup((char **) NULL, argv[1]);
+
 		if (!EFLibPath[0])	/* Put in a path if there wasn't one */
 		    (void) sprintf(EFLibPath, EXT_PATH, EFTech);
 		break;
 
 	    /* ext_style stylename */
 	    case EXT_STYLE:
-		if (EFStyle && EFStyle[0])
+#ifdef MAGIC_WRAPPER
+		if (ExtCompareStyle(argv[1]) == FALSE)
+		{
+		    TxError("Warning:  .ext file style %s is not known "
+				"in this technology!\n", argv[1]);
+		    if (EFStyle)
+		    {
+			freeMagic(EFStyle);
+			EFStyle = NULL;
+		    }
+		}
+#else
+		if (EFStyle)
 		{
 		    if (strcmp(EFStyle, argv[1]) != 0)
 		    {
@@ -379,7 +448,9 @@ resistChanged:
 			break;
 		    }
 		}
-		else EFStyle = StrDup((char **) NULL, argv[1]);
+#endif
+		else
+		    EFStyle = StrDup(&EFStyle, argv[1]);
 		break;
 
 	    /* version version-number */
@@ -417,10 +488,19 @@ resistChanged:
     }
     (void) fclose(inf);
 
+    /* Is there an "extresist" extract file? */
+    if (DoResist)
+    {
+	DoResist = FALSE;	/* do this only once */
+	inf = PaOpen(name, "r", ".res.ext", EFSearchPath, EFLibPath, &efReadFileName);
+	if (inf != NULL)
+	    goto readfile;
+    }
+
     /* Read in each def that has not yet been read in */
     for (use = def->def_uses; use; use = use->use_next)
 	if ((use->use_def->def_flags & DEF_AVAILABLE) == 0)
-	    if(efReadDef(use->use_def, ignore) != TRUE)
+	    if(efReadDef(use->use_def, dosubckt, resist) != TRUE)
 		rc = FALSE;
 
     return rc;
