@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFvisit.c,v 1.2 2001/01/12 22:12:36 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -31,6 +31,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/extflat/EFvisit.c,v 1.2 20
 #include "utils/utils.h"
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
+#include "extract/extract.h"
 
 /* Root of the tree being flattened */
 extern Def *efFlatRootDef;
@@ -119,15 +120,83 @@ efVisitSubcircuits(hc, ca)
 /*
  * ----------------------------------------------------------------------------
  *
- * EFVisitFets --
+ * EFGetLengthAndWidth --
  *
- * Visit all the fets in the circuit.
+ *	Estimate length and width for a device from area and perimeter values.
+ *
+ * Results: None
+ *
+ * Side Effects: values substituted for length and width
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+EFGetLengthAndWidth(dev, lptr, wptr)
+    Dev *dev;
+    int *lptr;
+    int *wptr;
+{
+    DevTerm *gate, *source, *drain;
+    int area, perim, l, w;
+
+    switch (dev->dev_class)
+    {
+	case DEV_MOSFET:
+	case DEV_BJT:
+	case DEV_SUBCKT:
+	    l = dev->dev_values.dev_lw.dev_length;
+	    w = dev->dev_values.dev_lw.dev_width;
+	    break;
+
+	case DEV_FET:
+	    area = dev->dev_values.dev_ap.dev_area;
+	    perim = dev->dev_values.dev_ap.dev_perim;
+
+	    gate = &dev->dev_terms[0];
+
+	    /*
+	     * L, W, and flat coordinates of a point inside the channel.
+	     * Handle FETs with two terminals (capacitors) separately.
+	     */
+
+	    if (dev->dev_nterm == 2)
+	    {
+		l = (perim - (int) sqrt((float)(perim * perim - 16 * area))) / 4;
+		w = area / l;
+	    }
+	    else
+	    {
+		source = drain = &dev->dev_terms[1];
+		if (dev->dev_nterm >= 3)
+		    drain = &dev->dev_terms[2];
+		l = gate->dterm_perim / 2;
+		w = (source->dterm_perim + drain->dterm_perim) / 2;
+	    }
+	    if (gate->dterm_attrs) efDevFixLW(gate->dterm_attrs, &l, &w);
+	    break;
+
+	default:
+	    l = w = 0;	/* RES and CAP devices shouldn't use l & w. */
+	    break;
+    }
+
+    *lptr = l;
+    *wptr = w;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * EFVisitDevs --
+ *
+ * Visit all the devs in the circuit.
  * Must be called after EFFlatBuild().
- * For each fet in the circuit, call the user-supplied procedure
- * (*fetProc)(), which should be of the following form:
+ * For each dev in the circuit, call the user-supplied procedure
+ * (*devProc)(), which should be of the following form:
  *
- *	(*fetProc)(fet, hierName, trans, l, w, cdata)
- *	    Fet *fet;
+ *	(*devProc)(dev, hierName, trans, l, w, cdata)
+ *	    Dev *dev;
  *	    HierName *hierName;
  *	    Transform *trans;
  *	    ClientData cdata;
@@ -137,7 +206,7 @@ efVisitSubcircuits(hc, ca)
  * The procedure should return 0 normally, or 1 to abort the
  * search.
  *
- * We ensure that no fets connected to killed nodes are passed
+ * We ensure that no devs connected to killed nodes are passed
  * to this procedure.
  *
  * Results:
@@ -145,25 +214,25 @@ efVisitSubcircuits(hc, ca)
  *	was aborted.
  *
  * Side effects:
- *	Whatever (*fetProc)() does.
+ *	Whatever (*devProc)() does.
  *
  * ----------------------------------------------------------------------------
  */
 
-EFVisitFets(fetProc, cdata)
-    int (*fetProc)();
+EFVisitDevs(devProc, cdata)
+    int (*devProc)();
     ClientData cdata;
 {
     CallArg ca;
 
-    ca.ca_proc = fetProc;
+    ca.ca_proc = devProc;
     ca.ca_cdata = cdata;
-    return efVisitFets(&efFlatContext, (ClientData) &ca);
+    return efVisitDevs(&efFlatContext, (ClientData) &ca);
 }
 
 /*
- * Procedure to visit recursively all fets in the design.
- * Does all the work of EFVisitFets() above.
+ * Procedure to visit recursively all devs in the design.
+ * Does all the work of EFVisitDevs() above.
  *
  * Results:
  *	Returns 0 to keep efHierSrUses going.
@@ -173,18 +242,17 @@ EFVisitFets(fetProc, cdata)
  */
 
 int
-efVisitFets(hc, ca)
+efVisitDevs(hc, ca)
     register HierContext *hc;
     register CallArg *ca;
 {
     Def *def = hc->hc_use->use_def;
-    FetTerm *gate, *source, *drain;
-    register Fet *fet;
-    int scale, l, w;
+    register Dev *dev;
+    int scale;
     Transform t;
 
-    /* Recursively visit fets in our children first */
-    if (efHierSrUses(hc, efVisitFets, (ClientData) ca))
+    /* Recursively visit devs in our children first */
+    if (efHierSrUses(hc, efVisitDevs, (ClientData) ca))
 	return 1;
 
     /*
@@ -203,34 +271,13 @@ efVisitFets(hc, ca)
 	t.t_d *= scale; t.t_e *= scale; t.t_f *= scale;
     }
 
-    /* Visit our own fets */
-    for (fet = def->def_fets; fet; fet = fet->fet_next)
+    /* Visit our own devices */
+    for (dev = def->def_devs; dev; dev = dev->dev_next)
     {
-	if (efFetKilled(fet, hc->hc_hierName))
+	if (efDevKilled(dev, hc->hc_hierName))
 	    continue;
 
-	gate = &fet->fet_terms[0];
-	source = drain = &fet->fet_terms[1];
-	if (fet->fet_nterm >= 3)
-	    drain = &fet->fet_terms[2];
-
-	/*
-	 * L, W, and flat coordinates of a point inside the channel.
-	 * Handle FETs with two terminals (capacitors) separately.
-	 */
-	if (fet->fet_nterm == 2)
-	{
-	    l = (fet->fet_perim - (int) sqrt((float)
-		(fet->fet_perim * fet->fet_perim - 16 * fet->fet_area))) / 4;
-	    w = fet->fet_area / l;
-	}
-	else
-	{
-	    l = gate->fterm_perim / 2;
-	    w = (source->fterm_perim + drain->fterm_perim)/2;
-	}
-	if (gate->fterm_attrs) efFetFixLW(gate->fterm_attrs, &l, &w);
-	if ((*ca->ca_proc)(fet, hc->hc_hierName, &t, l, w, ca->ca_cdata))
+	if ((*ca->ca_proc)(dev, hc->hc_hierName, &t, ca->ca_cdata))
 	    return 1;
     }
 
@@ -240,14 +287,14 @@ efVisitFets(hc, ca)
 /*
  * ----------------------------------------------------------------------------
  *
- * efFetKilled --
+ * efDevKilled --
  *
- * Check all of the nodes to which the fet 'fet' is connected (its
+ * Check all of the nodes to which the dev 'dev' is connected (its
  * hierarchical prefix is hc->hc_hierName).  If any of these nodes
- * have been killed, then the fet is also killed.
+ * have been killed, then the dev is also killed.
  *
  * Results:
- *	TRUE if the fet is connected to a killed node, FALSE if it's ok.
+ *	TRUE if the dev is connected to a killed node, FALSE if it's ok.
  *
  * Side effects:
  *	None.
@@ -256,8 +303,8 @@ efVisitFets(hc, ca)
  */
 
 bool
-efFetKilled(fet, prefix)
-    Fet *fet;
+efDevKilled(dev, prefix)
+    Dev *dev;
     HierName *prefix;
 {
     HierName *suffix;
@@ -265,9 +312,9 @@ efFetKilled(fet, prefix)
     EFNodeName *nn;
     int n;
 
-    for (n = 0; n < fet->fet_nterm; n++)
+    for (n = 0; n < dev->dev_nterm; n++)
     {
-	suffix = fet->fet_terms[n].fterm_node->efnode_name->efnn_hier;
+	suffix = dev->dev_terms[n].dterm_node->efnode_name->efnn_hier;
 	he = EFHNConcatLook(prefix, suffix, "kill");
 	if (he  && (nn = (EFNodeName *) HashGetValue(he))
 		&& (nn->efnn_node->efnode_flags & EF_KILLED))
@@ -280,10 +327,10 @@ efFetKilled(fet, prefix)
 /*
  * ----------------------------------------------------------------------------
  *
- * efFetFixLW --
+ * efDevFixLW --
  *
- * Called for any fets that have gate attributes; these attributes may
- * specify the L and W of the fet explicitly.  The attributes will be
+ * Called for any devs that have gate attributes; these attributes may
+ * specify the L and W of the dev explicitly.  The attributes will be
  * of the form ext:l=value or ext:w=value, where value is either numerical
  * or symbolic; if symbolic the symbol must have been defined via efSymAdd().
  * If the value is symbolic but wasn't defined by efSymAdd(), it's ignored.
@@ -300,7 +347,7 @@ efFetKilled(fet, prefix)
  */
 
 Void
-efFetFixLW(attrs, pL, pW)
+efDevFixLW(attrs, pL, pW)
     char *attrs;
     int *pL, *pW;
 {
