@@ -19,10 +19,16 @@
 
 #ifndef lint
 static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <math.h>
+
+#ifdef MAGIC_WRAPPER
+#include "magic/tclmagic.h"
+#endif
 #include "misc/magic.h"
 #include "utils/geometry.h"
 #include "utils/geofast.h"
@@ -40,6 +46,15 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 #include "misc/styles.h"
 #include "utils/stack.h"
 
+/* These must be in the order of "known devices" in extract.h		   */
+/* Note: "fet" refers to the original fet type; "mosfet" refers to the new */
+/* type.  The main difference is that "fet" records area/perimeter while   */
+/* "mosfet" records length/width.					   */
+/* Also: Note that this table is repeated in extflat/EFread.c when	   */
+/* ext2spice/ext2sim are compiled as separate programs (i.e., non-Tcl)	   */
+
+char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap", "subckt", NULL};
+
 /* --------------------- Data local to this file ---------------------- */
 
     /*
@@ -76,19 +91,38 @@ struct transRec
     int		 tr_termlen[MAXSD];	/* Length of each diff terminal edge,
 					 * used for computing L/W for the fet.
 					 */
+    Point	 tr_termvector[MAXSD];	/* Perimeter traversal vector, used to
+					 * find and calculate correct parameters
+					 * for annular (ring) devices and other
+					 * non-rectangular geometries.
+					 */
     int		 tr_perim;		/* Total perimeter */
     TermTilePos  tr_termpos[MAXSD];	/* lowest tile connecting to term */
 } extTransRec;
 
+typedef struct LB1
+{
+    Rect r;	/* Boundary segment */
+    int dir;	/* Direction of travel */
+    struct LB1 *b_next;
+} LinkedBoundary;
+
+LinkedBoundary **extSpecialBounds;		/* Linked Boundary List */
+
 #define	EDGENULL(r)	((r)->r_xbot > (r)->r_xtop || (r)->r_ybot > (r)->r_ytop)
 
 /* Forward declarations */
-int extOutputNodes();
+void extOutputNodes();
 int extTransTileFunc();
 int extTransPerimFunc();
 NodeRegion *extTransFindSubsNode();
 int extTransFindSubsFunc();
+
+int extAnnularTileFunc();
+int extAnnularPerimFunc();
+
 
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -177,8 +211,7 @@ extBasic(def, outFile)
 
     /* Output each node, along with its resistance and capacitance to GND */
     if (!SigInterruptPending)
-	if (extOutputNodes(nodeList, outFile))
-	    def->cd_flags |= CDSUBCIRCUIT;
+	extOutputNodes(nodeList, outFile);
 
     /* Output coupling capacitances */
     if (!SigInterruptPending && (ExtOptions&EXT_DOCOUPLING))
@@ -291,12 +324,12 @@ extSetResist(reg)
  * ----------------------------------------------------------------------------
  */
 
-int
+void
 extOutputNodes(nodeList, outFile)
     NodeRegion *nodeList;	/* Nodes */
     FILE *outFile;		/* Output file */
 {
-    ResValue rround = ExtCurStyle->exts_resistScale/2;
+    ResValue rround = ExtCurStyle->exts_resistScale / 2;
     CapValue finC;
     int intR;
     register NodeRegion *reg;
@@ -305,19 +338,17 @@ extOutputNodes(nodeList, outFile)
     register int n;
     Label *lab;
     char *text;
-    int portorder = 0;
 
     /* If this node is a subcircuit port, it gets special treatment.	*/
     /* There may be multiple ports per node.			 	*/
 
     for (reg = nodeList; reg && !SigInterruptPending; reg = reg->nreg_next)
-    {
 	for (ll = reg->nreg_labels; ll; ll = ll->ll_next)
 	    if (ll->ll_attr == LL_PORTATTR)
 	    {
 		(void) fprintf(outFile, "port \"%s\" %d %d %d %d %d %s\n",
 			ll->ll_label->lab_text,
-			portorder++,
+			ll->ll_label->lab_flags & PORT_NUM_MASK,
 			ll->ll_label->lab_rect.r_xbot,
 			ll->ll_label->lab_rect.r_ybot,
 			ll->ll_label->lab_rect.r_xtop,
@@ -337,7 +368,6 @@ extOutputNodes(nodeList, outFile)
 		    reg->nreg_type = ll->ll_label->lab_type;
 		}
 	    }
-    }
 
     for (reg = nodeList; reg && !SigInterruptPending; reg = reg->nreg_next)
     {
@@ -386,7 +416,6 @@ extOutputNodes(nodeList, outFile)
 		break;
 	    }
     }
-    return (portorder > 0) ? 1 : 0;
 }
 
 /*
@@ -590,6 +619,206 @@ Void ExtSortTerminals(tran, ll)
      }
      while( changed );
 }
+
+/*----------------------------------------------------------------------*/
+/*									*/
+/* extComputeEffectiveLW --						*/
+/*									*/
+/* Determine effective length and width of an annular (or otherwise	*/
+/* non-rectangular) transistor structure, based on the boundary vectors	*/
+/* stored in extSpecialBounds.						*/
+/*									*/
+/* Results:  None							*/
+/* Side Effects:  puts effective length and width into the pointers	*/
+/*		passed as arguments.					*/
+/*----------------------------------------------------------------------*/
+
+void
+extComputeEffectiveLW(rlengthptr, rwidthptr, numregions)
+   int *rlengthptr, *rwidthptr;
+   int numregions;
+{
+    int i;
+    LinkedBoundary *lb, *lb2;
+    int oppdir, length, loclength, testlen, width, testwid;
+    int segp, segn, segc, sege;
+
+    /* For each segment in the first list, find the closest */
+    /* segment in the other list which lies on the opposite */
+    /* side of the gate area.  Calculate the length, and    */
+    /* check for overlap, treating the length as a corner   */
+    /* extension.					    */
+
+    /* fprintf(stderr, "Annular transistor detailed L,W computaton:\n"); */
+
+    width = 0;
+    length = 0;
+    for (lb = extSpecialBounds[0]; lb != NULL; lb = lb->b_next)
+    {
+	loclength = INFINITY;
+	oppdir = (lb->dir + 2) % 4;
+
+	/* First pass: Find the distance of the closest segment within	*/
+	/* the range of	its corner extension.  We do two passes because */
+	/* there may be more than one segment at this distance.		*/
+
+        for (i = 1; i < numregions; i++)
+	{
+	    for (lb2 = extSpecialBounds[i]; lb2 != NULL; lb2 = lb2->b_next)
+	    {
+		if (lb2->dir == oppdir)
+		{
+		    switch (lb->dir)
+		    {
+			case BD_LEFT:
+			    if (lb2->r.r_xbot > lb->r.r_xbot)
+			    {
+				testlen = lb2->r.r_xbot - lb->r.r_xbot;
+				if (lb2->r.r_ybot < lb->r.r_ytop + testlen ||
+				    	lb2->r.r_ytop > lb->r.r_ybot - testlen)
+				    if (testlen < loclength) loclength = testlen;
+			    }
+			    break;
+			case BD_RIGHT:
+			    if (lb2->r.r_xtop < lb->r.r_xtop)
+			    {
+				testlen = lb->r.r_xtop - lb2->r.r_xtop;
+				if (lb2->r.r_ybot < lb->r.r_ytop + testlen ||
+				    	lb2->r.r_ytop > lb->r.r_ybot - testlen)
+				    if (testlen < loclength) loclength = testlen;
+			    }
+			    break;
+			case BD_TOP:
+			    if (lb2->r.r_ytop < lb->r.r_ytop)
+			    {
+				testlen = lb->r.r_ytop - lb2->r.r_ytop;
+				if (lb2->r.r_xbot < lb->r.r_xtop + testlen ||
+				    	lb2->r.r_xtop > lb->r.r_xbot - testlen)
+				    if (testlen < loclength) loclength = testlen;
+			    }
+			    break;
+			case BD_BOTTOM:
+			    if (lb2->r.r_ybot > lb->r.r_ybot)
+			    {
+				testlen = lb2->r.r_ybot - lb->r.r_ybot;
+				if (lb2->r.r_xbot < lb->r.r_xtop + testlen ||
+				    	lb2->r.r_xtop > lb->r.r_xbot - testlen)
+				    if (testlen < loclength) loclength = testlen;
+			    }
+			    break;
+		    }
+		}
+	    }
+	}
+
+	/* This segment should not be considered current-carrying; it	*/
+	/* only adds to the gate capacitance.  Should we output the	*/
+	/* extra capacitance somewhere?					*/
+
+	if (loclength == INFINITY) continue;
+
+	/* Note that the L/W calculation ignores the possibility that a	*/
+	/* transistor may have multiple lengths.  Such cases should	*/
+	/* either 1) scale the width to one of the lengths, or 2) out-	*/
+	/* put a separate transistor record for each length.		*/
+
+	if (length == 0)
+	    length = loclength;		/* Default length */
+
+	else if ((length != 0) && (length != loclength))
+	{
+	    TxError("Device has multiple lengths:  scaling"
+			" all widths to length %d", length);
+	}
+
+	/* fprintf(stderr, "   segment length = %d\n", loclength); */
+
+	/* Second pass:  All segments at "length" distance add to the	*/
+	/* length and width calculation.  Sides opposite and corner	*/
+	/* extensions are treated separately.  Areas outside the corner	*/
+	/* extension are ignored.					*/
+
+        for (i = 1; i < numregions; i++)
+	{
+	    for (lb2 = extSpecialBounds[i]; lb2 != NULL; lb2 = lb2->b_next)
+	    {
+		if (lb2->dir == oppdir)
+		{
+		    if (((lb->dir == BD_LEFT) &&
+			(lb2->r.r_xbot - lb->r.r_xbot == loclength)) ||
+			((lb->dir == BD_RIGHT) &&
+			(lb->r.r_xtop - lb2->r.r_xtop == loclength)))
+		    {
+			/* opposite */
+			segp = MIN(lb2->r.r_ytop, lb->r.r_ytop);
+			segn = MAX(lb2->r.r_ybot, lb->r.r_ybot);
+			testwid = segp - segn;
+			if (testwid > 0) width += testwid * 2;
+			if (testwid <= -loclength) continue;
+
+			/* corner extend top */
+			segc = MAX(lb2->r.r_ytop, lb->r.r_ytop);
+			sege = MAX(segp, segn);
+			testwid = segc - sege;
+			if (testwid > loclength) testwid = loclength;
+			if (testwid > 0) width += testwid;
+			
+			/* corner extend bottom */
+			segc = MIN(lb2->r.r_ybot, lb->r.r_ybot);
+			sege = MIN(segp, segn);
+			testwid = sege - segc;
+			if (testwid > loclength) testwid = loclength;
+			if (testwid > 0) width += testwid;
+		    }
+		    else if (((lb->dir == BD_TOP) &&
+			(lb->r.r_ytop - lb2->r.r_ytop == loclength)) ||
+			((lb->dir == BD_BOTTOM) &&
+			(lb2->r.r_ybot - lb->r.r_ybot == loclength)))
+		    {
+			/* opposite */
+			segp = MIN(lb2->r.r_xtop, lb->r.r_xtop);
+			segn = MAX(lb2->r.r_xbot, lb->r.r_xbot);
+			testwid = segp - segn;
+			if (testwid > 0) width += testwid * 2;
+			if (testwid <= -loclength) continue;
+
+			/* corner extend right */
+			segc = MAX(lb2->r.r_xtop, lb->r.r_xtop);
+			sege = MAX(segp, segn);
+			testwid = segc - sege;
+			if (testwid > loclength) testwid = loclength;
+			if (testwid > 0) width += testwid;
+			
+			/* corner extend left */
+			segc = MIN(lb2->r.r_xbot, lb->r.r_xbot);
+			sege = MIN(segp, segn);
+			testwid = sege - segc;
+			if (testwid > loclength) testwid = loclength;
+			if (testwid > 0) width += testwid;
+		    }
+		}
+	    }
+	}
+	/* if (width > 0)
+	    fprintf(stderr, "   segment width = %d\n", width); */
+
+	/* Width scaling for transistor sections with different lengths */
+	if (loclength != length)
+	{
+	    width *= length;
+	    width /= loclength;
+	}
+    }
+    if ((length > 0) && (width > 0))
+    {
+	*rlengthptr = length;
+	*rwidthptr = (width >> 1);
+
+	/* fprintf(stderr, "total L = %d, W = %d\n", length, width); */
+	/* fflush(stderr); */
+    }
+}
+	
 
 /*
  * ----------------------------------------------------------------------------
@@ -599,7 +828,7 @@ Void ExtSortTerminals(tran, ll)
  * For each TransRegion in the supplied list, corresponding to a single
  * transistor in the layout, compute and output:
  *	- Its type
- *	- Its area and perimeter
+ *	- Its area and perimeter OR length and width OR capacitance OR resistance
  *	- Its substrate node
  *	- For each of the gate, and the various diff terminals (eg,
  *	  source, drain):
@@ -634,7 +863,10 @@ extOutputTrans(def, transList, outFile)
     FindRegion arg;
     LabelList *ll;
     TileType t;
-    int nsd;
+    int nsd, length, width, n, i;
+    double dres, dcap;
+    char mesg[256];
+    bool isAnnular;
 
     for (reg = transList; reg && !SigInterruptPending; reg = reg->treg_next)
     {
@@ -678,10 +910,9 @@ extOutputTrans(def, transList, outFile)
 	nsd = ExtCurStyle->exts_transSDCount[t];
 	if (extTransRec.tr_nterm < nsd)
 	{
-	    char mesg[256];
 	    int missing = nsd - extTransRec.tr_nterm;
 
-	    (void) sprintf(mesg, "fet missing %d terminal%s", missing,
+	    (void) sprintf(mesg, "device missing %d terminal%s", missing,
 					missing == 1 ? "" : "s");
 	    if (extTransRec.tr_nterm > 0)
 	    {
@@ -699,7 +930,8 @@ extOutputTrans(def, transList, outFile)
 	}
 	else if (extTransRec.tr_nterm > nsd)
 	{
-	     
+	    /* It is not an error condition to have more terminals */
+	    /* than the minimum.				   */
 	}
 
 	/*
@@ -716,16 +948,161 @@ extOutputTrans(def, transList, outFile)
 	{
 	    subsName = extNodeName(subsNode);
 	}
-	(void) fprintf(outFile, "fet %s %d %d %d %d %d %d \"%s\"",
-		    ExtCurStyle->exts_transName[t],
-		    reg->treg_ll.p_x, reg->treg_ll.p_y, 
-		    reg->treg_ll.p_x + 1, reg->treg_ll.p_y + 1,
-		    reg->treg_area, extTransRec.tr_perim, subsName);
+#ifdef MAGIC_WRAPPER
+	/* If subsName is a Tcl variable (begins with "$"), make the	*/
+	/* variable substitution, if one exists.			*/
+	else if (subsName && subsName[0] == '$')
+	{
+	    char *varsub = Tcl_GetVar(magicinterp, &subsName[1], TCL_GLOBAL_ONLY);
+	    if (varsub != NULL) subsName = varsub;
+	}
+#endif
+
+	/* Original-style FET record backward compatibility */
+	if (ExtCurStyle->exts_deviceClass[t] != DEV_FET)
+	    fprintf(outFile, "device ");
+
+	fprintf(outFile, "%s %s %d %d %d %d",
+		extDevTable[ExtCurStyle->exts_deviceClass[t]],
+		ExtCurStyle->exts_transName[t],
+		reg->treg_ll.p_x, reg->treg_ll.p_y, 
+		reg->treg_ll.p_x + 1, reg->treg_ll.p_y + 1);
+
+	/* NOTE:  The following code makes unreasonable simplifying	*/
+	/* assumptions about how to calculate device length and	width.	*/
+	/* However, it is the same as was always used by ext2sim and	*/
+	/* ext2spice.  By putting it here, where all the tile		*/
+	/* information exists, it is at least theoretically possible to	*/
+	/* write better routines that can deal with bends in resistors	*/
+	/* and transistors, annular devices, multiple-drain devices,	*/
+	/* etc., etc.							*/
+	/*				Tim, 2/20/03			*/
+
+	switch (ExtCurStyle->exts_deviceClass[t])
+	{
+	    case DEV_FET:	/* old style, perimeter & area */
+		(void) fprintf(outFile, " %d %d \"%s\"",
+		    reg->treg_area, extTransRec.tr_perim, 
+				(subsName == NULL) ? "None" : subsName);
+		break;
+
+	    case DEV_MOSFET:	/* new style, length & width */
+	    case DEV_BJT:
+	    case DEV_SUBCKT:
+		length = extTransRec.tr_gatelen / 2;	/* (default) */
+		width = 0;
+		isAnnular = FALSE;
+
+		/* Note that width is accumulated on one tr_termlen	*/
+		/* record when nodes are merged, so proper behavior	*/
+		/* for transistors w/connected S-D is to count over	*/
+		/* non-NULL termnodes, not non-zero termlens.		*/
+
+		for (n = 0; extTransRec.tr_termnode[n] != NULL; n++)
+		{
+		    width += extTransRec.tr_termlen[n];
+		    if (extTransRec.tr_termvector[n].p_x == 0 &&
+				extTransRec.tr_termvector[n].p_y == 0)
+			isAnnular = TRUE;
+		}
+		if (n) width /= n;
+
+		/*------------------------------------------------------*/
+		/* Note that the tr_termvector says a lot about the	*/
+		/* device geometry.  If the sum of x and y for any	*/
+		/* vector is 0, then the terminal is enclosed (annular	*/
+		/* device).  If the sum of x and y for all vectors is	*/
+		/* zero, then we have a normal rectangular device.  But	*/
+		/* if the sum of all x and y is nonzero, then the	*/
+		/* device length changes along the device (including	*/
+		/* bends).  This is a trigger to do a more extensive	*/
+		/* boundary search to find the exact dimensions of the	*/
+		/* device.						*/
+		/*------------------------------------------------------*/
+
+		if (isAnnular)
+		{
+		    LinkedBoundary *lb;
+
+		    extSpecialBounds = (LinkedBoundary **)mallocMagic(n *
+				sizeof(LinkedBoundary *));
+
+		    for (i = 0; i < n; i++) extSpecialBounds[i] = NULL;
+
+		    /* Mark with reg and process each perimeter segment */
+
+		    arg.fra_uninit = (ClientData) extTransRec.tr_gatenode;
+		    arg.fra_region = (Region *) reg;
+		    arg.fra_each = extAnnularTileFunc;
+		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
+
+		    extComputeEffectiveLW(&length, &width, n);
+
+		    /* Free the lists */
+
+		    for (i = 0; i < n; i++)
+			for (lb = extSpecialBounds[i]; lb != NULL; lb = lb->b_next)
+			    FREE((char *)lb);
+		    FREE((char *)extSpecialBounds);
+
+		    /* Put the region list back the way we found it: */
+		    /* Re-mark with extTransRec.tr_gatenode */
+
+		    arg.fra_uninit = (ClientData) reg;
+		    arg.fra_region = (Region *) extTransRec.tr_gatenode;
+		    arg.fra_each = (int (*)()) NULL;
+		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
+		}
+
+		(void) fprintf(outFile, " %d %d \"%s\"", length, width,
+			(subsName == NULL) ? "None" : subsName);
+		break;
+
+	    case DEV_RES:
+		length = extTransRec.tr_perim;
+		width = 0;
+		for (n = 0; extTransRec.tr_termlen[n] != 0; n++)
+		{
+		    width += extTransRec.tr_termlen[n];
+		    length -= width;
+		}
+		if (width)
+		{
+		    dres = ExtCurStyle->exts_sheetResist[t] * (double)length /
+				(double)width;
+		    if (ExtDoWarn && (n > 2))
+		    {
+			sprintf(mesg, "Resistor has %d terminals: "
+				"extracted L/W will be wrong", n);
+			extTransBad(def, reg->treg_tile, mesg);
+		    }
+		}
+		else {
+		    dres = 0.0;
+		    if (ExtDoWarn)
+			extTransBad(def, reg->treg_tile, "Resistor has zero width");
+		}
+
+		(void) fprintf(outFile, " %g", dres / 1000.0); /* mOhms -> Ohms */
+		break;
+
+	    case DEV_CAP:
+		dcap = (ExtCurStyle->exts_transGateCap[t] * reg->treg_area) +
+			(ExtCurStyle->exts_transSDCap[t] * extTransRec.tr_perim);
+
+		(void) fprintf(outFile, " %g", dcap / 1000.0);  /* aF -> fF */
+		break;
+	}
 
 	/* gate */
 	node = (NodeRegion *) extGetRegion(reg->treg_tile);
 	ll = node->nreg_labels;
-	extTransOutTerminal((LabRegion *) node, ll, LL_GATEATTR,
+
+	/* device resistor does not output the resistor node itself,	*/
+	/* only the end terminals.					*/
+
+	if (ExtCurStyle->exts_deviceClass[t] != DEV_RES)
+	    extTransOutTerminal((LabRegion *) node, ll, LL_GATEATTR,
 			extTransRec.tr_gatelen, outFile);
 
 	ExtSortTerminals(&extTransRec, ll); 
@@ -867,6 +1244,8 @@ extTransPerimFunc(bp)
 	    extTransRec.tr_nterm++;
 	    extTransRec.tr_termnode[thisterm] = diffNode;
 	    extTransRec.tr_termlen[thisterm] = 0;
+	    extTransRec.tr_termvector[thisterm].p_x = 0;
+	    extTransRec.tr_termvector[thisterm].p_y = 0;
 	    extTransRec.tr_termpos[thisterm].pnum = DBPlane(TiGetType(bp->b_outside));
 	    extTransRec.tr_termpos[thisterm].pt = bp->b_outside->ti_ll;
 	}
@@ -893,6 +1272,22 @@ extTransPerimFunc(bp)
 	/* Add the length to this terminal's perimeter */
 	extTransRec.tr_termlen[thisterm] += len;
 
+	/* Update the boundary traversal vector */
+	switch(bp->b_direction) {
+	    case BD_LEFT:
+		extTransRec.tr_termvector[thisterm].p_y += len;
+		break;
+	    case BD_TOP:
+		extTransRec.tr_termvector[thisterm].p_x += len;
+		break;
+	    case BD_RIGHT:
+		extTransRec.tr_termvector[thisterm].p_y -= len;
+		break;
+	    case BD_BOTTOM:
+		extTransRec.tr_termvector[thisterm].p_x -= len;
+		break;
+	}
+
 	/*
 	 * Mark this attribute as belonging to this transistor
 	 * if it is either:
@@ -926,6 +1321,179 @@ extTransPerimFunc(bp)
 
     return (0);
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extAnnularTileFunc --
+ *
+ * Filter function called by ExtFindNeighbors for each tile in a
+ * transistor.  Responsible for doing an extensive boundary
+ * survey to determine the length of the transistor.
+ * This is basically a subset of the code in extTransTileFunc()
+ * but passes a different function to extEnumTilePerim().
+ *
+ * Results:
+ *	Returns 0 always.
+ *
+ * Side effects:
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+extAnnularTileFunc(tile)
+    Tile *tile;
+{
+    TileTypeBitMask mask;
+    TileType loctype;
+
+    /*
+     * Visit each segment of the perimeter of this tile that
+     * that borders on something of a different type.
+     */
+#ifdef NONMANHATTAN
+    if (IsSplit(tile))
+    {
+        loctype = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+    }
+    else
+#endif
+    loctype = TiGetTypeExact(tile);
+
+    mask = ExtCurStyle->exts_transConn[loctype];
+    TTMaskCom(&mask);
+    (void) extEnumTilePerim(tile, mask, extAnnularPerimFunc, (ClientData) NULL);
+    return (0);
+}
+
+/* Detailed boundary survey for unusual transistor geometries (esp. annular) */
+
+int
+extAnnularPerimFunc(bp)
+    Boundary *bp;
+{
+    TileType tinside = TiGetType(bp->b_inside);
+    TileType toutside = TiGetType(bp->b_outside);
+    NodeRegion *diffNode = (NodeRegion *) extGetRegion(bp->b_outside);
+    int thisterm;
+    int extended;
+    LinkedBoundary *newBound, *lb, *lastlb;
+
+    if (TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside], toutside))
+    {
+	/*
+	 * Since we're repeating the search, all terminals should be there.
+	 */
+	for (thisterm = 0; thisterm < extTransRec.tr_nterm; thisterm++)
+	    if (extTransRec.tr_termnode[thisterm] == diffNode)
+		break;
+	if (thisterm >= extTransRec.tr_nterm)
+	{
+	    TxError("Internal Error in Transistor Perimeter Boundary Search!\n");
+	    return 1;
+	}
+
+	/*
+	 * Check the existing segment list to see if this segment
+	 * extends an existing segment.
+	 */
+
+	extended = 0;
+	for (lb = extSpecialBounds[thisterm]; lb != NULL; lb = lb->b_next)
+	{
+	    if (bp->b_direction == lb->dir)
+	    {
+		switch(lb->dir)
+		{
+		    case BD_LEFT:
+		    case BD_RIGHT:
+			if (bp->b_segment.r_xbot == lb->r.r_xbot)
+			{
+			    if (bp->b_segment.r_ybot == lb->r.r_ytop)
+			    {
+				if (extended)
+				    lastlb->r.r_ybot = lb->r.r_ybot;
+				else
+				    lb->r.r_ytop = bp->b_segment.r_ytop;
+				extended++;
+				lastlb = lb;
+			    }
+			    else if (bp->b_segment.r_ytop == lb->r.r_ybot)
+			    {
+				if (extended)
+				    lastlb->r.r_ytop = lb->r.r_ytop;
+				else
+				    lb->r.r_ybot = bp->b_segment.r_ybot;
+				extended++;
+				lastlb = lb;
+			    }
+			}
+			break;
+		    case BD_TOP:
+		    case BD_BOTTOM:
+			if (bp->b_segment.r_ybot == lb->r.r_ybot)
+			{
+			    if (bp->b_segment.r_xbot == lb->r.r_xtop)
+			    {
+				if (extended)
+				    lastlb->r.r_xbot = lb->r.r_xbot;
+				else
+				    lb->r.r_xtop = bp->b_segment.r_xtop;
+				extended++;
+				lastlb = lb;
+			    }
+			    else if (bp->b_segment.r_xtop == lb->r.r_xbot)
+			    {
+				if (extended)
+				    lastlb->r.r_xtop = lb->r.r_xtop;
+				else
+				    lb->r.r_xbot = bp->b_segment.r_xbot;
+				extended++;
+				lastlb = lb;
+			    }
+			}
+			break;
+		}
+	    }
+	    if (extended == 2)
+	    {
+		/* Connected two existing entries---need to remove lastlb, */
+		/* which is now a redundant segment.			   */
+
+		if (lastlb == extSpecialBounds[thisterm])
+		    extSpecialBounds[thisterm] = lastlb->b_next;
+		else
+		{
+		    for (lb = extSpecialBounds[thisterm]; lb != NULL;
+				lb = lb->b_next)
+		    {
+			if (lastlb == lb->b_next)
+			{
+			    lb->b_next = lastlb->b_next;
+			    break;
+			}   
+		    }
+		}
+		FREE((char *)lastlb);
+	
+		/* New segment cannot extend more than two existing segments */
+		break;
+	    }
+	}
+
+	if (!extended)
+	{
+	    newBound = (LinkedBoundary *)mallocMagic(sizeof(LinkedBoundary));
+	    newBound->r = bp->b_segment;
+	    newBound->dir = bp->b_direction;
+	    newBound->b_next = extSpecialBounds[thisterm];
+	    extSpecialBounds[thisterm] = newBound;
+	}
+    }
+    return (0);
+}
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -1162,9 +1730,12 @@ extFindNodes(def, clipArea)
 
     SigDisableInterrupts();
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+    {
+	arg.fra_pNum = pNum;
 	(void) DBSrPaintClient((Tile *) NULL, def->cd_planes[pNum],
 		    &TiPlaneRect, &DBAllButSpaceBits, extUnInit,
 		    extNodeAreaFunc, (ClientData) &arg);
+    }
     SigEnableInterrupts();
 
     /* Compute resistance for last node */
@@ -1183,7 +1754,7 @@ extNodeAreaFunc(tile, arg)
     register TileTypeBitMask *mask, *resMask;
     register NodeRegion *reg;
     register Tile *tp;
-    TileType type, t;
+    TileType type, t, residue, tres;
     NodeRegion *old;
     Rect r;
 
@@ -1238,10 +1809,16 @@ extNodeAreaFunc(tile, arg)
 #endif              
         type = TiGetTypeExact(tile);
 
+	/* Contacts are replaced by their residues when calculating */
+	/* perimeter capacitance.				    */
+
+	residue = (DBIsContact(type)) ?
+		DBPlaneToResidue(type, arg->fra_pNum) : type;
+
+	tilePlaneNum = arg->fra_pNum;
 	mask = &ExtCurStyle->exts_nodeConn[type];
 	resMask = &ExtCurStyle->exts_typesResistChanged[type];
 	resistClass = ExtCurStyle->exts_typeToResistClass[type];
-	tilePlaneNum = DBPlane(type);
 
 	/*
 	 * Make sure the lower-leftmost point in the node is
@@ -1298,9 +1875,10 @@ topside:
             else
 #endif
 	    t = TiGetTypeExact(tp);
+	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
 	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
 		PUSHTILE(tp);
-	    if ((capval = ExtCurStyle->exts_perimCap[type][t]) != (CapValue) 0)
+	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
 	    if (TTMaskHasType(resMask, t) && resistClass != -1)
 		extResistPerim[resistClass] += len;
@@ -1330,9 +1908,10 @@ leftside:
             else
 #endif
 	    t = TiGetTypeExact(tp);
+	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
 	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
 		PUSHTILE(tp);
-	    if ((capval = ExtCurStyle->exts_perimCap[type][t]) != (CapValue) 0)
+	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
 	    if (TTMaskHasType(resMask, t) && resistClass != -1)
 		extResistPerim[resistClass] += len;
@@ -1363,9 +1942,10 @@ bottomside:
             else
 #endif
 	    t = TiGetTypeExact(tp);
+	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
 	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
 		PUSHTILE(tp);
-	    if ((capval = ExtCurStyle->exts_perimCap[type][t]) != (CapValue) 0)
+	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
 	    if (TTMaskHasType(resMask, t) && resistClass != -1)
 		extResistPerim[resistClass] += len;
@@ -1395,9 +1975,10 @@ rightside:
             else
 #endif
 	    t = TiGetTypeExact(tp);
+	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, arg->fra_pNum) : t;
 	    if (TTMaskHasType(mask, t) && tp->ti_client == extUnInit)
 		PUSHTILE(tp);
-	    if ((capval = ExtCurStyle->exts_perimCap[type][t]) != (CapValue) 0)
+	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
 	    if (TTMaskHasType(resMask, t) && resistClass != -1)
 		extResistPerim[resistClass] += len;
@@ -1409,7 +1990,7 @@ donesides:
 	    reg->nreg_cap = (CapValue) 0;
 
 	/* If this is a contact, visit all the other planes */
-	if (TTMaskHasType(&DBImageBits, type))
+	if (DBIsContact(type))
 	{
 	    pMask = DBConnPlanes[type];
 	    for (pNum = tilePlaneNum - 1; pNum <= tilePlaneNum + 1; pNum += 2)
