@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtCouple.c,v 1.3 2001/07/23 17:49:52 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -52,6 +52,34 @@ int extAddOverlap(), extAddCouple();
 int extSideLeft(), extSideRight(), extSideBottom(), extSideTop();
 int extSideOverlap();
 
+/* Structure to pass on to the coupling and sidewall capacitance	*/
+/* routines to include the current cell definition and the current	*/
+/* plane being searched.						*/
+
+typedef struct _ecs {
+    CellDef *def;
+    int	     plane;
+} extCapStruct;
+
+/* Structure to pass on two planes to check for coupling and the tile	*/
+/* which is doing the coupling.						*/
+
+typedef struct _ecpls {
+    Tile *tile;
+    int  plane_of_tile;
+    int  plane_checked;
+} extCoupleStruct;
+
+/* Structure to pass on two planes to check for coupling and the	*/
+/* boundary which initiated the check.					*/
+
+typedef struct _esws {
+    Boundary *bp;
+    int  plane_of_boundary;
+    int  plane_checked;
+} extSidewallStruct;
+
+
 /* --------------------- Debugging stuff ---------------------- */
 #define CAP_DEBUG	FALSE
 
@@ -138,20 +166,25 @@ extFindCoupling(def, table, clipArea)
 {
     Rect *searchArea;
     int pNum;
+    extCapStruct ecs;
+
+    ecs.def = def;
 
     extCoupleHashPtr = table;
     extCoupleSearchArea = clipArea;
     searchArea = clipArea ? clipArea : &TiPlaneRect;
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
     {
+	ecs.plane = pNum;
+
 	if (PlaneMaskHasPlane(ExtCurStyle->exts_overlapPlanes, pNum))
 	    (void) DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 			searchArea, &ExtCurStyle->exts_overlapTypes[pNum],
-			extBasicOverlap, (ClientData) def);
+			extBasicOverlap, (ClientData) &ecs);
 	if (PlaneMaskHasPlane(ExtCurStyle->exts_sidePlanes, pNum))
 	    (void) DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 			searchArea, &ExtCurStyle->exts_sideTypes[pNum],
-			extBasicCouple, (ClientData) def);
+			extBasicCouple, (ClientData) &ecs);
     }
 
 }
@@ -223,28 +256,35 @@ extOutputCoupling(table, outFile)
  * ----------------------------------------------------------------------------
  */
 
-extBasicOverlap(tile, def)
+extBasicOverlap(tile, ecs)
     Tile *tile;
-    CellDef *def;
+    extCapStruct *ecs;
 {
-    int thisType = TiGetType(tile), thisPlane = DBPlane(thisType);
+    int thisType = TiGetType(tile);
     int pNum, pMask = ExtCurStyle->exts_overlapOtherPlanes[thisType];
     TileTypeBitMask *tMask = &ExtCurStyle->exts_overlapOtherTypes[thisType];
     Rect r;
+    CellDef *def = ecs->def;
+    int thisPlane = ecs->plane;
+    extCoupleStruct ecpls;
 
     TITORECT(tile, &r);
     extOverlapDef = def;
     if (extCoupleSearchArea)
 	GEOCLIP(&r, extCoupleSearchArea);
 
+    ecpls.tile = tile;
+    ecpls.plane_of_tile = thisPlane;
+
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
     {
 	/* Skip if nothing interesting on the other plane */
 	if (pNum == thisPlane || !PlaneMaskHasPlane(pMask, pNum))
 	    continue;
 
+	ecpls.plane_checked = pNum;
 	(void) DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum], &r, tMask,
-		extAddOverlap, (ClientData) tile);
+		extAddOverlap, (ClientData) &ecpls);
     }
 
     return (0);
@@ -287,8 +327,9 @@ struct overlap
     TileTypeBitMask	 o_tmask;
 };
 
-extAddOverlap(tbelow, tabove)
-    register Tile *tbelow, *tabove;
+extAddOverlap(tbelow, ecpls)
+    Tile *tbelow;
+    extCoupleStruct *ecpls;
 {
     int extSubtractOverlap(), extSubtractOverlap2();
     register NodeRegion *rabove, *rbelow;
@@ -298,6 +339,7 @@ extAddOverlap(tbelow, tabove)
     CoupleKey ck;
     int pNum;
     CapValue c;
+    Tile *tabove = ecpls->tile;
 
     /* Do nothing if both tiles are connected */
     rabove = (NodeRegion *) extGetRegion(tabove);
@@ -317,6 +359,14 @@ extAddOverlap(tbelow, tabove)
     ta = TiGetType(tabove);
     tb = TiGetType(tbelow);
 
+    /* Revert any contacts to their residues */
+
+    if (DBIsContact(ta))
+	ta = DBPlaneToResidue(ta, ecpls->plane_of_tile);
+
+    if (DBIsContact(tb))
+	tb = DBPlaneToResidue(ta, ecpls->plane_checked);
+
     /*
      * Find whether rabove and rbelow are shielded by intervening material.
      * Deduct the area shielded from the area of the overlap, so we adjust
@@ -349,9 +399,9 @@ extAddOverlap(tbelow, tabove)
     if (ov.o_area > 0)
     {
 	int oa = 
-	    ExtCurStyle->exts_planeOrder[DBTypePlaneTbl[TiGetType(tabove)]];
+	    ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(tabove))];
 	int ob = 
-	    ExtCurStyle->exts_planeOrder[DBTypePlaneTbl[TiGetType(tbelow)]];
+	    ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(tbelow))];
 	if (oa > ob)
 	{
 	    /*
@@ -477,12 +527,12 @@ extSubtractOverlap2(tile, ov)
  * ----------------------------------------------------------------------------
  */
 
-extBasicCouple(tile, def)
+extBasicCouple(tile, ecs)
     Tile *tile;
-    CellDef *def;
+    extCapStruct *ecs;
 {
     (void) extEnumTilePerim(tile, ExtCurStyle->exts_sideEdges[TiGetType(tile)],
-			extAddCouple, (ClientData) def);
+			extAddCouple, (ClientData) ecs);
     return (0);
 }
 
@@ -518,14 +568,21 @@ extBasicCouple(tile, def)
 
 Rect extSideOverlapSearchArea;
 
-extAddCouple(bp, def)
+extAddCouple(bp, ecs)
     Boundary *bp;	/* Boundary being considered */
-    CellDef *def;	/* Def containing this boundary */
+    extCapStruct *ecs;
 {
     TileType tin = TiGetType(bp->b_inside), tout = TiGetType(bp->b_outside);
     int pNum, pMask, (*proc)();
     Boundary bpCopy;
     Rect r, ovr;
+    CellDef *def = ecs->def;
+
+    /* Revert any contacts to their residues */
+    if (DBIsContact(tin))
+	tin = DBPlaneToResidue(tin, ecs->plane);
+    if (DBIsContact(tout))
+	tin = DBPlaneToResidue(tout, ecs->plane);
 
     extCoupleList = ExtCurStyle->exts_sideCoupleCap[tin][tout];
     extOverlapList = ExtCurStyle->exts_sideOverlapCap[tin][tout];
@@ -612,10 +669,11 @@ extAddCouple(bp, def)
  * ----------------------------------------------------------------------------
  */
 
-extSideOverlap(tp, bp)
-    register Tile *tp;		/* Overlapped tile */
-    register Boundary *bp;	/* Overlapping edge */
+extSideOverlap(tp, esws)
+    Tile *tp;			/* Overlapped tile */
+    extSidewallStruct *esws;	/* Overlapping edge and plane information */
 {
+    Boundary *bp = esws->bp;	/* Overlapping edge */
     NodeRegion *rtp = (NodeRegion *) extGetRegion(tp);
     NodeRegion *rbp = (NodeRegion *) extGetRegion(bp->b_inside);
     TileType ta, tb;
@@ -650,6 +708,12 @@ extSideOverlap(tp, bp)
     ta = TiGetType(bp->b_inside);
     tb = TiGetType(tp);
 
+    /* Revert any contacts to their residues */
+    if (DBIsContact(ta))
+	ta = DBPlaneToResidue(ta, esws->plane_of_boundary);
+    if (DBIsContact(tb))
+	tb = DBPlaneToResidue(tb, esws->plane_checked);
+
     /* Apply each rule, incorporating shielding into the edge length. */
     cap = (CapValue) 0;
     for (e = extOverlapList; e; e = e->ec_next)
@@ -717,8 +781,8 @@ extSideOverlap(tp, bp)
 	if (CAP_DEBUG) extAdjustCouple(he, cap, "sideoverlap");
 	extSetCapValue(he, cap + extGetCapValue(he));
 
-	oa = ExtCurStyle->exts_planeOrder[DBTypePlaneTbl[TiGetType(bp->b_inside)]];
-	ob = ExtCurStyle->exts_planeOrder[DBTypePlaneTbl[TiGetType(tp)]];
+	oa = ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(bp->b_inside))];
+	ob = ExtCurStyle->exts_planeOrder[DBPlane(TiGetType(tp))];
 	if (oa > ob)
 	{
 	    /* If the overlapped tile is between the substrate and the boundary
@@ -726,10 +790,18 @@ extSideOverlap(tp, bp)
 	     * from rbp's region due to the area of the sideoverlap, since 
 	     * we now know it is shielded from the substrate.
 	     */
+	    CapValue subcap;
 	    TileType intype = TiGetType(bp->b_inside);
 	    TileType outtype = TiGetType(bp->b_outside);
-	    CapValue subcap = (ExtCurStyle->exts_perimCap[intype][outtype] *
-		MIN(areaAccountedFor, length));
+
+	    /* Decompose contacts into their residues */
+	    if (DBIsContact(intype))
+		intype = DBPlaneToResidue(intype, esws->plane_of_boundary);
+	    if (DBIsContact(outtype))
+		outtype = DBPlaneToResidue(outtype, esws->plane_of_boundary);
+
+	    subcap = (ExtCurStyle->exts_perimCap[intype][outtype] *
+			MIN(areaAccountedFor, length));
 	    rbp->nreg_cap -= subcap;
 	    if (CAP_DEBUG)
 		extNregAdjustCap(rbp, -subcap, "obsolete_perimcap");
