@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtHard.c,v 1.2 2001/01/12 22:12:37 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
