@@ -24,7 +24,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtLength.c,v 1.2 2001/01/12 22:12:37 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -823,7 +823,7 @@ extPathFlood(tile, p, distance, epa)
 	    extPathFloodTile(tile, p, distance, tp, epa);
 
     /* Try connections to other planes */
-    if (TTMaskHasType(&DBImageBits, type))
+    if (DBIsContact(type))
     {
 	for (pNum = tilePlaneNum - 1; pNum <= tilePlaneNum + 1; pNum += 2)
 	    if (PlaneMaskHasPlane(DBConnPlanes[type], pNum))
