@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtPerim.c,v 1.3 2001/07/23 17:49:52 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -93,7 +93,7 @@ extEnumTilePerim(tpIn, mask, func, cdata)
     b.b_direction = BD_TOP;
     for (tpOut = RT(tpIn); RIGHT(tpOut) > LEFT(tpIn); tpOut = BL(tpOut))
     {
-	if (TTMaskHasType(&mask, TiGetType(tpOut)))
+	if (TTMaskHasType(&mask, TiGetBottomType(tpOut)))
 	{
 	    b.b_segment.r_xbot = MAX(LEFT(tpIn), LEFT(tpOut));
 	    b.b_segment.r_xtop = MIN(RIGHT(tpIn), RIGHT(tpOut));
@@ -108,7 +108,7 @@ extEnumTilePerim(tpIn, mask, func, cdata)
     b.b_direction = BD_BOTTOM;
     for (tpOut = LB(tpIn); LEFT(tpOut) < RIGHT(tpIn); tpOut = TR(tpOut))
     {
-	if (TTMaskHasType(&mask, TiGetType(tpOut)))
+	if (TTMaskHasType(&mask, TiGetTopType(tpOut)))
 	{
 	    b.b_segment.r_xbot = MAX(LEFT(tpIn), LEFT(tpOut));
 	    b.b_segment.r_xtop = MIN(RIGHT(tpIn), RIGHT(tpOut));
@@ -123,7 +123,7 @@ extEnumTilePerim(tpIn, mask, func, cdata)
     b.b_direction = BD_LEFT;
     for (tpOut = BL(tpIn); BOTTOM(tpOut) < TOP(tpIn); tpOut = RT(tpOut))
     {
-	if (TTMaskHasType(&mask, TiGetType(tpOut)))
+	if (TTMaskHasType(&mask, TiGetRightType(tpOut)))
 	{
 	    b.b_segment.r_ybot = MAX(BOTTOM(tpIn), BOTTOM(tpOut));
 	    b.b_segment.r_ytop = MIN(TOP(tpIn), TOP(tpOut));
@@ -138,7 +138,7 @@ extEnumTilePerim(tpIn, mask, func, cdata)
     b.b_direction = BD_RIGHT;
     for (tpOut = TR(tpIn); TOP(tpOut) > BOTTOM(tpIn); tpOut = LB(tpOut))
     {
-	if (TTMaskHasType(&mask, TiGetType(tpOut)))
+	if (TTMaskHasType(&mask, TiGetLeftType(tpOut)))
 	{
 	    b.b_segment.r_ybot = MAX(BOTTOM(tpIn), BOTTOM(tpOut));
 	    b.b_segment.r_ytop = MIN(TOP(tpIn), TOP(tpOut));
