@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtRegion.c,v 1.2 2001/01/12 22:12:38 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -166,135 +166,7 @@ extRegionAreaFunc(tile, arg)
     return (0);
 }
 
-/*
- * ----------------------------------------------------------------------------
- *
- * extGetSubcircuitBoundary --
- *
- * Get a segment list describing the "subcircuit" layer boundary, if any.
- * This will be used to assign subcircuit ports to labels whose borders
- * are coincident with this boundary.
- *
- * ----------------------------------------------------------------------------
- */ 
 
-SegList *
-extGetSubcircuitBoundary(def)
-    CellDef *def;
-{
-   Rect *rsubckt = &(def->cd_bbox);
-   Plane *psubckt = def->cd_planes[PL_SUBCIRCUIT];
-   TileTypeBitMask smask;
-   SegList *slist;
-
-   if (psubckt == NULL) return NULL;
-
-   TTMaskSetOnlyType(&smask, TT_SUBCIRCUIT);
-
-   slist = DBGetLayerBoundary((Tile *)NULL, psubckt, &smask, rsubckt);
-
-   return(slist);
-}
-
-/*
- * ----------------------------------------------------------------------------
- *
- * ExtSegCoincident --
- *
- * Checks if a Rect (e.g., a label boundary) is coincident with the
- * given segment list (SegList linked list).  Returns TRUE if
- * coincident, FALSE if not.  Assumes that coincidence will be along a
- * Manhattan path.  Centerpoint of coincidence gets put in cpoint, if
- * non-NULL.
- *
- * ----------------------------------------------------------------------------
- */
-
-bool
-ExtSegCoincident(rect, seglist, cpoint)
-    Rect *rect;
-    SegList *seglist;
-    Point *cpoint;
-{
-    SegList *sl;
-    int plo, phi;
-
-    for (sl = seglist; sl != NULL; sl = sl->next)
-    {
-	if (sl->sp1.p_x == sl->sp2.p_x)	/* vertical boundary segment */
-	{
-	    if ((rect->r_xbot == sl->sp1.p_x) || (rect->r_xtop == sl->sp1.p_x))
-	    {
-		if (sl->sp1.p_y < sl->sp2.p_y)
-		{
-		    plo = sl->sp1.p_y;
-		    phi = sl->sp2.p_y;
-		}
-		else
-		{
-		    plo = sl->sp1.p_y;
-		    phi = sl->sp2.p_y;
-		}
-		if ((rect->r_ybot <= phi) && (rect->r_ytop >= plo))
-		{
-		    if (cpoint != NULL)
-		    {
-			cpoint->p_x = sl->sp1.p_x;
-			cpoint->p_y = (rect->r_ybot + rect->r_ytop) >> 1;
-		    }
-		    return TRUE;
-		}
-	    }
-	}
-	else if (sl->sp1.p_y == sl->sp2.p_y) /* horizontal boundary segment */
-	{
-	    if ((rect->r_ybot == sl->sp1.p_y) || (rect->r_ytop == sl->sp1.p_y))
-	    {
-		if (sl->sp1.p_x < sl->sp2.p_x)
-		{
-		    plo = sl->sp1.p_x;
-		    phi = sl->sp2.p_x;
-		}
-		else
-		{
-		    plo = sl->sp1.p_x;
-		    phi = sl->sp2.p_x;
-		}
-		if ((rect->r_xbot <= phi) && (rect->r_xtop >= plo))
-		{
-		    if (cpoint != NULL)
-		    {
-			cpoint->p_x = (rect->r_xbot + rect->r_xtop) >> 1;
-			cpoint->p_y = sl->sp1.p_y;
-		    }
-		    return TRUE;
-		}
-	    }
-	}
-    }
-    return FALSE;
-}
-
-/*
- * ----------------------------------------------------------------------------
- *
- * extAssignPort --
- *
- * Set the "LL_PORTATTR" attribute to a LabelList entry if the boundary
- * coincides with a subcircuit boundary.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-extAssignPort(ll, subckt)
-    LabelList *ll;
-    SegList *subckt;
-{
-    if (ExtSegCoincident(&(ll->ll_label->lab_rect), subckt, NULL))
-	ll->ll_attr = LL_PORTATTR;
-}
-	
 /*
  * ----------------------------------------------------------------------------
  *
@@ -330,11 +202,6 @@ ExtLabelRegions(def, connTo)
     LabRegion *reg;
     int quad, pNum;
     Point p;
-    SegList *subckt;
-
-    /* Generate segment list for subcircuit boundary, if any */
-
-    subckt = extGetSubcircuitBoundary(def);
 
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
     {
@@ -361,14 +228,15 @@ ExtLabelRegions(def, connTo)
 		MALLOC(LabelList *, ll, sizeof (LabelList));
 		ll->ll_label = lab;
 		ll->ll_next = reg->lreg_labels;
-		ll->ll_attr = LL_NOATTR;
 		reg->lreg_labels = ll;
-		extAssignPort(ll, subckt);
+		if (lab->lab_flags & PORT_DIR_MASK)
+		    ll->ll_attr = LL_PORTATTR;
+		else
+		    ll->ll_attr = LL_NOATTR;
 		break;
 	    }
 	}
     }
-    DBFreeLayerBoundary(subckt);
 }
 
 /*
@@ -398,12 +266,9 @@ ExtLabelOneRegion(def, connTo, reg)
     register Tile *tp;
     int quad, pNum;
     Point p;
-    SegList *subckt;
 
     /* Generate segment list for subcircuit boundary, if any */
 
-    subckt = extGetSubcircuitBoundary(def);
-
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
     {
 	pNum = DBPlane(lab->lab_type);
@@ -428,14 +293,15 @@ ExtLabelOneRegion(def, connTo, reg)
 		MALLOC(LabelList *, ll, sizeof (LabelList));
 		ll->ll_label = lab;
 		ll->ll_next = reg->nreg_labels;
-		ll->ll_attr = LL_NOATTR;
 		reg->nreg_labels = ll;
-		extAssignPort(ll, subckt);
+		if (lab->lab_flags & PORT_DIR_MASK)
+		    ll->ll_attr = LL_PORTATTR;
+		else
+		    ll->ll_attr = LL_NOATTR;
 		break;
 	    }
 	}
     }
-    DBFreeLayerBoundary(subckt);
 }
 
 
