@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char sccsid[] = "@(#)ExtTech.c	4.8 MAGIC (Berkeley) 10/26/85";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -64,19 +64,21 @@ void ExtLoadStyle();
 
 typedef enum
 {
-    AREAC, CONTACT, CSCALE, FET, FETRESIST, HEIGHT, LAMBDA, OVERC, PERIMC,
-    PLANEORDER, NOPLANEORDER, RESIST, RSCALE, SIDEHALO, SIDEOVERLAP,
+    AREAC, CONTACT, CSCALE, DEVICE, FET, FETRESIST, HEIGHT, LAMBDA, OVERC,
+    PERIMC, PLANEORDER, NOPLANEORDER, RESIST, RSCALE, SIDEHALO, SIDEOVERLAP,
     SIDEWALL, STEP, STYLE
 } Key;
 
-static struct keydesc
+typedef struct
 {
     char	*k_name;
-    Key		 k_key;
+    int		 k_key;
     int		 k_minargs;
     int		 k_maxargs;
     char	*k_usage;
-} keyTable[] = {
+} keydesc;
+
+static keydesc keyTable[] = {
     "areacap",		AREAC,		3,	3,
 "types capacitance",
 
@@ -86,6 +88,9 @@ static struct keydesc
     "cscale",		CSCALE,		2,	2,
 "capacitance-scalefactor",
 
+    "device",		DEVICE,		4,	9,
+"device dev-type types options...",
+
     "fet",		FET,		8,	9,
 "types terminal-types min-#-terminals name [subs-types] subs-node gscap gate-chan-cap",
 
@@ -135,6 +140,210 @@ static struct keydesc
 };
 
 
+/*
+ * Table used for parsing the "device" keyword types
+ */
+
+/* types are enumerated in extract.h */
+
+static keydesc devTable[] = {
+    "mosfet",		DEV_MOSFET,		5,	9,
+"name gate-types src-drn-types sub-types|None sub-node [gscap gccap]",
+
+    "bjt",		DEV_BJT,		5,	5,
+"name base-types emitter-types collector-types",
+
+    "capacitor",	DEV_CAP,		5,	6,
+"name top-types bottom-types [perimcap] areacap",
+
+    "resistor",		DEV_RES,		4,	4,
+"name|None res-types terminal-types",
+
+    "subcircuit",	DEV_SUBCKT,		3,	5,
+"name dev-types [term-types [sub-types]]",
+
+    0
+};
+
+#ifdef MAGIC_WRAPPER
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ *	This routine is designed to work with embedded exttosim and
+ *	exttospice.  It determines whether the current extract style
+ *	matches the string (picked up from the .ext file).  If so, it
+ *	returns TRUE.  If not, it checks whether the style exists in
+ *	the list of known files for this technology.  If so, it loads
+ *	the correct style and returns TRUE.  If not, it returns FALSE.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+ExtCompareStyle(stylename)
+    char *stylename;
+{
+    ExtKeep *style;
+
+    if (!strcmp(stylename, ExtCurStyle->exts_name))
+	return TRUE;
+    else
+    {
+	for (style = ExtAllStyles; style != NULL; style = style->exts_next)
+	{
+	    if (!strcmp(stylename, style->exts_name))
+	    {
+		ExtLoadStyle(stylename);
+		return TRUE;
+	    }
+	}
+    }
+    return FALSE;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ *	This routine is designed to work with the embedded exttosim and
+ *	exttospice commands under the Tcl-based magic, such that all
+ *	device information needed by these commands can be picked up
+ *	from the current extraction style (as it should!).  This
+ *	really should be set up when the extract file is read, which is
+ *	why the routine is here, although this is not a very efficient
+ *	way to do it (but it needs to be this way to keep backward
+ *	compatibility with the non-Tcl, standalone programs ext2sim and
+ *	ext2spice).
+ *
+ *	Note that finding the device by index ("idx") is horribly
+ *	inefficient, but keeps the netlist generator separated from
+ *	the extractor.  Some of this code is seriously schizophrenic,
+ *	and should not be investigated too closely.
+ * 
+ * Results:
+ *	Return FALSE if no device corresponds to index "idx".  TRUE
+ *	otherwise.
+ *
+ * Side Effects:
+ *	Fills values in the argument list.
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+ExtGetDevInfo(idx, devnameptr, sd_rclassptr, sub_rclassptr, subnameptr)
+    int idx;
+    char **devnameptr;
+    short *sd_rclassptr;
+    short *sub_rclassptr;
+    char **subnameptr;
+{
+    TileType t;
+    TileTypeBitMask rmask, tmask;
+    int n, i = 0, j;
+    bool repeat;
+    char *locdname;
+    char **uniquenamelist = (char **)mallocMagic(DBNumTypes * sizeof(char *));
+
+
+    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+    {
+	locdname = ExtCurStyle->exts_transName[t];
+	if (locdname != NULL)
+	{
+	    repeat = FALSE;
+	    for (j = 0; j < i; j++)
+		if (!strcmp(uniquenamelist[j], locdname))
+		{
+		    repeat = TRUE;
+		    break;
+		}
+	    if (repeat == FALSE)
+	    {
+		if (i == idx) break;
+		uniquenamelist[i] = locdname;
+		i++;
+	    }
+	}
+    }
+    if (t == DBNumTypes) return FALSE;
+
+    *devnameptr = locdname;
+    *subnameptr = ExtCurStyle->exts_transSubstrateName[t];
+
+    tmask = ExtCurStyle->exts_transSDTypes[t];
+    *sd_rclassptr = (short)(-1);	/* NO_RESCLASS */
+
+    for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
+    {
+        rmask = ExtCurStyle->exts_typesByResistClass[n];
+	if (TTMaskAndMask(&rmask, &tmask))
+	{
+	    *sd_rclassptr = (short)n;
+	    break;
+	}
+    }
+
+    tmask = ExtCurStyle->exts_transSubstrateTypes[t];
+    *sub_rclassptr = (short)(-1);	/* NO_RESCLASS */
+
+    for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
+    {
+        rmask = ExtCurStyle->exts_typesByResistClass[n];
+	if (TTMaskAndMask(&rmask, &tmask))
+	{
+	    *sub_rclassptr = (short)(n);
+	    break;
+	}
+    }
+
+    FREE(uniquenamelist);
+    return TRUE;
+}
+
+#endif /* MAGIC_WRAPPER */
+
+
+#ifdef THREE_D
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * ExtGetZAxis --
+ * 
+ *	Get the height and thickness parameters for a layer (used by the
+ *	graphics module which does not have access to internal variables
+ *	in the extract section).
+ *
+ * Results:
+ *	None
+ *
+ * Side Effects:
+ *	Fills values "height" and "thick" in argument list.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+ExtGetZAxis(tile, height, thick)
+    Tile *tile;
+    float *height, *thick;
+{
+    TileType ttype;
+
+    if (ExtCurStyle == NULL) return;
+
+    ttype = TiGetLeftType(tile);        /* Ignore non-Manhattan for now */
+
+    /* Note that w_scale is multiplied by 4096 to get fixed-point accuracy.     */
+    /* However, we downshift by only 9 (divide by 512) so that heights are      */
+    /* exaggerated in the layout by a factor of 8 (height exaggeration is       */
+    /* standard practice for VLSI cross-sections).                              */
+
+    *height = ExtCurStyle->exts_height[ttype];
+    *thick = ExtCurStyle->exts_thick[ttype];
+}
+#endif  /* THREE_D */
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -280,6 +489,7 @@ extTechStyleAlloc()
     {
 	style->exts_transSubstrateName[r] = (char *) NULL;
 	style->exts_transName[r] = (char *) NULL;
+	style->exts_deviceClass[r] = (char) 0;
 	style->exts_transResist[r].ht_table = (HashEntry **) NULL;
     }
 
@@ -364,6 +574,7 @@ extTechStyleInit(style)
 	    freeMagic(style->exts_transName[r]);
 	    style->exts_transName[r] = (char *) NULL;
 	}
+	style->exts_deviceClass[r] = (char)0;
 	if (style->exts_transResist[r].ht_table != (HashEntry **) NULL)
 	    HashKill(&style->exts_transResist[r]);
 	HashInit(&style->exts_transResist[r], 8, HT_STRINGKEYS);
@@ -474,7 +685,7 @@ ExtLoadStyle(stylename)
     /* no other tech client sections depend on the extract section.	*/
 
     invext = TechSectionGetMask("extract", NULL);
-    TechLoad(TechDefault, invext);
+    TechLoad(NULL, invext);
 
     extTechFinalStyle(ExtCurStyle);
     ExtTechScale(DBLambda[0], DBLambda[1]);
@@ -660,7 +871,7 @@ ExtTechLine(sectionName, argc, argv)
     TileTypeBitMask types1, types2, near, far, ov, shield, subsTypes;
     char *subsName, *transName, *cp, *endptr;
     register TileType s, t, r, o;
-    struct keydesc *kp;
+    keydesc *kp, *dv;
     bool isLinear;
     HashEntry *he;
     EdgeCap *cnew;
@@ -755,6 +966,9 @@ ExtTechLine(sectionName, argc, argv)
 	case SIDEOVERLAP:
 	    DBTechNoisyNameMask(argv[1], &types1);
 	    break;
+	case DEVICE:
+	    DBTechNoisyNameMask(argv[3], &types1);
+	    break;
 	case PLANEORDER:
 	case NOPLANEORDER:
 	default:
@@ -778,20 +992,20 @@ ExtTechLine(sectionName, argc, argv)
 		sidx = 3;
 		if (!StrIsInt(argv[2]))
 		{
-		    TechError("Contact border %s must be numeric\n", argv[3]);
+		    TechError("Contact border %s must be numeric\n", argv[2]);
 		    break;
 		}
 		border = atoi(argv[2]);
 		if (!StrIsInt(argv[4]))
 		{
-		    TechError("Contact border %s must be numeric\n", argv[4]);
+		    TechError("Contact spacing %s must be numeric\n", argv[4]);
 		    break;
 		}
 		spacing = atoi(argv[4]);
 	    }
 	    if (!StrIsInt(argv[sidx]))
 	    {
-		TechError("Contact minimum size %s must be numeric\n", argv[2]);
+		TechError("Contact minimum size %s must be numeric\n", argv[sidx]);
 		break;
 	    }
 	    size = atoi(argv[sidx]);
@@ -826,6 +1040,9 @@ ExtTechLine(sectionName, argc, argv)
 	    }
 	    break;
 	case FET:
+
+	    /* Original FET format, kept for backwards compatibility */
+
 	    DBTechNoisyNameMask(argv[2], &types2);
 	    nterm = atoi(argv[3]);
 	    transName = argv[4];
@@ -857,22 +1074,116 @@ ExtTechLine(sectionName, argc, argv)
 		    ExtCurStyle->exts_transSDCount[t] = nterm;
 		    ExtCurStyle->exts_transSDCap[t] = gscap;
 		    ExtCurStyle->exts_transGateCap[t] = gccap;
+		    ExtCurStyle->exts_deviceClass[t] = DEV_FET;
 		    ExtCurStyle->exts_transName[t] =
 			    StrDup((char **) NULL, transName);
 		    ExtCurStyle->exts_transSubstrateName[t] =
 			    StrDup((char **) NULL, subsName);
 		    ExtCurStyle->exts_transSubstrateTypes[t] = subsTypes;
 		    {
-		    	 int	z;
+			int z;
 
-			 for (z= TT_TECHDEPBASE; z < DBNumTypes;z++)
-			 {
-			      if (TTMaskHasType(&subsTypes,z))
-			      	TTMaskSetType(&ExtCurStyle->exts_subsTransistorTypes[z],t);
-			 }
+			for (z = TT_TECHDEPBASE; z < DBNumTypes; z++)
+			{
+			    if (TTMaskHasType(&subsTypes, z))
+				TTMaskSetType(&ExtCurStyle->exts_subsTransistorTypes[z],
+					t);
+			}
 		    }
 		}
 	    break;
+
+	case DEVICE:
+
+	    /* Parse second argument for device type */
+
+	    n = LookupStruct(argv[1], (LookupTable *) devTable, sizeof devTable[0]);
+	    if (n < 0)
+	    {
+		TechError("Illegal device.  Legal devices are:\n\t");
+		for (n = 0; devTable[n].k_name; n++)
+		    TxError(" %s", devTable[n].k_name);
+		TxError("\n");
+		return (TRUE);
+	    }
+
+	    dv = &devTable[n];
+	    if ((argc - 1) < dv->k_minargs || (argc - 1) > dv->k_maxargs)
+		goto usage;
+
+	    gscap = (CapValue) 0;
+	    gccap = (CapValue) 0;
+	    subsName = NULL;
+	    subsTypes = DBZeroTypeBits;
+	    transName = argv[2];
+
+	    switch (dv->k_key)
+	    {
+		case DEV_BJT:
+		    DBTechNoisyNameMask(argv[4], &types2);	/* emitter */
+		    DBTechNoisyNameMask(argv[5], &subsTypes);	/* collector */
+		    nterm = 1;	/* emitter is the only "terminal type" expected */
+		    break;
+		case DEV_MOSFET:
+		    DBTechNoisyNameMask(argv[4], &types2);	/* source/drain */
+		    if (strcmp(argv[5], "None"))
+			DBTechNoisyNameMask(argv[5], &subsTypes);   /* substrate */
+		    subsName = argv[6];
+		    if (argc > 7) gscap = aToCap(argv[7]);
+		    if (argc > 8) gccap = aToCap(argv[8]);
+		    nterm = 2;
+		    break;
+		case DEV_RES:
+		    DBTechNoisyNameMask(argv[4], &types2);	/* terminals */
+		    subsTypes = DBZeroTypeBits;
+		    nterm = 2;
+		    break;
+		case DEV_CAP:
+		    DBTechNoisyNameMask(argv[4], &types2);	/* bottom */
+		    subsTypes = DBZeroTypeBits;
+		    if (argc > 5) gscap = aToCap(argv[5]);	/* perimeter cap */
+		    gccap = aToCap(argv[argc - 1]);		/* area cap */
+		    nterm = 1;
+		    break;
+		case DEV_SUBCKT:
+		    nterm = 1;
+		    if (argc > 4)
+			DBTechNoisyNameMask(argv[4], &types2);	  /* terminals */
+		    if (argc > 5)
+			DBTechNoisyNameMask(argv[5], &subsTypes); /* substrate */
+		    break;
+	    }
+
+	    TTMaskSetMask(&ExtCurStyle->exts_transMask, &types1);
+	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	    {
+		if (TTMaskHasType(&types1, t))
+		{
+		    TTMaskSetMask(ExtCurStyle->exts_transConn + t, &types1);
+		    ExtCurStyle->exts_transSDTypes[t] = types2;
+		    ExtCurStyle->exts_transSDCount[t] = nterm;
+		    ExtCurStyle->exts_transSDCap[t] = gscap;
+		    ExtCurStyle->exts_transGateCap[t] = gccap;
+		    ExtCurStyle->exts_deviceClass[t] = dv->k_key;
+		    ExtCurStyle->exts_transName[t] =
+			    StrDup((char **) NULL, transName);
+		    ExtCurStyle->exts_transSubstrateName[t] =
+			    StrDup((char **) NULL, subsName);
+		    ExtCurStyle->exts_transSubstrateTypes[t] = subsTypes;
+		    {
+			int z;
+
+			for (z = TT_TECHDEPBASE; z < DBNumTypes; z++)
+			{
+			    if (TTMaskHasType(&subsTypes, z))
+				TTMaskSetType(&ExtCurStyle->
+					exts_subsTransistorTypes[z], t);
+			}
+		    }
+		}
+	    }
+	    break;
+
 	case FETRESIST:
 	    if (!StrIsInt(argv[3]))
 	    {
@@ -926,32 +1237,40 @@ ExtTechLine(sectionName, argc, argv)
 	    for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
 	    {
 		if (!TTMaskHasType(&types1, s)) continue;
+
+		/* Contact overlap caps are determined from residues */
+		if (DBIsContact(s)) continue;
+
 		for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 		{
 		    if (!TTMaskHasType(&types2, t)) continue;
+
+		    /* Contact overlap caps are determined from residues */
+		    if (DBIsContact(t)) continue;
+
 		    if (s == t)
 		    {
-			TechError(
-"Can't have overlap capacitance between tiles of the same type (%s)\n",
-			    DBTypeLongName(s));
+			TechError("Can't have overlap capacitance between"
+				" tiles of the same type (%s)\n",
+				DBTypeLongName(s));
 			bad = TRUE;
 			continue;
 		    }
 		    p1 = DBPlane(s), p2 = DBPlane(t);
 		    if (p1 == p2)
 		    {
-			TechError(
-"Can't have overlap capacitance between tiles on the same plane (%s, %s)\n",
-			    DBTypeLongName(s), DBTypeLongName(t));
+			TechError("Can't have overlap capacitance between"
+				" tiles on the same plane (%s, %s)\n",
+				DBTypeLongName(s), DBTypeLongName(t));
 			bad = TRUE;
 			continue;
 		    }
 		    if ((ExtCurStyle->exts_overlapCap[s][t]) != (CapValue) 0)
 		    {
-			TechError(
-"Only one of \"overlap %s %s\" or \"overlap %s %s\" allowed\n",
-			    DBTypeLongName(s), DBTypeLongName(t),
-			    DBTypeLongName(t), DBTypeLongName(s));
+			TechError("Only one of \"overlap %s %s\" or"
+				" \"overlap %s %s\" allowed\n",
+				DBTypeLongName(s), DBTypeLongName(t),
+				DBTypeLongName(t), DBTypeLongName(s));
 			bad = TRUE;
 			continue;
 		    }
@@ -969,11 +1288,14 @@ ExtTechLine(sectionName, argc, argv)
 		    {
 			if (TTMaskHasType(&shield, r))
 			{
+			    /* Shielding types are determined from residues */
+			    if (DBIsContact(r)) continue;
+
 			    p3 = DBPlane(r);
 			    if (p3 == p1 || p3 == p2)
 			    {
-				TechError(
-"Shielding type (%s) must be on a different plane from shielded types.\n",
+				TechError("Shielding type (%s) must be on a"
+					" different plane from shielded types.\n",
 					DBTypeLongName(r));
 				bad = TRUE;
 				continue;
@@ -991,8 +1313,7 @@ ExtTechLine(sectionName, argc, argv)
 	case SIDEOVERLAP:
 	    bad = FALSE;
 	    DBTechNoisyNameMask(argv[2], &types2);
-	    DBTechNoisyNameMask(argv[3], &ov);
-	    pov = extTechTypesToPlanes(&ov);
+	    pov = DBTechNoisyNameMask(argv[3], &ov);
 	    capVal = aToCap(argv[4]);
 	    shield = DBZeroTypeBits;
 	    if (argc == 6) DBTechNoisyNameMask(argv[5], &shield);
@@ -1018,6 +1339,10 @@ ExtTechLine(sectionName, argc, argv)
 	    {
 		if (!TTMaskHasType(&types1, s))
 		    continue;
+
+		/* Side overlap computed from residues */
+		if (DBIsContact(s)) continue;
+
 		p1 = DBPlane(s);
 		if (PlaneMaskHasPlane(pov, p1))
 		    goto diffplane;
@@ -1028,6 +1353,10 @@ ExtTechLine(sectionName, argc, argv)
 		{
 		    if (!TTMaskHasType(&types2, t))
 			continue;
+
+		    /* Side overlap computed from residues */
+		    if (DBIsContact(t)) continue;
+
 		    p2 = DBPlane(t);
 		    if (t != TT_SPACE && PlaneMaskHasPlane(pov, p2))
 			goto diffplane;
@@ -1047,11 +1376,14 @@ ExtTechLine(sectionName, argc, argv)
 		    {
 			if (TTMaskHasType(&shield, r))
 			{
+			    /* Side overlap shielding computed from residues */
+			    if (DBIsContact(r)) continue;
+
 			    p3 = DBPlane(r);
 			    if (p3 == p1 || p3 == p2)
 			    {
-				TechError(
-"Shielding type (%s) must be on a different plane from shielded types.\n",
+				TechError("Shielding type (%s) must be on"
+					" a different plane from shielded types.\n",
 					DBTypeLongName(r));
 				bad = TRUE;
 				continue;
@@ -1173,39 +1505,12 @@ usage:
     return (TRUE);
 
 diffplane:
-    TechError("Overlapped types in \"sideoverlap\" rule must be on\n\
-\tdifferent plane from intypes and outtypes.\n");
+    TechError("Overlapped types in \"sideoverlap\" rule must be on a\n"
+		"\tdifferent plane from intypes and outtypes.\n");
     return (TRUE);
 }
 
-/*
- * ----------------------------------------------------------------------------
- *
- * extTechTypesToPlanes --
- *
- * Like DBTechTypesToPlanes, but makes sure that TT_SPACE is turned off in
- * its argument mask.
- *
- * Results:
- *	Returns a mask of the planes occupied by the types in the
- *	mask pointed to by 'm'.
- *
- * Side effects:
- *	None.
- *
- * ----------------------------------------------------------------------------
- */
 
-extTechTypesToPlanes(m)
-    TileTypeBitMask *m;
-{
-    TileTypeBitMask mnew;
-
-    mnew = *m;
-    TTMaskClearType(&mnew, TT_SPACE);
-    return (DBTechTypesToPlanes(&mnew));
-}
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1276,10 +1581,9 @@ extTechFinalStyle(style)
     for (r = TT_TECHDEPBASE; r < DBNumTypes; r++)
     {
 	s = style->exts_typeToResistClass[r];
-      if( s > 0 ) {
-        TTMaskClearMask(&style->exts_typesResistChanged[r],
-                &style->exts_typesByResistClass[s]);
-      }
+	if (s >= 0)
+	    TTMaskClearMask(&style->exts_typesResistChanged[r],
+			&style->exts_typesByResistClass[s]);
     }
 
     /*
@@ -1327,8 +1631,9 @@ extTechFinalStyle(style)
 	if (indicis[p1] > 1) conflict = TRUE ;
 	if (indicis[p1] < 1) missing = TRUE ;
     }
-    if (!conflict && !missing) 
-	return /* Everything was ok */ ;
+    if (!conflict && !missing)		/* Everything was ok */
+	goto zinit; 	
+
     TxError ("\nWarning: Extraction Style %s\n", style -> exts_name);
     if (conflict) {
 	TxError ("  Conflicting planeorder for plane(s):\n   ");
@@ -1352,6 +1657,18 @@ extTechFinalStyle(style)
 	TxError(" %s=%d,",DBPlaneLongNameTbl[p1], style->exts_planeOrder[p1]);
     }
     TxError("\n");
+
+    /* Now that we have a plane ordering, we can apply default height	*/
+    /* and thickness values for those layers.				*/
+
+zinit:
+    for (r = TT_TECHDEPBASE; r < DBNumTypes; r++)
+    {
+	if (style->exts_thick[r] == 0)
+	    style->exts_thick[r] = 0.05;
+	if (style->exts_height[r] == 0)
+	    style->exts_height[r] = 0.1 * style->exts_planeOrder[DBPlane(r)];
+    }
 }
 
 /*
