@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/extract/ExtYank.c,v 1.2 2001/01/12 22:12:38 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -176,7 +176,7 @@ extHierYankFunc(use, trans, x, y, hy)
     scx.scx_y = y;
 
     /* Yank the paint */
-    if (!(use->cu_def->cd_flags & CDSUBCIRCUIT))
+    if (!DBIsSubcircuit(use->cu_def))
 	DBCellCopyAllPaint(&scx, &DBAllButSpaceBits, 0, hy->hy_target);
 
     /* Yank the labels */
@@ -237,7 +237,7 @@ extHierLabelFunc(scx, label, tpath, targetDef)
     /* Add paint inside label if this is a subcircuit */
     /* Caveat:  If label has zero area, it will be extended by 1 unit */
 
-    if (scx->scx_use->cu_def->cd_flags & CDSUBCIRCUIT)
+    if (DBIsSubcircuit(scx->scx_use->cu_def))
     {
 	int pNum;
 
