@@ -27,19 +27,19 @@
 
 #ifndef	_MAGIC
 int err0 = Need_to_include_misc/magic_header;
-#endif	_MAGIC
+#endif	/* _MAGIC */
 #ifndef	_DATABASE
 int err1 = Need_to_include_database/database_header;
-#endif	_DATABASE
+#endif	/* _DATABASE */
 #ifndef	_TILES
 int err2 = Need_to_include_tiles/tile_header;
-#endif	_TILES
+#endif	/* _TILES */
 #ifndef	_HASH
 int err3 = Need_to_include_utils/hash_header;
-#endif	_HASH
+#endif	/* _HASH */
 #ifndef	_GEOMETRY
 int err4 = Need_to_include_utils/geometry_header;
-#endif	_GEOMETRY
+#endif	/* _GEOMETRY */
 
 #undef	NT
 #define	NT	TT_MAXTYPES
@@ -772,6 +772,9 @@ typedef struct extstyle
 	/* Name of each transistor type as output in .ext file */
     char		*exts_transName[NT];
 
+	/* Device class for each layer type */
+    char		exts_deviceClass[NT];
+
 	/* Contains one for each type of fet, zero for all other types */
     TileTypeBitMask	 exts_transMask;
 
