@@ -17,7 +17,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/gcr/gcrDebug.c,v 1.2 2001/01/12 22:12:42 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -379,7 +379,7 @@ gcrPrDensity(ch, chanDensity)
     (void) fprintf(fp, "%3s %5s", "COL", "GLOB");
 #ifdef	IDENSITY
     (void) fprintf(fp, " %5s %5s", "INIT", "DIFF");
-#endif	IDENSITY
+#endif	/* IDENSITY */
     (void) fprintf(fp, " %5s\n", "CHAN");
     for (i = 1; i <= ch->gcr_length; i++)
     {
@@ -388,7 +388,7 @@ gcrPrDensity(ch, chanDensity)
 #ifdef	IDENSITY
 	diff -= ch->gcr_iRowsByCol[i];
 	(void) fprintf(fp, " %5d %5d", ch->gcr_iRowsByCol[i], diff);
-#endif	IDENSITY
+#endif	/* IDENSITY */
 	(void) fprintf(fp, "%5d%s\n", ch->gcr_density[i],
 		(diff != ch->gcr_density[i]) ? " *****" : "");
     }
@@ -397,7 +397,7 @@ gcrPrDensity(ch, chanDensity)
     (void) fprintf(fp, "%3s %5s", "ROW", "GLOB");
 #ifdef	IDENSITY
     (void) fprintf(fp, " %5s %5s", "INIT", "DIFF");
-#endif	IDENSITY
+#endif	/* IDENSITY */
     (void) fprintf(fp, "\n");
     for (i = 1; i <= ch->gcr_width; i++)
     {
@@ -405,7 +405,7 @@ gcrPrDensity(ch, chanDensity)
 #ifdef	IDENSITY
 	(void) fprintf(fp, " %5d %5d", ch->gcr_iColsByRow[i],
 			ch->gcr_dColsByRow[i] - ch->gcr_iColsByRow[i]);
-#endif	IDENSITY
+#endif	/* IDENSITY */
 	(void) fprintf(fp, "\n");
     }
 
