@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/graphics/grClip.c,v 1.8 2001/10/19 15:40:10 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -1099,20 +1099,16 @@ GrFastBox(prect)
 /*---------------------------------------------------------
  * GrClipTriangle:
  *	Returns an array of points representing a clipped
- *	triangle
+ *	triangle.  These are always arranged in counter-
+ *	clockwise order.
  *
  * Results:  Returns array of points and modifies int * np
  *	to hold the number of points in the array.
  *---------------------------------------------------------
  */
 
-#define xround(d) (((((d % height) << 1) >= height) ? 1 : 0) + (d / height))
-#define yround(d) (((((d % width) << 1) >= width) ? 1 : 0) + (d / width))
-
-/*
-#define xround(d)    (d / height)
-#define yround(d)    (d / width)
-*/
+#define xround(d) (int)(((((d % height) << 1) >= height) ? 1 : 0) + (d / height))
+#define yround(d) (int)(((((d % width) << 1) >= width) ? 1 : 0) + (d / width))
 
 Void
 GrClipTriangle(r, c, clipped, dir, side, points, np)
@@ -1157,30 +1153,30 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 
     if (clipped)
     {
-        int delx, dely;
-        int width = r->r_xtop - r->r_xbot;
-	int height = r->r_ytop - r->r_ybot;
+        dlong delx, dely;
+        dlong width = (dlong)(r->r_xtop - r->r_xbot);
+	dlong height = (dlong)(r->r_ytop - r->r_ybot);
 
 	switch((side << 1) | dir)
 	{
 	    case 0x0:	/* nw */
 		if (c->r_ytop < r->r_ytop)	/* clip top */
 		{
-		    delx = (points[1].p_y - c->r_ytop) * width;
+		    delx = (dlong)(points[1].p_y - c->r_ytop) * width;
 		    points[1].p_y = c->r_ytop;
 		    points[0].p_y = c->r_ytop;
 		    points[0].p_x -= xround(delx);
 		}
 		if (c->r_xbot > r->r_xbot)	/* clip left */
 		{
-		    dely = (c->r_xbot - points[2].p_x) * height;
+		    dely = (dlong)(c->r_xbot - points[2].p_x) * height;
 		    points[1].p_x = c->r_xbot;
 		    points[2].p_x = c->r_xbot;
 		    points[2].p_y += yround(dely);
 		}
 		if (c->r_ybot > points[2].p_y)	/* clip bottom */
 		{
-		    delx = (c->r_ybot - points[2].p_y) * width;
+		    delx = (dlong)(c->r_ybot - points[2].p_y) * width;
 		    points[2].p_y = c->r_ybot;
 		    points[3].p_y = c->r_ybot;
 		    points[3].p_x = points[2].p_x + xround(delx);
@@ -1193,7 +1189,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		    }
 		    else if (c->r_xtop < points[0].p_x) /* clip right, pentagon */
 		    {
-			dely = (points[0].p_x - c->r_xtop) * height;
+			dely = (dlong)(points[0].p_x - c->r_xtop) * height;
 			points[0].p_x = c->r_xtop;
 			points[4].p_x = c->r_xtop;
 			points[4].p_y = points[0].p_y - yround(dely);
@@ -1202,7 +1198,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		}
 		else if (c->r_xtop < points[0].p_x) /* clip right, quadrangle */
 		{
-		    dely = (points[0].p_x - c->r_xtop) * height;
+		    dely = (dlong)(points[0].p_x - c->r_xtop) * height;
 		    points[0].p_x = c->r_xtop;
 		    points[3].p_x = c->r_xtop;
 		    points[3].p_y = points[0].p_y - yround(dely);
@@ -1215,21 +1211,21 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 	    case 0x1:	/* sw */
 		if (c->r_xbot > r->r_xbot)	/* clip LEFT */
 		{
-		    dely = (c->r_xbot - points[1].p_x) * height;
+		    dely = (dlong)(c->r_xbot - points[1].p_x) * height;
 		    points[1].p_x = c->r_xbot;
 		    points[0].p_x = c->r_xbot;
 		    points[0].p_y -= yround(dely);
 		}
 		if (c->r_ybot > r->r_ybot)	/* clip BOTTOM */
 		{
-		    delx = (c->r_ybot - points[2].p_y) * width;
+		    delx = (dlong)(c->r_ybot - points[2].p_y) * width;
 		    points[1].p_y = c->r_ybot;
 		    points[2].p_y = c->r_ybot;
 		    points[2].p_x -= xround(delx);
 		}
 		if (c->r_xtop < points[2].p_x)	/* clip RIGHT */
 		{
-		    dely = (points[2].p_x - c->r_xtop) * height;
+		    dely = (dlong)(points[2].p_x - c->r_xtop) * height;
 		    points[2].p_x = c->r_xtop;
 		    points[3].p_x = c->r_xtop;
 		    points[3].p_y = points[1].p_y + yround(dely);
@@ -1242,7 +1238,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		    }
 		    else if (c->r_ytop < points[0].p_y) /* clip TOP, pentagon */
 		    {
-			delx = (points[0].p_y - c->r_ytop) * width;
+			delx = (dlong)(points[0].p_y - c->r_ytop) * width;
 			points[0].p_y = c->r_ytop;
 			points[4].p_y = c->r_ytop;
 			points[4].p_x = points[0].p_x + xround(delx);
@@ -1251,7 +1247,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		}
 		else if (c->r_ytop < points[0].p_y) /* clip TOP, quadrangle */
 		{
-		    delx = (points[0].p_y - c->r_ytop) * width;
+		    delx = (dlong)(points[0].p_y - c->r_ytop) * width;
 		    points[0].p_y = c->r_ytop;
 		    points[3].p_y = c->r_ytop;
 		    points[3].p_x = points[0].p_x + xround(delx);
@@ -1264,21 +1260,21 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 	    case 0x2:	/* se */	/* order: (bottom, right), (top, left) */
 		if (c->r_ybot > r->r_ybot)	/* clip BOTTOM */
 		{
-		    delx = (c->r_ybot - points[1].p_y) * width;
+		    delx = (dlong)(c->r_ybot - points[1].p_y) * width;
 		    points[1].p_y = c->r_ybot;
 		    points[0].p_y = c->r_ybot;
 		    points[0].p_x += xround(delx);
 		}
 		if (c->r_xtop < r->r_xtop)	/* clip RIGHT */
 		{
-		    dely = (points[2].p_x - c->r_xtop) * height;
+		    dely = (dlong)(points[2].p_x - c->r_xtop) * height;
 		    points[1].p_x = c->r_xtop;
 		    points[2].p_x = c->r_xtop;
 		    points[2].p_y -= yround(dely);
 		}
 		if (c->r_ytop < points[2].p_y)	/* clip TOP */
 		{
-		    delx = (points[2].p_y - c->r_ytop) * width;
+		    delx = (dlong)(points[2].p_y - c->r_ytop) * width;
 		    points[2].p_y = c->r_ytop;
 		    points[3].p_y = c->r_ytop;
 		    points[3].p_x = points[2].p_x - xround(delx);
@@ -1291,7 +1287,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		    }
 		    else if (c->r_xbot > points[0].p_x) /* clip LEFT, pentagon */
 		    {
-			dely = (c->r_xbot - points[0].p_x) * height;
+			dely = (dlong)(c->r_xbot - points[0].p_x) * height;
 			points[0].p_x = c->r_xbot;
 			points[4].p_x = c->r_xbot;
 			points[4].p_y = points[0].p_y + yround(dely);
@@ -1300,7 +1296,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		}
 		else if (c->r_xbot > points[0].p_x) /* clip LEFT, triangle */
 		{
-		    dely = (c->r_xbot - points[0].p_x) * height;
+		    dely = (dlong)(c->r_xbot - points[0].p_x) * height;
 		    points[0].p_x = c->r_xbot;
 		    points[3].p_x = c->r_xbot;
 		    points[3].p_y = points[0].p_y + yround(dely);
@@ -1313,21 +1309,21 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 	    case 0x3:	/* ne */	/* order: (top, right), (bottom, left) */
 		if (c->r_xtop < r->r_xtop)	/* clip RIGHT */
 		{
-		    dely = (points[1].p_x - c->r_xtop) * height;
+		    dely = (dlong)(points[1].p_x - c->r_xtop) * height;
 		    points[1].p_x = c->r_xtop;
 		    points[0].p_x = c->r_xtop;
 		    points[0].p_y += yround(dely);
 		}
 		if (c->r_ytop < r->r_ytop)	/* clip TOP */
 		{
-		    delx = (points[2].p_y - c->r_ytop) * width;
+		    delx = (dlong)(points[2].p_y - c->r_ytop) * width;
 		    points[1].p_y = c->r_ytop;
 		    points[2].p_y = c->r_ytop;
 		    points[2].p_x += xround(delx);
 		}
 		if (c->r_xbot > points[2].p_x)	/* clip LEFT */
 		{
-		    dely = (c->r_xbot - points[2].p_x) * height;
+		    dely = (dlong)(c->r_xbot - points[2].p_x) * height;
 		    points[2].p_x = c->r_xbot;
 		    points[3].p_x = c->r_xbot;
 		    points[3].p_y = points[2].p_y - yround(dely);
@@ -1340,7 +1336,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		    }
 		    else if (c->r_ybot > points[0].p_y) /* clip BOTTOM, pentagon */
 		    {
-			delx = (c->r_ybot - points[0].p_y) * width;
+			delx = (dlong)(c->r_ybot - points[0].p_y) * width;
 			points[0].p_y = c->r_ybot;
 			points[4].p_y = c->r_ybot;
 			points[4].p_x = points[0].p_x - xround(delx);
@@ -1349,7 +1345,7 @@ GrClipTriangle(r, c, clipped, dir, side, points, np)
 		}
 		else if (c->r_ybot > points[0].p_y) /* clip BOTTOM, quadrangle */
 		{
-		    delx = (c->r_ybot - points[0].p_y) * width;
+		    delx = (dlong)(c->r_ybot - points[0].p_y) * width;
 		    points[0].p_y = c->r_ybot;
 		    points[3].p_y = c->r_ybot;
 		    points[3].p_x = points[0].p_x - xround(delx);
@@ -1485,7 +1481,7 @@ GrDiagonal(prect, dir, side)
     }
 }
 
-#endif NONMANHATTAN
+#endif /* NONMANHATTAN */
 
 /*---------------------------------------------------------
  * GrClipBox:
