@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/graphics/grMain.c,v 1.8 2001/04/13 20:02:59 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 /*
  * The following display types are currently suported by Magic:
@@ -94,9 +94,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/graphics/grMain.c,v 1.8 2001
 #ifdef SUNVIEW
 #include <pixrect/pixrect.h>
 #endif
-#endif	sun
-
-#include <varargs.h>
+#endif	/* sun */
 
 extern char *getenv();
 extern int errno;
@@ -132,10 +130,10 @@ static char *grDisplayTypes[] = {
     "SUN110",
     "SUN160",
     "SUNBW",
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
 #ifdef	SUN120
     "SUN120",
-#endif	SUN120
+#endif	/* SUN120 */
 #ifdef	X11
     "XWIND",
     "X11", 	
@@ -160,14 +158,14 @@ static (*(grInitProcs[]))() = {
     sunWSetDisplay,
     sunWSetDisplay,
     sunWSetDisplay,
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
 #ifdef	SUN120
     sunSetDisplay,
-#endif	SUN120
+#endif	/* SUN120 */
 #ifdef	X11
     x11SetDisplay,  
     x11SetDisplay,  
-#endif	X11
+#endif	/* X11 */
 #ifdef  OGL
     oglSetDisplay,
     oglSetDisplay,
@@ -207,6 +205,7 @@ Void (*GrDamagedPtr)()		= NULL;
 Void (*GrUpdateIconPtr)()   	= NULL;
 bool (*GrEventPendingPtr)()   	= NULL;
 int (*GrWindowIdPtr)()   	= NULL;
+char *(*GrWindowNamePtr)()	= NULL;
 bool (*GrGetCursorPosPtr)()   	= NULL;
 
 
@@ -226,7 +225,7 @@ Void (*grSetLineStylePtr)()	= NULL;
 Void (*grSetCharSizePtr)()	= NULL;
 #ifdef NONMANHATTAN
 Void (*grFillPolygonPtr)()      = NULL;
-#endif NONMANHATTAN
+#endif /* NONMANHATTAN */
 
 /* The following variables are set by initialization routines for the
  * various displays.  They are strings that indicate what kind of
@@ -358,6 +357,61 @@ char *mouseName;		/* Filename used for communciation
     return res;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * GrIsDisplay --
+ *
+ *	Check if the first argument is the same type of display as the
+ *	second argument.
+ *
+ * Results:
+ *	TRUE if both strings represent the same display type, FALSE
+ *	otherwise.  "same display type" is defined as both display
+ *	strings in the grDisplayTypes list corresponding to the same
+ *	initialization procedure in grInitProcs.
+ *
+ * Side Effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+bool
+GrIsDisplay(disp1, disp2)
+    char *disp1, *disp2;
+{
+    char **ptr1, **ptr2;
+    int i, j;
+
+    /* See if the display type is in our table. */
+    ptr1 = grDisplayTypes;
+    for (i = 0; *ptr1; i++)
+    {
+	if (strncmp(*ptr1, disp1, strlen(*ptr1)) == 0) break;
+	ptr1++;
+    }
+    if (*ptr1 == NULL)
+    {
+	TxError("Unknown display type:  %s\n", disp1);
+	return FALSE;
+    }
+
+    ptr2 = grDisplayTypes;
+    for (j = 0; *ptr2; j++)
+    {
+	if (strncmp(*ptr2, disp2, strlen(*ptr2)) == 0) break;
+	ptr2++;
+    }
+    if (*ptr2 == NULL)
+    {
+	TxError("Unknown display type:  %s\n", disp2);
+	return FALSE;
+    }
+
+    if (grInitProcs[i] == grInitProcs[j]) return TRUE;
+    return FALSE;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -399,9 +453,9 @@ grSunFbDepth(device)
     depth = pr->pr_depth;
     pr_close(pr);
     return depth;
-#else	SUNVIEW
+#else	/* SUNVIEW */
     return 0;
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
 }
 
 
@@ -410,7 +464,7 @@ grSunFbDepth(device)
  * GrGuessDisplayType --
  *
  *	Try to guess what sort of machine we are on, and set the display
- *	ports and type appropriately.  This info is overrided by
+ *	ports and type appropriately.  This info is overridden by
  *	$CAD_HOME/lib/displays and by command line switches.
  *
  * Results:
@@ -455,7 +509,7 @@ GrGuessDisplayType(graphics, mouse, display, monitor)
 	    have2sunScreens = TRUE;
 	}
     }
-#endif	sun
+#endif	/* sun */
 
     if (haveX)
     {
@@ -492,7 +546,7 @@ GrGuessDisplayType(graphics, mouse, display, monitor)
 	        (void) close(fd);
 	    }
 	}
-#endif	sun
+#endif	/* sun */
     } 
     else if (onSun) {
 	/* CATCH ALL FOR SUNs: probably a black & white sun */
