@@ -886,7 +886,7 @@ GrX11Create(w)
 	    wmhints.input = True;
 	    XSetWMHints(grXdpy, wind, &wmhints);
         }
-#endif	sun
+#endif	/* sun */
 
 	/*
 	 * Signal xhelper to poll window.
