@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/graphics/grSunW1.c,v 1.3 2001/01/12 22:12:47 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #ifdef  sun
 
@@ -313,7 +313,7 @@ GrSunFigureLocation(rect)
 
 #ifdef	notdef	/* Bypass bug in wmgr_figuretoolrect */
     wmgr_figuretoolrect(fileno(grSunRootWindow), &r);
-#endif	notdef
+#endif	/* notdef */
 
 #ifdef	stillstrange
     wmgr_get_placeholders(&r, &rdummy);	/* Should fix figuretoolrect bug */
@@ -323,7 +323,7 @@ GrSunFigureLocation(rect)
     rect->r_xtop = r.r_left + r.r_width - 1;
     rect->r_ybot = INVERTY(r.r_top + r.r_height - 1);
     rect->r_ytop = INVERTY(r.r_top);
-#endif	stillstrange
+#endif	/* stillstrange */
 
 /* Cheap fix for wmgr_figuretoolrect problem */
     /*
@@ -859,4 +859,4 @@ sunWSetDisplay(type, colorScreenName, mouseName)
     return TRUE;
 }
 
-#endif	 sun
+#endif	 /* sun */
