@@ -1,4 +1,6 @@
 /* grTOGL1.c
+ *
+ * Copyright (C) 2003 Open Circuit Design, Inc., for MultiGiG Ltd.
  *
  * This file contains primitive functions for OpenGL running under
  * an X window system in a Tcl/Tk interpreter environment
@@ -40,17 +42,16 @@
 #include "database/database.h"
 #include "drc/drc.h"
 #include "macros/macros.h"
-#include "grTOGLInt.h"
+#include "graphics/grTOGLInt.h"
 #include "misc/paths.h"
+#include "graphics/grTkCommon.h"
 
 GLubyte		*grTOGLStipples[GR_NUM_STIPPLES];
-Display		*grXdpy;
 HashTable	grTOGLWindowTable;
 GLXContext	grXcontext;
-int		grXscrn;
 XVisualInfo	*grVisualInfo;
 
-TOGL_CURRENT toglCurrent= {(XFontStruct *)NULL, 0, 0, 0, 0,
+TOGL_CURRENT toglCurrent= {(Tk_Font)0, 0, 0, 0, 0,
 	(Tk_Window)0, (Window)0, (MagWindow *)NULL};
 
 extern bool TxTkConsole;
@@ -68,6 +69,7 @@ extern Void GrTOGLDelete(),GrTOGLConfigure(),GrTOGLRaise(),GrTOGLLower();
 extern Void GrTOGLLock(),GrTOGLUnlock(),GrTOGLIconUpdate();
 extern bool GrTOGLEventPending(), GrTOGLCreate(), grtoglGetCursorPos();
 extern int  GrTOGLWindowId();
+extern char *GrTkWindowName();
 
 /*---------------------------------------------------------
  * grtoglSetWMandC:
@@ -236,20 +238,31 @@ grtoglSetStipple (stipple)
 
 
 
-/*---------------------------------------------------------
+/*------------------------------------------------------------------------
  * GrTOGLInit:
  *	GrTOGLInit initializes the graphics display and clears its screen.
  *	Files must have been previously opened with GrSetDisplay();
  *
  * Results: TRUE if successful.
- *---------------------------------------------------------
+ *
+ * Notes: When 3D rendering is compiled in, we search for a double-buffered
+ *	configuration first, because it generates the smoothest graphics,
+ *	and fall back on a single-buffered configuration if necessary. 
+ *	For normal, 2D-only rendering, we look for a single-buffered
+ *	configuration first because we don't use the back buffer, so a
+ *	double-buffered configuration just wastes space.
+ *------------------------------------------------------------------------
  */
 
 bool
 GrTOGLInit ()
 {
     bool rstatus;
+#ifdef THREE_D
+    static int attributeList[] = { GLX_RGBA, GLX_DOUBLEBUFFER, None };
+#else
     static int attributeList[] = { GLX_RGBA, None, None };
+#endif
 
     toglCurrent.window = Tk_MainWindow(magicinterp);
     if (toglCurrent.window == NULL)
@@ -263,11 +276,17 @@ GrTOGLInit ()
     toglCurrent.depth = Tk_Depth(toglCurrent.window);
 
     grXscrn = DefaultScreen(grXdpy);
+
     grVisualInfo = glXChooseVisual(grXdpy, grXscrn, attributeList);
+
     if (!grVisualInfo)
     {
 	/* Try for a double-buffered confguration */
+#ifdef THREE_D
+	attributeList[1] = None;
+#else
 	attributeList[1] = GLX_DOUBLEBUFFER;
+#endif
 	grVisualInfo = glXChooseVisual(grXdpy, grXscrn, attributeList);
 	if (!grVisualInfo)
 	{
@@ -300,7 +319,7 @@ GrTOGLInit ()
 
     HashInit(&grTOGLWindowTable,8,HT_WORDKEYS);
 
-    return grtoglPreLoadFont();
+    return grTkLoadFont();
 }
 
 /*---------------------------------------------------------
@@ -317,6 +336,8 @@ GrTOGLClose ()
 {
     if (grXdpy == NULL) return;
     if (grVisualInfo != NULL) XFree(grVisualInfo);
+ 
+    grTkFreeFonts();
 
     /* Pop down Tk window but let Tcl/Tk */
     /* do XCloseDisplay()		 */
@@ -474,7 +495,7 @@ TOGLEventProc(clientData, xevent)
 		    SigInterruptOnSigIO = true;
 		    SigInterruptPending = false;
 
-		    (void) WindSendCommand((WindClient) NULL, &TxCurCommand);
+		    (void) WindSendCommand(mw, &TxCurCommand);
 		    WindUpdate();
 
 		    if (SigInterruptPending)
@@ -513,6 +534,14 @@ TOGLEventProc(clientData, xevent)
 		{
 		    keywstate = (keymod << 16) | (keysym & 0xffff);
 		}
+		else if (!strncmp(XKeysymToString(keysym), "KP_", 3))
+		{
+		    /* keypad key (special case---would like to		*/
+		    /* differentiate between shift-KP-# and # itself)	*/
+		    keymod &= ~ShiftMask;
+		    keywstate = (keymod << 16) | (keysym & 0xffff);
+		    nbytes = 0;
+		}
 		else			/* ASCII-valued character */
 		{
 		    if (!(keymod & (LockMask | Mod1Mask))) {
@@ -664,14 +693,12 @@ TOGLEventProc(clientData, xevent)
 			}
 			else
 			{
+			    int sl = strlen(macroDef);
+
 			    if (iMacro)
 			    {
-				int sl;
-
 				/* Echo macro to interpreter, then redirect keys */
 
-				sl = strlen(macroDef);
-
 				if (TxTkConsole)
 				{
 				    char *outstring = Tcl_Alloc(sl + 20);
@@ -693,10 +720,17 @@ TOGLEventProc(clientData, xevent)
 			    {
 				/* TxParseString is defined by magic/magicwrap.i */
 				/* and calls Tcl_Eval()				 */
+
+				char *outstring = Tcl_Alloc(sl +
+					 strlen(Tk_PathName(tkwind)) + 2);
+				sprintf(outstring, "%s %s", 
+					Tk_PathName(tkwind),
+					macroDef);
 				TxSetPoint(KeyPressedEvent->x,
 					glTransY(mw, KeyPressedEvent->y),
 					mw->w_wid);
-				TxParseString(macroDef, NULL, NULL);
+				TxParseString(outstring, NULL, NULL);
+				Tcl_Free(outstring);
 			    }
 			    freeMagic(macroDef);
 			}
@@ -792,6 +826,7 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     char *fullname;
     FILE* f;
     bool execFailed = false;
+    int x, y, width, height;
 
     WindPackageType = WIND_X_WINDOWS;  /* to be changed? */
     TxInputRedirect = False;
@@ -825,12 +860,13 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     GrUpdateIconPtr = GrTOGLIconUpdate; 
     GrEventPendingPtr = GrTOGLEventPending;
     GrWindowIdPtr = GrTOGLWindowId;
+    GrWindowNamePtr = GrTkWindowName;		/* from grTkCommon.c */
     GrGetCursorPosPtr = grtoglGetCursorPos;
 
     /* local indirections */
     grSetSPatternPtr = grtoglSetSPattern;
     grPutTextPtr = grtoglPutText;
-    grDefineCursorPtr = grtoglDefineCursor;
+    grDefineCursorPtr = grTkDefineCursor;
     GrBitBltPtr = GrTOGLBitBlt;
     grDrawGridPtr = grtoglDrawGrid;
     grDrawLinePtr = grtoglDrawLine;
@@ -853,10 +889,11 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
 	return false;
     };
 
-    GrScreenRect.r_xbot = 0;
-    GrScreenRect.r_ybot = 0;
-    GrScreenRect.r_xtop = DisplayWidth(grXdpy,grXscrn);
-    GrScreenRect.r_ytop = DisplayHeight(grXdpy,grXscrn);
+    Tk_GetVRootGeometry(Tk_MainWindow(magicinterp), &x, &y, &width, &height);
+    GrScreenRect.r_xbot = x;
+    GrScreenRect.r_ybot = y;
+    GrScreenRect.r_xtop = width + x;
+    GrScreenRect.r_ytop = height + y;
 
     return Tk_MainWindow(magicinterp) ? true : false;
 }
@@ -983,7 +1020,7 @@ GrTOGLCreate(w, name)
 	toglCurrent.windowid = wind;
 	glXMakeCurrent(grXdpy, (GLXDrawable)wind, grXcontext);
 
-        XDefineCursor(grXdpy, wind, toglCurrent.cursor);
+        Tk_DefineCursor(tkwind, toglCurrent.cursor);
 	GrTOGLIconUpdate(w, w->w_caption); 
 
 	WindowNumber++;
@@ -1000,7 +1037,7 @@ GrTOGLCreate(w, name)
 
 	/* set up commands to be passed expressly to this window */  
 
-	MakeWindowCommand((name == NULL) ? windowname : name);
+	MakeWindowCommand((name == NULL) ? windowname : name, w);
 
 	return (WindowNumber == 1) ? grtoglLoadFont() : 1;
     }
@@ -1085,12 +1122,12 @@ GrTOGLConfigure(w)
 GrTOGLRaise(w)
     MagWindow *w;
 {
-    Window wind;
+    Tk_Window tkwind;
 
     if (w->w_flags & WIND_OFFSCREEN) return;
 
-    wind = Tk_WindowId((Tk_Window)w->w_grdata);
-    XRaiseWindow(grXdpy, wind);  /* See Blt code */
+    tkwind = (Tk_Window)w->w_grdata;
+    Tk_RestackWindow(tkwind, Above, NULL);
 }
 
 /*
@@ -1112,12 +1149,12 @@ GrTOGLRaise(w)
 GrTOGLLower(w)
     MagWindow *w;
 {
-    Window wind;
+    Tk_Window tkwind;
 
     if (w->w_flags & WIND_OFFSCREEN) return;
 
-    wind = Tk_WindowId((Tk_Window)w->w_grdata);
-    XLowerWindow(grXdpy, wind);  /* See Blt code */
+    tkwind = (Tk_Window)w->w_grdata;
+    Tk_RestackWindow(tkwind, Below, NULL);
 }
 
 
