@@ -1,16 +1,6 @@
 /* grTOGL3.c -
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
- *     *********************************************************************
+ * Copyright 2003 Open Circuit Design, Inc., for MultiGiG Ltd.
  *
  * This file contains additional functions to manipulate an X window system
  * color display.  Included here are device-dependent routines to draw and 
@@ -33,17 +23,13 @@
 #include "signals/signals.h"
 #include "utils/utils.h"
 #include "utils/hash.h"
-#include "grTOGLInt.h"
+#include "graphics/grTOGLInt.h"
+#include "graphics/grTkCommon.h"
 
 extern Display *grXdpy;
 
 /* locals */
 
-static XFontStruct *grXFonts[4];
-#define	grSmallFont	grXFonts[0]
-#define	grMediumFont	grXFonts[1]
-#define	grLargeFont	grXFonts[2]
-#define	grXLargeFont	grXFonts[3]
 GLuint  grXBases[4];
 
 
@@ -116,59 +102,6 @@ grtoglDrawGrid (prect, outline, clip)
     return true;
 }
 
-
-/*---------------------------------------------------------
- * grtoglPreLoadFont
- *	This local routine loads the X fonts used by Magic.
- *	At this time, we need the font information to size
- *	the window (making room for the title at the top),
- *	but with no rendering context set (no window to
- *	draw to) we defer transferring the font bitmaps to
- *	OpenGL display lists until later.
- *
- * Results:	None.
- *
- * Side Effects:    None.
- *---------------------------------------------------------
- */
-
-Void
-grtoglPreLoadFont()
-{
-    XFontStruct *fontInfo;
-    int i;
-    char *s;
-    char *unable = "Unable to load font";
-
-    static char *fontnames[4] = {
-      TK_FONT_SMALL,
-      TK_FONT_MEDIUM,
-      TK_FONT_LARGE,
-      TK_FONT_XLARGE };
-    static char *optionnames[4] = {
-      "small",
-      "medium",
-      "large",
-      "xlarge"};
-
-    for (i = 0; i < 4; i++)
-    {
-    	s = XGetDefault(grXdpy,"magic",optionnames[i]);
-	if (s) fontnames[i] = s;
-	if ((fontInfo = XLoadQueryFont(grXdpy, fontnames[i])) == NULL)
-	{
-	    TxError("%s %s\n",unable,fontnames[i]);
-            if ((grXFonts[i]= XLoadQueryFont(grXdpy,GR_DEFAULT_FONT))==NULL) 
-	    {
-		TxError("%s %s\n",unable,GR_DEFAULT_FONT);
-		return false;
-	    }
-        }
-	grXFonts[i] = fontInfo;
-    }
-    return true;
-}
-
 
 /*---------------------------------------------------------
  * grtoglLoadFont
@@ -185,22 +118,18 @@ grtoglPreLoadFont()
 bool
 grtoglLoadFont()
 {
-    XFontStruct *fontInfo;
     Font id;
-    unsigned int first, last, i;
+    unsigned int i;
 
     for (i = 0; i < 4; i++) {
-	fontInfo = grXFonts[i];
-	id = fontInfo->fid;
-	first = fontInfo->min_char_or_byte2;
-	last = fontInfo->max_char_or_byte2;
+	id = Tk_FontId(grTkFonts[i]);
 
-	grXBases[i] = glGenLists(last+1);
+	grXBases[i] = glGenLists(256);
 	if (grXBases[i] == 0) {
 	    TxError("Out of display lists!\n");
 	    return false;
 	}
-	glXUseXFont(id, first, last-first+1, grXBases[i]+first);
+	glXUseXFont(id, 0, 256, grXBases[i]);
     }
     return true;
 }
@@ -267,9 +196,9 @@ GrTOGLTextSize(text, size, r)
     int size;
     Rect *r;
 {
-    XCharStruct overall;
-    XFontStruct *font;
-    int dir,fa,fd;
+    Tk_FontMetrics overall;
+    Tk_Font font;
+    int width;
     
     switch (size) {
     case GR_TEXT_DEFAULT:
@@ -291,11 +220,12 @@ GrTOGLTextSize(text, size, r)
 	break;
     }
     if (font == NULL) return;
-    XTextExtents(font, text, strlen(text), &dir, &fa, &fd, &overall);
+    Tk_GetFontMetrics(font, &overall);
+    width = Tk_TextWidth(font, text, strlen(text));
     r->r_ytop = overall.ascent;
     r->r_ybot = -overall.descent;
-    r->r_xtop = overall.width - overall.lbearing;
-    r->r_xbot = -overall.lbearing - 1;
+    r->r_xtop = width;
+    r->r_xbot = 0;
 }
 
 
