@@ -1,16 +1,6 @@
 /* grTk1.c
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
- *     *********************************************************************
+ * Copyright 2003 Open Circuit Design, Inc., for MultiGiG Ltd.
  *
  * This file contains primitive functions to manipulate an X window system
  * Included here are initialization and closing
@@ -51,13 +41,10 @@
 #include "macros/macros.h"
 #include "grTkInt.h"
 #include "misc/paths.h"
-
-Display *grXdpy;
-int	grXscrn;
+#include "graphics/grTkCommon.h"
 
 GR_CURRENT grCurrent = {
-	(XFontStruct *)NULL,
-	0, 0, 0, 0, 0,
+	(Tk_Font)0, 0, 0, 0, 0, 0,
 	(MagWindow *)NULL
 };
 
@@ -91,7 +78,7 @@ extern bool GrTkInstalledCMap;
 extern bool TxTkConsole;
 
 /* machine-dependent constants - see below */
-#ifdef macII
+#ifdef macosx
 #define X_COLORMAP_BASE		128
 #define X_COLORMAP_RESERVED	4
 #else
@@ -117,6 +104,7 @@ extern Void GrTkDelete(),GrTkConfigure(),GrTkRaise(),GrTkLower();
 extern Void GrTkLock(),GrTkUnlock(),GrTkIconUpdate();
 extern bool GrTkEventPending(), GrTkCreate(), grtkGetCursorPos();
 extern int  GrTkWindowId();
+extern char *GrTkWindowName();
 
 
 /*---------------------------------------------------------
@@ -274,11 +262,14 @@ grtkSetSPattern (stipple, pattern)
     int stipple;			/* The stipple number, 1-15. */
     int pattern[8];			/* 8 8-bit patterns integers */
 {
+    Tk_Window tkwind;
     Pixmap p;
     int x,y,pat;
 
-    p = XCreatePixmap(grXdpy, XDefaultRootWindow(grXdpy), 8, 8, 1);
+    tkwind = Tk_MainWindow(magicinterp);
+    p = Tk_GetPixmap(grXdpy, Tk_WindowId(tkwind), 8, 8, 1);
     if (grGCStipple == 0) {
+	/* grGCStipple = Tk_GetGC(tkwind, 0, 0); */
 	grGCStipple = XCreateGC(grXdpy, p, 0, 0);
     }
     for (y = 0; y < 8; y++) {
@@ -645,7 +636,7 @@ GrTkInit ()
     grBitPlaneMask = (1 << grDisplay.depth) - 1;
 
     HashInit(&grTkWindowTable,8,HT_WORDKEYS);
-    rstatus = grtkLoadFont();
+    rstatus = grTkLoadFont();
     return rstatus;
 }
 
@@ -661,10 +652,15 @@ GrTkInit ()
 Void
 GrTkClose ()
 {
+    int i;
+
     if (grXdpy == NULL) return;
 
     grGCStipple = 0;
 
+    /* Free Tk fonts from the font cache */
+    grTkFreeFonts();
+
     /* XCloseDisplay(grXdpy); */  /* Pop down Tk window but let Tcl/Tk	*/
 				  /* do XCloseDisplay()			*/
 }
@@ -769,7 +765,7 @@ MagicEventProc(clientData, xevent)
 		    SigInterruptOnSigIO = true;
 		    SigInterruptPending = false;
 
-		    (void) WindSendCommand((WindClient) NULL, &TxCurCommand);
+		    (void) WindSendCommand(mw, &TxCurCommand);
 		    WindUpdate();
 
 		    if (SigInterruptPending)
@@ -806,10 +802,19 @@ MagicEventProc(clientData, xevent)
 		keymod = (LockMask | ControlMask | ShiftMask | Mod1Mask)
 				& KeyPressedEvent->state;
 
-		if (nbytes == 0)	/* No ASCII equivalent */
+		if (nbytes == 0)
 		{
+		    /* No ASCII equivalent */
 		    keywstate = (keymod << 16) | (keysym & 0xffff);
 		}
+		else if (!strncmp(XKeysymToString(keysym), "KP_", 3))
+		{
+		    /* keypad key (special case---would like to		*/
+		    /* differentiate between shift-KP-# and # itself)	*/
+		    keymod &= ~ShiftMask;
+		    keywstate = (keymod << 16) | (keysym & 0xffff);
+		    nbytes = 0;
+		}
 		else			/* ASCII-valued character */
 		{
 		    if (!(keymod & (LockMask | Mod1Mask))) {
@@ -965,14 +970,11 @@ MagicEventProc(clientData, xevent)
 			}
 			else
 			{
+			    int sl = strlen(macroDef);
 			    if (iMacro)
 			    {
-				int sl;
-
 				/* Echo macro to interpreter, then redirect keys */
 
-				sl = strlen(macroDef);
-
 				if (TxTkConsole)
 				{
 				    char *outstring = Tcl_Alloc(sl + 20);
@@ -994,10 +996,15 @@ MagicEventProc(clientData, xevent)
 			    {
 				/* TxParseString is defined by magic/magicwrap.i */
 				/* and calls Tcl_Eval()				 */
+				char *outstring = Tcl_Alloc(sl +
+					strlen(Tk_PathName(wind)) + 2);
+				sprintf(outstring, "%s %s",
+					Tk_PathName(wind), macroDef);
 				TxSetPoint(KeyPressedEvent->x,
-						grXToMagic(KeyPressedEvent->y),
-						grCurrent.mw->w_wid);
-				TxParseString(macroDef, NULL, NULL);
+					grXToMagic(KeyPressedEvent->y),
+					grCurrent.mw->w_wid);
+				TxParseString(outstring, NULL, NULL);
+				Tcl_Free(outstring);
 			    }
 			    freeMagic(macroDef);
 			}
@@ -1091,8 +1098,9 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     char *fullname;
     FILE* f;
     bool execFailed = false;
+    int x, y, width, height;
 
-    WindPackageType = WIND_X_WINDOWS;  /* to be changed? */
+    WindPackageType = WIND_X_WINDOWS;
     TxInputRedirect = False;
     grCursorType = "bw";
     WindScrollBarWidth = 14;
@@ -1121,12 +1129,13 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     GrUpdateIconPtr = GrTkIconUpdate; 
     GrEventPendingPtr = GrTkEventPending;
     GrWindowIdPtr = GrTkWindowId;
+    GrWindowNamePtr = GrTkWindowName;
     GrGetCursorPosPtr = grtkGetCursorPos;
 
     /* local indirections */
     grSetSPatternPtr = grtkSetSPattern;
     grPutTextPtr = grtkPutText;
-    grDefineCursorPtr = grtkDefineCursor;
+    grDefineCursorPtr = grTkDefineCursor;
     GrBitBltPtr = GrTkBitBlt;
     grDrawGridPtr = grtkDrawGrid;
     grDrawLinePtr = grtkDrawLine;
@@ -1145,12 +1154,15 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
 	return false;
     }
 
-    if(!GrTkInit()){
+    if(!GrTkInit()) {
 	return false;
     };
 
-    GrScreenRect.r_xtop = DisplayWidth(grXdpy,grXscrn);
-    GrScreenRect.r_ytop = DisplayHeight(grXdpy,grXscrn);
+    Tk_GetVRootGeometry(Tk_MainWindow(magicinterp), &x, &y, &width, &height);
+    GrScreenRect.r_xbot = x;
+    GrScreenRect.r_ybot = y;
+    GrScreenRect.r_xtop = x + width;
+    GrScreenRect.r_ytop = y + height;
 
     return Tk_MainWindow(magicinterp) ? true : false;
 }
@@ -1289,7 +1301,7 @@ GrTkCreate(w, name)
         }
 
 	XSetPlaneMask(grXdpy, grGCGlyph, AllPlanes);
-        XDefineCursor(grXdpy, wind, grCurrent.cursor);
+        Tk_DefineCursor(tkwind, grCurrent.cursor);
 	GrTkIconUpdate(w, w->w_caption); 
 
 
@@ -1333,7 +1345,7 @@ GrTkCreate(w, name)
 
 	/* set up commands to be passed expressly to this window */
 
-	MakeWindowCommand((name == NULL) ? windowname : name);
+	MakeWindowCommand((name == NULL) ? windowname : name, w);
 
 	/*--------------------------------------------------------------*/
 	/* In a PseudoColor visual + GUI wrapper environment, we need	*/
@@ -1365,6 +1377,11 @@ GrTkCreate(w, name)
 	    if (clist != NULL) XFree(clist);
 	}
 
+	/* Install the colormap in all associated windows, for pseudocolor */
+	/* visuals, if required 					   */
+
+	if (grClass == 3) XInstallColormap(grXdpy, grXcmap);
+
 	return 1;
     }
     else
@@ -1449,12 +1466,12 @@ GrTkConfigure(w)
 GrTkRaise(w)
     MagWindow *w;
 {
-    Window wind;
+    Tk_Window tkwind;
 
     if (w->w_flags & WIND_OFFSCREEN) return;
 
-    wind = Tk_WindowId((Tk_Window)w->w_grdata);
-    XRaiseWindow(grXdpy, wind);  /* See Blt code */
+    tkwind = (Tk_Window)w->w_grdata;
+    Tk_RestackWindow(tkwind, Above, NULL);
 }
 
 /*
@@ -1476,11 +1493,12 @@ GrTkRaise(w)
 GrTkLower(w)
     MagWindow *w;
 {
-    Window wind;
+    Tk_Window tkwind;
+
     if (w->w_flags & WIND_OFFSCREEN) return;
 
-    wind = Tk_WindowId((Tk_Window)w->w_grdata);
-    XLowerWindow(grXdpy, wind);  /* See Blt code */
+    tkwind = (Tk_Window)w->w_grdata;
+    Tk_RestackWindow(tkwind, Below, NULL);
 }
 
 /*
@@ -1639,3 +1657,4 @@ int GrTkWindowId(tkname)
     }
     return id;
 }
+
