@@ -72,7 +72,7 @@ HashTable     grX11WindowTable;
 
 /* machine-dependent constants - see below */
 
-#ifdef macII
+#ifdef macosx
 #define X_COLORMAP_BASE		128
 #define X_COLORMAP_RESERVED	4
 #else
@@ -984,6 +984,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     GrOverWindowPtr = GrX11Raise;
     GrUnderWindowPtr = GrX11Lower;
     GrUpdateIconPtr = GrX11IconUpdate; 
+    GrGetCursorPosPtr = grx11GetCursorPos;
 
     /* local indirections */
     grSetSPatternPtr = grx11SetSPattern;
@@ -1130,7 +1131,7 @@ GrX11Create(w, name)
 	    wmhints.input = True;
 	    XSetWMHints(grXdpy, wind, &wmhints);
         }
-#endif	sun
+#endif	/* sun */
 
 	/*
 	 * Signal xhelper to poll window.
