@@ -23,7 +23,7 @@
 
 #ifndef lint
 static char sccsid[] = "@(#)grouteNet.c	4.3 MAGIC (Berkeley) 12/6/85";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -523,7 +523,7 @@ glFinalPropagate(inPt, loc)
 	if (!glJogsAcrossChannel(inPt->gl_pin, destPin))
 	    return (FALSE);
     }
-#endif	notdef
+#endif	/* notdef */
 
     cost = glCrossPenalty(cost, destCh, (GCRChannel *) NULL,
 		    inPt->gl_pin, destPin);
@@ -743,7 +743,7 @@ glNormalPropagate(inPt, inCh, heapCost)
 	inPin->gcr_pId = GCR_BLOCKEDNETID;
 	if (inPin->gcr_linked)
 	    inPin->gcr_linked->gcr_pId = GCR_BLOCKEDNETID;
-#endif	notdef
+#endif	/* notdef */
 	return;
     }
 
@@ -1345,7 +1345,7 @@ glPropagateFn(outCh, outPin, inPt)
 #ifdef	notdef
     if (outPin->gcr_point.p_x == iPt.p_x && outPin->gcr_point.p_y == iPt.p_y)
 	TxPrintf("Bingo! cost=%d\n", cost);
-#endif	notdef
+#endif	/* notdef */
     outPin->gcr_cost = cost;
     if (outPin->gcr_linked)
 	outPin->gcr_linked->gcr_cost = cost;
