@@ -19,7 +19,7 @@
 
 #ifndef	lint
 static char rcsid[] = "$Header: /ufs/repository/magic/grouter/groutePen.c,v 1.2 2001/01/12 22:12:52 jsolomon Exp $";
-#endif	lint
+#endif	/* lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -237,7 +237,7 @@ glPenCompute(chanList, netList)
 	glDMCopy(&gc->gc_prevDens[CZ_COL], &gc->gc_postDens[CZ_COL]);
 	glDMCopy(&gc->gc_prevDens[CZ_ROW], &gc->gc_postDens[CZ_ROW]);
     }
-#endif	notdef
+#endif	/* notdef */
 }
 
 /*
