@@ -24,7 +24,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/irouter/irCommand.c,v 1.2 2001/01/12 22:12:53 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
