@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/irouter/irRoute.c,v 1.2 2001/01/12 22:12:53 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 /* --- Includes --- */
 
