@@ -20,6 +20,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <unistd.h>
 #include <signal.h>
 #include <string.h>
@@ -39,6 +40,7 @@
 #include "signals/signals.h"
 #include "graphics/graphics.h"
 #include "utils/malloc.h" 
+#include "dbwind/dbwind.h"
 
 /*
  * String containing the version number of magic.  Don't change the string
@@ -72,14 +74,19 @@ static int TagCallback(interp, tkpath, argc, argv)
 {
     int argidx, result = TCL_OK;
     char *postcmd, *substcmd, *newcmd, *sptr, *sres;
-    char *croot = argv[0];
+    char *croot;
     HashEntry *entry;
     Tcl_SavedResult state;
     bool reset = FALSE;
     int cmdnum;
 
+    /* No command, no action */
+
+    if (argc == 0) return TCL_OK;
+
     /* Skip over namespace qualifier, if any */
 
+    croot = argv[0];
     if (!strncmp(croot, "::", 2)) croot += 2;
     if (!strncmp(croot, "magic::", 7)) croot += 7;
 
@@ -118,12 +125,12 @@ static int TagCallback(interp, tkpath, argc, argv)
 		    /* path has been passed in the argument.  If not, get */
 		    /* the window path if there is only one window.       */
 		    /* Otherwise, the window is unknown so we substitute  */
-		    /* nothing.						  */ 
+		    /* a null list "{}".				  */ 
 
 		    if (tkpath == NULL)
 		    {
 			MagWindow *w = NULL;
-			windCheckOnlyWindow(&w);
+			windCheckOnlyWindow(&w, DBWclientID);
 			if (w != NULL && !(w->w_flags & WIND_OFFSCREEN))
 			{
 			    Tk_Window tkwind = (Tk_Window) w->w_grdata;
@@ -131,19 +138,18 @@ static int TagCallback(interp, tkpath, argc, argv)
 			}
 		    }
 		    if (tkpath == NULL)
-			newcmd = (char *)mallocMagic(strlen(substcmd));
+			newcmd = (char *)mallocMagic(strlen(substcmd) + 2);
 		    else
 			newcmd = (char *)mallocMagic(strlen(substcmd) + strlen(tkpath));
 
 		    strcpy(newcmd, substcmd);
 
 		    if (tkpath == NULL)
-			strcpy(newcmd + (int)(sptr - substcmd), sptr + 2);
+			strcpy(newcmd + (int)(sptr - substcmd), "{}");
 		    else
-		    {
 			strcpy(newcmd + (int)(sptr - substcmd), tkpath);
-			strcat(newcmd, sptr + 2);
-		    }
+
+		    strcat(newcmd, sptr + 2);
 		    freeMagic(substcmd);
 		    substcmd = newcmd;
 		    sptr = substcmd;
@@ -258,15 +264,167 @@ static int AddCommandTag(ClientData clientData,
     return TCL_OK;
 }
 
-/*--------------------------------------*/
-/* Main startup procedure		*/
-/*--------------------------------------*/
- 
-static int _magic_start(ClientData clientData,
+/*--------------------------------------------------------------*/
+/* Dispatch a command from Tcl					*/
+/* See TxTclDispatch() in textio/txCommands.c			*/
+/*--------------------------------------------------------------*/
+
+static int _tcl_dispatch(ClientData clientData,
         Tcl_Interp *interp, int argc, char *argv[])
 {
+    int result, idx;
+    Tcl_Obj *objv0;
+    char *argv0;
+
+    /* Check command (argv[0]) against known conflicting */
+    /* command names.  If the command is potentially a	 */
+    /* Tcl/Tk command, try it as such, first.  If a Tcl	 */
+    /* error is returned, then try it as a magic	 */
+    /* command.  Note that the other way (try the magic	 */
+    /* command first) would necessitate setting Tcl 	 */
+    /* results for every magic command.  Too much work.	 */
+
+    static char *conflicts[] =
+    {
+	"clockwise", "flush", "load", "label", "array", "grid", NULL
+    };
+    static char *resolutions[] =
+    {
+	"orig_clock", "tcl_flush", "tcl_load", "tcl_label", "tcl_array",
+	"tcl_grid", NULL
+    };
+
+    typedef enum
+    {
+	IDX_CLOCK, IDX_FLUSH, IDX_LOAD, IDX_LABEL, IDX_ARRAY,
+	IDX_GRID
+    } conflictCommand;
+
+    /* Skip any "::" namespace prefix before parsing */
+    argv0 = argv[0];
+    if (!strncmp(argv0, "::", 2)) argv0 += 2;
+
+    objv0 = Tcl_NewStringObj(argv0, strlen(argv0));
+    if (Tcl_GetIndexFromObj(interp, objv0, conflicts,
+	"overloaded command", 0, &idx) == TCL_OK)
+    {
+	int i;
+	Tcl_Obj **objv = (Tcl_Obj **)Tcl_Alloc(argc * sizeof(Tcl_Obj *));
+	
+	/* Create a Tcl_Obj array suitable for calling Tcl_EvalObjv.	*/
+	/* The first argument is changed from the magic command name to	*/
+	/* "tcl" + the command name.  This assumes that all conflicting	*/
+	/* command names have been so renamed in the startup script!	*/
+
+	objv[0] = Tcl_NewStringObj(resolutions[idx], strlen(resolutions[idx]));
+	Tcl_IncrRefCount(objv[0]);
+
+	for (i = 1; i < argc; i++)
+	{
+	    objv[i] = Tcl_NewStringObj(argv[i], strlen(argv[i]));
+	    Tcl_IncrRefCount(objv[i]);
+	}
+
+	result = Tcl_EvalObjv(interp, argc, objv, 0);
+
+	for (i = 0; i < argc; i++)
+	    Tcl_DecrRefCount(objv[i]);
+	Tcl_Free((char *)objv);
+
+	if (result == TCL_OK)
+	    return result;
+
+	/* The rule is to execute Magic commands for any Tcl command 	*/
+	/* with the same name that returns an error.  However, this	*/
+	/* rule hangs magic when the "load" command is used on a shared	*/
+	/* object file that fails to load properly.  So if the filename	*/
+	/* has an extension which is not ".mag", we will return the 	*/
+	/* error.							*/
+
+	if (idx == IDX_LOAD)
+	{
+	    char *dotptr;
+	    if ((argc >= 2) && (dotptr = strchr(argv[1], '.')) != NULL)
+		if (strcmp(dotptr + 1, "mag"))
+		    return result;
+	}
+    }
+    Tcl_ResetResult(interp);
+
+    TxInputRedirect = False;
+    TxTclDispatch(clientData, argc, argv);
+    return TagCallback(interp, NULL, argc, argv);
+}
+
+/*--------------------------------------------------------------*/
+/* Dispatch a window-related command.  The first argument is	*/
+/* the window to which the command should be directed, so we	*/
+/* determine which window this is, set "TxCurCommand" values	*/
+/* to point to the window, then dispatch the command.		*/
+/*--------------------------------------------------------------*/
+
+static int _tk_dispatch(ClientData clientData,
+        Tcl_Interp *interp, int argc, char *argv[])
+{
+    int id;
+    char *tkpath;
+    char *arg0;
+    Point txp;
+
+    if (GrWindowIdPtr)
+    {
+	/* Key macros set the point from the graphics module code but	*/
+	/* set up the command to be dispatched via _tk_dispatch().	*/
+	/* Therefore it is necessary to check if a point position	*/
+	/* has already been set for this command.  If not, then the	*/
+	/* command was probably called from the command entry window,	*/
+	/* so we choose an arbitrary point which is somewhere in the	*/
+	/* window, so that command functions have a point of reference.	*/
+
+	id = (*GrWindowIdPtr)(argv[0]);
+
+	if (TxGetPoint(&txp) != id)
+	{
+	    /* This is a point in the window, inside the	*/
+	    /* scrollbars if they are managed by magic.		*/
+
+	    txp.p_x = 20;
+	    txp.p_y = 20;
+	}
+	TxSetPoint(txp.p_x, txp.p_y, id);
+	arg0 = argv[0];
+	argc--;
+	argv++;
+    }
+    TxTclDispatch(clientData, argc, argv);
+
+    /* Get pathname of window and pass to TagCallback */
+    return TagCallback(interp, arg0, argc, argv);
+}
+
+/*--------------------------------------------------------------*/
+/* Set up a window to use commands via _tk_dispatch		*/
+/*--------------------------------------------------------------*/
+
+void
+MakeWindowCommand(char *wname, MagWindow *mw)
+{
+    Tcl_CreateCommand(magicinterp, wname, _tk_dispatch,
+		(ClientData)mw, (Tcl_CmdDeleteProc *) NULL);
+}
+
+/*------------------------------------------------------*/
+/* Main startup procedure				*/
+/*------------------------------------------------------*/
+ 
+static int _magic_initialize(ClientData clientData,
+        Tcl_Interp *interp, int argc, char *argv[])
+{
+    int n;
+    char keyword[100];
+    char *kwptr = keyword + 7;
+    char **commandTable;
     int result;
-    Tcl_ChannelType *inChannel;
 
     /* Is magic being executed in a slave interpreter? */
 
@@ -293,6 +451,65 @@ static int _magic_start(ClientData clientData,
 
     if (mainInitAfterArgs() != 0) goto magicfatal;
 
+    /* Registration of commands is performed after calling the	*/
+    /* start function, not after initialization, as the command */
+    /* modularization requires magic initialization to get a	*/
+    /* valid DBWclientID and windClientID.			*/
+
+    sprintf(keyword, "magic::");
+    commandTable = WindGetCommandTable(DBWclientID);
+    for (n = 0; commandTable[n] != NULL; n++)
+    {
+	sscanf(commandTable[n], "%s ", kwptr); /* get first word */
+	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
+			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
+    }
+
+    commandTable = WindGetCommandTable(windClientID);
+    for (n = 0; commandTable[n] != NULL; n++)
+    {
+	sscanf(commandTable[n], "%s ", kwptr); /* get first word */
+	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
+			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
+    }
+
+    /* Extra commands provided by the Tk graphics routines	*/
+    /* (See graphics/grTkCommon.c)				*/
+
+    RegisterTkCommands(interp);
+    return TCL_OK;
+
+magicfatal:
+    TxResetTerminal();
+    Tcl_SetResult(interp, "Magic initialization encountered a fatal error.", NULL);
+    return TCL_ERROR;
+}
+
+/*--------------------------------------------------------------*/
+/* Post-initialization:  read in the magic startup files and	*/
+/* load any initial layout.  Note that this is not done via	*/
+/* script, but probably should be.				*/
+/*--------------------------------------------------------------*/
+
+static int _magic_startup(ClientData clientData,
+        Tcl_Interp *interp, int argc, char *argv[])
+{
+    Tcl_ChannelType *inChannel;
+
+    /* Execute contents of startup files and load any initial cell */
+
+    if (mainInitFinal() != 0)
+    {
+	/* We don't want mainInitFinal errors to return TCL_ERROR from	*/
+	/* magic::start; otherwise, the window won't come up.  As long	*/
+	/* as we have successfully passed mainInitAfterArgs(), magic is	*/
+	/* fundamentally sound.						*/
+
+	Tcl_SetResult(interp,
+		"Magic encountered problems with the startup files.",
+		NULL);
+    }
+
     TxResetTerminal();
 
     if (TxTkConsole)
@@ -310,65 +527,6 @@ static int _magic_start(ClientData clientData,
     }
 
     return TCL_OK;
-
-magicfatal:
-    TxResetTerminal();
-    Tcl_SetResult(interp, "Magic startup encountered a fatal error.", NULL);
-    return TCL_ERROR;
-}
-
-/*--------------------------------------------------------------*/
-/* Dispatch a command from Tcl					*/
-/* See TxTclDispatch() in textio/txCommands.c			*/
-/*--------------------------------------------------------------*/
-
-static int _tcl_dispatch(ClientData clientData,
-        Tcl_Interp *interp, int argc, char *argv[])
-{
-    TxTclDispatch(clientData, argc, argv);
-    return TagCallback(interp, NULL, argc, argv);
-}
-
-/*--------------------------------------------------------------*/
-/* Dispatch a window-related command.  The first argument is	*/
-/* the window to which the command should be directed, so we	*/
-/* determine which window this is, set "TxCurCommand" values	*/
-/* to point to the window, then dispatch the command.		*/
-/*--------------------------------------------------------------*/
-
-static int _tk_dispatch(ClientData clientData,
-        Tcl_Interp *interp, int argc, char *argv[])
-{
-    int id;
-    char *tkpath;
-    char *arg0;
-
-    if (GrWindowIdPtr)
-    {
-	id = (*GrWindowIdPtr)(argv[0]);
-
-	/* Find the bottommost point of the window? */
-	/* Probably not necessary.  Need to clear the scrollbar, though. */
-	TxSetPoint(20, 20, id);
-	arg0 = argv[0];
-	argc--;
-	argv++;
-    }
-    TxTclDispatch(clientData, argc, argv);
-
-    /* Get pathname of window and pass to TagCallback */
-    return TagCallback(interp, arg0, argc, argv);
-}
-
-/*--------------------------------------------------------------*/
-/* Set up a window to use commands via _tk_dispatch		*/
-/*--------------------------------------------------------------*/
-
-void
-MakeWindowCommand(char *wname)
-{
-    Tcl_CreateCommand(magicinterp, wname, _tk_dispatch,
-		(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
 }
 
 /*--------------------------------------------------------------*/
@@ -465,8 +623,16 @@ TxGetLinePfix(dest, maxChars, prefix)
 
     if (TxTkConsole)
     {
-	/* Use dialog function (must be defined!) */
-	Tcl_Eval(magicinterp, "magic::dialog \"prefix\"\n");
+	/* Use dialog function (must be defined in magic.tcl!)	*/
+        if (prefix != NULL)
+	{
+	    string = Tcl_Alloc(20 + strlen(prefix));
+	    sprintf(string, "magic::dialog \"%s\"\n", prefix);
+	    Tcl_Eval(magicinterp, string);
+	    Tcl_Free(string);
+	}
+	else
+	    Tcl_Eval(magicinterp, "magic::dialog\n");
     }
     else
     {
@@ -544,7 +710,7 @@ TxFlushErr()
     Tcl_SavedResult state;
 
     Tcl_SaveResult(magicinterp, &state);
-    Tcl_Eval(magicinterp, "::flush stderr");
+    Tcl_Eval(magicinterp, "::tcl_flush stderr");
     Tcl_RestoreResult(magicinterp, &state);
 }
 
@@ -556,7 +722,7 @@ TxFlushOut()
     Tcl_SavedResult state;
 
     Tcl_SaveResult(magicinterp, &state);
-    Tcl_Eval(magicinterp, "::flush stdout");
+    Tcl_Eval(magicinterp, "::tcl_flush stdout");
     Tcl_RestoreResult(magicinterp, &state);
 }
 
@@ -577,11 +743,9 @@ TxFlush()
 /*--------------------------------------------------------------*/
 
 int
-Tcl_printf(f, fmt, args)
-    FILE *f;
-    char *fmt;
-    va_list args;
+Tcl_printf(FILE *f, char *fmt, va_list args_in)
 {
+    va_list args;
     static char outstr[128] = "puts -nonewline std";
     char *outptr, *bigstr = NULL, *finalstr = NULL;
     int i, nchars, result, escapes = 0, limit;
@@ -589,14 +753,19 @@ Tcl_printf(f, fmt, args)
 
     strcpy (outstr + 19, (f == stderr) ? "err \"" : "out \"");
 
+    va_copy(args, args_in);
     outptr = outstr;
     nchars = vsnprintf(outptr + 24, 102, fmt, args);
+    va_end(args);
+
     if (nchars >= 102)
     {
+	va_copy(args, args_in);
 	bigstr = Tcl_Alloc(nchars + 26);
 	strncpy(bigstr, outptr, 24);
 	outptr = bigstr;
 	vsnprintf(outptr + 24, nchars + 2, fmt, args);
+	va_end(args);
     }
     else if (nchars == -1) nchars = 126;
 
@@ -732,10 +901,6 @@ int
 Tclmagic_Init(interp)
     Tcl_Interp *interp;
 {
-    extern char *windCommands[];
-    int n;
-    char keyword[100];
-    char *kwptr = keyword + 7;
     char *cadhome;
 
     /* Sanity check! */
@@ -753,30 +918,12 @@ Tclmagic_Init(interp)
     TxCurCommand.tx_p.p_y = 20;
     TxCurCommand.tx_wid = WIND_UNKNOWN_WINDOW;
 
-    /* Use namespace to avoid conflicts with existing commands */
-    strcpy(keyword, "magic::");
 
-    for (n = 0; CmdLongCommands[n] != NULL; n++)
-    {
-	sscanf(CmdLongCommands[n], "%s ", kwptr); /* get first word */
-	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
+    /* Initialization and Startup commands */
+    Tcl_CreateCommand(interp, "magic::initialize", _magic_initialize,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
-    }
-    for (n = 0; windCommands[n] != NULL; n++)
-    {
-	sscanf(windCommands[n], "%s ", kwptr); /* get first word */
-	Tcl_CreateCommand(interp, keyword, _tcl_dispatch,
+    Tcl_CreateCommand(interp, "magic::startup", _magic_startup,
 			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
-    }
-
-    /* Start command */
-    Tcl_CreateCommand(interp, "magic::start", _magic_start,
-			(ClientData)NULL, (Tcl_CmdDeleteProc *) NULL);
-
-    /* Extra commands provided by the Tk graphics routines	*/
-    /* (See graphics/grTkCommon.c)				*/
-
-    RegisterTkCommands(interp);
 
     /* Initialize the command-tag callback feature */
 
@@ -788,10 +935,6 @@ Tclmagic_Init(interp)
 
     Tcl_Eval(interp, "lappend auto_path " TCL_DIR );
 
-    /* Export the namespace commands */
-
-    Tcl_Eval(interp, "namespace eval magic namespace export *");
-
     /* Set $CAD_HOME as a Tcl variable */
 
     cadhome = getenv("CAD_HOME");
@@ -802,3 +945,15 @@ Tclmagic_Init(interp)
     Tcl_PkgProvide(interp, "Tclmagic", "7.2");
     return TCL_OK;
 }
+
+/*--------------------------------------------------------------*/
+/* Define a "safe init" function for those platforms that	*/
+/* require it.							*/
+/*--------------------------------------------------------------*/
+
+int
+Tclmagic_SafeInit(interp)
+    Tcl_Interp *interp; 
+{
+    return Tclmagic_Init(interp);
+}
