@@ -20,7 +20,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/main/main.c,v 1.6 2001/08/14 18:24:04 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -57,9 +57,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/main/main.c,v 1.6 2001/08/14
 #include "extract/extract.h"
 #include "undo/undo.h"
 #include "netmenu/netmenu.h"
-#ifdef PLOW_MODULE
 #include "plow/plow.h"
-#endif
 #include "misc/paths.h"
 #include "wiring/wiring.h"
 #ifdef PLOT_MODULE
@@ -125,6 +123,8 @@ global char *MainMouseFile = NULL;
 /* the name of the first window, if it is not to be a top-level window */
 global bool MakeMainWindow = TRUE;
 global bool TxTkConsole = TRUE;
+#else
+#define MakeMainWindow TRUE
 #endif
 
 /* information about the color display. */
@@ -253,6 +253,7 @@ char **argv;
 		    if ((cp = mainArg(&argc, &argv, "technology")) == NULL)
 			return 1;
 		    TechDefault = StrDup((char **) NULL, cp);
+		    TechOverridesDefault = TRUE;
 		    break;
 
 		/* 
@@ -297,7 +298,7 @@ char **argv;
 		    return 1;
 	    }
 	}
-	else
+	else if (MakeMainWindow)
 	{
 	    if (MainFileName == NULL) {
 		MainFileName = StrDup((char **) NULL, *argv);
@@ -405,6 +406,7 @@ mainInitBeforeArgs(argc, argv)
     int argc;
     char *argv[];
 {
+    TechOverridesDefault = FALSE;
     if (Path == NULL)
 	Path = StrDup((char **) NULL, ".");
 
@@ -453,9 +455,6 @@ mainInitBeforeArgs(argc, argv)
 int
 mainInitAfterArgs()
 {
-    char *home;
-    char startupFileName[100];
-    FILE *f;
     Void (*nullProc)() = 0;
     SectionID sec_tech, sec_planes, sec_types, sec_styles;
     SectionID sec_connect, sec_contact, sec_compose;
@@ -552,13 +551,8 @@ mainInitAfterArgs()
 #endif
     TechAddClient("drc", DRCTechInit, DRCTechAddRule, DRCTechFinal,
 			sec_types|sec_planes, &sec_drc, FALSE);
-#ifndef PLOW_MODULE
-    TechAddClient("drc", nullProc,nullProc,nullProc,
-			(SectionID) 0, (SectionID *) 0, FALSE);
-#else
     TechAddClient("drc", PlowDRCInit, PlowDRCLine, PlowDRCFinal,
 			(SectionID) 0, (SectionID *) 0, FALSE);
-#endif
 #ifdef ROUTE_MODULE
     TechAddClient("drc", MZDRCInit, MZDRCLine, nullProc,
 			sec_mzrouter, (SectionID *) 0, FALSE);
@@ -584,13 +578,8 @@ mainInitAfterArgs()
     TechAddClient("router", nullProc,nullProc,nullProc,
 			sec_types, &sec_router, FALSE);
 #endif
-#ifndef PLOW_MODULE
-    TechAddClient("plowing", nullProc,nullProc,nullProc,
-			sec_types|sec_connect|sec_contact, &sec_plow, FALSE);
-#else
     TechAddClient("plowing", PlowTechInit, PlowTechLine, PlowTechFinal,
 			sec_types|sec_connect|sec_contact, &sec_plow, FALSE);
-#endif	
 #ifdef PLOT_MODULE
     TechAddClient("plot", PlotTechInit, PlotTechLine, PlotTechFinal,
 			sec_types, &sec_plot, FALSE);
@@ -625,8 +614,14 @@ mainInitAfterArgs()
 
     /* Initalize the interface between windows and its clients */
     DBWinit();
+#ifdef USE_READLINE
+    TxInitReadline();
+#endif
     CMWinit();
     DBWElementInit();
+#ifdef THREE_D
+    W3Dinit();
+#endif
 
     /* Initialize the circuit extractor */
 #ifndef NO_EXT
@@ -634,9 +629,7 @@ mainInitAfterArgs()
 #endif
 
     /* Initialize plowing */
-#ifdef PLOW_MODULE
     PlowInit();
-#endif
 
     /* Initialize selection */
     SelectInit();
@@ -668,8 +661,34 @@ mainInitAfterArgs()
     SimInit();
 #endif
 
-    /* Read in system startup file, if it exists. */
     TxSetPoint(GR_CURSOR_X, GR_CURSOR_Y, WIND_UNKNOWN_WINDOW);
+
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * mainInitFinal:
+ *
+ *	Final initialization; reads startup files and any initial file
+ *	specified.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	All sorts of initialization.
+ * ----------------------------------------------------------------------------
+ */
+
+int
+mainInitFinal()
+{
+    char *home;
+    char startupFileName[100];
+    FILE *f;
+
+    /* Read in system startup file, if it exists. */
     f = PaOpen(MAGIC_SYS_DOT, "r", (char *) NULL, ".",
 	    (char *) NULL, (char **) NULL);
     if (f != NULL) { 
@@ -704,10 +723,11 @@ mainInitAfterArgs()
     }
 
     /*
-     * Bring in a new cell to start up if one was given
-     * on the command line
+     * Bring in a new cell or cells to start up if one was given
+     * on the command line, and magic is handling creation of its
+     * own windows.
      */
-    if (MainFileName) {
+    if (MainFileName && MakeMainWindow) {
 	FileName *temporary;
 
 	while(CurrentName != NULL) {
@@ -720,11 +740,8 @@ mainInitAfterArgs()
     
     /* Create an initial box. */
 
-#ifdef MAGIC_WRAPPER
     if (MakeMainWindow)
-#endif
-
-    DBWSetBox(EditCellUse->cu_def, &EditCellUse->cu_def->cd_bbox);
+	DBWSetBox(EditCellUse->cu_def, &EditCellUse->cu_def->cd_bbox);
 
     /* Set the initial fence for undo-ing:  don't want to be able to
      * undo past this point.
@@ -783,12 +800,10 @@ magicMain(argc, argv)
 {
     int rstatus;
 
-#if defined(macII) && !defined(macosx)
-    set42sig();
-#endif macII
     if ((rstatus = mainInitBeforeArgs(argc, argv)) != 0) MainExit(rstatus);
     if ((rstatus = mainDoArgs(argc, argv)) != 0) MainExit(rstatus);
     if ((rstatus = mainInitAfterArgs()) != 0) MainExit(rstatus);
+    if ((rstatus = mainInitFinal()) != 0) MainExit(rstatus);
     TxDispatch( (FILE *) NULL);
     mainFinished();
 }
