@@ -19,7 +19,7 @@
  */
 #ifndef lint
 static char rcsid[] = "$Header:";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
