@@ -21,7 +21,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/netmenu/NMwiring.c,v 1.2 2001/01/12 22:13:07 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
