@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/plot/plotCmd.c,v 1.0 2001/03/29 19:50:05 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -106,7 +106,7 @@ CmdPlot(w, cmd)
 #ifdef	LLNL
 	"pixels [width [layers]]     generate plot in pix format, width pixels\n\
 		                     wide, for layers underneath box",
-#endif	LLNL
+#endif	/* LLNL */
 	"parameters [name value]     set or print out plotting parameters",
 	"help                        print this help information",
 	NULL
@@ -138,7 +138,7 @@ CmdPlot(w, cmd)
 	window = ToolGetPoint((Point *) NULL, (Rect *) NULL);
 	if (window == NULL)
 	{
-	    windCheckOnlyWindow(&window);
+	    windCheckOnlyWindow(&window, DBWclientID);
 	    if ((window == (MagWindow *) NULL) || (window->w_client != DBWclientID))
 	    {
 		TxError("The cursor must be over a layout window to plot.\n");
@@ -277,7 +277,7 @@ CmdPlot(w, cmd)
 	    }
 	    PlotPixels( &scx, &mask, crec->dbw_bitmask, iwidth);
 	    return;
-#endif	LLNL
+#endif	/* LLNL */
     }
 }
 
