@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plot/plotPNM.c,v 1.5 2001/07/23 20:37:01 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -776,12 +776,12 @@ PlotLoadStyles()
     char *which;
 
     which = "display styles";
-    (void) sprintf(fullName, "%.100s.7bit.mraster_dstyle6", DBWStyleType);
+    (void) sprintf(fullName, "%.100s.7bit.mraster_dstyle", DBWStyleType);
     inp = PaOpen(fullName, "r", (char *)NULL, ".", SysLibPath, (char **) NULL);
     if (inp == NULL)
       goto err;
 
-    /* read in the dstyle6 file for mraster */
+    /* read in the dstyle file for mraster */
     newsec = FALSE;
     while (fgets (buf, 256, inp)) {
       if (buf[0] == '#') continue;
@@ -813,7 +813,7 @@ PlotLoadStyles()
     fclose (inp);
 
     /* read in color map */
-    (void) sprintf(fullName, "%.100s.7bit.mraster.cmap1", DBWStyleType);
+    (void) sprintf(fullName, "%.100s.7bit.mraster.cmap", DBWStyleType);
     inp = PaOpen(fullName, "r", (char *) NULL, ".", SysLibPath, (char **) NULL);
     if (inp == NULL)
       {
