@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plot/plotPS.c,v 1.11 2001/08/13 19:31:25 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -1262,7 +1262,7 @@ PlotPS(fileName, scx, layers, xMask)
     fprintf(file, "%%%%EndResource\n%%%%EndProlog\n\n");
     fprintf(file, "%%%%Page: 1 1\n");
     fprintf(file, "/pgsave save def bop\n");
-    fprintf(file, "%% 0 0 offsets\n");
+    fprintf(file, "%% 0 0 offsets\nninit\n");
     fprintf(file, "%d %d translate\n", PlotPSMargin + xnmargin, PlotPSMargin
 		+ ynmargin);
     fprintf(file, "%.3f %.3f scale\nminit\n", fscale, fscale);
