@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plot/plotVers.c,v 1.2 2001/01/12 22:13:10 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -40,7 +40,6 @@ static char rcsid[]="$Header: /ufs/repository/magic/plot/plotVers.c,v 1.2 2001/0
 
 /* Library imports: */
 
-extern char *mktemp();
 extern int rasFileByteCount; 
   
 /* Records of the following type are used to describe how to generate
@@ -339,7 +338,7 @@ PlotVersTechLine(sectionName, argc, argv)
 	    new->vs_stipple[i] = (value<<16) | (value & 0xffff);
 #ifndef IS_BIG_ENDIAN
 	    new->vs_stipple[i] = PlotSwapBytes(new->vs_stipple[i]);
-#endif  IS_BIG_ENDIAN
+#endif  /* IS_BIG_ENDIAN */
 	}
     }
 
@@ -436,7 +435,7 @@ PlotColorVersTechLine(sectionName, argc, argv)
 	    new->vs_stipple[i] = (value<<16) | (value & 0xffff);
 #ifndef IS_BIG_ENDIAN
 	    new->vs_stipple[i] = PlotSwapBytes(new->vs_stipple[i]);
-#endif  IS_BIG_ENDIAN
+#endif  /* IS_BIG_ENDIAN */
 	}
     }
 
@@ -976,7 +975,7 @@ PlotVersatec(scx, layers, xMask, width)
     char fileName[200], command[300], answer[32];
     float length, mBytes;
     Transform tinv;
-    int action;
+    int action, result;
     FILE *file;
     VersatecColor color;
     bool haveColorMessage;
@@ -1065,7 +1064,12 @@ PlotVersatec(scx, layers, xMask, width)
     /* Compute the name of the file to use for output, and open it. */
 
     sprintf(fileName, "%s/magicPlotXXXXXX", PlotTempDirectory);
-    mktemp(fileName);
+    result = mkstemp(fileName);
+    if (result == -1)
+    {
+	TxError("Failed to create temporary filename for %s\n", fileName);
+	return;
+    }
     file = PaOpen(fileName, "w", (char *) NULL, (char *) NULL, (char *) NULL,
 	    (char **) NULL);
     if (file == NULL)
