@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/plow/PlowTest.c,v 1.3 2001/06/15 15:33:40 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -757,7 +757,7 @@ plowShowOutline(outline, clipArea)
 	    || outline->o_rect.r_xbot < clipArea->r_xbot
 	    || outline->o_rect.r_ybot < clipArea->r_ybot)
 	return (1);
-#endif	notdef
+#endif	/* notdef */
 
     /* Are we back at the starting point? */
     switch (outline->o_currentDir)
