@@ -1,6 +1,6 @@
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMain.c,v 1.2 2001/01/12 22:13:16 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -16,7 +16,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMain.c,v 1.2 2001
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "database/databaseInt.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
 #include "extract/extract.h"
@@ -137,43 +136,43 @@ ResGetReCell()
 ResDissolveContacts(contacts)
 	ResContactPoint *contacts;
 {
-    TileType t,oldtype;
+    TileType t, oldtype;
     Tile *tp;
-    LayerInfo	*lp;
+    TileTypeBitMask *residues;
 
     for (; contacts != (ResContactPoint *) NULL; contacts = contacts->cp_nextcontact)
 
     {
-          oldtype=contacts->cp_type;
+        oldtype=contacts->cp_type;
 #ifdef PARANOID
-	  if (oldtype == TT_SPACE)
-	  {
-	       TxError("Error in Contact Dissolving for %s \n",ResCurrentNode);
-	  }
+	if (oldtype == TT_SPACE)
+	{
+	    TxError("Error in Contact Dissolving for %s \n",ResCurrentNode);
+	}
 #endif
-	  lp = dbLayerInfo+oldtype;
-	  if (TTMaskHasType(&ExtCurStyle->exts_transMask,lp->l_images[0])) 
-	  							   continue;
-	  if (TTMaskHasType(&ExtCurStyle->exts_transMask,lp->l_images[1])) 
-	  							   continue;
-	  if (TTMaskHasType(&ExtCurStyle->exts_transMask,lp->l_images[2])) 
-	  							   continue;
-	  DBErase(ResUse->cu_def,&(contacts->cp_rect),oldtype);
-	  DBPaint(ResUse->cu_def,&(contacts->cp_rect),lp->l_residue);	
-	  if ((t=lp->l_rbelow) != TT_SPACE)
-	       DBPaint(ResUse->cu_def,&(contacts->cp_rect),t);
-	  if ((t=lp->l_rabove) != TT_SPACE)
-	       DBPaint(ResUse->cu_def,&(contacts->cp_rect),t);
-	  tp = ResDef->cd_planes[DBPlane(contacts->cp_type)]->pl_hint;
-	  GOTOPOINT(tp,&(contacts->cp_rect.r_ll));
+	
+	residues = DBResidueMask(oldtype);
+
+	DBErase(ResUse->cu_def, &(contacts->cp_rect), oldtype);
+	for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	{
+	    if (TTMaskHasType(residues, t))
+	    {
+		if (TTMaskHasType(&ExtCurStyle->exts_transMask, t))
+		    continue;
+		DBPaint(ResUse->cu_def, &(contacts->cp_rect), t);	
+	    }
+	}
+
+	tp = ResDef->cd_planes[DBPlane(contacts->cp_type)]->pl_hint;
+	GOTOPOINT(tp,&(contacts->cp_rect.r_ll));
 #ifdef PARANOID
-	  if (TiGetTypeExact(tp) == contacts->cp_type)
-	  {
-	       TxError("Error in Contact Preprocess Routines\n");
-	  }
+	if (TiGetTypeExact(tp) == contacts->cp_type)
+	{
+	    TxError("Error in Contact Preprocess Routines\n");
+	}
 #endif
     }
-     
 }
 
 /*
@@ -670,7 +669,7 @@ ResExtractNet(startlist,goodies,cellname)
     {
     	 for (fix = startlist; fix != NULL;fix=fix->fp_next)
 	 {
-     	      fix->fp_tile = ResUse->cu_def->cd_planes[DBTypePlaneTbl[fix->fp_ttype]]->pl_hint;
+     	      fix->fp_tile = ResUse->cu_def->cd_planes[DBPlane(fix->fp_ttype)]->pl_hint;
 	      GOTOPOINT(fix->fp_tile,&fix->fp_loc);
 	      if (TiGetTypeExact(fix->fp_tile) == TT_SPACE) fix->fp_tile = NULL;
 	 }
@@ -774,7 +773,7 @@ FindStartTile(goodies, SourcePoint)
      
     workingPoint.p_x = goodies->rg_tranloc->p_x;
     workingPoint.p_y = goodies->rg_tranloc->p_y;
-    pnum = DBTypePlaneTbl[goodies->rg_ttype];
+    pnum = DBPlane(goodies->rg_ttype);
 
     /* for drivepoints, we don't have to find a transistor */
     if (goodies->rg_status & DRIVEONLY)
