@@ -1,7 +1,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResMakeRes.c,v 1.2 2001/01/12 22:13:16 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
