@@ -10,7 +10,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResSimple.c,v 1.2 2001/01/12 22:13:17 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
