@@ -1,7 +1,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 2001/01/12 22:13:17 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -17,7 +17,6 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
-#include "database/databaseInt.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
 #include "extract/extract.h"
@@ -27,7 +26,7 @@ static char rcsid[] = "$Header: /ufs/repository/magic/resis/ResUtils.c,v 1.2 200
 #include "utils/stack.h"
 #include "tech/tech.h"
 #include "textio/txcommands.h"
-#include	"resis/resis.h"
+#include "resis/resis.h"
 
 
 /*
@@ -65,7 +64,7 @@ ResFirst(tile, arg)
 #endif
     t = TiGetType(tile);
 
-    if ( (DBConnPlanes[t] !=0) && TTMaskHasType(&DBContactBits,t))
+    if ((DBConnPlanes[t] !=0) && DBIsContact(t))
     {
 	    MALLOC(ResContactPoint *, reg, sizeof(ResContactPoint));
 	    reg->cp_center.p_x = (LEFT(tile)+RIGHT(tile))>>1;
@@ -578,19 +577,23 @@ ResPreProcessTransistors(TileList,TransistorList,Def)
      while (TileList != (ResTranTile *) NULL)
      {
 	  tt = TileList->type;
-	  pNum = DBPlane(tt);
 	  if (!TTMaskHasType(&ExtCurStyle->exts_transMask,tt)) 
 	  /* this layer must be a contact; find which image is transistor. */
 	  {
-	       LayerInfo	*lp = &dbLayerInfo[tt];
-	       int		origin;
+	      TileType origin, residue;
+	      TileTypeBitMask *ttresidues = DBResidueMask(tt);
 	       
-	       for (origin = 0; origin != lp->l_nresidues; origin++)
-	       {
-	       	    if (lp->l_images[origin] == tt) break;
-	       }
-	       pNum = (TTMaskHasType(&ExtCurStyle->exts_transMask,
-			lp->l_images[origin+1]))?pNum+1:pNum-1;
+	      for (residue = TT_TECHDEPBASE; residue < DBNumTypes; residue++)
+	      {
+		   if (TTMaskHasType(ttresidues, residue))
+		   {
+			if (TTMaskHasType(&ExtCurStyle->exts_transMask, residue))
+			{
+			    pNum = DBPlane(residue);
+			    break;
+			}
+		   }
+	      }
 	  }
 	  tile = (Def->cd_planes[pNum])->pl_hint;
 	  GOTOPOINT(tile, &(TileList->area.r_ll));
