@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/router/rtrVia.c,v 1.2 2001/01/12 22:13:19 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 
 #include <stdio.h>
