@@ -23,6 +23,7 @@
 #include "windows/windows.h"
 #include "commands/commands.h"
 #include "tech/tech.h"
+#include "dbwind/dbwind.h"
 
 #include "router/router.h"
 #include "gcr/gcr.h"
@@ -48,7 +49,6 @@ int
 Tclroute_Init(interp)
     Tcl_Interp *interp;
 {
-    int n;
     SectionID invsec;
 
     /* Sanity checks! */
@@ -62,27 +62,15 @@ Tclroute_Init(interp)
     /* Replace the auto-load function with the ones defined in	*/
     /* this package in the command functions list.		*/
 
-    for (n = 0; CmdLongCommands[n] != NULL; n++)
-    {
-	if (!strncmp(CmdLongCommands[n], "route", 5))
-	    CmdFuncs[n] = CmdRoute;
-	else if (!strncmp(CmdLongCommands[n], "garoute", 7))
-	    CmdFuncs[n] = CmdGaRoute;
-	else if (!strncmp(CmdLongCommands[n], "iroute", 6))
-	    CmdFuncs[n] = CmdIRoute;
-	else if (!strncmp(CmdLongCommands[n], "channels", 8))
-	    CmdFuncs[n] = CmdChannel;
-	else if (!strncmp(CmdLongCommands[n], "*garoute", 8))
-	    CmdFuncs[n] = CmdGARouterTest;
-	else if (!strncmp(CmdLongCommands[n], "*groute", 7))
-	    CmdFuncs[n] = CmdGRouterTest;
-	else if (!strncmp(CmdLongCommands[n], "*iroute", 7))
-	    CmdFuncs[n] = CmdIRouterTest;
-	else if (!strncmp(CmdLongCommands[n], "*mzroute", 8))
-	    CmdFuncs[n] = CmdMZRouterTest;
-	else if (!strncmp(CmdLongCommands[n], "*seeflags", 9))
-	    CmdFuncs[n] = CmdSeeFlags;
-    }
+    WindReplaceCommand(DBWclientID, "route", CmdRoute);
+    WindReplaceCommand(DBWclientID, "garoute", CmdRoute);
+    WindReplaceCommand(DBWclientID, "iroute", CmdIRoute);
+    WindReplaceCommand(DBWclientID, "channels", CmdChannel);
+    WindReplaceCommand(DBWclientID, "*garoute", CmdGARouterTest);
+    WindReplaceCommand(DBWclientID, "*groute", CmdGRouterTest);
+    WindReplaceCommand(DBWclientID, "*iroute", CmdIRouterTest);
+    WindReplaceCommand(DBWclientID, "*mzroute", CmdMZRouterTest);
+    WindReplaceCommand(DBWclientID, "*seeflags", CmdSeeFlags);
 
     /* Now we need to do TechAddClient and reload the tech file */
 
@@ -101,7 +89,7 @@ Tclroute_Init(interp)
     invsec = TechSectionGetMask("drc", NULL);
     invsec &= TechSectionGetMask("mzrouter", NULL);
     invsec &= TechSectionGetMask("router", NULL);
-    if (!TechLoad(TechDefault, invsec)) return TCL_ERROR;
+    if (!TechLoad(NULL, invsec)) return TCL_ERROR;
 
     Tcl_PkgProvide(interp, "Route", "7.2");
     return TCL_OK;
