@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/select/selOps.c,v 1.4 2001/03/29 19:50:11 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -34,14 +34,10 @@ static char rcsid[]="$Header: /ufs/repository/magic/select/selOps.c,v 1.4 2001/0
 #include "select/selInt.h"
 #include "textio/textio.h"
 #include "undo/undo.h"
+#include "plow/plow.h"
 #include "utils/malloc.h"
 #include "drc/drc.h"
 
-/* This one variable is declared outside of the plow module, because it	*/
-/* is used by the select function.					*/
-
-TileTypeBitMask 	PlowFixedTypes;
-
 /* The following variables are shared between SelectStretch and the
  * search functions that it causes to be invoked.
  */
@@ -355,7 +351,7 @@ selTransLabelFunc(label, cellDef, defTransform, transform)
     GeoTransRect(transform, &rootArea, &finalArea);
     finalPos = GeoTransPos(transform, rootPos);
     (void) DBPutLabel(Select2Def, &finalArea, finalPos, label->lab_text,
-	    label->lab_type);
+	    label->lab_type, label->lab_flags);
     return 0;
 }
 
@@ -656,7 +652,7 @@ selArrayLFunc(label, def, transform, arrayInfo)
 		-1, nmPutNums(label->lab_text, labx + xi, laby +yi));
 	    (void) DBPutLabel(Select2Def, &current, rootPos,
 		nmPutNums(label->lab_text, labx + xi, laby + yi),
-		label->lab_type);
+		label->lab_type, label->lab_flags);
 	    current.r_ybot += arrayInfo->ar_ysep;
 	    current.r_ytop += arrayInfo->ar_ysep;
 	    xi += only1;
@@ -726,7 +722,7 @@ SelectStretch(x, y)
     
     selStretchX = x;
     selStretchY = y;
-    for (plane = PL_SELECTBASE; plane < DBNumPlanes; plane += 1)
+    for (plane = PL_SELECTBASE; plane < DBNumPlanes; plane++)
     {
 	(void) DBSrPaintArea((Tile *) NULL, Select2Def->cd_planes[plane],
 		&TiPlaneRect, &DBAllButSpaceAndDRCBits, selStretchEraseFunc,
@@ -741,7 +737,7 @@ SelectStretch(x, y)
      */
 
     selStretchList = NULL;
-    for (plane = PL_SELECTBASE; plane < DBNumPlanes; plane += 1)
+    for (plane = PL_SELECTBASE; plane < DBNumPlanes; plane++)
     {
 	(void) DBSrPaintArea((Tile *) NULL, Select2Def->cd_planes[plane],
 		&TiPlaneRect, &DBAllButSpaceAndDRCBits, selStretchFillFunc,
@@ -811,9 +807,20 @@ selStretchEraseFunc(tile)
     /* Translate into edit coords and erase all material on the
      * tile's plane.
      */
-    
+
+    /*
+     * Stacking contact types do not get erased
+     */
+
     GeoTransRect(&RootToEditTransform, &area, &editArea);
-    DBEraseMask(EditCellUse->cu_def, &editArea, &DBPlaneTypes[DBPlane(type)]);
+
+    DBErase(EditCellUse->cu_def, &editArea, type);
+
+    /* Was previously DBEraseMask().  Don't know why.  If you trust the */
+    /* paint/erase tables, it should not be necessary to erase anything	*/
+    /* other than the type itself.					*/
+
+    /* DBEraseMask(EditCellUse->cu_def, &editArea, &DBPlaneTypes[DBPlane(type)]); */
 
     return 0;
 }
@@ -917,6 +924,8 @@ selStretchFillFunc2(tile, area)
 					 */
 {
     Rect spaceArea, editArea;
+    int pNum;
+
     extern int selStretchFillFunc3();
 
     TiToRect(tile, &spaceArea);
@@ -934,10 +943,13 @@ selStretchFillFunc2(tile, area)
     spaceArea.r_ytop -= selStretchY;
     GeoTransRect(&RootToEditTransform, &spaceArea, &editArea);
 
-    (void) DBSrPaintArea((Tile *) NULL,
-	    EditCellUse->cu_def->cd_planes[DBPlane(selStretchType)],
-	    &editArea, &DBAllButSpaceAndDRCBits, selStretchFillFunc3,
-	    (ClientData) &spaceArea);
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+	if (DBPaintOnPlane(selStretchType, pNum))
+	    (void) DBSrPaintArea((Tile *) NULL,
+//			EditCellUse->cu_def->cd_planes[DBPlane(selStretchType)],
+			EditCellUse->cu_def->cd_planes[pNum],
+			&editArea, &DBAllButSpaceAndDRCBits, selStretchFillFunc3,
+			(ClientData) &spaceArea);
 
     return 0;
 }
@@ -1013,15 +1025,21 @@ selStretchFillFunc3(tile, area)
 #endif
     type = TiGetType(tile);
 
-    if (TTMaskHasType(&PlowFixedTypes, type)
-	    || !TTMaskHasType(&PlowFixedTypes, selStretchType))
-	type = selStretchType;
+    /* If have type and stretch type are both contacts (fixed types)	*/
+    /* and there exists a stacking contact type between them, then	*/
+    /* we should be filling in the original type.			*/
 
-    if (type >= DBNumUserLayers)
+    if (!DBIsContact(type) || !DBIsContact(selStretchType) ||
+		(DBTechFindStacking(type, selStretchType) < DBNumUserLayers))
     {
-	mask = &DBLayerTypeMaskTbl[type];
-	for (type = TT_SELECTBASE; !TTMaskHasType(mask, type); type += 1)
-	    /* Null loop body. */;
+	/* If type is a residue of selStretchType then we always fill	*/
+	/* with type.  Otherwise, we check the list of plow types to	*/
+	/* see what is and isn't fixed.					*/
+
+	if (!TTMaskHasType(DBResidueMask(selStretchType), type))
+	    if (TTMaskHasType(&PlowFixedTypes, type)
+			|| !TTMaskHasType(&PlowFixedTypes, selStretchType))
+		type = selStretchType;
     }
 
     /* Save around the area we just found. */
