@@ -203,7 +203,7 @@ selRemoveLabelPaintFunc(tile, label)
      Label *label;
 {
   (void) DBPutLabel(Select2Def, &label->lab_rect, label->lab_pos,
-		    &label->lab_text[0], label->lab_type);
+		    &label->lab_text[0], label->lab_type, label->lab_flags);
 
   return 1;
 }
