@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/tech/tech.c,v 1.3 2001/01/12 22:13:25 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <ctype.h>
@@ -33,6 +33,8 @@ static char rcsid[] = "$Header: /ufs/repository/magic/tech/tech.c,v 1.3 2001/01/
 #include <varargs.h>
 
 global char *TechDefault = NULL;
+global int  TechFormatVersion;
+global bool TechOverridesDefault;
 
 int techLineNumber;
 char *TechFileName = NULL;
@@ -303,16 +305,48 @@ TechLoad(filename, initmask)
 	return (FALSE);
     }
 
-    /* TECH_VERSION is defined in tech/Makefile by appending to DFLAGS */
-    (void) sprintf(suffix, ".tech%d", TECH_VERSION);
+    /* If NULL is passed to argument "filename", this is a reload and	*/
+    /* we should read TechFileName verbatim.				*/
 
-    tf = PaOpen(filename, "r", suffix, ".", SysLibPath, &realname);
-    if (tf == (FILE *) NULL) {
-	TxError("Could not find file '%s%s' in any of these directories:\n         %s\n",
-	    filename, suffix, SysLibPath);
-	return (FALSE);
+    if (filename == NULL)
+    {
+	tf = PaOpen(TechFileName, "r", (char *)NULL, ".", SysLibPath, &realname);
+	if (tf == (FILE *) NULL)
+	{
+	    TxError("Could not find file '%s' in any of these "
+			"directories:\n         %s\n",
+			TechFileName, SysLibPath);
+	    return (FALSE);
+	}
+    }
+    else
+    {
+	/* TECH_VERSION in the filename is deprecated as of magic version	*/
+	/* 7.2.27;  TECH_VERSION is no longer defined in the tech/Makefile.	*/
+	/* It has been changed to TECH_FORMAT_VERSION, left at version 27,	*/
+	/* and placed in tech/tech.h.  It is needed for backward		*/
+	/* compatibility with *.tech27 files, of which there are many.	*/
+
+	(void) sprintf(suffix, ".tech");
+	tf = PaOpen(filename, "r", suffix, ".", SysLibPath, &realname);
+	if (tf == (FILE *) NULL)
+	{
+	    /* Try looking for tech files from the last version to	*/
+	    /* put the version number into the filename itself.	*/
+
+	    (void) sprintf(suffix, ".tech%d", TECH_FORMAT_VERSION);
+
+	    tf = PaOpen(filename, "r", suffix, ".", SysLibPath, &realname);
+	    if (tf == (FILE *) NULL)
+	    {
+		TxError("Could not find file '%s.tech' in any of these "
+			"directories:\n         %s\n",
+			filename, SysLibPath);
+		return (FALSE);
+	    }
+	}
+	(void) StrDup(&TechFileName, realname);
     }
-    (void) StrDup(&TechFileName, realname);
 
     /*
      * Mark all sections as being unread.
