@@ -40,17 +40,19 @@ extern int TxCurButtons;
 #define TX_LONG_CMD2	';'	/* Alternate way of invoking a long command. */
 
 /* printing procedures */
-extern void TxPrintf();
 extern bool TxPrintOn();  	/* enables TxPrintf output */
 extern bool TxPrintOff();	/* disables TxPrintf output */
-extern void TxError();
 extern void TxFlush();
 extern void TxFlushOut();
 extern void TxFlushErr();
 extern void TxVisChar();
 extern void TxUseMore();
 extern void TxStopMore();
-extern char *TxPrintString();
+
+/* printing procedures with variable arguments lists */
+extern void TxError(char *, ...);
+extern void TxPrintf(char *, ...);
+extern char *TxPrintString(char *, ...);
 
 /* input procedures */
 extern char *TxGetLinePrompt();
@@ -82,5 +84,8 @@ extern bool TxStdoutIsatty;
 
 /* Misc procs */
 void TxInit();
+#ifdef USE_READLINE
+void TxInitReadline();
+#endif
 
 #define   TX_MAX_OPEN_FILES       20
