@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/textio/txCommands.c,v 1.7 2001/07/23 20:37:01 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #ifdef  SYSV
@@ -1105,7 +1105,17 @@ txGetFileCommand(f, queue)
     } while ((*current == 0) || (*current == '#'));
 
     linep = inputLine;
+
+#ifdef MAGIC_WRAPPER
+    /* Don't muck with Tcl's interpretation of semicolon as a	*/
+    /* comment character.  Watch for single-colon entries for	*/
+    /* backwards-compatibility with the old format, but watch	*/
+    /* for the double-colon, which is a Tcl namespace marker.	*/
+
+    if ((inputLine[0] == ':') && (inputLine[1] != ':')) linep++;
+#else
     if ((inputLine[0] == ':') || (inputLine[1] == ';')) linep++;
+#endif
 
     TxParseString(linep, queue, (TxInputEvent *) NULL);
 }
@@ -1158,7 +1168,7 @@ TxTclDispatch(clientData, argc, argv)
 	tclcmd->tx_wid = TxCurCommand.tx_wid;
     }
 
-    result = WindSendCommand((WindClient)NULL, tclcmd);
+    result = WindSendCommand((MagWindow *)clientData, tclcmd);
     TxFreeCommand(tclcmd);
     TxCommandNumber++;
 
@@ -1349,7 +1359,7 @@ TxDispatch(f)
 	      }
 	    else {
 #endif
-	      (void) WindSendCommand((WindClient) NULL, cmd);
+	      (void) WindSendCommand((MagWindow *) NULL, cmd);
 	      TxFreeCommand(cmd);
 	      TxCommandNumber++;
 
@@ -1442,7 +1452,7 @@ TxLispDispatch (argc,argv,trace, inFile)
   }
   if (trace)
     TxPrintCommand (cmd);
-  ret = WindSendCommand((WindClient) NULL, cmd);
+  ret = WindSendCommand((MagWindow *) NULL, cmd);
   /* restore the original values */
   cmd->tx_p = tx_p;
   cmd->tx_wid = wid;
