@@ -18,18 +18,19 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/textio/txOutput.c,v 1.4 2001/02/21 15:52:52 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
+#include <stdarg.h>
+
 #ifndef	SYSV
 #include <sys/wait.h>
 #include <strings.h>
 #else
 #include <string.h>
-#endif SYSV
+#endif /* SYSV */
 #include <sys/stat.h>
 #include <unistd.h>
-#include <varargs.h>
 #include "misc/magsgtty.h"
 
 #include "misc/magic.h"
@@ -76,18 +77,13 @@ static bool txPrintFlag = TRUE;
  * ----------------------------------------------------------------------------
  */ 
 
- /*VARARGS0*/
-
 void
-txFprintfBasic(va_alist)
-va_dcl
+txFprintfBasic(FILE *f, ...)
 {
     va_list args;
     char *fmt;
-    FILE *f;
 
-    va_start(args);
-    f = va_arg(args, FILE *);
+    va_start(args, f);
     fmt = va_arg(args, char *);
     (void) GrVfprintf(f, fmt, args);
     va_end(args);
@@ -115,14 +111,10 @@ va_dcl
  * ----------------------------------------------------------------------------
  */ 
 
- /*VARARGS0*/
-
 void
-TxPrintf(va_alist)
-va_dcl
+TxPrintf(char *fmt, ...)
 {
     va_list args;
-    char *fmt;
     FILE *f;
 
     if (txPrintFlag)
@@ -139,16 +131,14 @@ va_dcl
 	if (txHavePrompt)
 	{
 	    TxUnPrompt();
-	    va_start(args);
-	    fmt = va_arg(args, char *);
+	    va_start(args, fmt);
 	    (void) GrVfprintf(f, fmt, args);
 	    va_end(args);
 	    TxPrompt();
 	}
 	else 
 	{
-	    va_start(args);
-	    fmt = va_arg(args, char *);
+	    va_start(args, fmt);
 	    (void) GrVfprintf(f, fmt, args);
 	    va_end(args);
 	}
@@ -172,14 +162,10 @@ va_dcl
  * ----------------------------------------------------------------------------
  */ 
 
- /*VARARGS0*/
-
 char *
-TxPrintString(va_alist)
-va_dcl
+TxPrintString(char *fmt, ...)
 {
     va_list args;
-    char *fmt;
     static char *outstr = NULL;
     static int outlen;
     int nchars;
@@ -190,10 +176,9 @@ va_dcl
 	MALLOC(char *, outstr, outlen + 1);
     }
 
-    va_start(args);
-    fmt = va_arg(args, char *);
-
+    va_start(args, fmt);
     nchars = vsnprintf(outstr, outlen, fmt, args);
+
     if (nchars >= outlen)
     {
 	outlen = nchars + 1;
@@ -201,10 +186,10 @@ va_dcl
 	MALLOC(char *, outstr, outlen + 1);
 	vsnprintf(outstr, outlen, fmt, args);
     }
+    va_end(args);
     if (nchars == -1)
 	return NULL;
 
-    va_end(args);
     return outstr;
 }
 
@@ -315,14 +300,11 @@ TxFlush()
  *	a Pyramid machine.
  * ----------------------------------------------------------------------------
  */
- /*VARARGS0*/
 
 void
-TxError(va_alist)
-va_dcl
+TxError(char *fmt, ...)
 {
     va_list args;
-    char *fmt;
     FILE *f;
 
     TxFlushOut();
@@ -330,21 +312,17 @@ va_dcl
 	f = TxMoreFile;
     else
 	f = stderr;
+    va_start(args, fmt);
     if (txHavePrompt)
     {
 	TxUnPrompt();
-	va_start(args);
-	fmt = va_arg(args, char *);
 	(void) GrVfprintf(f, fmt, args);
-	va_end(args);
 	TxPrompt();
     }
     else {
-	va_start(args);
-	fmt = va_arg(args, char *);
 	(void) GrVfprintf(f, fmt, args);
-	va_end(args);
     }
+    va_end(args);
     TxFlushErr();
 }
 
@@ -461,13 +439,15 @@ done:
 void
 TxStopMore()
 {
+
 /*
 #if defined(SYSV) | defined(ALPHA)
     int status;
 #else
     union wait status;
-#endif SYSV
+#endif
 */
+
     /* TxMoreFile may be NULL if the "more" executable was not found */
     if (TxMoreFile == NULL) return;
 
@@ -487,16 +467,14 @@ TxStopMore()
 /*  while (wait(&status) != txMorePid) */ /* try again */; 
 }
 
-#endif !MAGIC_WRAPPER
+#endif /* !MAGIC_WRAPPER */
 
 #ifdef	NEED_VFPRINTF
 
 int
-vfprintf(iop, fmt, ap)
-    FILE *iop;
-    char *fmt;
-    va_list ap;
+vfprintf(FILR *iop, char *fmt, va_list args_in)
 {
+    va_list ap;
     int len;
 #if defined(MIPSEB) && defined(SYSTYPE_BSD43)
     unsigned char localbuf[BUFSIZ];
@@ -504,6 +482,7 @@ vfprintf(iop, fmt, ap)
     char localbuf[BUFSIZ];
 #endif
 
+    va_copy(ap, args_in);
     if (iop->_flag & _IONBF) {
 	iop->_flag &= ~_IONBF;
 	iop->_ptr = iop->_base = localbuf;
@@ -516,7 +495,8 @@ vfprintf(iop, fmt, ap)
     } else
 	len = _doprnt(fmt, ap, iop);
 
+    va_end(ap);
     return (ferror(iop) ? EOF : len);
 }
-#endif  NEED_VFPRINTF
+#endif  /* NEED_VFPRINTF */
 
