@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/utils/geometry.c,v 1.2 2001/01/12 22:13:31 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -277,9 +277,14 @@ GeoNameToPos(name, manhattan, verbose)
     }
     positions[] =
     {
+	"bl",		GEO_SOUTHWEST,		FALSE,
 	"bottom",	GEO_SOUTH,		TRUE,
+	"br",		GEO_SOUTHEAST,		FALSE,
 	"center",	GEO_CENTER,		FALSE,
+	"d",		GEO_SOUTH,		TRUE,
+	"dl",		GEO_SOUTHWEST,		FALSE,
 	"down",		GEO_SOUTH,		TRUE,
+	"dr",		GEO_SOUTHEAST,		FALSE,
 	"e",		GEO_EAST,		TRUE,
 	"east",		GEO_EAST,		TRUE,
 	"left",		GEO_WEST,		TRUE,
@@ -296,8 +301,13 @@ GeoNameToPos(name, manhattan, verbose)
 	"southeast",	GEO_SOUTHEAST,		FALSE,
 	"southwest",	GEO_SOUTHWEST,		FALSE,
 	"sw",		GEO_SOUTHWEST,		FALSE,
+	"tl",		GEO_NORTHWEST,		FALSE,
 	"top",		GEO_NORTH,		TRUE,
+	"tr",		GEO_NORTHEAST,		FALSE,
+	"u",		GEO_NORTH,		TRUE,
+	"ul",		GEO_NORTHWEST,		FALSE,
 	"up",		GEO_NORTH,		TRUE,
+	"ur",		GEO_NORTHEAST,		FALSE,
 	"w",		GEO_WEST,		TRUE,
 	"west",		GEO_WEST,		TRUE,
 	0
