@@ -23,6 +23,32 @@
 #define TRUE 1
 #define FALSE 0
 
+/*-------------------------------------------------------------------
+ * Define a double integer.  This ought to replace the horrible
+ * mess in utils/doubleint.c and doubleint.h. . .
+ *
+ * This defines "dlong" to be an 8-byte double-precision integer.
+ *-------------------------------------------------------------------
+ */
+
+#ifdef HAVE_LIMITS_H
+#include <limits.h>
+#else
+  #ifdef HAVE_SYS_PARAM_H
+  #include <sys/param.h>
+  #endif
+#endif
+
+#if defined(LONG_MAX) && defined(INT_MAX) 
+  #if LONG_MAX == INT_MAX
+  typedef long long dlong;
+  #else
+  typedef long dlong;
+  #endif
+#else
+typedef long long dlong;
+#endif
+
 /*-------------------------------------------------------------------
  * Structure definition for Point (an x,y pair).
  *-------------------------------------------------------------------
