@@ -36,7 +36,7 @@
 
 #ifndef lint
 static char rcsid[] = "$Header: /ufs/repository/magic/utils/malloc.c,v 1.6 2001/08/23 13:48:58 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <errno.h>
@@ -96,19 +96,18 @@ mallocMagic(nbytes)
 {
     char *p; 
     
-    if (freeDelayedItem) {
-/*
-	fprintf(stderr, "freed 0x%x\n", freeDelayedItem); fflush(stderr);
-*/
+    if (freeDelayedItem)
+    {
+	/* fprintf(stderr, "freed 0x%x (delayed)\n", freeDelayedItem); fflush(stderr); */
+
 	FreeRoutine(freeDelayedItem);
 	freeDelayedItem=NULL;
     }
 
     if ((p = (char *)MallocRoutine(nbytes)) != NULL) 
     {
-/*
-	fprintf(stderr, "alloc'd %u bytes at 0x%x\n", nbytes, p); fflush(stderr);
-*/
+	/* fprintf(stderr, "alloc'd %u bytes at 0x%x\n", nbytes, p); fflush(stderr); */
+
 	return p;
     }
     else
@@ -125,6 +124,7 @@ mallocMagic(nbytes)
  *---------------------------------------------------------------------
  */
 
+void
 freeMagic(cp)
     char *cp;
 {
@@ -132,9 +132,8 @@ freeMagic(cp)
 	TxError("freeMagic called with NULL argument.\n");
     if (freeDelayedItem)
     {
-/*
-	fprintf(stderr, "freed 0x%x\n", freeDelayedItem); fflush(stderr);
-*/
+	/* fprintf(stderr, "freed 0x%x\n", freeDelayedItem); fflush(stderr); */
+
 	FreeRoutine(freeDelayedItem);
     }
     freeDelayedItem=cp;
