@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/windows/windClient.c,v 1.3 2001/01/12 22:13:34 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -50,58 +50,10 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windClient.c,v 1.3 2
 #define WIND_ILG -1
 
 /* our window client ID */
-static WindClient windClientID = NULL;
+global WindClient windClientID = NULL;
 
-global char *windCommands[] =
-    {
-	"*crash			cause a core dump",
-	"*files			print out currently open files",
-	"*grstats		print out stats on graphics",
-	"*pause	[args]		print args and wait for <cr>",
-	"*winddebug		set debugging mode",
-	"*winddump		print out debugging info",
-
-	"center [x y]		center window on the cursor or indicated coordinate",
-	"closewindow		close a window",
-	"echo [-n] [strings]	print text on the terminal",
-	"grow			blow a window up to full-screen size or back again",
-	"help [pattern]		print out synopses for all commands valid\n\
-			in the current window (or just those\n\
-			containing pattern)",
-	"imacro [char [string]]	define or print an interactive macro called char",
-	"logcommands [file [update]]\n\
-			log all commands into a file",
-	"macro [char [string]]	define or print a macro called char",
-	"openwindow [cell][name]\n\
-			open a new window with indicated name, bound to indicated cell",
-	"over			move a window over (on top of) the rest",
-	"pushbutton button act	push a mouse button",
-	"redo [count]		redo commands",
-	"redraw			redraw the display",
-	"reset			reset the display",
-	"scroll dir [amount]	scroll the window",
-	"send type command	send a command to a certain window type",
-	"setpoint [x y [WID]]	force to cursor (point) to x,y in window WID",
-	"sleep seconds		sleep for a number of seconds",
-	"source filename		read in commands from file",
-	"specialopen [coords] type [args]\n\
-			open a special window",
-	"quit			exit magic",
-	"underneath		move a window underneath the rest",
-	"undo [count]		undo commands",
-	"updatedisplay		force the display to be updated",
-	"version			print out version info",
-	"view [get]              zoom window out so everything is visible",
-	"windowcaption [on|off]	toggle title caption for new windows",	
-	"windowscrollbars [on|off]\n\
-			toggle scroll bars for new windows",
-	"windowpositions [file]	print out window positions",
-	"xview               	zoom window out so everything is unexpanded",
-	"zoom amount		zoom window by amount",
-	0
-    };
-
-extern int windCaptionCmd(), windCrashCmd();
+extern int windBorderCmd();
+extern int windCaptionCmd(), windCrashCmd(), windCursorCmd();
 extern int windFilesCmd(), windCloseCmd(), windOpenCmd();
 extern int windQuitCmd(), windRedrawCmd();
 extern int windResetCmd(), windSpecialOpenCmd();
@@ -118,47 +70,9 @@ extern int windCenterCmd(), windScrollCmd();
 extern int windVersionCmd(), windViewCmd(), windXviewCmd(), windZoomCmd();
 extern int windScrollBarsCmd(), windPositionsCmd();
 
-static int (*windFuncs[])() =
-    {
-	windCrashCmd,
-	windFilesCmd,
-	windGrstatsCmd,
-	windPauseCmd,
-	windDebugCmd,
-	windDumpCmd,
-
-	windCenterCmd,
-	windCloseCmd,
-	windEchoCmd,
-	windGrowCmd,
-	windHelpCmd,
-	windIntMacroCmd,
-	windLogCommandsCmd,
-	windMacroCmd,
-	windOpenCmd,
-	windOverCmd,
-	windPushbuttonCmd,
-	windRedoCmd,
-	windRedrawCmd,
-	windResetCmd,
-	windScrollCmd,
-	windSendCmd,
-	windSetpointCmd,
-	windSleepCmd,
-	windSourceCmd,
-	windSpecialOpenCmd,
-	windQuitCmd,
-	windUnderCmd,
-	windUndoCmd,
-	windUpdateCmd,
-	windVersionCmd,
-	windViewCmd,
-	windCaptionCmd,
-	windScrollBarsCmd,
-	windPositionsCmd,
-	windXviewCmd,
-	windZoomCmd
-    };
+#ifdef MAGIC_WRAPPER
+extern int windNamesCmd();
+#endif
 
 static Rect windFrameRect;
 static MagWindow *windFrameWindow;
@@ -601,7 +515,7 @@ windClientButtons(w, cmd)
 	case WIND_SUN_WINDOWS:
 #ifdef	SUNVIEW
 	    windSunHandleCaptionInput(w, cmd);
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
 	    break;
 	
 	default:
@@ -663,14 +577,8 @@ windCmdInterp(w, cmd)
 	    windClientButtons(w, cmd);
 	    break;
 	case TX_NO_BUTTON:
-	    if (cmd->tx_argc > 0)
-	    {
-		cmdNum = Lookup(cmd->tx_argv[0], windCommands);
-		ASSERT(cmdNum >= 0, "windCmdInterp");
-
-		(void) (*windFuncs[cmdNum])(w, cmd);
+	    if (WindExecute(w, windClientID, cmd) >= 0)
 		UndoNext();
-	    }
 	    break;
 	default:
 	    ASSERT(FALSE, "windCmdInterp");
@@ -699,5 +607,139 @@ windClientInit()
     windClientID = WindAddClient(WINDOW_CLIENT, ( Void (*)() ) NULL,
 	( Void (*)() ) NULL, ( Void (*)() ) NULL, windCmdInterp, 
 	( Void (*)() ) NULL, ( Void (*)() ) NULL, ( Void (*)() ) NULL,
-	windCommands, (GrGlyph *) NULL);
+	(GrGlyph *) NULL);
+
+    /* Set up all of the window commands */
+
+    WindAddCommand(windClientID,
+	"*crash			cause a core dump",
+	windCrashCmd, FALSE);
+    WindAddCommand(windClientID,
+	"*files			print out currently open files",
+	windFilesCmd, FALSE);
+    WindAddCommand(windClientID,
+	"*grstats		print out stats on graphics",
+	windGrstatsCmd, FALSE);
+    WindAddCommand(windClientID,
+	"*pause	[args]		print args and wait for <cr>",
+	windPauseCmd, FALSE);
+    WindAddCommand(windClientID,
+	"*winddebug		set debugging mode",
+	windDebugCmd, FALSE);
+    WindAddCommand(windClientID,
+	"*winddump		print out debugging info",
+	windDumpCmd, FALSE);
+
+    WindAddCommand(windClientID,
+	"center [x y]		center window on the cursor or indicated coordinate",
+	windCenterCmd, FALSE);
+    WindAddCommand(windClientID,
+	"closewindow		close a window",
+	windCloseCmd, FALSE);
+    WindAddCommand(windClientID,
+	"cursor			return magic coordinates of the cursor",
+	windCursorCmd, FALSE);
+    WindAddCommand(windClientID,
+	"grow			blow a window up to full-screen size or back again",
+	windGrowCmd, FALSE);
+    WindAddCommand(windClientID,
+	"help [pattern]		print out synopses for all commands valid\n\
+			in the current window (or just those\n\
+			containing pattern)",
+	windHelpCmd, FALSE);
+    WindAddCommand(windClientID,
+	"imacro [char [string]]	define or print an interactive macro called char",
+	windIntMacroCmd, FALSE);
+    WindAddCommand(windClientID,
+	"logcommands [file [update]]\n\
+			log all commands into a file",
+	windLogCommandsCmd, FALSE);
+    WindAddCommand(windClientID,
+	"macro [char [string]]	define or print a macro called char",
+	windMacroCmd, FALSE);
+    WindAddCommand(windClientID,
+	"openwindow [cell][name]\n\
+			open a new window with indicated name, bound to indicated cell",
+	windOpenCmd, FALSE);
+    WindAddCommand(windClientID,
+	"over			move a window over (on top of) the rest",
+	windOverCmd, FALSE);
+    WindAddCommand(windClientID,
+	"pushbutton button act	push a mouse button",
+	windPushbuttonCmd, FALSE);
+    WindAddCommand(windClientID,
+	"redo [count]		redo commands",
+	windRedoCmd, FALSE);
+    WindAddCommand(windClientID,
+	"redraw			redraw the display",
+	windRedrawCmd, FALSE);
+    WindAddCommand(windClientID,
+	"reset			reset the display",
+	windResetCmd, FALSE);
+    WindAddCommand(windClientID,
+	"scroll dir [amount]	scroll the window",
+	windScrollCmd, FALSE);
+    WindAddCommand(windClientID,
+	"setpoint [x y [WID]]	force to cursor (point) to x,y in window WID",
+	windSetpointCmd, FALSE);
+    WindAddCommand(windClientID,
+	"sleep seconds		sleep for a number of seconds",
+	windSleepCmd, FALSE);
+    WindAddCommand(windClientID,
+	"specialopen [coords] type [args]\n\
+			open a special window",
+	windSpecialOpenCmd, FALSE);
+    WindAddCommand(windClientID,
+	"quit			exit magic",
+	windQuitCmd, FALSE);
+    WindAddCommand(windClientID,
+	"underneath		move a window underneath the rest",
+	windUnderCmd, FALSE);
+    WindAddCommand(windClientID,
+	"undo [count]		undo commands",
+	windUndoCmd, FALSE);
+    WindAddCommand(windClientID,
+	"updatedisplay		force the display to be updated",
+	windUpdateCmd, FALSE);
+    WindAddCommand(windClientID,
+	"version			print out version info",
+	windVersionCmd, FALSE);
+    WindAddCommand(windClientID,
+	"view [get]              zoom window out so everything is visible",
+	windViewCmd, FALSE);
+    WindAddCommand(windClientID,
+	"windowborder [on|off]	toggle border drawing for new windows",	
+	windBorderCmd, FALSE);
+    WindAddCommand(windClientID,
+	"windowcaption [on|off]	toggle title caption for new windows",	
+	windCaptionCmd, FALSE);
+    WindAddCommand(windClientID,
+	"windowscrollbars [on|off]\n\
+			toggle scroll bars for new windows",
+	windScrollBarsCmd, FALSE);
+    WindAddCommand(windClientID,
+	"windowpositions [file]	print out window positions",
+	windPositionsCmd, FALSE);
+    WindAddCommand(windClientID,
+	"xview               	zoom window out so everything is unexpanded",
+	windXviewCmd, FALSE);
+    WindAddCommand(windClientID,
+	"zoom amount		zoom window by amount",
+	windZoomCmd, FALSE);
+
+#ifdef MAGIC_WRAPPER
+    WindAddCommand(windClientID,
+	"windownames [all|type]	get name of current or all windows",
+	windNamesCmd, FALSE);
+#else
+    WindAddCommand(windClientID,
+	"echo [-n] [strings]	print text on the terminal",
+	windEchoCmd, FALSE);
+    WindAddCommand(windClientID,
+	"send type command	send a command to a certain window type",
+	windSendCmd, FALSE);
+    WindAddCommand(windClientID,
+	"source filename		read in commands from file",
+	windSourceCmd, FALSE);
+#endif
 }
