@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdAM.c,v 1.7 2001/04/25 14:38:06 tim Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -48,6 +48,76 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdAM.c,v 1.7 20
 #include "dbwind/dbwind.h"
 #include "utils/utils.h"
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * windBorderCmd --
+ *
+ *	Change the flag which says whether new windows will have a border.
+ *
+ * Usage:
+ *	windborder [on|off]
+ *
+ * Results:
+ *	None.  In Tcl, if no options are presented, the border status
+ *	is returned.
+ *
+ * Side effects:
+ *	A flag is changed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+ /*ARGSUSED*/
+
+Void
+windBorderCmd(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    int place;
+    bool value;
+    static char *onoff[] = {"on", "off", 0};
+    static bool truth[] = {TRUE, FALSE};
+    extern int windDefaultFlags;
+
+    if (cmd->tx_argc > 2) goto usage;
+    else if (cmd->tx_argc == 1)
+    {
+	if (w == (MagWindow *)NULL)
+	{
+	    TxError("No window specified for caption command\n");
+	    goto usage;
+	}
+
+	value = (w->w_flags & WIND_BORDER) ? TRUE : FALSE;
+
+#ifdef MAGIC_WRAPPER
+	Tcl_SetResult(magicinterp, onoff[value], TCL_STATIC);
+#else
+	TxPrintf("Window border is %s\n", onoff[value]);
+#endif
+	return;
+    }
+
+    place = Lookup(cmd->tx_argv[1], onoff);
+    if (place < 0)
+    goto usage;
+
+    if (truth[place])
+    {
+        windDefaultFlags |= WIND_BORDER;
+	TxPrintf("New windows will have a border.\n");
+    }
+    else {
+        windDefaultFlags &= ~WIND_BORDER;
+	TxPrintf("New windows will not have a border.\n");
+    }
+    return;
+
+    usage:
+	TxError("Usage: %s [on|off]\n", cmd->tx_argv[0]);
+	return;
+}
 /*
  * ----------------------------------------------------------------------------
  *
@@ -291,6 +361,47 @@ windCrashCmd(w, cmd)
     niceabort();
 }
 
+
+/*
+ * ----------------------------------------------------------------------------
+ * windCursorCmd --
+ *
+ *	Report the cursor position in Magic (internal) coordinates
+ *
+ * Results:
+ *	Returns the cursor position as a list (Tcl version)
+ *
+ * Side effects:
+ *	Prints coordinates (non-Tcl version)
+ * ----------------------------------------------------------------------------
+ */
+
+windCursorCmd(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    Point p_in, p_out;
+    bool result;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *listxy;
+#endif
+
+    if (GrGetCursorPosPtr == NULL)
+	return;
+
+    GrGetCursorPos(w, &p_in);
+    WindPointToSurface(w, &p_in, &p_out, (Rect *)NULL);
+
+#ifdef MAGIC_WRAPPER
+    listxy = Tcl_NewListObj(0, NULL);
+    Tcl_ListObjAppendElement(magicinterp, listxy, Tcl_NewIntObj(p_out.p_x));
+    Tcl_ListObjAppendElement(magicinterp, listxy, Tcl_NewIntObj(p_out.p_y));
+    Tcl_SetObjResult(magicinterp, listxy);
+#else
+    TxPrintf("%d %d\n", p_out.p_x, p_out.p_y);
+#endif
+}
+
 /*
  * ----------------------------------------------------------------------------
  * windDebugCmd --
@@ -342,6 +453,7 @@ windDumpCmd(w, cmd)
     (void) windDump();
 }
 
+#ifndef MAGIC_WRAPPER
 
 /*
  * ----------------------------------------------------------------------------
@@ -382,6 +494,8 @@ windEchoCmd(w, cmd)
     TxFlush();
 }
 
+#endif
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -419,14 +533,10 @@ windFilesCmd(w, cmd)
     open = unopen = 0;
     for (fd = 0; fd < NUM_FD; fd++) {
 	if (fstat(fd, &buf) != 0) {
-	    if (errno == EBADF) {
+	    if (errno == EBADF)
 		unopen++;
-	    } else {
-		if (errno < sys_nerr)
-		    TxError("file descriptor %d: %s\n", fd, STRERROR(errno));
-		else
-		    TxError("file descriptor %d: unknown error\n", fd);
-	    }
+	    else
+		TxError("file descriptor %d: %s\n", fd, strerror(errno));
 	}
 	else {
 	    char *type;
