@@ -18,7 +18,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdSZ.c,v 1.3 2001/01/12 22:13:34 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include <math.h>
@@ -38,6 +38,7 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdSZ.c,v 1.3 20
 #include "utils/hash.h"
 #include "database/database.h"
 #include "dbwind/dbwind.h"
+#include "graphics/graphics.h"
 #include "misc/patchlevel.h"
 
 
@@ -49,13 +50,18 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windCmdSZ.c,v 1.3 20
  *	Scroll the view around
  *
  * Usage:
- *	scroll [dir [amount]]
+ *	scroll dir [amount [units]]
  *
  * Results:
  *	None.
  *
  * Side effects:
  *	The window underneath the cursor is changed.
+ *	Note: behavior has been changed from original.  To have "amount"
+ *	parsed as a fractional scroll amount, "units" *must* be declared
+ *	as "w".  Otherwise, no units implies that "amount" is an absolute
+ *	value.
+ *	
  *
  * ----------------------------------------------------------------------------
  */
@@ -70,11 +76,11 @@ windScrollCmd(w, cmd)
     Point p;
     int pos, locargc = cmd->tx_argc;
     float amount;
-    char unitc = 'w';	/* Default units in window width or height */
+    bool doFractional = FALSE;
 
-    if ( (cmd->tx_argc < 1) || (cmd->tx_argc > 4) )
+    if ( (cmd->tx_argc < 2) || (cmd->tx_argc > 4) )
     {
-	TxError("Usage: %s [direction [amount [units]]]\n", cmd->tx_argv[0]);
+	TxError("Usage: %s direction [amount [units]]\n", cmd->tx_argv[0]);
 	return 0;
     }
 
@@ -93,26 +99,47 @@ windScrollCmd(w, cmd)
     if (pos < 0 || pos == GEO_CENTER)
 	return 0;
 
-    if (cmd->tx_argc >= 3)
+    if (cmd->tx_argc == 2)	/* default = half-screen pan */
     {
-	if (sscanf(cmd->tx_argv[2], "%f", &amount) != 1)
+	r = w->w_screenArea;
+	amount = 0.5;
+	doFractional = TRUE;
+    }
+    else if (cmd->tx_argc == 4)
+    {
+	char unitc = cmd->tx_argv[3][0];
+
+	if (unitc == 'w')
+	    r = w->w_screenArea;
+	else if (unitc == 'l')
+	    r = *(w->w_bbox);
+	else
 	{
-	    TxError("Usage: %s [direction [amount [units]]]\n", cmd->tx_argv[0]);
+	    TxError("Usage: %s direction [amount [units]]\n", cmd->tx_argv[0]);
+	    TxError("  'units' must be one of 'w' (window) or 'l' (layout);\n");
 	    return 0;
 	}
-	if (cmd->tx_argc == 4)
-	    unitc = cmd->tx_argv[3][0];
+
+	if (sscanf(cmd->tx_argv[2], "%f", &amount) != 1)
+	{
+	    TxError("Usage: %s direction [amount [units]]\n", cmd->tx_argv[0]);
+	    TxError("  'amount' is a fractional value.\n");
+	    return 0;
+	}
+	doFractional = TRUE;
+    }
+
+    if (doFractional)
+    {
+	xsize = (r.r_xtop - r.r_xbot) * amount;
+	ysize = (r.r_ytop - r.r_ybot) * amount;
     }
     else
-	amount = 0.5;
-	
-    if (unitc == 'w')
-	r = w->w_screenArea;
-    else if (unitc == 'l')
-	r = *(w->w_bbox);
+    {
+	/* Alternate syntax: parse for integer coordinate amount to scroll */
+	xsize = ysize = cmdParseCoord(cmd->tx_argv[2]);
+    }
 
-    xsize = (r.r_xtop - r.r_xbot) * amount;
-    ysize = (r.r_ytop - r.r_ybot) * amount;
     p.p_x = 0;
     p.p_y = 0;
 
@@ -148,7 +175,16 @@ windScrollCmd(w, cmd)
 	    break;
     }
 
-    WindScroll(w, (Point *) NULL, &p);
+    if (doFractional)
+	WindScroll(w, (Point *) NULL, &p);
+    else
+    {
+	/* Direction is reversed w/respect to above call to WindScroll() */
+	p.p_x = -p.p_x;
+	p.p_y = -p.p_y;
+	WindScroll(w, &p, (Point *) NULL);
+    }
+
     return 0;
 }
 
@@ -267,6 +303,8 @@ windSleepCmd(w, cmd)
     }
 }
 
+#ifndef MAGIC_WRAPPER
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -311,6 +349,8 @@ windSourceCmd(w, cmd)
     };
 }
 
+#endif
+
 /*
  * ----------------------------------------------------------------------------
  * windSpecialOpenCmd --
@@ -385,23 +425,75 @@ usage:
 #ifdef MAGIC_WRAPPER
 /*
  * ----------------------------------------------------------------------------
- *  windTkPathCmd --
+ *  windNamesCmd --
  *
- *	Register or retrieve the Tk path associated with the layout window
+ *	Register or retrieve the name associated with the layout window
  *
  * Results:
- *	Returns the Tk path as a Tcl result
+ *	Returns the name as a Tcl string result.  If "all" is selected,
+ *	or if w is NULL and cannot be determined, returns a list of all
+ *	window names.  The first argument may also be a window client
+ *	type, in which case only windows of that type are returned.
+ *
+ * Note:
+ *	This routine can easily be made "generic", not Tcl-specific.
+ *	However, of the graphics interfaces available, only Tcl/Tk keeps
+ *	track of windows by name.
  *
  * ----------------------------------------------------------------------------
  */
 
-void
-windTkPathCmd(w, cmd)
+windNamesCmd(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-}
+    bool doforall = False;
+    WindClient wc = NULL;
+    MagWindow *sw;
 
+    if (cmd->tx_argc > 2)
+    {
+	TxError("Usage:  windownames [all | client_type]\n");
+	return;
+    }
+    if (cmd->tx_argc == 2)
+    {
+	if (!strncmp(cmd->tx_argv[1], "all", 3))
+	    doforall = True;
+	else
+	{
+	    wc = WindGetClient(cmd->tx_argv[1]);
+	    if (wc == (WindClient) NULL)
+	    {
+		TxError("Usage:  windownames [all | client_type]\n");
+		TxPrintf("Valid window types are:\n");
+		WindPrintClientList(FALSE);
+		return;
+	    }
+	    doforall = True;
+	}
+    }
+
+    if (cmd->tx_argc == 1)
+    {
+	wc = DBWclientID;
+	windCheckOnlyWindow(&w, wc);
+	if (w == (MagWindow *)NULL)
+	    doforall = True;
+    }
+
+    if (doforall == True)
+    {
+	for (sw = windTopWindow; sw != NULL; sw = sw->w_nextWindow)
+	    if ((wc == NULL) || (sw->w_client == wc))
+		Tcl_AppendElement(magicinterp, (*GrWindowNamePtr)(sw));
+    }
+    else
+    {
+	if (GrWindowNamePtr)
+	    Tcl_SetResult(magicinterp, (*GrWindowNamePtr)(w), NULL);
+    }
+}
 #endif
 
 
@@ -593,56 +685,7 @@ windViewCmd(w, cmd)
     if (w == NULL)
 	return;
 
-    if ((cmd->tx_argc == 2) && (!strcmp(cmd->tx_argv[1], "get")))
-    {
-	int viewl, viewu, bbox;
-	float f1, f2;
-#ifdef MAGIC_WRAPPER
-	Tcl_Obj *listxy, *fval;
-	
-	listxy = Tcl_NewListObj(0, NULL);
-#endif
-
-	bbox = MAX(1, w->w_bbox->r_xtop - w->w_bbox->r_xbot + 1);
-	viewl = w->w_surfaceArea.r_xbot - w->w_bbox->r_xbot + 1;
-	viewu = w->w_surfaceArea.r_xtop - w->w_bbox->r_xbot + 1;
-
-	f1 = (float)viewl / (float)bbox;
-	if (f1 < 0.0) f1 = 0.0;
-	if (f1 > 1.0) f1 = 1.0;
-
-	f2 = (float)viewu / (float)bbox;
-	if (f2 < 0.0) f2 = 0.0;
-	if (f2 > 1.0) f2 = 1.0;
-
-#ifndef MAGIC_WRAPPER
-	TxPrintf("X = (%2.1f, %2.1f) ", f1, f2);
-#else
-	Tcl_ListObjAppendElement(magicinterp, listxy, Tcl_NewDoubleObj((double)f1));
-	Tcl_ListObjAppendElement(magicinterp, listxy, Tcl_NewDoubleObj((double)f2));
-#endif
-
-	bbox = MAX(1, w->w_bbox->r_ytop - w->w_bbox->r_ybot + 1);
-	viewl = w->w_surfaceArea.r_ybot - w->w_bbox->r_ybot + 1;
-	viewu = w->w_surfaceArea.r_ytop - w->w_bbox->r_ybot + 1;
-
-	f1 = 1.0 - ((float)viewu / (float)bbox);
-	if (f1 < 0.0) f1 = 0.0;
-	if (f1 > 1.0) f1 = 1.0;
-
-	f2 = 1.0 - ((float)viewl / (float)bbox);
-	if (f2 < 0.0) f2 = 0.0;
-	if (f2 > 1.0) f2 = 1.0;
-
-#ifndef MAGIC_WRAPPER
-	TxPrintf("Y = (%2.1f, %2.1f)\n", f1, f2);
-#else
-	Tcl_ListObjAppendElement(magicinterp, listxy, Tcl_NewDoubleObj((double)f1));
-	Tcl_ListObjAppendElement(magicinterp, listxy, Tcl_NewDoubleObj((double)f2));
-	Tcl_SetObjResult(magicinterp, listxy);
-#endif
-    }
-    else if (cmd->tx_argc == 1)
+    if (cmd->tx_argc == 1)
     {
 	if ((w->w_flags & WIND_SCROLLABLE) == 0) {
 	    TxError("Sorry, can't zoom out this window.\n");
@@ -650,6 +693,66 @@ windViewCmd(w, cmd)
 	}
 	WindView(w);
     }
+    else if (cmd->tx_argc == 2)
+    {
+#ifdef MAGIC_WRAPPER
+	Tcl_Obj *listxy, *fval;
+	
+	listxy = Tcl_NewListObj(0, NULL);
+#endif
+
+	if (!strncmp(cmd->tx_argv[1], "get", 3))
+	{
+#ifndef MAGIC_WRAPPER
+	    TxPrintf("(%d, %d) to (%d, %d)\n", 
+			w->w_surfaceArea.r_xbot, w->w_surfaceArea.r_ybot,
+			w->w_surfaceArea.r_xtop, w->w_surfaceArea.r_ytop);
+#else
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_surfaceArea.r_xbot));
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_surfaceArea.r_ybot));
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_surfaceArea.r_xtop));
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_surfaceArea.r_ytop));
+	    Tcl_SetObjResult(magicinterp, listxy);
+#endif
+	}
+	else if (!strncmp(cmd->tx_argv[1], "bbox", 4))
+	{
+#ifndef MAGIC_WRAPPER
+	    TxPrintf("(%d, %d) to (%d, %d)\n", 
+			w->w_bbox->r_xbot, w->w_bbox->r_ybot,
+			w->w_bbox->r_xtop, w->w_bbox->r_ytop);
+#else
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_bbox->r_xbot));
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_bbox->r_ybot));
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_bbox->r_xtop));
+	    Tcl_ListObjAppendElement(magicinterp, listxy,
+			Tcl_NewIntObj((int)w->w_bbox->r_ytop));
+	    Tcl_SetObjResult(magicinterp, listxy);
+#endif
+	}
+    }
+    else if (cmd->tx_argc == 5)
+    {
+	Rect r;
+	r.r_xbot = cmdParseCoord(cmd->tx_argv[1]);
+        r.r_ybot = cmdParseCoord(cmd->tx_argv[2]);
+        r.r_xtop = cmdParseCoord(cmd->tx_argv[3]);
+        r.r_ytop = cmdParseCoord(cmd->tx_argv[4]);
+
+	/* Redisplay */
+	WindMove(w, &r);
+    }
+    else
+    {
+	TxError("Usage: view [get|bbox|llx lly urx ury]\n");
+    }
 }
 
 
@@ -768,6 +871,9 @@ windScrollBarsCmd(w, cmd)
 	TxError("Usage: %s [on|off]\n", cmd->tx_argv[0]);
 	return;
 }
+
+#ifndef MAGIC_WRAPPER
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -818,7 +924,7 @@ windSendCmd(w, cmd)
     }
     newcmd.tx_wid = WIND_UNKNOWN_WINDOW;
     if (toWindow != NULL) newcmd.tx_wid = toWindow->w_wid;
-    (void) WindSendCommand(client, &newcmd);
+    (void) WindSendCommand(toWindow, &newcmd);
     return;
 
     usage:
@@ -837,6 +943,16 @@ windSendCmdFunc(w, cd)
     return 1;
 }
 
+#endif
+
+/* Structure used by "position" command to pass to WindSearch	*/
+/* as client data.						*/
+
+typedef struct _cdwpos {
+    FILE *file;
+    bool doFrame;
+} cdwpos;
+   
 
 /*
  * ----------------------------------------------------------------------------
@@ -863,26 +979,39 @@ windPositionsCmd(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    FILE *file;
     extern int windPositionsFunc();
-    bool haveFile;
+    char *filename = NULL;
+    cdwpos windpos;
 
-    if (cmd->tx_argc > 2) goto usage;
-    haveFile = (cmd->tx_argc == 2);
+    windpos.doFrame = FALSE;
+    windpos.file = stdout;
 
-    if (haveFile) {
-	file = fopen(cmd->tx_argv[1], "w");
-	if (file == (FILE *) NULL) {
-	    TxError("Could not open file %s for writing.\n", cmd->tx_argv[1]);
+    if (cmd->tx_argc > 3) goto usage;
+    if (cmd->tx_argc > 1)
+    {
+	if (!strncmp(cmd->tx_argv[1], "frame", 5))
+	{
+	   windpos.doFrame = TRUE;
+	   if (cmd->tx_argc == 3)
+		filename = cmd->tx_argv[2];
+	}
+	else if (cmd->tx_argc == 2)
+	   filename = cmd->tx_argv[1];
+	else
+	    goto usage;
+    }
+
+    if (filename) {
+	windpos.file = fopen(filename, "w");
+	if (windpos.file == (FILE *) NULL)
+	{
+	    TxError("Could not open file %s for writing.\n", filename);
 	    return;
 	};
     }
-    else {
-	file = stdout;
-    }
     (void) WindSearch((WindClient) NULL, (ClientData) NULL, (Rect *) NULL, 
-	windPositionsFunc, (ClientData) file);
-    if (haveFile) (void) fclose(file);
+	windPositionsFunc, (ClientData) &windpos);
+    if (filename) (void) fclose(windpos.file);
     return;
     
 usage:
@@ -895,26 +1024,38 @@ windPositionsFunc(w, cdata)
     MagWindow *w;
     ClientData cdata;
 {
-    if ((FILE *)cdata == stdout)
+    cdwpos *windpos = (cdwpos *)cdata;
+    Rect r;
+
+    if (windpos->doFrame)
+	r = w->w_frameArea;
+    else
+	r = w->w_screenArea;
+
+    if (windpos->file == stdout)
 #ifdef MAGIC_WRAPPER
     {
-	char *estr = (char *)Tcl_Alloc(50);
-	sprintf(estr, "%d %d %d %d %s\n", 
-		w->w_frameArea.r_xbot, w->w_frameArea.r_ybot, 
-		w->w_frameArea.r_xtop, w->w_frameArea.r_ytop, 
-		((clientRec *) w->w_client)->w_clientName);
-	Tcl_SetResult(magicinterp, estr, TCL_DYNAMIC);
+	Tcl_Obj *lobj;
+
+	lobj = Tcl_NewListObj(0, NULL);
+
+	Tcl_ListObjAppendElement(magicinterp, lobj, Tcl_NewIntObj((int)r.r_xbot));
+	Tcl_ListObjAppendElement(magicinterp, lobj, Tcl_NewIntObj((int)r.r_ybot));
+	Tcl_ListObjAppendElement(magicinterp, lobj, Tcl_NewIntObj((int)r.r_xtop));
+	Tcl_ListObjAppendElement(magicinterp, lobj, Tcl_NewIntObj((int)r.r_ytop));
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+		Tcl_NewStringObj(((clientRec *)w->w_client)->w_clientName,
+		strlen(((clientRec *)w->w_client)->w_clientName)));
+	Tcl_SetObjResult(magicinterp, lobj);
     }
 #else
 	TxPrintf("specialopen %d %d %d %d %s\n", 
-		w->w_frameArea.r_xbot, w->w_frameArea.r_ybot, 
-		w->w_frameArea.r_xtop, w->w_frameArea.r_ytop, 
+		r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop,
 		((clientRec *) w->w_client)->w_clientName);
 #endif
     else
 	fprintf((FILE *)cdata, "specialopen %d %d %d %d %s\n", 
-		w->w_frameArea.r_xbot, w->w_frameArea.r_ybot, 
-		w->w_frameArea.r_xtop, w->w_frameArea.r_ytop, 
+		r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop,
 		((clientRec *) w->w_client)->w_clientName);
     return 0;
 }
