@@ -17,7 +17,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/windows/windDisp.c,v 1.3 2001/01/12 22:13:35 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 
@@ -70,15 +70,31 @@ bool windSomeSeparateRedisplay = FALSE;
  * generate bothersome messages about the cursor not being in a window,
  * because there is no confusion.
  *
+ * We want to make sure that we don't count windows which are not of
+ * the same client type.
+ *
  * ----------------------------------------------------------------------------
  */
 
-Void windCheckOnlyWindow(MagWindow **w)
+Void windCheckOnlyWindow(MagWindow **w, WindClient client)
 {
-   if (*w == NULL)
-      if (windTopWindow != NULL)
-         if (windTopWindow->w_nextWindow == NULL)
-	    *w = windTopWindow;
+    MagWindow *sw, *tw;
+    int wct = 0;
+
+    if (*w != NULL) return;
+
+    if (windTopWindow != NULL)
+    {
+	for (sw = windTopWindow; sw != NULL; sw = sw->w_nextWindow)
+	{
+	    if (sw->w_client == client)
+	    {
+		wct++;
+		tw = sw;
+	    }
+	}
+	if (wct == 1) *w = tw;
+    }
 }
 
 
@@ -440,47 +456,54 @@ windDrawBorder(w, clip)
     int bar, bbox, viewl, viewu;
     Point capp;
     Rect capr;
-#ifdef MAGIC_WRAPPER
-    char tclcommand[150];
-#endif
 
     GrLock(w, FALSE);
     GrClipTo(clip);
 
+
     /* Redisplay the caption if it overlaps the area. */
+
     capr = w->w_allArea;
     capr.r_ybot = capr.r_ytop - TOP_BORDER(w) + GrPixelCorrect;
     capp.p_x = (capr.r_xbot + capr.r_xtop) / 2;
     capp.p_y = (capr.r_ybot + capr.r_ytop + 1) / 2;
     if (GEO_TOUCH(&capr, clip)) {
-	GrClipBox(&capr, STYLE_BORDER);
+	if (w->w_flags & WIND_BORDER)
+	    GrClipBox(&capr, STYLE_BORDER);
 	if ((w->w_flags & WIND_CAPTION) && (w->w_caption != NULL)) {
 	    (void) GrPutText(w->w_caption, STYLE_CAPTION, &capp, 
-			GEO_CENTER, GR_TEXT_DEFAULT, FALSE, &capr, (Rect *) NULL);
+			GEO_CENTER, GR_TEXT_DEFAULT, FALSE, &capr,
+			(Rect *) NULL);
 	}
     }
 
-    /* right border */
-    r = w->w_allArea;
-    r.r_xbot = w->w_allArea.r_xtop - RIGHT_BORDER(w) + GrPixelCorrect;
-    r.r_ytop = w->w_allArea.r_ytop - TOP_BORDER(w);
-    if (GEO_TOUCH(&r, clip)) GrClipBox(&r, STYLE_BORDER);
-
-    if ((w->w_flags & WIND_SCROLLBARS) == 0) {
-	/* windows without scroll bars */
-
-	/* left border */
+    if ((w->w_flags & WIND_BORDER) != 0)
+    {
+	/* right border */
 	r = w->w_allArea;
-	r.r_xtop = w->w_allArea.r_xbot + LEFT_BORDER(w) - GrPixelCorrect;
+	r.r_xbot = w->w_allArea.r_xtop - RIGHT_BORDER(w) + GrPixelCorrect;
 	r.r_ytop = w->w_allArea.r_ytop - TOP_BORDER(w);
 	if (GEO_TOUCH(&r, clip)) GrClipBox(&r, STYLE_BORDER);
 
-	/* bottom border */
-	r = w->w_allArea;
-	r.r_ytop = w->w_allArea.r_ybot + BOT_BORDER(w) - GrPixelCorrect;
-	if (GEO_TOUCH(&r, clip)) GrClipBox(&r, STYLE_BORDER);
+	if ((w->w_flags & WIND_SCROLLBARS) == 0)
+	{
+	    /* windows without scroll bars */
+
+	    /* left border */
+	    r = w->w_allArea;
+	    r.r_xtop = w->w_allArea.r_xbot + LEFT_BORDER(w) - GrPixelCorrect;
+	    r.r_ytop = w->w_allArea.r_ytop - TOP_BORDER(w);
+	    if (GEO_TOUCH(&r, clip)) GrClipBox(&r, STYLE_BORDER);
+
+	    /* bottom border */
+	    r = w->w_allArea;
+	    r.r_ytop = w->w_allArea.r_ybot + BOT_BORDER(w) - GrPixelCorrect;
+	    if (GEO_TOUCH(&r, clip)) GrClipBox(&r, STYLE_BORDER);
+	}
     }
-    else {
+
+    if ((w->w_flags & WIND_SCROLLBARS) != 0)
+    {
 	/* windows with scroll bars */
 
 	/* left vertical lines */
@@ -796,7 +819,8 @@ WindUpdate()
 	}
     }
 
-    if (WindPackageType == WIND_MAGIC_WINDOWS) {
+    if (WindPackageType == WIND_MAGIC_WINDOWS)
+    {
 	/* For any tiles left that intersect the screen, draw the background
 	 * color (there are no windows over these tiles).
 	 */
