@@ -45,7 +45,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/windows/windMain.c,v 1.2 2001/01/12 22:13:35 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -60,6 +60,8 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windMain.c,v 1.2 200
 #include "textio/textio.h"
 #include "graphics/graphics.h"
 #include "utils/malloc.h"
+#include "utils/utils.h"
+#include "textio/txcommands.h"
 
 /* The type of windows that this package will implement */
 int WindPackageType = WIND_MAGIC_WINDOWS;
@@ -195,7 +197,7 @@ WindInit()
  *	( A command has been issued to this window.  The client should
  *	  process it.  It is split into Unix-style argc and argv words. )
  *
- *	command(w, cmd)
+ *	command(w, client, cmd)
  *	    MagWindow *w;
  *	    TxCommand *cmd;
  *	{
@@ -215,7 +217,7 @@ WindInit()
 
 WindClient
 WindAddClient(clientName, create, delete, redisplay, command, update, 
-exitproc, reposition, table, icon)
+exitproc, reposition, icon)
     char *clientName;		/* A textual name for the client.  This
 				 * name will be visable in the user
 				 * interface as the name to use to switch
@@ -228,9 +230,6 @@ exitproc, reposition, table, icon)
     Void (*update)();
     Void (*exitproc)();
     Void (*reposition)();
-    char *(table[]);		/* A table of commands that this client
-				 * is willing to accept.
-				 */
     GrGlyph *icon;		/* An icon to draw when the window is closed.
 				 * (currently for Sun Windows only).
 				 */
@@ -250,9 +249,18 @@ exitproc, reposition, table, icon)
     res->w_update = update;
     res->w_exit = exitproc;
     res->w_reposition = reposition;
-    res->w_commandTable = table;
     res->w_icon = icon;
     res->w_nextClient = windFirstClientRec;
+
+    /* The command and function tables are dynamically allocated.  */
+    /* Commands and functions should be registered with the client */
+    /* using the WindAddCommand() function.			   */
+
+    res->w_commandTable = (char **)mallocMagic(sizeof(char *));
+    *(res->w_commandTable) = NULL;
+    res->w_functionTable = (Void (**)())mallocMagic(sizeof(Void (*)()));
+    *(res->w_functionTable) = NULL;
+
     windFirstClientRec = res;
 
     return (WindClient) res;
@@ -324,3 +332,199 @@ WindPrintClientList(wizard)
 	    TxError("	%s\n", cr->w_clientName);
 	}
 }
+
+/*
+ * ----------------------------------------------------------------------------
+ * WindExecute --
+ *
+ *	Execute the command associated with a windClient
+ *
+ * Results:
+ *	Returns the command index on success.  Returns -1 if the
+ *	command was not found in the client's command list.  Returns
+ *	-2 if the procedure was sent an empty command.
+ *
+ * Side effects:
+ *	Whatever is done by the command execution.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+WindExecute(w, rc, cmd)
+    MagWindow *w;
+    WindClient rc;
+    TxCommand *cmd;
+{
+    int cmdNum;
+    clientRec *client = (clientRec *) rc;
+    char **commandTable = client->w_commandTable;
+    Void (**functionTable)() = client->w_functionTable;
+
+    if (cmd->tx_argc > 0)
+    {
+	cmdNum = Lookup(cmd->tx_argv[0], commandTable);
+
+	if (cmdNum >= 0)
+	{
+	    (void) (*functionTable[cmdNum])(w, cmd);
+	    return cmdNum;
+	}
+	return -1;
+    }
+    return -2;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * WindAddCommand --
+ *
+ *	Add a command to the indicated client.  The command is passed
+ *	in "text", which also contains the (1-line) help text for the
+ *	command.  "func" is a function pointer, and "volatile" is TRUE
+ *	if the command "text" is dynamically allocated and must be
+ *	copied before adding to the client.
+ *
+ * Results:
+ *	None
+ *
+ * Side effects:
+ *	The memory allocated to the command and function pointers may
+ *	be reallocated and the entries in the client record updated.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+WindAddCommand(rc, text, func, dynamic)
+    WindClient rc;
+    char *text;
+    Void (*func)();
+    bool dynamic;
+{
+    int cidx, numCommands = 0;
+    clientRec *client = (clientRec *) rc;
+    char **commandTable = client->w_commandTable;
+    Void (**functionTable)() = client->w_functionTable;
+    char **newcmdTable;
+    Void (**newfnTable)();
+
+    /* Find the number of commands and functions, increment by one, and */
+    /* Allocate a new array of pointers.				*/
+
+    while (commandTable[numCommands] != NULL) numCommands++;
+    numCommands++;
+
+    newcmdTable = (char **)mallocMagic((numCommands + 1) * sizeof(char *));
+    newfnTable = (Void (**)())mallocMagic((numCommands + 1) * sizeof(Void (*)()));
+
+    /* Copy the old values, inserting the new command in alphabetical	*/
+    /* order.								*/
+
+    for (cidx = 0; (commandTable[cidx] != NULL) && 
+		(strcmp(commandTable[cidx], text) < 0); cidx++)
+    {
+	newcmdTable[cidx] = commandTable[cidx];
+	newfnTable[cidx] = functionTable[cidx];
+    }
+
+    if (dynamic)
+	newcmdTable[cidx] = StrDup(NULL, text);
+    else
+	newcmdTable[cidx] = text;
+    newfnTable[cidx] = func;
+
+    for (; commandTable[cidx] != NULL; cidx++)
+    {
+	newcmdTable[cidx + 1] = commandTable[cidx];
+	newfnTable[cidx + 1] = functionTable[cidx];
+    }
+
+    newcmdTable[cidx + 1] = NULL;
+
+    /* Release memory for the original pointers, and replace the	*/
+    /* pointers in the client record.					*/
+
+    freeMagic(commandTable);
+    freeMagic(functionTable);
+
+    client->w_commandTable = newcmdTable;
+    client->w_functionTable = newfnTable;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * WindReplaceCommand --
+ *
+ *	Change the function for the indicated command.  This routine
+ *	is mainly used by the Tcl module interface, where commands
+ *	are registered with the command interpreter pointing to an
+ *	"auto" function which loads the module, then calls this routine
+ *	to replace the auto-load function with the real one.
+ *
+ *      Note that this routine matches to the length of "command", then
+ *	checks one character beyond in the command table to ensure that
+ *	we don't inadvertently change a command which happens to be a
+ *	substring of the intended command.  In cases where this is
+ *	intended (e.g., "ext" and "extract"), the routine must be called
+ *	separately for each command string.
+ *
+ * Results:
+ *	0 on success, -1 if the command was not found.
+ *
+ * Side effects:
+ *	The clientRec structure for the DBWind interface is altered.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+WindReplaceCommand(rc, command, newfunc)
+    WindClient rc;
+    char *command;
+    Void (*newfunc)();
+{
+    int cidx, clen;
+    clientRec *client = (clientRec *) rc;
+    char **commandTable = client->w_commandTable;
+    Void (**functionTable)() = client->w_functionTable;
+
+    clen = strlen(command);
+
+    for (cidx = 0; commandTable[cidx] != NULL; cidx++)
+	if (!strncmp(commandTable[cidx], command, clen))
+	    if (!isalnum(*(commandTable[cidx] + clen)))
+	    {
+		functionTable[cidx] = newfunc;
+		return 0;
+	    }
+
+    return -1;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ * WindGetCommandTable --
+ * 
+ *	For functions wishing to parse the command table of a client
+ *	directly, this routine returns a pointer to the top of the
+ *	table.  The only purpose of this routine is to export the
+ *	w_commandTable value inside the clientRec structure, which is
+ *	not itself exported.
+ *
+ * Results:
+ *	A pointer to the top of the command table of the indicated
+ *	client.
+ *
+ * Side effects:
+ *	None.
+ * ----------------------------------------------------------------------------
+ */
+
+char **
+WindGetCommandTable(rc)
+    WindClient rc;
+{
+    clientRec *client = (clientRec *) rc;
+    return client->w_commandTable;
+}
