@@ -19,7 +19,7 @@
 
 #ifndef lint
 static char rcsid[]="$Header: /ufs/repository/magic/windows/windMove.c,v 1.2 2001/01/12 22:13:36 jsolomon Exp $";
-#endif  not lint
+#endif  /* not lint */
 
 #include <stdio.h>
 #include "misc/magic.h"
@@ -41,9 +41,12 @@ static char rcsid[]="$Header: /ufs/repository/magic/windows/windMove.c,v 1.2 200
 static MagWindow *sharedW;
 
 /*
- * Should new windows have scroll bars and/or a title caption?
+ * By default, new windows have scroll bars, border, a title caption,
+ * and allow standard window commands.
  */
-bool windDefaultFlags = WIND_SCROLLBARS | WIND_CAPTION;
+bool windDefaultFlags = WIND_SCROLLBARS | WIND_CAPTION |
+			WIND_BORDER | WIND_COMMANDS |
+			WIND_SCROLLABLE;
 
 /*
  * A mask of the current window IDs, as well as a limit on the number of
@@ -198,7 +201,7 @@ windSetWindowPosition(w)
 	case WIND_SUN_WINDOWS:
 #ifdef	SUNVIEW
 	    windSunSetWindowPosition(w);
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
 	    break;
     }
 }
@@ -287,7 +290,6 @@ WindCreate(client, frameArea, isHint, argc, argv)
     w = (MagWindow *) mallocMagic( sizeof(MagWindow) );
     w->w_client = client;
     w->w_flags = windDefaultFlags;
-    w->w_flags |= WIND_SCROLLABLE;
     w->w_clipAgainst = (LinkedRect *) NULL;
     w->w_caption = (char *) NULL;
     w->w_stippleOrigin.p_x = 0;
@@ -328,7 +330,7 @@ WindCreate(client, frameArea, isHint, argc, argv)
     /* Don't set window positions ourselves. */
     if ((WindPackageType == WIND_SUN_WINDOWS) && isHint)
 	GrSunFigureLocation(&w->w_frameArea);
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
     WindSetWindowAreas(w);
 
     /* now link the window in on top */
@@ -343,6 +345,10 @@ WindCreate(client, frameArea, isHint, argc, argv)
     /* notify the client */
     OK = ( (cr->w_create == NULL) || (*(cr->w_create))(w, argc, argv) );
 
+#ifdef THREE_D
+    if (strcmp(cr->w_clientName, "wind3d"))
+#endif
+
     if (GrCreateWindowPtr != NULL) OK &= (*GrCreateWindowPtr)(w,
 		(argc > 1) ? argv[1] : NULL);
 
@@ -451,7 +457,7 @@ WindUnder(w)
 	case WIND_SUN_WINDOWS:
 #ifdef	SUNVIEW
 	    windSunUnder(w);
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
 	    break;
 	default:
 	    /* Mark for redisplay all the areas that this window
@@ -514,7 +520,7 @@ WindOver(w)
 	case WIND_SUN_WINDOWS:
 #ifdef	SUNVIEW
 	    windSunOver(w);
-#endif	SUNVIEW
+#endif	/* SUNVIEW */
 	    break;
 	
 	default:
