@@ -23,10 +23,10 @@
 
 #ifndef	_MAGIC
 int err0 = Need_to_include_misc/magic_header;
-#endif	_MAGIC
+#endif	/* _MAGIC */
 #ifndef	_DATABASE
 int err0 = Need_to_include_database/database_header;
-#endif	_DATABASE
+#endif	/* _DATABASE */
 
 /* Table that defines the shape of contacts and the layers that they
  * connect.  This definition allows some layers to extend around the
